@@ -1,3 +1,4 @@
+/* -*- C -*- */
 static const char THANKS [] = ""
 "To Sveta and Max Muntyan,\n"
 "\n"
@@ -5,11 +6,14 @@ static const char THANKS [] = ""
 "\n"
 "Andreas Hanke\n"
 "Andrey Fedoseev\n"
+"Arnau Sanchez\n"
 "Chaosye\n"
 "Christian Dywan\n"
 "Collilieux\n"
+"Daniel Butzu\n"
 "David Hough\n"
 "Dmytro Savchuk\n"
+"Everaldo Coelho\n"
 "Federico Mena Quintero\n"
 "Geoffrey French\n"
 "Jeroen Zwartepoorte\n"
@@ -25,4 +29,5 @@ static const char THANKS [] = ""
 "Thomas Gilgin\n"
 "Tim Janik\n"
 "Tim-Philipp Müller\n"
+"Vlastimil Ott\n"
 ;
