@@ -1,12 +1,11 @@
 /*
  *   mooapp/mooapp.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -23,20 +22,21 @@
 #include "mooedit/mooeditprefs.h"
 #include "mooedit/mooeditor.h"
 #include "mooedit/mooplugin.h"
-#include "mooedit/mooedit-script.h"
 #include "mooedit/moousertools.h"
 #include "mooedit/moousertools-prefs.h"
 #include "mooedit/plugins/mooeditplugins.h"
 #include "mooutils/mooprefsdialog.h"
 #include "mooutils/moopython.h"
 #include "mooutils/moomarshals.h"
-#include "mooutils/moocompat.h"
 #include "mooutils/moodialogs.h"
 #include "mooutils/moostock.h"
 #include "mooutils/mooutils-fs.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/mooutils-debug.h"
 #include "mooutils/mooi18n.h"
 #include "mooutils/xdgmime/xdgmime.h"
+#include "mooutils/moohelp.h"
+#include <glib/gmappedfile.h>
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
@@ -58,31 +58,46 @@
 #define MOO_ACTIONS_FILE    "actions"
 #endif
 
-static MooApp *moo_app_instance = NULL;
-static MooAppInput *moo_app_input = NULL;
+#define SESSION_VERSION "1.0"
+
+#ifndef __WIN32__
+#define TMPL_RC_FILE    "%src"
+#else
+#define TMPL_RC_FILE    "%s.ini"
+#endif
+#define TMPL_STATE_FILE "%s.state"
+
+static struct {
+    MooApp *instance;
+    MooAppInput *input;
+    gboolean atexit_installed;
+} moo_app_data;
 
 static volatile int signal_received;
 
-
 struct _MooAppPrivate {
     char      **argv;
     int         exit_code;
     MooEditor  *editor;
     MooAppInfo *info;
     char       *rc_files[2];
+
     gboolean    run_input;
+    char       *instance_name;
 
     gboolean    running;
     gboolean    in_try_quit;
 
+    int         use_session;
     EggSMClient *sm_client;
+    char       *session_file;
+    MooMarkupDoc *session;
 
     MooUIXML   *ui_xml;
     char       *default_ui;
     guint       quit_handler_id;
     gboolean    use_editor;
-
-    char       *tmpdir;
+    gboolean    quit_on_editor_close;
 };
 
 
@@ -100,6 +115,8 @@ static void     moo_app_info_free       (MooAppInfo         *info);
 static void     install_common_actions  (void);
 static void     install_editor_actions  (void);
 
+static void     moo_app_help            (GtkWidget          *window);
+
 static void     moo_app_set_property    (GObject            *object,
                                          guint               prop_id,
                                          const GValue       *value,
@@ -120,6 +137,10 @@ static void     moo_app_exec_cmd_real   (MooApp             *app,
                                          char                cmd,
                                          const char         *data,
                                          guint               len);
+static void     moo_app_load_session_real (MooApp           *app,
+                                         MooMarkupNode      *xml);
+static void     moo_app_save_session_real (MooApp           *app,
+                                         MooMarkupNode      *xml);
 static GtkWidget *moo_app_create_prefs_dialog (MooApp       *app);
 
 static void     moo_app_load_prefs      (MooApp             *app);
@@ -133,6 +154,12 @@ static void     moo_app_set_description (MooApp             *app,
 static void     moo_app_set_version     (MooApp             *app,
                                          const char         *version);
 
+static void     moo_app_save_session    (MooApp             *app);
+static void     moo_app_write_session   (MooApp             *app);
+
+static void     moo_app_install_cleanup (void);
+static void     moo_app_cleanup         (void);
+
 static void     start_input             (MooApp             *app);
 
 
@@ -174,12 +201,15 @@ enum {
     PROP_VERSION,
     PROP_DESCRIPTION,
     PROP_RUN_INPUT,
+    PROP_USE_SESSION,
     PROP_USE_EDITOR,
+    PROP_QUIT_ON_EDITOR_CLOSE,
     PROP_DEFAULT_UI,
     PROP_LOGO,
     PROP_WEBSITE,
     PROP_WEBSITE_LABEL,
-    PROP_CREDITS
+    PROP_CREDITS,
+    PROP_INSTANCE_NAME
 };
 
 enum {
@@ -189,6 +219,8 @@ enum {
     TRY_QUIT,
     PREFS_DIALOG,
     EXEC_CMD,
+    LOAD_SESSION,
+    SAVE_SESSION,
     LAST_SIGNAL
 };
 
@@ -214,6 +246,8 @@ moo_app_class_init (MooAppClass *klass)
     klass->try_quit = moo_app_try_quit_real;
     klass->prefs_dialog = moo_app_create_prefs_dialog;
     klass->exec_cmd = moo_app_exec_cmd_real;
+    klass->load_session = moo_app_load_session_real;
+    klass->save_session = moo_app_save_session_real;
 
     g_object_class_install_property (gobject_class,
                                      PROP_ARGV,
@@ -287,6 +321,22 @@ moo_app_class_init (MooAppClass *klass)
                                              TRUE,
                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+    g_object_class_install_property (gobject_class,
+                                     PROP_USE_SESSION,
+                                     g_param_spec_int ("use-session",
+                                             "use-session",
+                                             "use-session",
+                                             -1, 1, -1,
+                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_INSTANCE_NAME,
+                                     g_param_spec_string ("instance-name",
+                                             "instance-name",
+                                             "instance-name",
+                                             NULL,
+                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
     g_object_class_install_property (gobject_class,
                                      PROP_USE_EDITOR,
                                      g_param_spec_boolean ("use-editor",
@@ -295,6 +345,14 @@ moo_app_class_init (MooAppClass *klass)
                                              TRUE,
                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+    g_object_class_install_property (gobject_class,
+                                     PROP_QUIT_ON_EDITOR_CLOSE,
+                                     g_param_spec_boolean ("quit-on-editor-close",
+                                             "quit-on-editor-close",
+                                             "quit-on-editor-close",
+                                             FALSE,
+                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
     g_object_class_install_property (gobject_class,
                                      PROP_DEFAULT_UI,
                                      g_param_spec_string ("default-ui",
@@ -367,21 +425,42 @@ moo_app_class_init (MooAppClass *klass)
                           G_TYPE_CHAR,
                           G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
                           G_TYPE_UINT);
+
+    signals[LOAD_SESSION] =
+            g_signal_new ("load-session",
+                          G_OBJECT_CLASS_TYPE (klass),
+                          G_SIGNAL_RUN_LAST,
+                          G_STRUCT_OFFSET (MooAppClass, load_session),
+                          NULL, NULL,
+                          _moo_marshal_VOID__POINTER,
+                          G_TYPE_NONE, 1,
+                          G_TYPE_POINTER);
+
+    signals[SAVE_SESSION] =
+            g_signal_new ("save-session",
+                          G_OBJECT_CLASS_TYPE (klass),
+                          G_SIGNAL_RUN_LAST,
+                          G_STRUCT_OFFSET (MooAppClass, save_session),
+                          NULL, NULL,
+                          _moo_marshal_VOID__POINTER,
+                          G_TYPE_NONE, 1,
+                          G_TYPE_POINTER);
 }
 
 
 static void
 moo_app_instance_init (MooApp *app)
 {
-    g_return_if_fail (moo_app_instance == NULL);
+    g_return_if_fail (moo_app_data.instance == NULL);
 
     _moo_stock_init ();
 
-    moo_app_instance = app;
+    moo_app_data.instance = app;
 
     app->priv = g_new0 (MooAppPrivate, 1);
-    app->priv->info = moo_app_info_new ();
+    app->priv->use_session = -1;
 
+    app->priv->info = moo_app_info_new ();
     app->priv->info->version = g_strdup (APP_VERSION);
     app->priv->info->website = g_strdup ("http://ggap.sourceforge.net/");
     app->priv->info->website_label = g_strdup ("http://ggap.sourceforge.net");
@@ -415,7 +494,7 @@ moo_app_constructor (GType           type,
     GObject *object;
     MooApp *app;
 
-    if (moo_app_instance != NULL)
+    if (moo_app_data.instance != NULL)
     {
         g_critical ("attempt to create second instance of application class");
         g_critical ("going to crash now");
@@ -433,6 +512,7 @@ moo_app_constructor (GType           type,
 #if defined(HAVE_SIGNAL) && defined(SIGINT)
     setup_signals (sigint_handler);
 #endif
+    moo_app_install_cleanup ();
 
     install_common_actions ();
     install_editor_actions ();
@@ -448,13 +528,17 @@ moo_app_finalize (GObject *object)
 
     moo_app_quit_real (app);
 
-    moo_app_instance = NULL;
+    moo_app_data.instance = NULL;
 
     g_free (app->priv->rc_files[0]);
     g_free (app->priv->rc_files[1]);
     moo_app_info_free (app->priv->info);
     g_free (app->priv->default_ui);
 
+    g_free (app->priv->session_file);
+    if (app->priv->session)
+        moo_markup_doc_unref (app->priv->session);
+
     if (app->priv->argv)
         g_strfreev (app->priv->argv);
     if (app->priv->editor)
@@ -462,6 +546,7 @@ moo_app_finalize (GObject *object)
     if (app->priv->ui_xml)
         g_object_unref (app->priv->ui_xml);
 
+    g_free (app->priv->instance_name);
     g_free (app->priv);
 
     G_OBJECT_CLASS (moo_app_parent_class)->finalize (object);
@@ -513,10 +598,23 @@ moo_app_set_property (GObject        *object,
             app->priv->run_input = g_value_get_boolean (value);
             break;
 
+        case PROP_USE_SESSION:
+            app->priv->use_session = g_value_get_int (value);
+            break;
+
+        case PROP_INSTANCE_NAME:
+            g_free (app->priv->instance_name);
+            app->priv->instance_name = g_value_dup_string (value);
+            break;
+
         case PROP_USE_EDITOR:
             app->priv->use_editor = g_value_get_boolean (value);
             break;
 
+        case PROP_QUIT_ON_EDITOR_CLOSE:
+            app->priv->quit_on_editor_close = g_value_get_boolean (value);
+            break;
+
         case PROP_DEFAULT_UI:
             g_free (app->priv->default_ui);
             app->priv->default_ui = g_strdup (g_value_get_string (value));
@@ -573,11 +671,21 @@ moo_app_get_property (GObject        *object,
         case PROP_RUN_INPUT:
             g_value_set_boolean (value, app->priv->run_input);
             break;
+        case PROP_USE_SESSION:
+            g_value_set_int (value, app->priv->use_session);
+            break;
+        case PROP_INSTANCE_NAME:
+            g_value_set_string (value, app->priv->instance_name);
+            break;
 
         case PROP_USE_EDITOR:
             g_value_set_boolean (value, app->priv->use_editor);
             break;
 
+        case PROP_QUIT_ON_EDITOR_CLOSE:
+            g_value_set_boolean (value, app->priv->quit_on_editor_close);
+            break;
+
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -587,7 +695,7 @@ moo_app_get_property (GObject        *object,
 MooApp*
 moo_app_get_instance (void)
 {
-    return moo_app_instance;
+    return moo_app_data.instance;
 }
 
 
@@ -610,39 +718,6 @@ moo_app_set_exit_code (MooApp      *app,
 }
 
 
-const char *
-moo_app_get_input_pipe_name (G_GNUC_UNUSED MooApp *app)
-{
-    return moo_app_input ? _moo_app_input_get_name (moo_app_input) : NULL;
-}
-
-
-char *
-moo_app_create_user_data_dir (MooApp *app)
-{
-    char *dir;
-    GError *error = NULL;
-
-    g_return_val_if_fail (MOO_IS_APP (app), NULL);
-
-    dir = moo_get_user_data_dir ();
-    g_return_val_if_fail (dir != NULL, NULL);
-
-    if (g_file_test (dir, G_FILE_TEST_IS_DIR))
-        return dir;
-
-    if (!_moo_create_dir (dir, &error))
-    {
-        g_warning ("%s: %s", G_STRLOC, error->message);
-        g_error_free (error);
-        g_free (dir);
-        dir = NULL;
-    }
-
-    return dir;
-}
-
-
 static gboolean
 moo_app_python_run_file (MooApp      *app,
                          const char  *filename)
@@ -654,7 +729,7 @@ moo_app_python_run_file (MooApp      *app,
     g_return_val_if_fail (filename != NULL, FALSE);
     g_return_val_if_fail (moo_python_running (), FALSE);
 
-    file = _moo_fopen (filename, "r");
+    file = _moo_fopen (filename, "rb");
     g_return_val_if_fail (file != NULL, FALSE);
 
     res = moo_python_run_file (file, filename, NULL, NULL);
@@ -714,10 +789,36 @@ moo_app_get_info (MooApp     *app)
 
 
 #ifdef MOO_BUILD_EDIT
+static gboolean
+close_editor_window (MooApp *app)
+{
+    GSList *windows;
+    gboolean ret = FALSE;
+
+    if (!app->priv->running || app->priv->in_try_quit ||
+        !app->priv->quit_on_editor_close)
+        return FALSE;
+
+    windows = moo_editor_list_windows (app->priv->editor);
+
+    if (windows && !windows->next)
+    {
+        moo_app_quit (app);
+        ret = TRUE;
+    }
+
+    g_slist_free (windows);
+    return ret;
+}
+
 static void
 moo_app_init_editor (MooApp *app)
 {
     app->priv->editor = moo_editor_create_instance ();
+
+    g_signal_connect_swapped (app->priv->editor, "close-window",
+                              G_CALLBACK (close_editor_window), app);
+
     /* if ui_xml wasn't set yet, then moo_app_get_ui_xml()
        will get editor's xml */
     moo_editor_set_ui_xml (app->priv->editor,
@@ -726,9 +827,7 @@ moo_app_init_editor (MooApp *app)
                              app->priv->info->short_name);
 
     moo_plugin_read_dirs ();
-
-    _moo_edit_load_user_tools (MOO_USER_TOOL_MENU);
-    _moo_edit_load_user_tools (MOO_USER_TOOL_CONTEXT);
+    _moo_edit_load_user_tools ();
 }
 #endif /* MOO_BUILD_EDIT */
 
@@ -791,14 +890,21 @@ moo_app_init_real (MooApp *app)
     gdk_set_program_class (app->priv->info->full_name);
     gtk_window_set_default_icon_name (app->priv->info->short_name);
 
+    _moo_set_app_instance_name (app->priv->instance_name);
+
     moo_app_load_prefs (app);
     moo_app_init_ui (app);
 
 #ifdef MOO_BUILD_EDIT
     if (app->priv->use_editor)
         moo_app_init_editor (app);
+
+    if (app->priv->use_session == -1)
+        app->priv->use_session = moo_prefs_get_bool (moo_edit_setting (MOO_EDIT_PREFS_SAVE_SESSION));
 #endif
 
+    if (app->priv->use_session)
+        app->priv->run_input = TRUE;
     start_input (app);
 
     return TRUE;
@@ -809,16 +915,16 @@ static void
 start_input (MooApp *app)
 {
     if (app->priv->run_input)
-    {
-        moo_app_input = _moo_app_input_new (app->priv->info->short_name);
+        moo_app_data.input =
+            _moo_app_input_new (app->priv->info->short_name,
+                                app->priv->instance_name,
+                                TRUE);
+}
 
-        if (!_moo_app_input_start (moo_app_input))
-        {
-            g_critical ("%s: oops", G_STRLOC);
-            _moo_app_input_unref (moo_app_input);
-            moo_app_input = NULL;
-        }
-    }
+const char *
+moo_app_get_input_pipe_name (void)
+{
+    return moo_app_data.input ? _moo_app_input_get_path (moo_app_data.input) : NULL;
 }
 
 
@@ -839,27 +945,29 @@ moo_app_send_files (MooApp     *app,
                     char      **files,
                     guint32     line,
                     guint32     stamp,
-                    const char *pid)
+                    const char *pid,
+                    guint       options)
 {
     gboolean result;
     GString *msg;
+    char **p;
 
     g_return_val_if_fail (MOO_IS_APP (app), FALSE);
 
-    _moo_message ("moo_app_send_files: got %d files to pid %s",
-                  files ? g_strv_length (files) : 0,
+    _moo_message ("moo_app_send_files: got %u files to pid %s",
+                  files ? g_strv_length (files) : 0u,
                   pid ? pid : "NONE");
 
     msg = g_string_new (NULL);
 
-    g_string_append_printf (msg, "%s%08x%08x", CMD_OPEN_URIS, stamp, line);
+    g_string_append_printf (msg, "%s%08x%08x", CMD_OPEN_URIS, stamp, 0u);
 
-    while (files && *files)
+    for (p = files; p && *p; ++p)
     {
         char *freeme = NULL, *uri;
         const char *basename, *filename;
 
-        basename = *files++;
+        basename = *p;
 
         if (g_path_is_absolute (basename))
         {
@@ -873,7 +981,10 @@ moo_app_send_files (MooApp     *app,
             g_free (dir);
         }
 
-        uri = g_filename_to_uri (filename, NULL, NULL);
+        if (p != files)
+            line = 0;
+
+        uri = _moo_edit_filename_to_uri (filename, line, options);
 
         if (uri)
         {
@@ -909,9 +1020,10 @@ check_signal (void)
 {
     if (signal_received)
     {
-        printf ("%s\n", g_strsignal (signal_received));
-        MOO_APP_GET_CLASS(moo_app_instance)->quit (moo_app_instance);
-        exit (0);
+        g_print ("%s\n", g_strsignal (signal_received));
+        if (moo_app_data.instance)
+            MOO_APP_GET_CLASS(moo_app_data.instance)->quit (moo_app_data.instance);
+        exit (EXIT_FAILURE);
     }
 
     return TRUE;
@@ -989,6 +1101,8 @@ moo_app_try_quit_real (MooApp *app)
     if (!app->priv->running)
         return FALSE;
 
+    moo_app_save_session (app);
+
 #ifdef MOO_BUILD_EDIT
     if (!moo_editor_close_all (app->priv->editor, TRUE, TRUE))
         return TRUE;
@@ -998,6 +1112,33 @@ moo_app_try_quit_real (MooApp *app)
 }
 
 
+static void
+moo_app_install_cleanup (void)
+{
+    if (!moo_app_data.atexit_installed)
+    {
+        moo_app_data.atexit_installed = TRUE;
+        atexit (moo_app_cleanup);
+    }
+}
+
+static void
+moo_app_cleanup (void)
+{
+    if (moo_app_data.input)
+    {
+        _moo_app_input_free (moo_app_data.input);
+        moo_app_data.input = NULL;
+    }
+
+#ifdef MOO_USE_XDGMIME
+    xdg_mime_shutdown ();
+#endif
+
+    moo_cleanup ();
+}
+
+
 static void
 moo_app_quit_real (MooApp *app)
 {
@@ -1011,13 +1152,6 @@ moo_app_quit_real (MooApp *app)
     g_object_unref (app->priv->sm_client);
     app->priv->sm_client = NULL;
 
-    if (moo_app_input)
-    {
-        _moo_app_input_shutdown (moo_app_input);
-        _moo_app_input_unref (moo_app_input);
-        moo_app_input = NULL;
-    }
-
 #ifdef MOO_BUILD_EDIT
     moo_editor_close_all (app->priv->editor, FALSE, FALSE);
 
@@ -1027,6 +1161,7 @@ moo_app_quit_real (MooApp *app)
     app->priv->editor = NULL;
 #endif /* MOO_BUILD_EDIT */
 
+    moo_app_write_session (app);
     moo_app_save_prefs (app);
 
     if (app->priv->quit_handler_id)
@@ -1039,24 +1174,7 @@ moo_app_quit_real (MooApp *app)
         i++;
     }
 
-    if (app->priv->tmpdir)
-    {
-        GError *error = NULL;
-        _moo_remove_dir (app->priv->tmpdir, TRUE, &error);
-
-        if (error)
-        {
-            g_warning ("%s: %s", G_STRLOC, error->message);
-            g_error_free (error);
-        }
-
-        g_free (app->priv->tmpdir);
-        app->priv->tmpdir = NULL;
-    }
-
-#ifdef MOO_USE_XDGMIME
-    xdg_mime_shutdown ();
-#endif
+    moo_app_cleanup ();
 }
 
 
@@ -1162,6 +1280,15 @@ install_common_actions (void)
                                  "closure-callback", moo_app_about_dialog,
                                  NULL);
 
+#ifdef MOO_ENABLE_HELP
+    moo_window_class_new_action (klass, "Help", NULL,
+                                 "label", GTK_STOCK_HELP,
+                                 "accel", "F1",
+                                 "stock-id", GTK_STOCK_HELP,
+                                 "closure-callback", moo_app_help,
+                                 NULL);
+#endif
+
     moo_window_class_new_action (klass, "SystemInfo", NULL,
                                  /* menu item label */
                                  "label", _("System Info"),
@@ -1314,7 +1441,8 @@ _moo_app_exec_cmd (MooApp     *app,
 static void
 moo_app_new_file (MooApp       *app,
                   const char   *filename,
-                  guint32       line)
+                  guint32       line,
+                  guint         options)
 {
 #ifdef MOO_BUILD_EDIT
     MooEditor *editor = moo_app_get_editor (app);
@@ -1352,10 +1480,7 @@ moo_app_new_file (MooApp       *app,
                 *colon = 0;
             }
 
-            if (line > 0)
-                moo_editor_open_file_line (editor, norm_name, line - 1, NULL);
-            else
-                moo_editor_new_file (editor, NULL, NULL, norm_name, NULL);
+            _moo_editor_open_file (editor, norm_name, line, options);
         }
 
         g_free (norm_name);
@@ -1373,6 +1498,134 @@ moo_app_new_file (MooApp       *app,
 }
 
 
+static void
+moo_app_load_session_real (MooApp        *app,
+                           MooMarkupNode *xml)
+{
+#ifdef MOO_BUILD_EDIT
+    MooEditor *editor;
+    editor = moo_app_get_editor (app);
+    g_return_if_fail (editor != NULL);
+    _moo_editor_load_session (editor, xml);
+#endif /* MOO_BUILD_EDIT */
+}
+
+static void
+moo_app_save_session_real (MooApp        *app,
+                           MooMarkupNode *xml)
+{
+#ifdef MOO_BUILD_EDIT
+    MooEditor *editor;
+    editor = moo_app_get_editor (app);
+    g_return_if_fail (editor != NULL);
+    _moo_editor_save_session (editor, xml);
+#endif /* MOO_BUILD_EDIT */
+}
+
+static void
+moo_app_save_session (MooApp *app)
+{
+    MooMarkupNode *root;
+
+    if (!app->priv->session_file)
+        return;
+
+    if (app->priv->session)
+        moo_markup_doc_unref (app->priv->session);
+
+    app->priv->session = moo_markup_doc_new ("session");
+    root = moo_markup_create_root_element (app->priv->session, "session");
+    moo_markup_set_prop (root, "version", SESSION_VERSION);
+
+    g_signal_emit (app, signals[SAVE_SESSION], 0, root);
+}
+
+static void
+moo_app_write_session (MooApp *app)
+{
+    char *string;
+    GError *error = NULL;
+
+    if (!app->priv->session_file)
+        return;
+
+    if (!app->priv->session)
+    {
+        char *file = moo_get_user_cache_file (app->priv->session_file);
+        _moo_unlink (file);
+        g_free (file);
+        return;
+    }
+
+    string = moo_markup_format_pretty (app->priv->session, 1);
+    moo_save_user_cache_file (app->priv->session_file, string, -1, &error);
+
+    if (error)
+    {
+        char *file = moo_get_user_cache_file (app->priv->session_file);
+        g_critical ("could not save session file %s: %s", file, error->message);
+        g_free (file);
+        g_error_free (error);
+    }
+
+    g_free (string);
+}
+
+void
+moo_app_load_session (MooApp *app)
+{
+    MooMarkupDoc *doc;
+    MooMarkupNode *root;
+    GError *error = NULL;
+    const char *version;
+    char *session_file;
+
+    g_return_if_fail (MOO_IS_APP (app));
+
+    if (!app->priv->use_session)
+        return;
+
+    if (!app->priv->session_file)
+    {
+        if (app->priv->instance_name)
+            app->priv->session_file = g_strdup_printf ("%s.session.%s",
+                                                       g_get_prgname (),
+                                                       app->priv->instance_name);
+        else
+            app->priv->session_file = g_strdup_printf ("%s.session",
+                                                       g_get_prgname ());
+    }
+
+    session_file = moo_get_user_cache_file (app->priv->session_file);
+
+    if (!g_file_test (session_file, G_FILE_TEST_EXISTS) ||
+        !(doc = moo_markup_parse_file (session_file, &error)))
+    {
+        if (error)
+        {
+            g_warning ("could not open session file %s: %s",
+                       session_file, error->message);
+            g_error_free (error);
+        }
+
+        g_free (session_file);
+        return;
+    }
+
+    if (!(root = moo_markup_get_root_element (doc, "session")) ||
+        !(version = moo_markup_get_prop (root, "version")))
+        g_warning ("malformed session file %s, ignoring", session_file);
+    else if (strcmp (version, SESSION_VERSION) != 0)
+        g_warning ("invalid session file version %s in %s, ignoring",
+                   version, session_file);
+    else
+        g_signal_emit (app, signals[LOAD_SESSION], 0, root);
+
+    moo_markup_doc_unref (doc);
+    g_free (session_file);
+}
+
+
 static void
 moo_app_present (MooApp *app)
 {
@@ -1414,24 +1667,28 @@ moo_app_open_uris (MooApp     *app,
     {
         char **p;
 
-        for (p = uris; p && *p; ++p)
+        for (p = uris; p && *p && **p; ++p)
         {
-            char *filename = g_filename_from_uri (*p, NULL, NULL);
+            guint line_here = 0;
+            guint options = 0;
+            char *filename;
+
+            filename = _moo_edit_uri_to_filename (*p, &line_here, &options);
+
+            if (p != uris)
+                line = 0;
+            if (line_here)
+                line = line_here;
 
             if (filename)
-            {
-                if (p == uris && line > 0)
-                    moo_app_new_file (app, filename, line);
-                else
-                    moo_app_new_file (app, filename, 0);
-            }
+                moo_app_new_file (app, filename, line, options);
 
             g_free (filename);
         }
     }
     else
     {
-        moo_app_new_file (app, NULL, 0);
+        moo_app_new_file (app, NULL, 0, 0);
     }
 
     moo_editor_present (app->priv->editor, stamp);
@@ -1445,7 +1702,8 @@ void
 moo_app_open_files (MooApp     *app,
                     char      **files,
                     guint32     line,
-                    guint32     stamp)
+                    guint32     stamp,
+                    guint       options)
 {
 #ifdef MOO_BUILD_EDIT
     char **p;
@@ -1456,9 +1714,9 @@ moo_app_open_files (MooApp     *app,
     for (p = files; p && *p; ++p)
     {
         if (p == files && line > 0)
-            moo_app_new_file (app, *p, line);
+            moo_app_new_file (app, *p, line, options);
         else
-            moo_app_new_file (app, *p, 0);
+            moo_app_new_file (app, *p, 0, options);
     }
 
     moo_editor_present (app->priv->editor, stamp);
@@ -1500,7 +1758,7 @@ moo_app_exec_cmd_real (MooApp             *app,
             break;
 
         case MOO_APP_CMD_OPEN_FILE:
-            moo_app_new_file (app, data, 0);
+            moo_app_new_file (app, data, 0, 0);
             break;
         case MOO_APP_CMD_OPEN_URIS:
             moo_app_open_uris (app, data);
@@ -1522,56 +1780,11 @@ moo_app_exec_cmd_real (MooApp             *app,
 }
 
 
-char*
-moo_app_tempnam (MooApp *app)
+static void
+moo_app_help (GtkWidget *window)
 {
-    int i;
-    char *basename;
-    char *filename;
-
-    g_return_val_if_fail (MOO_IS_APP (app), NULL);
-
-    if (!app->priv->tmpdir)
-    {
-        char *dirname = NULL;
-
-        for (i = 0; i < 1000; ++i)
-        {
-            basename = g_strdup_printf ("%s-%08x",
-                                        app->priv->info->short_name,
-                                        g_random_int ());
-            dirname = g_build_filename (g_get_tmp_dir (), basename, NULL);
-            g_free (basename);
-
-            if (_moo_mkdir (dirname))
-            {
-                g_free (dirname);
-                dirname = NULL;
-            }
-            else
-            {
-                break;
-            }
-        }
-
-        g_return_val_if_fail (dirname != NULL, NULL);
-        app->priv->tmpdir = dirname;
-    }
-
-    for (i = 0; i < 1000; ++i)
-    {
-        basename = g_strdup_printf ("%08x.tmp", g_random_int ());
-        filename = g_build_filename (app->priv->tmpdir, basename, NULL);
-        g_free (basename);
-
-        if (g_file_test (filename, G_FILE_TEST_EXISTS))
-            g_free (filename);
-        else
-            return filename;
-    }
-
-    g_warning ("%s: could not generate temp file name", G_STRLOC);
-    return NULL;
+    GtkWidget *focus = gtk_window_get_focus (GTK_WINDOW (window));
+    moo_help_open_any (focus ? focus : window);
 }
 
 
@@ -1610,7 +1823,7 @@ moo_app_create_prefs_dialog (MooApp *app)
 
 #ifdef MOO_BUILD_EDIT
     moo_prefs_dialog_append_page (dialog, moo_edit_prefs_page_new (moo_app_get_editor (app)));
-    moo_prefs_dialog_append_page (dialog, _moo_user_tools_prefs_page_new ());
+    moo_prefs_dialog_append_page (dialog, moo_user_tools_prefs_page_new ());
     moo_plugin_attach_prefs (GTK_WIDGET (dialog));
 #endif
 
@@ -1647,8 +1860,8 @@ move_rc_files (MooApp *app)
         char *new_file;
         char *old_file;
 
-        new_file = g_strdup_printf ("%s/%src", g_get_user_config_dir (), g_get_prgname ());
-        old_file = g_strdup_printf ("%s/.%src", g_get_home_dir (), g_get_prgname ());
+        new_file = g_strdup_printf ("%s/" TMPL_RC_FILE, g_get_user_config_dir (), g_get_prgname ());
+        old_file = g_strdup_printf ("%s/." TMPL_RC_FILE, g_get_home_dir (), g_get_prgname ());
 
         if (!g_file_test (new_file, G_FILE_TEST_EXISTS) &&
             g_file_test (old_file, G_FILE_TEST_EXISTS) &&
@@ -1675,10 +1888,20 @@ move_rc_files (MooApp *app)
 
     {
         const char *new_file;
-        char *old_file = g_strdup_printf ("%s/.%s.state", g_get_home_dir (), g_get_prgname ());
+        char *old_file = g_strdup_printf ("%s/." TMPL_STATE_FILE, g_get_home_dir (), g_get_prgname ());
+
+        if (app->priv->instance_name)
+            app->priv->rc_files[MOO_PREFS_STATE] =
+                g_strdup_printf ("%s/" TMPL_STATE_FILE ".%s",
+                                 cache_dir,
+                                 g_get_prgname (),
+                                 app->priv->instance_name);
+        else
+            app->priv->rc_files[MOO_PREFS_STATE] =
+                g_strdup_printf ("%s/" TMPL_STATE_FILE,
+                                 cache_dir,
+                                 g_get_prgname ());
 
-        app->priv->rc_files[MOO_PREFS_STATE] =
-            g_strdup_printf ("%s/%s.state", cache_dir, g_get_prgname ());
         new_file = app->priv->rc_files[MOO_PREFS_STATE];
 
         if (!g_file_test (new_file, G_FILE_TEST_EXISTS) &&
@@ -1697,25 +1920,57 @@ move_rc_files (MooApp *app)
 #endif
 
 
+static char **
+get_rc_files (void)
+{
+    char *prefix;
+    char *var;
+    const char *value;
+    char **files = NULL;
+
+    prefix = g_ascii_strup (g_get_prgname (), -1);
+    var = g_strdup_printf ("%s_RC_FILES", prefix);
+    value = g_getenv (var);
+
+    if (value && value[0])
+    {
+        files = g_strsplit (value, G_SEARCHPATH_SEPARATOR_S, 0);
+    }
+    else
+    {
+        char *tmpl = g_strdup_printf (TMPL_RC_FILE, g_get_prgname ());
+        files = moo_get_data_files (tmpl, MOO_DATA_SHARE, NULL);
+        g_free (tmpl);
+    }
+
+    g_free (var);
+    g_free (prefix);
+    return files;
+}
+
 static void
 moo_app_load_prefs (MooApp *app)
 {
     GError *error = NULL;
+    char **sys_files;
 
 #ifndef __WIN32__
     move_rc_files (app);
 #else
     app->priv->rc_files[MOO_PREFS_RC] =
-        g_strdup_printf ("%s/%s.ini", g_get_user_config_dir (), g_get_prgname ());
+        g_strdup_printf ("%s/" TMPL_RC_FILE, g_get_user_config_dir (), g_get_prgname ());
     app->priv->rc_files[MOO_PREFS_STATE] =
-        g_strdup_printf ("%s/%s.state", g_get_user_config_dir (), g_get_prgname ());
+        g_strdup_printf ("%s/" TMPL_STATE_FILE, g_get_user_config_dir (), g_get_prgname ());
 #endif
 
-    if (!moo_prefs_load (app->priv->rc_files[MOO_PREFS_RC],
+    sys_files = get_rc_files ();
+
+    if (!moo_prefs_load (sys_files,
+                         app->priv->rc_files[MOO_PREFS_RC],
                          app->priv->rc_files[MOO_PREFS_STATE],
                          &error))
     {
-        g_warning ("%s: could not read config file", G_STRLOC);
+        g_warning ("%s: could not read config files", G_STRLOC);
 
         if (error)
         {
@@ -1723,6 +1978,8 @@ moo_app_load_prefs (MooApp *app)
             g_error_free (error);
         }
     }
+
+    g_strfreev (sys_files);
 }
 
 
@@ -1735,7 +1992,7 @@ moo_app_save_prefs (MooApp *app)
                          app->priv->rc_files[MOO_PREFS_STATE],
                          &error))
     {
-        g_warning ("%s: could not save config file", G_STRLOC);
+        g_warning ("%s: could not save config files", G_STRLOC);
 
         if (error)
         {
