@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -134,9 +133,6 @@ static gboolean moo_html_motion             (GtkWidget      *widget,
 
 static gboolean moo_html_load_url_real      (MooHtml        *html,
                                              const char     *url);
-static gboolean moo_html_load_file          (GtkTextView    *view,
-                                             const char     *file,
-                                             const char     *encoding);
 static void     moo_html_clear              (GtkTextView    *view);
 static void     moo_html_set_doc            (GtkTextView    *view,
                                              htmlDocPtr      doc);
@@ -532,7 +528,8 @@ moo_html_load_url (GtkTextView *view,
     if (!scheme)
         scheme = g_strdup ("file://");
 
-    if (!strcmp (scheme, "mailto://"))
+    if (!strcmp (scheme, "mailto:") ||
+        !strcmp (scheme, "mailto://"))
     {
         result = moo_open_email (base, NULL, NULL);
         goto out;
@@ -554,7 +551,7 @@ moo_html_load_url (GtkTextView *view,
         {
             char *filename = g_build_filename (data->dirname, base, NULL);
 
-            result = moo_html_load_file (view, filename, NULL);
+            result = _moo_html_load_file (view, filename, NULL);
 
             if (result && anchor)
                 moo_html_goto_anchor (view, anchor);
@@ -564,7 +561,7 @@ moo_html_load_url (GtkTextView *view,
     }
     else
     {
-        result = moo_html_load_file (view, base, NULL);
+        result = _moo_html_load_file (view, base, NULL);
 
         if (result && anchor)
             moo_html_goto_anchor (view, anchor);
@@ -677,10 +674,10 @@ _moo_html_load_memory (GtkTextView *view,
 }
 
 
-static gboolean
-moo_html_load_file (GtkTextView *view,
-                    const char  *file,
-                    const char  *encoding)
+gboolean
+_moo_html_load_file (GtkTextView *view,
+                     const char  *file,
+                     const char  *encoding)
 {
     htmlDocPtr doc;
     MooHtmlData *data = moo_html_get_data (view);
@@ -1224,7 +1221,7 @@ moo_html_parse_url (const char     *url,
     g_return_val_if_fail (url != NULL, FALSE);
     g_return_val_if_fail (scheme && base && anchor, FALSE);
 
-    regex = g_regex_new ("^([a-zA-Z]+://)?([^#]*)(#(.*))?$", 0, 0, NULL);
+    regex = g_regex_new ("^([a-zA-Z]+:(//)?)?([^#]*)(#(.*))?$", 0, 0, NULL);
     g_return_val_if_fail (regex != NULL, FALSE);
 
     if (!g_regex_match (regex, url, 0, &match_info))
@@ -1235,8 +1232,8 @@ moo_html_parse_url (const char     *url,
     }
 
     *scheme = g_match_info_fetch (match_info, 1);
-    *base = g_match_info_fetch (match_info, 2);
-    *anchor = g_match_info_fetch (match_info, 4);
+    *base = g_match_info_fetch (match_info, 3);
+    *anchor = g_match_info_fetch (match_info, 5);
 
     if (!*scheme || !**scheme) {g_free (*scheme); *scheme = NULL;}
     if (!*base || !**base) {g_free (*base); *base = NULL;}
@@ -1801,7 +1798,7 @@ static ProcessElm
 get_proc_elm_func (xmlNode *node)
 {
     if (IS_ELEMENT (node))
-        return g_hash_table_lookup (proc_elm_funcs__, node->name);
+        return (ProcessElm) g_hash_table_lookup (proc_elm_funcs__, node->name);
     else
         return NULL;
 }
