@@ -27,6 +27,8 @@
 
 #include "eggdesktopfile.h"
 
+#include <mooutils/moo-environ.h>
+
 #include <string.h>
 #include <unistd.h>
 
@@ -932,8 +934,6 @@ set_startup_notification_timeout (GdkDisplay *display,
 }
 #endif /* HAVE_GDK_X11_DISPLAY_BROADCAST_STARTUP_MESSAGE */
 
-extern char **environ;
-
 static GPtrArray *
 array_putenv (GPtrArray *env, char *variable)
 {
@@ -941,10 +941,12 @@ array_putenv (GPtrArray *env, char *variable)
 
   if (!env)
     {
+      char **environ_ptr = environ;
+
       env = g_ptr_array_new ();
 
-      for (i = 0; environ[i]; i++)
-	g_ptr_array_add (env, g_strdup (environ[i]));
+      for (i = 0; environ_ptr[i]; i++)
+	g_ptr_array_add (env, g_strdup (environ_ptr[i]));
     }
 
   keylen = strcspn (variable, "=");
@@ -974,7 +976,7 @@ egg_desktop_file_launchv (EggDesktopFile *desktop_file,
 			  GError **error)
 {
   EggDesktopFileLaunchOption option;
-  GSList *translated_documents, *docs;
+  GSList *translated_documents = NULL, *docs;
   char *command, **argv;
   int argc, i, screen_num;
   gboolean success, current_success;
