@@ -34,7 +34,12 @@
 
 #include <string.h>
 #include <fcntl.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#ifdef G_OS_WIN32
+#include <io.h>
+#endif
 #include <libxml/xmlreader.h>
 #include <glib/gstdio.h>
 #include "gtksourceview-i18n.h"
@@ -62,9 +67,12 @@ struct _ParserState
 {
 	/* The args passed to _file_parse_version2() */
 	xmlTextReader *reader;
+	char *filename;
 	GtkSourceLanguage *language;
 	GtkSourceContextData *ctx_data;
 
+	gchar *language_decoration;
+
 	/* A stack of id that representing parent contexts */
 	GQueue *curr_parents;
 
@@ -75,17 +83,25 @@ struct _ParserState
 	 * resolve references (the key is the id) */
 	GHashTable *defined_regexes;
 
-	/* The mapping between style ids and their default styles.
-	 * If lang file 'mama' contains this:
-	 * <style id="foo" map-to="def:blah"/>
-	 * <style id="bar"/>
-	 * then in styles_mapping: "mama:foo"->"def:blah", "mama:bar"->"mama:bar". */
+	/* Maps style ids to GtkSourceStyleInfo objects.
+	 * Contains all the styles defined in the lang files parsed
+	 * while parsing the main language file. For example, if the main
+	 * language is C, also styles in def.lang, gtk-doc.lang, etc. are
+	 * stored in this hash table. For styles defined in language files
+	 * different from the main one, the name is _not_ stored, since it
+	 * is not used in other places of the code. So, if the main language is
+	 * C, only the name of styles defined in c.lang is stored, while for
+	 * the styles defined in def.lang, etc. the name is not stored. */
 	GHashTable *styles_mapping;
 
 	/* The list of loaded languages (the item are xmlChar pointers),
 	 * mapping is id -> id */
 	GHashTable *loaded_lang_ids;
 
+	/* The list of replacements. The queue object is owned by the caller,
+	 * so parser_state only adds stuff to it */
+	GQueue *replacements;
+
 	/* A serial number incremented to get unique generated names */
 	guint id_cookie;
 
@@ -94,9 +110,11 @@ struct _ParserState
 
 	gchar *opening_delimiter;
 	gchar *closing_delimiter;
-};
-typedef struct _ParserState ParserState;
 
+	GError *error;
+};
+
+typedef struct _ParserState ParserState;
 
 
 static GQuark     parser_error_quark           (void);
@@ -111,7 +129,9 @@ static ParserState *parser_state_new           (GtkSourceLanguage      *language
                                                 GtkSourceContextData   *ctx_data,
                                                 GHashTable             *defined_regexes,
                                                 GHashTable             *styles_mapping,
+						GQueue                 *replacements,
                                                 xmlTextReader          *reader,
+						const char             *filename,
                                                 GHashTable             *loaded_lang_ids);
 static void       parser_state_destroy         (ParserState *parser_state);
 
@@ -121,11 +141,12 @@ static gboolean   file_parse                   (gchar                  *filename
                                                 GHashTable             *defined_regexes,
                                                 GHashTable             *styles,
                                                 GHashTable             *loaded_lang_ids,
+						GQueue                 *replacements,
                                                 GError                **error);
 
 static GRegexCompileFlags
 		  update_regex_flags	       (GRegexCompileFlags flags,
-						const gchar   *option_name,
+						const xmlChar *option_name,
 						const xmlChar *bool_value);
 
 static gboolean   create_definition            (ParserState *parser_state,
@@ -134,17 +155,13 @@ static gboolean   create_definition            (ParserState *parser_state,
                                                 gchar *style,
 						GError **error);
 
-static void       handle_context_element       (ParserState *parser_state,
-                                                GError **error);
-static void       handle_language_element      (ParserState *parser_state,
-                                                GError **error);
-static void       handle_define_regex_element  (ParserState *parser_state,
-                                                GError **error);
+static void       handle_context_element       (ParserState *parser_state);
+static void       handle_language_element      (ParserState *parser_state);
+static void       handle_define_regex_element  (ParserState *parser_state);
 static void       handle_default_regex_options_element
-                                               (ParserState *parser_state,
-                                                GError **error);
-static void       element_start                (ParserState *parser_state,
-						GError **error);
+                                               (ParserState *parser_state);
+static void       handle_replace_element       (ParserState *parser_state);
+static void       element_start                (ParserState *parser_state);
 static void       element_end                  (ParserState *parser_state);
 static gboolean   replace_by_id                (const GMatchInfo *match_info,
 						GString *expanded_regex,
@@ -230,7 +247,6 @@ lang_id_is_already_loaded (ParserState *parser_state, gchar *lang_id)
 	return g_hash_table_lookup (parser_state->loaded_lang_ids, lang_id) != NULL;
 }
 
-
 static GRegexCompileFlags
 get_regex_flags (xmlNode             *node,
 		 GRegexCompileFlags flags)
@@ -241,26 +257,8 @@ get_regex_flags (xmlNode             *node,
 	{
 		g_return_val_if_fail (attribute->children != NULL, flags);
 
-		if (xmlStrcmp (BAD_CAST "extended", attribute->name) == 0)
-		{
-			flags = update_regex_flags (flags, "extended",
-						    attribute->children->content);
-		}
-		else if (xmlStrcmp (BAD_CAST "case-insensitive", attribute->name) == 0)
-		{
-			flags = update_regex_flags (flags, "case-insensitive",
-						    attribute->children->content);
-		}
-		else if (xmlStrcmp (BAD_CAST "dot-match-all", attribute->name) == 0)
-		{
-			flags = update_regex_flags (flags, "dot-match-all",
-						    attribute->children->content);
-		}
-		else if (xmlStrcmp (BAD_CAST "dupnames", attribute->name) == 0)
-		{
-			flags = update_regex_flags (flags, "dupnames",
-						    attribute->children->content);
-		}
+		flags = update_regex_flags (flags, attribute->name,
+					    attribute->children->content);
 	}
 
 	return flags;
@@ -502,14 +500,9 @@ add_ref (ParserState               *parser_state,
 	 const gchar               *style,
 	 GError	                  **error)
 {
-	gchar *container_id;
 	gboolean all = FALSE;
 	gchar *ref_id;
-	gchar *lang_id;
-
-	GtkSourceLanguageManager *lm;
-	GtkSourceLanguage *imported_language;
-
+	gchar *lang_id = NULL;
 	GError *tmp_error = NULL;
 
 	/* Return if an error is already set */
@@ -519,8 +512,11 @@ add_ref (ParserState               *parser_state,
 	{
 		if (!lang_id_is_already_loaded (parser_state, lang_id))
 		{
+			GtkSourceLanguageManager *lm;
+			GtkSourceLanguage *imported_language;
+
 			lm = _gtk_source_language_get_language_manager (parser_state->language);
-			imported_language = gtk_source_language_manager_get_language_by_id (lm, lang_id);
+			imported_language = gtk_source_language_manager_get_language (lm, lang_id);
 
 			if (imported_language == NULL)
 			{
@@ -538,6 +534,7 @@ add_ref (ParserState               *parser_state,
 					    parser_state->defined_regexes,
 					    parser_state->styles_mapping,
 					    parser_state->loaded_lang_ids,
+					    parser_state->replacements,
 					    &tmp_error);
 
 				if (tmp_error != NULL)
@@ -554,7 +551,6 @@ add_ref (ParserState               *parser_state,
 			}
 		}
 		ref_id = g_strdup (ref);
-		g_free (lang_id);
 	}
 	else
 	{
@@ -575,12 +571,14 @@ add_ref (ParserState               *parser_state,
 				     PARSER_ERROR_WRONG_STYLE,
 				     "style override used with wildcard context reference"
 				     " in language '%s' in ref '%s'",
-				     lang_id, ref);
+				     lang_id != NULL ? lang_id : parser_state->current_lang_id, ref);
 		}
 	}
 
 	if (tmp_error == NULL && parser_state->ctx_data != NULL)
 	{
+		gchar *container_id;
+
 		container_id = g_queue_peek_head (parser_state->curr_parents);
 
 		/* If the document is validated container_id is never NULL */
@@ -597,6 +595,7 @@ add_ref (ParserState               *parser_state,
 		DEBUG (g_message ("appended %s in %s", ref_id, container_id));
 	}
 
+	g_free (lang_id);
 	g_free (ref_id);
 
 	if (tmp_error != NULL)
@@ -649,8 +648,7 @@ create_sub_pattern (ParserState *parser_state,
 }
 
 static void
-handle_context_element (ParserState *parser_state,
-			GError **error)
+handle_context_element (ParserState *parser_state)
 {
 	gchar *id, *parent_id, *style_ref;
 	xmlChar *ref, *sub_pattern, *tmp;
@@ -661,11 +659,11 @@ handle_context_element (ParserState *parser_state,
 
 	GError *tmp_error = NULL;
 
-	g_return_if_fail (error != NULL && *error == NULL);
+	g_return_if_fail (parser_state->error == NULL);
 
 	ref = xmlTextReaderGetAttribute (parser_state->reader, BAD_CAST "ref");
 	sub_pattern = xmlTextReaderGetAttribute (parser_state->reader,
-			BAD_CAST "sub-pattern");
+						 BAD_CAST "sub-pattern");
 
 	tmp = xmlTextReaderGetAttribute (parser_state->reader, BAD_CAST "ignore-style");
 	if (tmp != NULL && str_to_bool (tmp))
@@ -681,7 +679,8 @@ handle_context_element (ParserState *parser_state,
 
 	if (ignore_style && ref == NULL)
 	{
-		g_set_error (error, PARSER_ERROR,
+		g_set_error (&parser_state->error,
+			     PARSER_ERROR,
 			     PARSER_ERROR_WRONG_STYLE,
 			     "ignore-style used not in a reference to context");
 
@@ -696,18 +695,24 @@ handle_context_element (ParserState *parser_state,
 		options |= GTK_SOURCE_CONTEXT_IGNORE_STYLE;
 
 		if (style_ref != NULL)
-			g_warning ("style-ref and ignore-style used simultaneously");
+			g_warning ("in file %s: style-ref and ignore-style used simultaneously",
+				   parser_state->filename);
 	}
 
 	/* XXX */
 	if (!ignore_style && style_ref != NULL &&
 	    g_hash_table_lookup (parser_state->styles_mapping, style_ref) == NULL)
 	{
-		g_warning ("style '%s' not defined", style_ref);
+		g_warning ("in file %s: style '%s' not defined", parser_state->filename, style_ref);
 	}
 
 	if (ref != NULL)
 	{
+		tmp = xmlTextReaderGetAttribute (parser_state->reader, BAD_CAST "original");
+		if (tmp != NULL && str_to_bool (tmp))
+			options |= GTK_SOURCE_CONTEXT_REF_ORIGINAL;
+		xmlFree (tmp);
+
 		if (style_ref != NULL)
 			options |= GTK_SOURCE_CONTEXT_OVERRIDE_STYLE;
 
@@ -748,8 +753,19 @@ handle_context_element (ParserState *parser_state,
 				is_empty = xmlTextReaderIsEmptyElement (
 						parser_state->reader);
 
-				success = create_definition (parser_state, id, parent_id,
-						style_ref, &tmp_error);
+				if (is_empty)
+					success = _gtk_source_context_data_define_context (parser_state->ctx_data,
+											   id,
+											   parent_id,
+											   "$^",
+											   NULL,
+											   NULL,
+											   NULL,
+											   0,
+											   &tmp_error);
+				else
+					success = create_definition (parser_state, id, parent_id,
+								     style_ref, &tmp_error);
 
 				if (success && !is_empty)
 				{
@@ -757,7 +773,7 @@ handle_context_element (ParserState *parser_state,
 					 * stack only if other contexts can be
 					 * defined inside it */
 					g_queue_push_head (parser_state->curr_parents,
-							g_strdup (id));
+							   g_strdup (id));
 				}
 			}
 		}
@@ -765,46 +781,65 @@ handle_context_element (ParserState *parser_state,
 		g_free (id);
 	}
 
-
 	g_free (style_ref);
 	xmlFree (sub_pattern);
 	xmlFree (ref);
 
 	if (tmp_error != NULL)
-	{
-		g_propagate_error (error, tmp_error);
-		return;
-	}
+		g_propagate_error (&parser_state->error, tmp_error);
 }
 
 static void
-handle_language_element (ParserState *parser_state,
-			 GError **error)
+handle_replace_element (ParserState *parser_state)
+{
+	xmlChar *id, *ref;
+	GtkSourceContextReplace *repl;
+	gchar *replace_with;
+
+	id = xmlTextReaderGetAttribute (parser_state->reader, BAD_CAST "id");
+	ref = xmlTextReaderGetAttribute (parser_state->reader, BAD_CAST "ref");
+
+	if (id_is_decorated ((gchar*) ref, NULL))
+		replace_with = g_strdup ((gchar *) ref);
+	else
+		replace_with = decorate_id (parser_state, (gchar*) ref);
+
+	repl = _gtk_source_context_replace_new ((const gchar *) id, replace_with);
+	g_queue_push_tail (parser_state->replacements, repl);
+
+	g_free (replace_with);
+	xmlFree (ref);
+	xmlFree (id);
+}
+
+static void
+handle_language_element (ParserState *parser_state)
 {
 	/* FIXME: check that the language name, version, etc. are the
 	 * right ones - Paolo */
 	xmlChar *lang_id, *lang_version;
 	xmlChar *expected_version = BAD_CAST "2.0";
 
-	g_return_if_fail (error != NULL && *error == NULL);
+	g_return_if_fail (parser_state->error == NULL);
 
 	lang_version = xmlTextReaderGetAttribute (parser_state->reader, BAD_CAST "version");
 
 	if (lang_version == NULL ||
-			(xmlStrcmp (expected_version, lang_version) != 0))
+	    xmlStrcmp (expected_version, lang_version) != 0)
 	{
-		g_set_error (error,
+		g_set_error (&parser_state->error,
 			     PARSER_ERROR,
 			     PARSER_ERROR_WRONG_VERSION,
 			     "wrong language version '%s', expected '%s'",
 			     lang_version ? (gchar*) lang_version : "(none)",
 			     (gchar*) expected_version);
 	}
-
-	lang_id = xmlTextReaderGetAttribute (parser_state->reader, BAD_CAST "id");
-
-	parser_state->current_lang_id = g_strdup ((gchar *) lang_id);
-	g_hash_table_insert (parser_state->loaded_lang_ids, lang_id, lang_id);
+	else
+	{
+		lang_id = xmlTextReaderGetAttribute (parser_state->reader, BAD_CAST "id");
+		parser_state->current_lang_id = g_strdup ((gchar *) lang_id);
+		g_hash_table_insert (parser_state->loaded_lang_ids, lang_id, lang_id);
+	}
 
 	xmlFree (lang_version);
 }
@@ -864,25 +899,35 @@ replace_by_id (const GMatchInfo *match_info,
 
 static GRegexCompileFlags
 update_regex_flags (GRegexCompileFlags  flags,
-		    const gchar        *option_name,
+		    const xmlChar      *option_name,
 		    const xmlChar      *value)
 {
 	GRegexCompileFlags single_flag;
+	gboolean set_flag;
 
 	DEBUG (g_message ("setting the '%s' regex flag to %s", option_name, value));
 
-	if (strcmp ("case-insensitive", option_name) == 0)
-		single_flag = G_REGEX_CASELESS;
-	else if (strcmp ("extended", option_name) == 0)
-		single_flag = G_REGEX_EXTENDED;
-	else if (strcmp ("dot-match-all", option_name) == 0)
-		single_flag = G_REGEX_DOTALL;
-	else if (strcmp ("dupnames", option_name) == 0)
-		single_flag = G_REGEX_DUPNAMES;
-	else
-		return flags;
+	set_flag = str_to_bool (value);
 
-	if (str_to_bool (value))
+	if (xmlStrcmp (BAD_CAST "case-sensitive", option_name) == 0)
+	{
+		single_flag = G_REGEX_CASELESS;
+		set_flag = !set_flag;
+	}
+	else if (xmlStrcmp (BAD_CAST "extended", option_name) == 0)
+	{
+		single_flag = G_REGEX_EXTENDED;
+	}
+	else if (xmlStrcmp (BAD_CAST "dupnames", option_name) == 0)
+	{
+		single_flag = G_REGEX_DUPNAMES;
+	}
+	else
+	{
+		return flags;
+	}
+
+	if (set_flag)
 		flags |= single_flag;
 	else
 		flags &= ~single_flag;
@@ -890,50 +935,6 @@ update_regex_flags (GRegexCompileFlags  flags,
 	return flags;
 }
 
-static gboolean
-calc_regex_flags (const gchar *flags_string, GRegexCompileFlags *flags)
-{
-	gboolean add = TRUE;
-	while (*flags_string != '\0')
-	{
-		GRegexCompileFlags single_flag = 0;
-
-		switch (g_utf8_get_char (flags_string))
-		{
-			case '-':
-				add = FALSE;
-				break;
-			case '+':
-				add = TRUE;
-				break;
-			case 'i':
-				single_flag = G_REGEX_CASELESS;
-				break;
-			case 'x':
-				single_flag = G_REGEX_EXTENDED;
-				break;
-			case 's':
-				/* FIXME: do we need this? */
-				single_flag = G_REGEX_DOTALL;
-				break;
-			default:
-				return FALSE;
-		}
-
-		if (single_flag != 0)
-		{
-			if (add)
-				*flags |= single_flag;
-			else
-				*flags &= ~single_flag;
-		}
-
-		flags_string = g_utf8_next_char (flags_string);
-	}
-
-	return TRUE;
-}
-
 static gchar *
 expand_regex_vars (ParserState *parser_state, gchar *regex, gint len, GError **error)
 {
@@ -969,7 +970,7 @@ expand_regex_vars (ParserState *parser_state, gchar *regex, gint len, GError **e
 	if (regex == NULL)
 		return NULL;
 
-	egg_re = g_regex_new (re, 0, 0, NULL);
+	egg_re = g_regex_new (re, G_REGEX_NEWLINE_LF, 0, NULL);
 
 	data.parser_state = parser_state;
 	data.error = NULL;
@@ -1049,12 +1050,13 @@ expand_regex_delimiters (ParserState *parser_state,
 	 */
 	const gchar *re = "(?<!\\\\)(\\\\\\\\)*\\\\%(\\[|\\])";
 	gchar *expanded_regex;
-	GRegex *egg_re;
+	static GRegex *egg_re;
 
 	if (regex == NULL)
 		return NULL;
 
-	egg_re = g_regex_new (re, 0, 0, NULL);
+	if (egg_re == NULL)
+		egg_re = g_regex_new (re, G_REGEX_NEWLINE_LF | G_REGEX_OPTIMIZE, 0, NULL);
 
 	expanded_regex = g_regex_replace_eval (egg_re, regex, len, 0, 0,
 					       replace_delimiter, parser_state, NULL);
@@ -1062,8 +1064,6 @@ expand_regex_delimiters (ParserState *parser_state,
 	DEBUG (g_message ("expanded regex delims '%s' to '%s'",
 				regex, expanded_regex));
 
-	g_regex_unref (egg_re);
-
 	return expanded_regex;
 }
 
@@ -1077,8 +1077,7 @@ expand_regex (ParserState *parser_state,
 {
 	gchar *tmp_regex;
 	GString *expanded_regex;
-	/* Length in *bytes* of the regex passed to expand_regex_vars. */
-	gint len;
+	static GRegex *backref_re = NULL;
 
 	g_assert (parser_state != NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
@@ -1086,14 +1085,20 @@ expand_regex (ParserState *parser_state,
 	if (regex == NULL)
 		return NULL;
 
-	if (g_regex_match_simple ("(?<!\\\\)(\\\\\\\\)*\\\\[0-9]", regex, 0, 0))
+	if (backref_re == NULL)
+		backref_re = g_regex_new ("(?<!\\\\)(\\\\\\\\)*\\\\[0-9]",
+					  G_REGEX_OPTIMIZE | G_REGEX_NEWLINE_LF,
+					  0,
+					  NULL);
+
+	if (g_regex_match (backref_re, regex, 0, NULL))
 	{
 		/* This may be a backreference, or it may be an octal character */
 		GRegex *compiled;
 
-		compiled = g_regex_new (regex, flags, 0, error);
+		compiled = g_regex_new (regex, flags | G_REGEX_NEWLINE_LF, 0, error);
 
-		if (!compiled)
+		if (compiled == NULL)
 			return NULL;
 
 		if (g_regex_get_max_backref (compiled) > 0)
@@ -1108,40 +1113,20 @@ expand_regex (ParserState *parser_state,
 		g_regex_unref (compiled);
 	}
 
-	if (g_utf8_get_char (regex) == '/')
-	{
-		gchar *regex_end = g_utf8_strrchr (regex, -1, '/');
-		if (regex_end == regex)
-		{
-			g_set_error (error,
-					PARSER_ERROR, PARSER_ERROR_MALFORMED_REGEX,
-					_ ("malformed regex '%s'"), regex);
-			return NULL;
-		}
-		regex = g_utf8_next_char (regex);
-		len = regex_end - regex;
-	}
-	else
-	{
-		len = -1;
-	}
-
 	if (do_expand_vars)
 	{
-		tmp_regex = expand_regex_vars (parser_state, regex, len, error);
+		tmp_regex = expand_regex_vars (parser_state, regex, -1, error);
 
 		if (tmp_regex == NULL)
 			return NULL;
 	}
 	else
 	{
-		if (len == -1)
-			len = strlen (regex);
-		tmp_regex = g_strndup (regex, len);
+		tmp_regex = g_strdup (regex);
 	}
 
 	regex = tmp_regex;
-	tmp_regex = expand_regex_delimiters (parser_state, regex, len);
+	tmp_regex = expand_regex_delimiters (parser_state, regex, -1);
 	g_free (regex);
 
 	/* Set the options and add not capturing parentheses if
@@ -1151,30 +1136,28 @@ expand_regex (ParserState *parser_state,
 	if (insert_parentheses)
 		g_string_append (expanded_regex, "(?:");
 	g_string_append (expanded_regex, "(?");
+
 	if (flags != 0)
 	{
 		if (flags & G_REGEX_CASELESS)
 			g_string_append (expanded_regex, "i");
 		if (flags & G_REGEX_EXTENDED)
 			g_string_append (expanded_regex, "x");
-		if (flags & G_REGEX_DOTALL)
-			g_string_append (expanded_regex, "s");
 		/* J is added here if it's used, but -J isn't added
 		 * below */
 		if (flags & G_REGEX_DUPNAMES)
 			g_string_append (expanded_regex, "J");
 	}
-	if (flags != (G_REGEX_CASELESS | G_REGEX_EXTENDED |
-		      G_REGEX_DOTALL | G_REGEX_DUPNAMES))
+
+	if ((flags & (G_REGEX_CASELESS | G_REGEX_EXTENDED)) != (G_REGEX_CASELESS | G_REGEX_EXTENDED))
 	{
 		g_string_append (expanded_regex, "-");
 		if (!(flags & G_REGEX_CASELESS))
 			g_string_append (expanded_regex, "i");
 		if (!(flags & G_REGEX_EXTENDED))
 			g_string_append (expanded_regex, "x");
-		if (!(flags & G_REGEX_DOTALL))
-			g_string_append (expanded_regex, "s");
 	}
+
 	g_string_append (expanded_regex, ")");
 	g_string_append (expanded_regex, tmp_regex);
 	if (insert_parentheses)
@@ -1194,21 +1177,20 @@ expand_regex (ParserState *parser_state,
 }
 
 static void
-handle_define_regex_element (ParserState *parser_state,
-			     GError **error)
+handle_define_regex_element (ParserState *parser_state)
 {
 	gchar *id;
 	xmlChar *regex;
 	xmlChar *tmp;
 	gchar *expanded_regex;
 	int i;
-	const gchar *regex_options[] = {"extended", "case-insensitive", "dot-match-all", NULL};
+	const gchar *regex_options[] = {"extended", "case-sensitive", "dupnames", NULL};
 	GRegexCompileFlags flags;
 	GError *tmp_error = NULL;
 
 	int type;
 
-	g_return_if_fail (error != NULL && *error == NULL);
+	g_return_if_fail (parser_state->error == NULL);
 
 	if (parser_state->ctx_data == NULL)
 		return;
@@ -1232,11 +1214,14 @@ handle_define_regex_element (ParserState *parser_state,
 		tmp = xmlTextReaderGetAttribute (parser_state->reader,
 						 BAD_CAST regex_options[i]);
 		if (tmp != NULL)
-			flags = update_regex_flags (flags, regex_options[i], tmp);
+		{
+			flags = update_regex_flags (flags,
+						    BAD_CAST regex_options[i],
+						    tmp);
+		}
 		xmlFree (tmp);
 	}
 
-
 	xmlTextReaderRead (parser_state->reader);
 
 	type = xmlTextReaderNodeType (parser_state->reader);
@@ -1247,7 +1232,7 @@ handle_define_regex_element (ParserState *parser_state,
 		regex = xmlStrdup (BAD_CAST "");
 
 	expanded_regex = expand_regex (parser_state, (gchar*) regex, flags,
-				       FALSE, TRUE, &tmp_error);
+				       TRUE, TRUE, &tmp_error);
 
 	if (tmp_error == NULL)
 	{
@@ -1258,70 +1243,43 @@ handle_define_regex_element (ParserState *parser_state,
 	xmlFree (regex);
 
 	if (tmp_error != NULL)
-	{
-		g_propagate_error (error, tmp_error);
-		return;
-	}
+		g_propagate_error (&parser_state->error, tmp_error);
 }
 
 static void
-handle_default_regex_options_element (ParserState *parser_state,
-				      GError     **error)
+handle_default_regex_options_element (ParserState *parser_state)
 {
-	xmlChar *options;
-	int ret, type;
+	xmlNode *elm;
 
-	g_return_if_fail (error != NULL && *error == NULL);
+	g_return_if_fail (parser_state->error == NULL);
 
 	if (parser_state->ctx_data == NULL)
 		return;
 
-	do {
-		ret = xmlTextReaderRead (parser_state->reader);
-		g_assert (ret == 1);
+	elm = xmlTextReaderCurrentNode (parser_state->reader);
 
-		type = xmlTextReaderNodeType (parser_state->reader);
-
-	} while (type != XML_READER_TYPE_TEXT &&
-		 type != XML_READER_TYPE_CDATA);
-
-	options = xmlTextReaderValue (parser_state->reader);
-
-	g_strstrip ((gchar*) options);
-
-	if (!calc_regex_flags ((gchar*) options, &(parser_state->regex_compile_flags)))
-	{
-		g_set_error (error,
-			     PARSER_ERROR,
-			     PARSER_ERROR_MALFORMED_REGEX,
-			     _ ("malformed regex options '%s'"),
-			     (gchar*) options);
-	}
-
-	xmlFree (options);
+	parser_state->regex_compile_flags = get_regex_flags (elm, 0);
 }
 
 static void
 parse_language_with_id (ParserState *parser_state,
-		gchar *lang_id,
-		GError **error)
+			gchar       *lang_id)
 {
 	GtkSourceLanguageManager *lm;
 	GtkSourceLanguage *imported_language;
-	GError *tmp_error = NULL;
 
-	g_return_if_fail (error != NULL && *error == NULL);
+	g_return_if_fail (parser_state->error == NULL);
 
 	lm = _gtk_source_language_get_language_manager (parser_state->language);
-	imported_language = gtk_source_language_manager_get_language_by_id (lm, lang_id);
+	imported_language = gtk_source_language_manager_get_language (lm, lang_id);
 
 	if (imported_language == NULL)
 	{
-		g_set_error (&tmp_error,
-				PARSER_ERROR,
-				PARSER_ERROR_WRONG_ID,
-				"unable to resolve language '%s'",
-				lang_id);
+		g_set_error (&parser_state->error,
+			     PARSER_ERROR,
+			     PARSER_ERROR_WRONG_ID,
+			     "unable to resolve language '%s'",
+			     lang_id);
 	}
 	else
 	{
@@ -1331,27 +1289,20 @@ parse_language_with_id (ParserState *parser_state,
 			    parser_state->defined_regexes,
 			    parser_state->styles_mapping,
 			    parser_state->loaded_lang_ids,
-			    &tmp_error);
-	}
-
-	if (tmp_error != NULL)
-	{
-		g_propagate_error (error, tmp_error);
-		return;
+			    parser_state->replacements,
+			    &parser_state->error);
 	}
 }
 
 static void
-parse_style (ParserState *parser_state,
-	     GError     **error)
+parse_style (ParserState *parser_state)
 {
 	gchar *id;
 	xmlChar *name, *map_to;
 	xmlChar *tmp;
 	gchar *lang_id = NULL;
-	GError *tmp_error = NULL;
 
-	g_return_if_fail (error != NULL && *error == NULL);
+	g_return_if_fail (parser_state->error == NULL);
 
 	tmp = xmlTextReaderGetAttribute (parser_state->reader,
 					 BAD_CAST "id");
@@ -1366,7 +1317,6 @@ parse_style (ParserState *parser_state,
 	name = xmlTextReaderGetAttribute (parser_state->reader,
 					  BAD_CAST "_name");
 
-	/* FIXME: actually use this name somehow */
 	if (name != NULL)
 	{
 		gchar *tmp2 = _gtk_source_language_translate_string (parser_state->language,
@@ -1387,49 +1337,64 @@ parse_style (ParserState *parser_state,
 
 	if (map_to != NULL && !id_is_decorated ((gchar*) map_to, &lang_id))
 	{
-		g_set_error (&tmp_error,
+		g_set_error (&parser_state->error,
 			     PARSER_ERROR,
 			     PARSER_ERROR_MALFORMED_MAP_TO,
 			     "the map-to attribute '%s' for the style '%s' lacks the prefix",
 			     map_to, id);
 	}
 
-	if (tmp_error == NULL && lang_id != NULL && lang_id[0] == 0)
+	if (parser_state->error == NULL && lang_id != NULL && lang_id[0] == 0)
 	{
 		g_free (lang_id);
 		lang_id = NULL;
 	}
 
-	if (tmp_error == NULL && lang_id != NULL && !lang_id_is_already_loaded (parser_state, lang_id))
-		parse_language_with_id (parser_state, lang_id, &tmp_error);
+	if (parser_state->error == NULL && lang_id != NULL &&
+	    !lang_id_is_already_loaded (parser_state, lang_id))
+	{
+		parse_language_with_id (parser_state, lang_id);
+	}
 
 	DEBUG (g_message ("style %s (%s) to be mapped to '%s'",
 			  name, id, map_to ? (char*) map_to : "(null)"));
 
-	if (tmp_error == NULL)
-		g_hash_table_insert (parser_state->styles_mapping, g_strdup (id),
-				     map_to ? g_strdup ((char*) map_to) : g_strdup (id));
+	if (map_to != NULL &&
+	    g_hash_table_lookup (parser_state->styles_mapping, map_to) == NULL)
+	{
+		g_warning ("in file %s: style '%s' not defined", parser_state->filename, map_to);
+	}
+
+	if (parser_state->error == NULL)
+	{
+
+		GtkSourceStyleInfo *info;
+
+		/* Remember the style name only if the style has been defined in
+		 * the lang file we are parsing */
+		if (g_str_has_prefix (id, parser_state->language_decoration))
+			info = _gtk_source_style_info_new ((gchar *) name,
+							   (gchar *) map_to);
+		else
+			info = _gtk_source_style_info_new (NULL,
+							   (gchar *) map_to);
+
+		g_hash_table_insert (parser_state->styles_mapping, g_strdup (id), info);
+	}
 
 	g_free (lang_id);
 	g_free (id);
 	xmlFree (name);
 	xmlFree (map_to);
-
-	if (tmp_error != NULL)
-	{
-		g_propagate_error (error, tmp_error);
-		return;
-	}
 }
 
 static void
-handle_keyword_char_class_element (ParserState *parser_state,
-				   GError     **error)
+handle_keyword_char_class_element (ParserState *parser_state)
 {
 	xmlChar *char_class;
 	int ret, type;
 
-	g_return_if_fail (error != NULL && *error == NULL);
+	g_return_if_fail (parser_state->error == NULL);
 
 	if (parser_state->ctx_data == NULL)
 		return;
@@ -1455,24 +1420,21 @@ handle_keyword_char_class_element (ParserState *parser_state,
 }
 
 static void
-handle_styles_element (ParserState *parser_state,
-		       GError     **error)
+handle_styles_element (ParserState *parser_state)
 {
-	int ret, type;
+	int type;
 	const xmlChar *tag_name;
-	GError *tmp_error = NULL;
 
-	g_return_if_fail (error != NULL && *error == NULL);
+	g_return_if_fail (parser_state->error == NULL);
 
-	while (TRUE)
+	while (parser_state->error == NULL)
 	{
-		ret = xmlTextReaderRead (parser_state->reader);
-
-		xmlTextReaderIsValid (parser_state->reader);
-
 		/* FIXME: is xmlTextReaderIsValid call needed here or
 		 * error func will be called? */
-		if (*error != NULL)
+		xmlTextReaderRead (parser_state->reader);
+		xmlTextReaderIsValid (parser_state->reader);
+
+		if (parser_state->error != NULL)
 			break;
 
 		tag_name = xmlTextReaderConstName (parser_state->reader);
@@ -1488,60 +1450,35 @@ handle_styles_element (ParserState *parser_state,
 			continue;
 
 		/* Handle <style> elements */
-		parse_style (parser_state, &tmp_error);
-
-		if (tmp_error != NULL)
-			break;
-	}
-
-	if (tmp_error != NULL)
-	{
-		g_propagate_error (error, tmp_error);
-		return;
+		parse_style (parser_state);
 	}
 }
 
 
 static void
-element_start (ParserState *parser_state,
-	       GError     **error)
+element_start (ParserState *parser_state)
 {
 	const xmlChar *name;
-	GError *tmp_error = NULL;
+
+	g_return_if_fail (parser_state->error == NULL);
 
 	/* TODO: check the namespace and ignore everithing is not in our namespace */
 	name = xmlTextReaderConstName (parser_state->reader);
 
 	if (xmlStrcmp (BAD_CAST "context", name) == 0)
-	{
-		handle_context_element (parser_state, &tmp_error);
-	}
+		handle_context_element (parser_state);
+	else if (xmlStrcmp (BAD_CAST "replace", name) == 0)
+		handle_replace_element (parser_state);
 	else if (xmlStrcmp (BAD_CAST "define-regex", name) == 0)
-	{
-		handle_define_regex_element (parser_state, &tmp_error);
-	}
+		handle_define_regex_element (parser_state);
 	else if (xmlStrcmp (BAD_CAST "language", name) == 0)
-	{
-		handle_language_element (parser_state, &tmp_error);
-	}
+		handle_language_element (parser_state);
 	else if (xmlStrcmp (BAD_CAST "styles", name) == 0)
-	{
-		handle_styles_element (parser_state, &tmp_error);
-	}
+		handle_styles_element (parser_state);
 	else if (xmlStrcmp (BAD_CAST "keyword-char-class", name) == 0)
-	{
-		handle_keyword_char_class_element (parser_state, &tmp_error);
-	}
+		handle_keyword_char_class_element (parser_state);
 	else if (xmlStrcmp (BAD_CAST "default-regex-options", name) == 0)
-	{
-		handle_default_regex_options_element (parser_state, &tmp_error);
-	}
-
-	if (tmp_error != NULL)
-	{
-		g_propagate_error (error, tmp_error);
-		return;
-	}
+		handle_default_regex_options_element (parser_state);
 }
 
 static void
@@ -1552,7 +1489,7 @@ element_end (ParserState *parser_state)
 
 	name = xmlTextReaderConstName (parser_state->reader);
 
-	if (!xmlStrcmp (name, BAD_CAST "context"))
+	if (xmlStrcmp (name, BAD_CAST "context") == 0)
 	{
 		/* pop the first element in the curr_parents list */
 		popped_id = g_queue_pop_head (parser_state->curr_parents);
@@ -1561,16 +1498,22 @@ element_end (ParserState *parser_state)
 }
 
 static void
-text_reader_structured_error_func (GError     **gerror,
+text_reader_structured_error_func (ParserState *parser_state,
 				   xmlErrorPtr  error)
 {
 	/* FIXME: does someone know how to use libxml api? */
 
-	if (*gerror == NULL)
-		g_set_error (gerror, PARSER_ERROR, PARSER_ERROR_INVALID_DOC, "parser error");
-
-	/* XXX: g_print now because of --g-fatal-warnings */
-	g_print ("in file %s on line %d: %s\n", error->file, error->line, error->message);
+	if (parser_state->error == NULL)
+		g_set_error (&parser_state->error,
+			     PARSER_ERROR,
+			     PARSER_ERROR_INVALID_DOC,
+			     "in file %s on line %d: %s\n",
+			     error->file,
+			     error->line,
+			     error->message);
+	else
+		/* extra warning won't hurt and it could give a clue about what's wrong */
+		g_warning ("in file %s on line %d: %s\n", error->file, error->line, error->message);
 }
 
 static gboolean
@@ -1580,11 +1523,11 @@ file_parse (gchar                     *filename,
 	    GHashTable                *defined_regexes,
 	    GHashTable                *styles,
 	    GHashTable                *loaded_lang_ids,
+	    GQueue                    *replacements,
 	    GError                   **error)
 {
 	ParserState *parser_state;
 	xmlTextReader *reader = NULL;
-	int ret;
 	int fd = -1;
 	GError *tmp_error = NULL;
 	GtkSourceLanguageManager *lm;
@@ -1635,20 +1578,21 @@ file_parse (gchar                     *filename,
 
 	parser_state = parser_state_new (language, ctx_data,
 					 defined_regexes, styles,
-					 reader, loaded_lang_ids);
+					 replacements, reader,
+					 filename, loaded_lang_ids);
 	xmlTextReaderSetStructuredErrorHandler (reader,
 						(xmlStructuredErrorFunc) text_reader_structured_error_func,
-						&tmp_error);
+						parser_state);
 
-	while (!tmp_error && (ret = xmlTextReaderRead (parser_state->reader)) == 1)
+	while (parser_state->error == NULL &&
+	       xmlTextReaderRead (parser_state->reader) == 1)
 	{
 		int type;
 
-		xmlTextReaderIsValid (reader);
+		/* FIXME: does xmlTextReaderRead already do it? */
+		xmlTextReaderIsValid (parser_state->reader);
 
-		/* FIXME: is xmlTextReaderIsValid call needed here or
-		 * error func will be called? */
-		if (tmp_error != NULL)
+		if (parser_state->error != NULL)
 			break;
 
 		type = xmlTextReaderNodeType (parser_state->reader);
@@ -1656,7 +1600,7 @@ file_parse (gchar                     *filename,
 		switch (type)
 		{
 			case XML_READER_TYPE_ELEMENT:
-				element_start (parser_state, &tmp_error);
+				element_start (parser_state);
 				break;
 			case XML_READER_TYPE_END_ELEMENT:
 				element_end (parser_state);
@@ -1664,6 +1608,12 @@ file_parse (gchar                     *filename,
 		}
 	}
 
+	if (parser_state->error != NULL)
+	{
+		g_propagate_error (&tmp_error, parser_state->error);
+		parser_state->error = NULL;
+	}
+
 	parser_state_destroy (parser_state);
 
 	if (tmp_error != NULL)
@@ -1685,24 +1635,32 @@ parser_state_new (GtkSourceLanguage       *language,
 		  GtkSourceContextData    *ctx_data,
 		  GHashTable              *defined_regexes,
 		  GHashTable              *styles_mapping,
+		  GQueue                  *replacements,
 		  xmlTextReader	          *reader,
+		  const char              *filename,
 		  GHashTable              *loaded_lang_ids)
 {
 	ParserState *parser_state;
-	parser_state = g_new (ParserState, 1);
+	parser_state = g_slice_new0 (ParserState);
 
 	parser_state->language = language;
 	parser_state->ctx_data = ctx_data;
 
+	g_return_val_if_fail (language->priv->id != NULL, NULL);
+	parser_state->language_decoration = g_strdup_printf ("%s:", language->priv->id);
+
 	parser_state->current_lang_id = NULL;
 
 	parser_state->id_cookie = 0;
 	parser_state->regex_compile_flags = 0;
 
 	parser_state->reader = reader;
+	parser_state->filename = g_filename_display_name (filename);
+	parser_state->error = NULL;
 
 	parser_state->defined_regexes = defined_regexes;
 	parser_state->styles_mapping = styles_mapping;
+	parser_state->replacements = replacements;
 
 	parser_state->loaded_lang_ids = loaded_lang_ids;
 
@@ -1720,27 +1678,28 @@ parser_state_destroy (ParserState *parser_state)
 	if (parser_state->reader != NULL)
 		xmlFreeTextReader (parser_state->reader);
 
+	if (parser_state->error != NULL)
+		g_error_free (parser_state->error);
+
 	g_queue_free (parser_state->curr_parents);
 	g_free (parser_state->current_lang_id);
 
 	g_free (parser_state->opening_delimiter);
 	g_free (parser_state->closing_delimiter);
 
-	g_free (parser_state);
+	g_free (parser_state->language_decoration);
+	g_free (parser_state->filename);
+
+	g_slice_free (ParserState, parser_state);
 }
 
 static gboolean
-steal_styles_mapping (char       *style,
-		      char       *map_to,
-		      GHashTable *styles)
+steal_styles_mapping (gchar              *style_id,
+		      GtkSourceStyleInfo *info,
+		      GHashTable         *styles)
 {
-	if (strcmp (style, map_to) != 0)
-	{
-		g_hash_table_insert (styles, style, map_to);
-		return TRUE;
-	}
-
-	return FALSE;
+	g_hash_table_insert (styles, style_id, info);
+	return TRUE;
 }
 
 gboolean
@@ -1752,6 +1711,7 @@ _gtk_source_language_file_parse_version2 (GtkSourceLanguage       *language,
 	GError *error = NULL;
 	gchar *filename;
 	GHashTable *loaded_lang_ids;
+	GQueue *replacements;
 
 	g_return_val_if_fail (ctx_data != NULL, FALSE);
 
@@ -1768,24 +1728,30 @@ _gtk_source_language_file_parse_version2 (GtkSourceLanguage       *language,
 
 	defined_regexes = g_hash_table_new_full (g_str_hash, g_str_equal,
 						 g_free, g_free);
-	styles = g_hash_table_new_full (g_str_hash, g_str_equal,
-					g_free, g_free);
+	styles = g_hash_table_new_full (g_str_hash,
+					g_str_equal,
+					g_free,
+					(GDestroyNotify) _gtk_source_style_info_free);
 	loaded_lang_ids = g_hash_table_new_full (g_str_hash, g_str_equal,
 						 (GDestroyNotify) xmlFree,
 						 NULL);
+	replacements = g_queue_new ();
 
 	success = file_parse (filename, language, ctx_data,
 			      defined_regexes, styles,
-			      loaded_lang_ids, &error);
+			      loaded_lang_ids, replacements,
+			      &error);
 
 	if (success)
-		success = _gtk_source_context_data_resolve_refs (ctx_data, &error);
+		success = _gtk_source_context_data_finish_parse (ctx_data, replacements->head, &error);
 
 	if (success)
 		g_hash_table_foreach_steal (styles,
 					    (GHRFunc) steal_styles_mapping,
 					    language->priv->styles);
 
+	g_queue_foreach (replacements, (GFunc) _gtk_source_context_replace_free, NULL);
+	g_queue_free (replacements);
 	g_hash_table_destroy (loaded_lang_ids);
 	g_hash_table_destroy (defined_regexes);
 	g_hash_table_destroy (styles);
