@@ -3,25 +3,32 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #define MOOEDIT_COMPILATION
 #include "mooedit/moocommand-private.h"
-#include "mooedit/moocommand-script.h"
-#include "mooedit/moocommand-exe.h"
+#include "mooedit/moocommand-lua.h"
 #include "mooedit/mooeditwindow.h"
 #include "mooedit/moooutputfilterregex.h"
 #include "mooedit/mooedit-enums.h"
+#include "mooutils/mooutils-debug.h"
 #include "mooutils/mooutils-misc.h"
-#include <gtk/gtkwindow.h>
-#include <gtk/gtktextview.h>
+#ifdef MOO_OS_UNIX
+#include "mooedit/moocommand-unx.h"
+#endif
+#include <gtk/gtk.h>
+#include <glib/gstdio.h>
 #include <string.h>
+#include <stdio.h>
 
 
 #define KEY_TYPE    "type"
@@ -61,6 +68,7 @@ struct _MooCommandData {
 };
 
 typedef struct {
+    char *id;
     char *name;
     MooCommandFilterFactory factory_func;
     gpointer data;
@@ -102,7 +110,8 @@ void
 moo_command_factory_register (const char        *name,
                               const char        *display_name,
                               MooCommandFactory *factory,
-                              char             **keys)
+                              char             **keys,
+                              const char        *extension)
 {
     MooCommandFactoryClass *klass;
 
@@ -136,6 +145,7 @@ moo_command_factory_register (const char        *name,
     factory->display_name = g_strdup (display_name);
     factory->keys = g_strdupv (keys);
     factory->n_keys = keys ? g_strv_length (keys) : 0;
+    factory->extension = g_strdup (extension);
 
     g_hash_table_insert (registered_factories, g_strdup (name), g_object_ref (factory));
 }
@@ -190,10 +200,56 @@ moo_command_factory_finalize (GObject *object)
     G_OBJECT_CLASS (moo_command_factory_parent_class)->finalize (object);
 }
 
+static GtkWidget *
+dummy_create_widget (G_GNUC_UNUSED MooCommandFactory *factory)
+{
+    return gtk_vbox_new (FALSE, FALSE);
+}
+
+static void
+dummy_load_data (G_GNUC_UNUSED MooCommandFactory *factory,
+                 G_GNUC_UNUSED GtkWidget *page,
+                 G_GNUC_UNUSED MooCommandData *data)
+{
+}
+
+static gboolean
+dummy_save_data (G_GNUC_UNUSED MooCommandFactory *factory,
+                 G_GNUC_UNUSED GtkWidget *page,
+                 G_GNUC_UNUSED MooCommandData *data)
+{
+    return FALSE;
+}
+
+static gboolean
+default_data_equal (MooCommandFactory *factory,
+                    MooCommandData    *data1,
+                    MooCommandData    *data2)
+{
+    guint i;
+    const char *val1, *val2;
+
+    for (i = 0; i < factory->n_keys; ++i)
+    {
+        val1 = moo_command_data_get (data1, i);
+        val2 = moo_command_data_get (data2, i);
+        if (!_moo_str_equal (val1, val2))
+            return FALSE;
+    }
+
+    val1 = moo_command_data_get_code (data1);
+    val2 = moo_command_data_get_code (data2);
+    return _moo_str_equal (val1, val2);
+}
+
 static void
 moo_command_factory_class_init (MooCommandFactoryClass *klass)
 {
     G_OBJECT_CLASS (klass)->finalize = moo_command_factory_finalize;
+    klass->create_widget = dummy_create_widget;
+    klass->load_data = dummy_load_data;
+    klass->save_data = dummy_save_data;
+    klass->data_equal = default_data_equal;
 }
 
 
@@ -312,7 +368,7 @@ moo_command_check_sensitive_real (MooCommand *cmd,
     if ((cmd->options & MOO_COMMAND_NEED_DOC) && !doc)
         return FALSE;
 
-    if ((cmd->options & MOO_COMMAND_NEED_FILE) && (!MOO_IS_EDIT (doc) || !moo_edit_get_filename (doc)))
+    if ((cmd->options & MOO_COMMAND_NEED_FILE) && (!MOO_IS_EDIT (doc) || moo_edit_is_untitled (doc)))
         return FALSE;
 
     return TRUE;
@@ -411,7 +467,7 @@ check_context (MooCommandOptions options,
         return FALSE;
 
     if ((options & MOO_COMMAND_NEED_FILE) &&
-        !(MOO_IS_EDIT (doc) && moo_edit_get_filename (doc) != NULL))
+        !(MOO_IS_EDIT (doc) && !moo_edit_is_untitled (doc)))
             return FALSE;
 
     if ((options & MOO_COMMAND_NEED_SAVE) && !MOO_IS_EDIT (doc))
@@ -442,9 +498,7 @@ moo_command_run (MooCommand         *cmd,
     }
     else if (cmd->options & MOO_COMMAND_NEED_SAVE)
     {
-        if (!save_one (doc))
-            return;
-        if (!moo_edit_get_filename (doc))
+        if (!save_one (doc) || moo_edit_is_untitled (doc))
             return;
     }
 
@@ -615,7 +669,6 @@ moo_command_context_get (MooCommandContext  *ctx,
 }
 
 
-#if 0
 const char *
 moo_command_context_get_string (MooCommandContext  *ctx,
                                 const char         *name)
@@ -633,7 +686,6 @@ moo_command_context_get_string (MooCommandContext  *ctx,
     g_return_val_if_fail (G_VALUE_TYPE (&var->value) == G_TYPE_STRING, NULL);
     return g_value_get_string (&var->value);
 }
-#endif
 
 
 void
@@ -856,6 +908,204 @@ find_key (MooCommandFactory *factory,
     return -1;
 }
 
+
+static void
+get_one_option (MooCommandFactory *factory,
+                MooCommandData    *cmd_data,
+                const char        *key,
+                const char        *value,
+                GPtrArray        **options)
+{
+    int index;
+
+    index = find_key (factory, key);
+
+    if (index >= 0)
+    {
+        moo_command_data_set (cmd_data, index, value);
+    }
+    else
+    {
+        if (!*options)
+            *options = g_ptr_array_new ();
+        g_ptr_array_add (*options, g_strdup (key));
+        g_ptr_array_add (*options, g_strdup (value));
+    }
+}
+
+static void
+get_options_from_contents (MooCommandFactory *factory,
+                           MooCommandData    *cmd_data,
+                           const char        *contents,
+                           GPtrArray        **options,
+                           const char        *filename)
+{
+    guint i;
+    guint line_ends;
+    const char *start = NULL;
+    const char *end = NULL;
+    char **p, **comps;
+    char *opt_string;
+
+    for (i = 0, line_ends = 0; i < 2048 && contents[i]; i++)
+    {
+        if (contents[i] == '\n' || contents[i] == '\r')
+        {
+            if (line_ends)
+                break;
+            if (contents[i] == '\r' && contents[i+1] == '\n')
+                i += 1;
+            line_ends += 1;
+        }
+        else if (contents[i] == '!' && contents[i+1] == '!')
+        {
+            if (!start)
+            {
+                start = contents + i + 2;
+                i += 1;
+            }
+            else
+            {
+                end = contents + i;
+                break;
+            }
+        }
+    }
+
+    if (!start || !end)
+        return;
+
+    opt_string = g_strndup (start, end - start);
+    comps = g_strsplit (opt_string, ";", 0);
+
+    for (p = comps; p && *p; ++p)
+    {
+        char *eq = strchr (*p, '=');
+
+        if (!eq)
+        {
+            g_warning ("%s: invalid option '%s' in file %s", G_STRFUNC, *p, filename);
+        }
+        else
+        {
+            char *key = *p;
+            char *value = eq + 1;
+            *eq = 0;
+            g_strstrip (key);
+            g_strstrip (value);
+            get_one_option (factory, cmd_data, key, value, options);
+        }
+    }
+
+    g_strfreev (comps);
+    g_free (opt_string);
+}
+
+static void
+get_options_from_file (MooCommandFactory *factory,
+                       MooCommandData    *cmd_data,
+                       const char        *filename,
+                       GPtrArray        **options)
+{
+    FILE *file;
+    char buf[2048];
+
+    if (!(file = g_fopen (filename, "rb")))
+    {
+        g_warning ("%s: could not open file %s", G_STRFUNC, filename);
+        return;
+    }
+
+    buf[0] = 0;
+    buf[sizeof buf - 1] = '\1';
+
+    if (fgets (buf, sizeof buf, file) && buf[sizeof buf - 1] != 0)
+    {
+        int len = strlen (buf);
+        fgets (buf + len, sizeof buf - len, file);
+    }
+
+    get_options_from_contents (factory, cmd_data, buf, options, filename);
+
+    fclose (file);
+}
+
+MooCommandData *
+_moo_command_parse_file (const char         *filename,
+                         MooCommandFactory **factory_p,
+                         char             ***params)
+{
+    GSList *factories;
+    MooCommandFactory *factory = NULL;
+    MooCommandData *cmd_data = NULL;
+    GPtrArray *array = NULL;
+
+    g_return_val_if_fail (filename != NULL, NULL);
+    g_return_val_if_fail (factory_p != NULL, NULL);
+    g_return_val_if_fail (params != NULL, NULL);
+
+    *factory_p = NULL;
+    *params = NULL;
+
+    factories  = moo_command_list_factories ();
+    while (factories)
+    {
+        factory = factories->data;
+
+        if (factory->extension && g_str_has_suffix (filename, factory->extension))
+            break;
+
+        factory = NULL;
+        factories = g_slist_delete_link (factories, factories);
+    }
+    g_slist_free (factories);
+
+    if (factory)
+    {
+        char *contents;
+        GError *error = NULL;
+
+        if (!g_file_get_contents (filename, &contents, NULL, &error))
+        {
+            g_warning ("could not read file %s: %s", filename, error->message);
+            g_error_free (error);
+            return NULL;
+        }
+
+        cmd_data = moo_command_data_new (factory->n_keys);
+        moo_command_data_take_code (cmd_data, contents);
+        get_options_from_contents (factory, cmd_data, contents, &array, filename);
+    }
+
+#ifndef __WIN32__
+    if (!factory)
+    {
+        if (g_file_test (filename, G_FILE_TEST_IS_EXECUTABLE))
+            factory = moo_command_factory_lookup ("exe");
+
+        if (factory)
+        {
+            cmd_data = moo_command_data_new (factory->n_keys);
+            moo_command_data_set_code (cmd_data, filename);
+            get_options_from_file (factory, cmd_data, filename, &array);
+        }
+    }
+#endif
+
+    if (!factory)
+        g_warning ("%s: unknown file type: %s", G_STRFUNC, filename);
+
+    if (array)
+    {
+        g_ptr_array_add (array, NULL);
+        *params = (char**) g_ptr_array_free (array, FALSE);
+    }
+
+    *factory_p = factory;
+    return cmd_data;
+}
+
+
 static void
 item_foreach_func (const char *key,
                    const char *val,
@@ -920,8 +1170,9 @@ _moo_command_parse_item (MooKeyFileItem     *item,
 
     if (!factory)
     {
-        g_warning ("unknown command type %s in item %s in file %s",
-                   factory_name, name, filename);
+        if (strcmp (factory_name, "python") != 0)
+            g_warning ("unknown command type %s in item %s in file %s",
+                       factory_name, name, filename);
         goto error;
     }
 
@@ -1045,14 +1296,9 @@ moo_command_data_set (MooCommandData *data,
                       guint           index,
                       const char     *value)
 {
-    char *tmp;
-
     g_return_if_fail (data != NULL);
     g_return_if_fail (index < data->len);
-
-    tmp = data->data[index];
-    data->data[index] = g_strdup (value);
-    g_free (tmp);
+    MOO_ASSIGN_STRING (data->data[index], value);
 }
 
 
@@ -1070,13 +1316,9 @@ static void
 moo_command_data_take_code (MooCommandData *data,
                             char           *code)
 {
-    char *tmp;
-
     g_return_if_fail (data != NULL);
-
-    tmp = data->code;
+    g_free (data->code);
     data->code = code;
-    g_free (tmp);
 }
 
 
@@ -1117,11 +1359,13 @@ _moo_command_init (void)
 
     if (!been_here)
     {
-        g_type_class_unref (g_type_class_ref (MOO_TYPE_COMMAND_SCRIPT));
-#ifndef __WIN32__
-        g_type_class_unref (g_type_class_ref (MOO_TYPE_COMMAND_EXE));
+#ifdef MOO_BUILD_LUA
+        g_type_class_unref (g_type_class_ref (MOO_TYPE_COMMAND_LUA));
 #endif
+#ifdef MOO_OS_UNIX
+        g_type_class_unref (g_type_class_ref (MOO_TYPE_COMMAND_UNX));
         _moo_command_filter_regex_load ();
+#endif
         been_here = TRUE;
     }
 }
@@ -1169,6 +1413,7 @@ moo_command_filter_register (const char             *id,
     }
 
     info = g_new0 (FilterInfo, 1);
+    info->id = g_strdup (id);
     info->name = g_strdup (name);
     info->factory_func = factory_func;
     info->data = data;
@@ -1199,6 +1444,7 @@ moo_command_filter_unregister (const char *id)
         info->data_notify (info->data);
 
     g_free (info->name);
+    g_free (info->id);
     g_free (info);
 }
 
@@ -1217,22 +1463,48 @@ moo_command_filter_lookup (const char *id)
 
 
 static void
-prepend_filter_id (const char *id,
-                   G_GNUC_UNUSED gpointer info,
-                   GSList    **list)
+prepend_filter_info (G_GNUC_UNUSED const char *id,
+                     FilterInfo *info,
+                     GSList    **list)
 {
-    *list = g_slist_prepend (*list, g_strdup (id));
+    *list = g_slist_prepend (*list, info);
+}
+
+static int
+compare_filter_names (FilterInfo *fi1,
+                      FilterInfo *fi2)
+{
+    if (strcmp (fi1->id, "default") == 0)
+        return -1;
+    else if (strcmp (fi2->id, "default") == 0)
+        return 1;
+    else if (strcmp (fi1->id, "none") == 0)
+        return -1;
+    else if (strcmp (fi2->id, "none") == 0)
+        return 1;
+    else
+        return g_utf8_collate (fi1->name, fi2->name);
 }
 
 GSList *
 moo_command_filter_list (void)
 {
     GSList *list = NULL;
+    GSList *ids = NULL;
 
     if (registered_filters)
-        g_hash_table_foreach (registered_filters, (GHFunc) prepend_filter_id, &list);
+        g_hash_table_foreach (registered_filters, (GHFunc) prepend_filter_info, &list);
 
-    return list;
+    list = g_slist_sort (list, (GCompareFunc) compare_filter_names);
+
+    while (list)
+    {
+        FilterInfo *fi = list->data;
+        ids = g_slist_prepend (ids, g_strdup (fi->id));
+        list = g_slist_delete_link (list, list);
+    }
+
+    return g_slist_reverse (ids);
 }
 
 
