@@ -1,12 +1,11 @@
 /*
  *   mooedit.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -23,8 +22,6 @@
 #include "mooedit/mooeditfiltersettings.h"
 #include "mooedit/mooeditor-private.h"
 #include "mooutils/moomarshals.h"
-#include "mooutils/moocompat.h"
-#include "mooutils/mooutils-gobject.h"
 #include "mooutils/mooglade.h"
 #include "mooutils/mooi18n.h"
 #include "mooutils/mooutils-misc.h"
@@ -82,6 +79,9 @@ static GtkTextBuffer *get_buffer                (MooEdit        *edit);
 static void     modified_changed_cb             (GtkTextBuffer  *buffer,
                                                  MooEdit        *edit);
 
+static void     moo_edit_apply_style_scheme     (MooTextView        *view,
+                                                 MooTextStyleScheme *scheme);
+
 
 enum {
     DOC_STATUS_CHANGED,
@@ -101,7 +101,8 @@ enum {
     PROP_0,
     PROP_EDITOR,
     PROP_ENABLE_BOOKMARKS,
-    PROP_HAS_COMMENTS
+    PROP_HAS_COMMENTS,
+    PROP_ENCODING
 };
 
 /* MOO_TYPE_EDIT */
@@ -128,6 +129,7 @@ moo_edit_class_init (MooEditClass *klass)
     widget_class->focus_out_event = moo_edit_focus_out;
 
     textview_class->line_mark_clicked = _moo_edit_line_mark_clicked;
+    textview_class->apply_style_scheme = moo_edit_apply_style_scheme;
 
     klass->filename_changed = moo_edit_filename_changed;
     klass->config_notify = moo_edit_config_notify;
@@ -158,6 +160,14 @@ moo_edit_class_init (MooEditClass *klass)
                                              FALSE,
                                              G_PARAM_READABLE));
 
+    g_object_class_install_property (gobject_class,
+                                     PROP_ENCODING,
+                                     g_param_spec_string ("encoding",
+                                             "encoding",
+                                             "encoding",
+                                             NULL,
+                                             G_PARAM_READWRITE));
+
     signals[CONFIG_NOTIFY] =
             g_signal_new ("config-notify",
                           G_OBJECT_CLASS_TYPE (klass),
@@ -299,7 +309,6 @@ moo_edit_finalize (GObject *object)
     MooEdit *edit = MOO_EDIT (object);
 
     g_free (edit->priv->filename);
-    g_free (edit->priv->basename);
     g_free (edit->priv->display_filename);
     g_free (edit->priv->display_basename);
     g_free (edit->priv->encoding);
@@ -361,12 +370,6 @@ moo_edit_dispose (GObject *object)
 
     _moo_edit_delete_bookmarks (edit, TRUE);
 
-    if (edit->priv->menu)
-    {
-        gtk_widget_destroy (GTK_WIDGET (edit->priv->menu));
-        edit->priv->menu = NULL;
-    }
-
     if (edit->priv->actions)
     {
         g_object_unref (edit->priv->actions);
@@ -463,8 +466,8 @@ _moo_edit_set_status (MooEdit        *edit,
 
 
 MooEditFileInfo*
-moo_edit_file_info_new (const char         *filename,
-                        const char         *encoding)
+moo_edit_file_info_new (const char *filename,
+                        const char *encoding)
 {
     MooEditFileInfo *info = g_new0 (MooEditFileInfo, 1);
     info->filename = g_strdup (filename);
@@ -474,7 +477,7 @@ moo_edit_file_info_new (const char         *filename,
 
 
 MooEditFileInfo*
-moo_edit_file_info_copy (const MooEditFileInfo  *info)
+moo_edit_file_info_copy (const MooEditFileInfo *info)
 {
     MooEditFileInfo *copy;
     g_return_val_if_fail (info != NULL, NULL);
@@ -485,7 +488,7 @@ moo_edit_file_info_copy (const MooEditFileInfo  *info)
 }
 
 void
-moo_edit_file_info_free (MooEditFileInfo    *info)
+moo_edit_file_info_free (MooEditFileInfo *info)
 {
     if (info)
     {
@@ -503,7 +506,7 @@ moo_edit_is_empty (MooEdit *edit)
 
     g_return_val_if_fail (MOO_IS_EDIT (edit), FALSE);
 
-    if (MOO_EDIT_IS_BUSY (edit) || MOO_EDIT_IS_MODIFIED (edit) || edit->priv->filename)
+    if (MOO_EDIT_IS_BUSY (edit) || MOO_EDIT_IS_MODIFIED (edit) || !MOO_EDIT_IS_UNTITLED (edit))
         return FALSE;
 
     gtk_text_buffer_get_bounds (get_buffer (edit), &start, &end);
@@ -511,6 +514,13 @@ moo_edit_is_empty (MooEdit *edit)
     return !gtk_text_iter_compare (&start, &end);
 }
 
+gboolean
+moo_edit_is_untitled (MooEdit *edit)
+{
+    g_return_val_if_fail (MOO_IS_EDIT (edit), FALSE);
+    return MOO_EDIT_IS_UNTITLED (edit);
+}
+
 
 MooEditStatus
 moo_edit_get_status (MooEdit *edit)
@@ -538,6 +548,10 @@ moo_edit_set_property (GObject        *object,
             moo_edit_set_enable_bookmarks (edit, g_value_get_boolean (value));
             break;
 
+        case PROP_ENCODING:
+            _moo_edit_set_encoding (edit, g_value_get_string (value));
+            break;
+
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
             break;
@@ -567,6 +581,10 @@ moo_edit_get_property (GObject        *object,
             g_value_set_boolean (value, _moo_edit_has_comments (edit, NULL, NULL));
             break;
 
+        case PROP_ENCODING:
+            g_value_set_string (value, edit->priv->encoding);
+            break;
+
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
             break;
@@ -574,6 +592,15 @@ moo_edit_get_property (GObject        *object,
 }
 
 
+static void
+moo_edit_apply_style_scheme (MooTextView        *view,
+                             MooTextStyleScheme *scheme)
+{
+    MOO_TEXT_VIEW_CLASS (moo_edit_parent_class)->apply_style_scheme (view, scheme);
+    _moo_edit_update_bookmarks_style (MOO_EDIT (view));
+}
+
+
 static gboolean
 moo_edit_focus_in (GtkWidget     *widget,
                    GdkEventFocus *event)
@@ -618,22 +645,15 @@ moo_edit_file_info_get_type (void)
 }
 
 
-const char *
+char *
 moo_edit_get_filename (MooEdit *edit)
 {
     g_return_val_if_fail (MOO_IS_EDIT (edit), NULL);
-    return edit->priv->filename;
+    return g_strdup (edit->priv->filename);
 }
 
 const char *
-moo_edit_get_basename (MooEdit *edit)
-{
-    g_return_val_if_fail (MOO_IS_EDIT (edit), NULL);
-    return edit->priv->basename;
-}
-
-const char *
-moo_edit_get_display_filename (MooEdit *edit)
+moo_edit_get_display_name (MooEdit *edit)
 {
     g_return_val_if_fail (MOO_IS_EDIT (edit), NULL);
     return edit->priv->display_filename;
@@ -646,14 +666,6 @@ moo_edit_get_display_basename (MooEdit *edit)
     return edit->priv->display_basename;
 }
 
-const char *
-moo_edit_get_encoding (MooEdit *edit)
-{
-    g_return_val_if_fail (MOO_IS_EDIT (edit), NULL);
-    return edit->priv->encoding;
-}
-
-
 char *
 moo_edit_get_uri (MooEdit *edit)
 {
@@ -665,6 +677,27 @@ moo_edit_get_uri (MooEdit *edit)
         return NULL;
 }
 
+const char *
+moo_edit_get_encoding (MooEdit *edit)
+{
+    g_return_val_if_fail (MOO_IS_EDIT (edit), NULL);
+    return edit->priv->encoding;
+}
+
+void
+_moo_edit_set_encoding (MooEdit    *edit,
+                        const char *encoding)
+{
+    g_return_if_fail (MOO_IS_EDIT (edit));
+    g_return_if_fail (encoding != NULL);
+
+    if (!_moo_str_equal (encoding, edit->priv->encoding))
+    {
+        MOO_ASSIGN_STRING (edit->priv->encoding, encoding);
+        g_object_notify (G_OBJECT (edit), "encoding");
+    }
+}
+
 
 static GtkTextBuffer *
 get_buffer (MooEdit *edit)
@@ -969,6 +1002,7 @@ moo_edit_apply_config (MooEdit *edit)
     GtkWrapMode wrap_mode;
     gboolean line_numbers;
     guint tab_width;
+    char *word_chars;
 
     moo_edit_apply_lang_config (edit);
 
@@ -976,11 +1010,15 @@ moo_edit_apply_config (MooEdit *edit)
                          "wrap-mode", &wrap_mode,
                          "show-line-numbers", &line_numbers,
                          "tab-width", &tab_width,
+                         "word-chars", &word_chars,
                          NULL);
 
     gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (edit), wrap_mode);
     moo_text_view_set_show_line_numbers (MOO_TEXT_VIEW (edit), line_numbers);
     moo_text_view_set_tab_width (MOO_TEXT_VIEW (edit), tab_width);
+    moo_text_view_set_word_chars (MOO_TEXT_VIEW (edit), word_chars);
+
+    g_free (word_chars);
 }
 
 
@@ -1046,7 +1084,7 @@ moo_edit_filename_changed (MooEdit    *edit,
     gboolean lang_changed = FALSE;
     MooLang *lang = NULL, *old_lang = NULL;
     const char *lang_id = NULL;
-    const char *filter_config = NULL;
+    char *filter_config = NULL;
 
     old_lang = moo_text_view_get_lang (MOO_TEXT_VIEW (edit));
 
@@ -1087,6 +1125,8 @@ moo_edit_filename_changed (MooEdit    *edit,
     }
 
     _moo_edit_thaw_config_notify (edit);
+
+    g_free (filter_config);
 }
 
 
@@ -1494,6 +1534,22 @@ moo_edit_uncomment (MooEdit *edit)
 }
 
 
+void
+_moo_edit_ensure_newline (MooEdit *edit)
+{
+    GtkTextBuffer *buffer;
+    GtkTextIter iter;
+
+    g_return_if_fail (MOO_IS_EDIT (edit));
+
+    buffer = get_buffer (edit);
+    gtk_text_buffer_get_end_iter (buffer, &iter);
+
+    if (!gtk_text_iter_starts_line (&iter) || gtk_text_iter_is_start (&iter))
+        gtk_text_buffer_insert (buffer, &iter, "\n", -1);
+}
+
+
 /*****************************************************************************/
 /* popup menu
  */
@@ -1573,48 +1629,39 @@ popup_position_func (GtkMenu   *menu,
     *push_in = FALSE;
 }
 
-static void
-popup_detach_func (MooEdit *edit)
-{
-    edit->priv->menu = NULL;
-}
-
 void
 _moo_edit_do_popup (MooEdit        *edit,
                     GdkEventButton *event)
 {
     MooUIXML *xml;
     MooEditWindow *window;
+    GtkMenu *menu;
 
     window = moo_edit_get_window (edit);
     xml = moo_editor_get_doc_ui_xml (edit->priv->editor);
     g_return_if_fail (xml != NULL);
 
-    if (edit->priv->menu)
-        gtk_widget_destroy (GTK_WIDGET (edit->priv->menu));
-
-    edit->priv->menu =
-        moo_ui_xml_create_widget (xml, MOO_UI_MENU, "Editor/Popup", edit->priv->actions,
-                                  window ? MOO_WINDOW(window)->accel_group : NULL);
-    g_return_if_fail (edit->priv->menu != NULL);
-    gtk_menu_attach_to_widget (edit->priv->menu, GTK_WIDGET (edit),
-                               (GtkMenuDetachFunc) popup_detach_func);
+    menu = moo_ui_xml_create_widget (xml, MOO_UI_MENU, "Editor/Popup", edit->priv->actions,
+                                     window ? MOO_WINDOW(window)->accel_group : NULL);
+    g_return_if_fail (menu != NULL);
+    MOO_OBJECT_REF_SINK (menu);
 
     _moo_edit_check_actions (edit);
 
     if (event)
     {
-        gtk_menu_popup (edit->priv->menu,
-                        NULL, NULL, NULL, NULL,
+        gtk_menu_popup (menu, NULL, NULL, NULL, NULL,
                         event->button, event->time);
     }
     else
     {
-        gtk_menu_popup (edit->priv->menu, NULL, NULL,
+        gtk_menu_popup (menu, NULL, NULL,
                         popup_position_func, edit,
                         0, gtk_get_current_event_time ());
-        gtk_menu_shell_select_first (GTK_MENU_SHELL (edit->priv->menu), FALSE);
+        gtk_menu_shell_select_first (GTK_MENU_SHELL (menu), FALSE);
     }
+
+    g_object_unref (menu);
 }
 
 
@@ -1713,7 +1760,7 @@ show_progress (MooEdit *edit)
 
     g_return_val_if_fail (!edit->priv->progress, FALSE);
 
-    xml = moo_glade_xml_new_from_buf (MOO_EDIT_PROGRESS_GLADE_XML, -1,
+    xml = moo_glade_xml_new_from_buf (mooeditprogress_glade_xml, -1,
                                       "eventbox", GETTEXT_PACKAGE, NULL);
     g_return_val_if_fail (xml != NULL, FALSE);
 
