@@ -1,12 +1,11 @@
 /*
  *   mooeditaction-factory.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -18,9 +17,9 @@
 #include "mooutils/mooutils-gobject.h"
 #include "mooutils/mooactionfactory.h"
 #include "mooutils/mooactionbase.h"
-#include "mooutils/moocompat.h"
 #include "mooutils/moomenuaction.h"
 #include "mooutils/mooi18n.h"
+#include "mooutils/moocompat.h"
 #include <string.h>
 #include <gobject/gvaluecollector.h>
 
@@ -225,7 +224,10 @@ moo_edit_class_install_action (MooEditClass      *klass,
             GtkAction *action = create_action (action_id, info, l->data);
 
             if (action)
+            {
                 moo_edit_add_action (l->data, action);
+                g_object_unref (action);
+            }
         }
     }
 }
@@ -542,7 +544,10 @@ add_action (const char *id,
     GtkAction *action = create_action (id, info, edit);
 
     if (action)
+    {
         moo_edit_add_action (edit, action);
+        g_object_unref (action);
+    }
 }
 
 void
@@ -589,10 +594,11 @@ create_input_methods_menu_item (GtkAction *action)
     gtk_im_multicontext_append_menuitems (GTK_IM_MULTICONTEXT (edit->im_context),
 					  GTK_MENU_SHELL (menu));
 
-    if (!gtk_check_version (2, 10, 0))
-        g_object_get (gtk_widget_get_settings (GTK_WIDGET (edit)),
-                      "gtk-show-input-method-menu", &visible,
-                      NULL);
+#if GTK_CHECK_VERSION(2, 10, 0)
+    g_object_get (gtk_widget_get_settings (GTK_WIDGET (edit)),
+                  "gtk-show-input-method-menu", &visible,
+                  NULL);
+#endif
     g_object_set (action, "visible", visible, NULL);
 
     return item;
@@ -614,10 +620,11 @@ create_special_chars_menu_item (GtkAction *action)
     gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
     append_special_char_menuitems (GTK_MENU_SHELL (menu), edit);
 
-    if (!gtk_check_version (2, 10, 0))
-        g_object_get (gtk_widget_get_settings (GTK_WIDGET (edit)),
-                      "gtk-show-unicode-menu", &visible,
-                      NULL);
+#if GTK_CHECK_VERSION(2, 10, 0)
+    g_object_get (gtk_widget_get_settings (GTK_WIDGET (edit)),
+                  "gtk-show-unicode-menu", &visible,
+                  NULL);
+#endif
     g_object_set (action, "visible", visible, NULL);
 
     return item;
@@ -694,16 +701,16 @@ static const struct BidiMenuEntry {
   const char *label;
   gunichar ch;
 } bidi_menu_entries[] = {
-  { N_("LRM _Left-to-right mark"), 0x200E },
-  { N_("RLM _Right-to-left mark"), 0x200F },
-  { N_("LRE Left-to-right _embedding"), 0x202A },
-  { N_("RLE Right-to-left e_mbedding"), 0x202B },
-  { N_("LRO Left-to-right _override"), 0x202D },
-  { N_("RLO Right-to-left o_verride"), 0x202E },
-  { N_("PDF _Pop directional formatting"), 0x202C },
-  { N_("ZWS _Zero width space"), 0x200B },
-  { N_("ZWJ Zero width _joiner"), 0x200D },
-  { N_("ZWNJ Zero width _non-joiner"), 0x200C }
+  { "LRM _Left-to-right mark", 0x200E },
+  { "RLM _Right-to-left mark", 0x200F },
+  { "LRE Left-to-right _embedding", 0x202A },
+  { "RLE Right-to-left e_mbedding", 0x202B },
+  { "LRO Left-to-right _override", 0x202D },
+  { "RLO Right-to-left o_verride", 0x202E },
+  { "PDF _Pop directional formatting", 0x202C },
+  { "ZWS _Zero width space", 0x200B },
+  { "ZWJ Zero width _joiner", 0x200D },
+  { "ZWNJ Zero width _non-joiner", 0x200C }
 };
 
 static void
