@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -17,8 +16,9 @@
 #include "mooedit/mooeditprefs.h"
 #include "mooedit/mootextfind-glade.h"
 #include "mooedit/mooeditfileops.h"
-#include "mooedit/mooeditsavemultiple-glade.h"
+#include "mooedit/mooeditsavemult-glade.h"
 #include "mooutils/moodialogs.h"
+#include "mooutils/moocompat.h"
 #include "mooutils/moostock.h"
 #include "mooutils/mooglade.h"
 #include "mooutils/mooi18n.h"
@@ -39,29 +39,47 @@ open_dialog_created (G_GNUC_UNUSED MooFileDialog *fd,
 
 GSList *
 _moo_edit_open_dialog (GtkWidget      *widget,
-                       MooFilterMgr   *mgr)
+                       MooFilterMgr   *mgr,
+                       MooEdit        *current_doc)
 {
     MooFileDialog *dialog;
-    const char *start, *encoding;
+    const char *start_dir = NULL;
+    const char *encoding;
     char *new_start;
-    GSList *filenames, *infos = NULL, *l;
+    char *freeme = NULL;
+    GSList *filenames = NULL;
+    GSList *infos = NULL;
+    GSList *l;
 
     moo_prefs_create_key (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR), MOO_PREFS_STATE, G_TYPE_STRING, NULL);
-    start = moo_prefs_get_filename (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR));
+
+    if (current_doc && moo_prefs_get_bool (moo_edit_setting (MOO_EDIT_PREFS_DIALOGS_OPEN_FOLLOWS_DOC)))
+    {
+        char *filename = moo_edit_get_filename (current_doc);
+
+        if (filename)
+        {
+            freeme = g_path_get_dirname (filename);
+            start_dir = freeme;
+            g_free (filename);
+        }
+    }
+
+    if (!start_dir)
+        start_dir = moo_prefs_get_filename (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR));
 
     dialog = moo_file_dialog_new (MOO_FILE_DIALOG_OPEN, widget,
-                                  TRUE, "Open", start, NULL);
+                                  TRUE, GTK_STOCK_OPEN, start_dir,
+                                  NULL);
     g_object_set (dialog, "enable-encodings", TRUE, NULL);
     g_signal_connect (dialog, "dialog-created", G_CALLBACK (open_dialog_created), NULL);
+    moo_file_dialog_set_help_id (dialog, "dialog-open");
 
     if (mgr)
         moo_file_dialog_set_filter_mgr (dialog, mgr, "MooEdit");
 
     if (!moo_file_dialog_run (dialog))
-    {
-        g_object_unref (dialog);
-        return NULL;
-    }
+        goto out;
 
     encoding = moo_file_dialog_get_encoding (dialog);
 
@@ -79,8 +97,11 @@ _moo_edit_open_dialog (GtkWidget      *widget,
     moo_prefs_set_filename (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR), new_start);
     g_free (new_start);
 
+out:
+    g_free (freeme);
     g_object_unref (dialog);
     g_slist_foreach (filenames, (GFunc) g_free, NULL);
+    g_slist_free (filenames);
     return infos;
 }
 
@@ -90,8 +111,6 @@ _moo_edit_save_as_dialog (MooEdit        *edit,
                           MooFilterMgr   *mgr,
                           const char     *display_basename)
 {
-    /* Save dialog title */
-    const char *title = _("Save As");
     const char *start = NULL;
     const char *filename = NULL;
     const char *encoding;
@@ -104,9 +123,10 @@ _moo_edit_save_as_dialog (MooEdit        *edit,
     start = moo_prefs_get_filename (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR));
 
     dialog = moo_file_dialog_new (MOO_FILE_DIALOG_SAVE, GTK_WIDGET (edit),
-                                  FALSE, title, start, display_basename);
+                                  FALSE, GTK_STOCK_SAVE_AS, start, display_basename);
     g_object_set (dialog, "enable-encodings", TRUE, NULL);
     moo_file_dialog_set_encoding (dialog, moo_edit_get_encoding (edit));
+    moo_file_dialog_set_help_id (dialog, "dialog-save");
 
     if (mgr)
         moo_file_dialog_set_filter_mgr (dialog, mgr, "MooEdit");
@@ -131,102 +151,12 @@ _moo_edit_save_as_dialog (MooEdit        *edit,
 }
 
 
-MooEditDialogResponse
+MooSaveChangesDialogResponse
 _moo_edit_save_changes_dialog (MooEdit *edit)
 {
-    GtkDialog *dialog = NULL;
-    int response;
-    const char *name = moo_edit_get_display_basename (edit);
-
-    g_return_val_if_fail (MOO_IS_EDIT (edit), MOO_EDIT_RESPONSE_CANCEL);
-    name = moo_edit_get_display_basename (edit);
-
-#if GTK_CHECK_VERSION(2,6,0)
-    dialog = GTK_DIALOG (gtk_message_dialog_new (
-        GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (edit))),
-        GTK_DIALOG_MODAL,
-        GTK_MESSAGE_WARNING,
-        GTK_BUTTONS_NONE,
-        name ?
-            _("Save changes to document \"%s\" before closing?") :
-            _("Save changes to the document before closing?"),
-        name));
-
-    gtk_message_dialog_format_secondary_text (
-        GTK_MESSAGE_DIALOG (dialog),
-        _("If you don't save, changes will be discarded"));
-#elif GTK_CHECK_VERSION(2,4,0)
-    {
-        char *question, *markup;
-
-        question = name ?
-            g_strdup_printf (_("Save changes to document \"%s\" before closing?"), name) :
-            g_strdup (_("Save changes to the document before closing?"));
-        markup = g_markup_printf_escaped ("<span weight=\"bold\" size=\"larger\">%s</span>\n%s",
-                                          question,
-                                          _("If you don't save, changes will be discarded"));
-
-        dialog = GTK_DIALOG (gtk_message_dialog_new_with_markup (
-            GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (edit))),
-            GTK_DIALOG_MODAL,
-            GTK_MESSAGE_WARNING,
-            GTK_BUTTONS_NONE,
-            "%s", markup);
-
-        g_free (markup);
-        g_free (question);
-    }
-#else /* !GTK_CHECK_VERSION(2,4,0) */
-    {
-        char *markup;
-
-        question = name ?
-            g_strdup_printf (_("Save changes to document \"%s\" before closing?"), name) :
-            g_strdup (_("Save changes to the document before closing?"));
-
-        dialog = GTK_DIALOG (gtk_message_dialog_new (
-            GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (edit))),
-            GTK_DIALOG_MODAL,
-            GTK_MESSAGE_WARNING,
-            GTK_BUTTONS_NONE,
-            "%s\n%s", question,
-            _("If you don't save, changes will be discarded"));
-
-        g_free (question);
-    }
-#endif /* !GTK_CHECK_VERSION(2,4,0) */
-
-    gtk_dialog_add_buttons (dialog,
-        /* Translators: this is a button label (it's "Do Not Save" in some toolkits), try to keep it short */
-        _("Close _without Saving"), GTK_RESPONSE_NO,
-        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-        GTK_STOCK_SAVE, GTK_RESPONSE_YES,
-        NULL);
-
-#if GTK_CHECK_VERSION(2,6,0)
-    gtk_dialog_set_alternative_button_order (dialog,
-        GTK_RESPONSE_YES, GTK_RESPONSE_NO, GTK_RESPONSE_CANCEL, -1);
-#endif /* GTK_CHECK_VERSION(2,6,0) */
-
-    gtk_dialog_set_default_response (dialog, GTK_RESPONSE_YES);
-
-    moo_window_set_parent (GTK_WIDGET (dialog), GTK_WIDGET (edit));
-    response = gtk_dialog_run (dialog);
-    if (response == GTK_RESPONSE_DELETE_EVENT)
-        response = GTK_RESPONSE_CANCEL;
-    gtk_widget_destroy (GTK_WIDGET (dialog));
-
-    switch (response)
-    {
-        case GTK_RESPONSE_NO:
-            return MOO_EDIT_RESPONSE_DONT_SAVE;
-        case GTK_RESPONSE_CANCEL:
-            return MOO_EDIT_RESPONSE_CANCEL;
-        case GTK_RESPONSE_YES:
-            return MOO_EDIT_RESPONSE_SAVE;
-    }
-
-    g_return_val_if_reached (MOO_EDIT_RESPONSE_CANCEL);
+    g_return_val_if_fail (MOO_IS_EDIT (edit), MOO_SAVE_CHANGES_RESPONSE_CANCEL);
+    return moo_save_changes_dialog (moo_edit_get_display_basename (edit),
+                                    GTK_WIDGET (edit));
 }
 
 
@@ -388,7 +318,7 @@ files_treeview_get_to_save (GtkWidget *treeview)
 }
 
 
-MooEditDialogResponse
+MooSaveChangesDialogResponse
 _moo_edit_save_multiple_changes_dialog (GSList  *docs,
                                         GSList **to_save)
 {
@@ -396,17 +326,17 @@ _moo_edit_save_multiple_changes_dialog (GSList  *docs,
     GtkWidget *dialog, *label, *treeview;
     char *msg, *question;
     int response;
-    MooEditDialogResponse retval;
+    MooSaveChangesDialogResponse retval;
     MooGladeXML *xml;
 
-    g_return_val_if_fail (docs != NULL, MOO_EDIT_RESPONSE_CANCEL);
-    g_return_val_if_fail (docs->next != NULL, MOO_EDIT_RESPONSE_CANCEL);
-    g_return_val_if_fail (to_save != NULL, MOO_EDIT_RESPONSE_CANCEL);
+    g_return_val_if_fail (docs != NULL, MOO_SAVE_CHANGES_RESPONSE_CANCEL);
+    g_return_val_if_fail (docs->next != NULL, MOO_SAVE_CHANGES_RESPONSE_CANCEL);
+    g_return_val_if_fail (to_save != NULL, MOO_SAVE_CHANGES_RESPONSE_CANCEL);
 
     for (l = docs; l != NULL; l = l->next)
-        g_return_val_if_fail (MOO_IS_EDIT (l->data), MOO_EDIT_RESPONSE_CANCEL);
+        g_return_val_if_fail (MOO_IS_EDIT (l->data), MOO_SAVE_CHANGES_RESPONSE_CANCEL);
 
-    xml = moo_glade_xml_new_from_buf (MOO_EDIT_SAVE_MULTIPLE_GLADE_UI, -1,
+    xml = moo_glade_xml_new_from_buf (mooeditsavemult_glade_xml, -1,
                                       "dialog", GETTEXT_PACKAGE, NULL);
     dialog = moo_glade_xml_get_widget (xml, "dialog");
 
@@ -420,12 +350,10 @@ _moo_edit_save_multiple_changes_dialog (GSList  *docs,
 
     gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
 
-#if GTK_CHECK_VERSION(2,6,0)
     gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
                                              GTK_RESPONSE_YES,
                                              GTK_RESPONSE_NO,
                                              GTK_RESPONSE_CANCEL, -1);
-#endif /* GTK_CHECK_VERSION(2,6,0) */
 
     label = moo_glade_xml_get_widget (xml, "label");
     question = g_strdup_printf (dngettext (GETTEXT_PACKAGE,
@@ -449,14 +377,14 @@ _moo_edit_save_multiple_changes_dialog (GSList  *docs,
     switch (response)
     {
         case GTK_RESPONSE_NO:
-            retval = MOO_EDIT_RESPONSE_DONT_SAVE;
+            retval = MOO_SAVE_CHANGES_RESPONSE_DONT_SAVE;
             break;
         case GTK_RESPONSE_YES:
             *to_save = files_treeview_get_to_save (treeview);
-            retval = MOO_EDIT_RESPONSE_SAVE;
+            retval = MOO_SAVE_CHANGES_RESPONSE_SAVE;
             break;
         default:
-            retval = MOO_EDIT_RESPONSE_CANCEL;
+            retval = MOO_SAVE_CHANGES_RESPONSE_CANCEL;
     }
 
     g_free (question);
@@ -618,32 +546,22 @@ moo_edit_question_dialog (MooEdit    *edit,
     GtkWindow *parent = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (edit)));
     GtkWidget *dialog;
 
-#if !GTK_CHECK_VERSION(2,6,0)
-#warning "Implement me: moo_edit_question_dialog"
-    dialog = gtk_message_dialog_new (parent, GTK_DIALOG_MODAL,
-                                     GTK_MESSAGE_WARNING,
-                                     GTK_BUTTONS_NONE,
-                                     text);
-#else
     dialog = gtk_message_dialog_new (parent, GTK_DIALOG_MODAL,
                                      GTK_MESSAGE_WARNING,
                                      GTK_BUTTONS_NONE,
                                      text);
     gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
                                               "%s", secondary);
-#endif
 
     gtk_dialog_add_buttons (GTK_DIALOG (dialog),
                             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                             button, GTK_RESPONSE_YES,
                             NULL);
 
-#if GTK_CHECK_VERSION(2,6,0)
     gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
                                              GTK_RESPONSE_YES,
                                              GTK_RESPONSE_CANCEL,
                                              -1);
-#endif /* GTK_CHECK_VERSION(2,6,0) */
 
     res = gtk_dialog_run (GTK_DIALOG (dialog));
     gtk_widget_destroy (dialog);
@@ -730,12 +648,10 @@ _moo_text_search_from_start_dialog (GtkWidget *widget,
                             GTK_STOCK_YES, GTK_RESPONSE_YES,
                             NULL);
     gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
-#if GTK_CHECK_VERSION(2,6,0)
     gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
                                              GTK_RESPONSE_YES,
                                              GTK_RESPONSE_CANCEL,
                                              -1);
-#endif /* GTK_CHECK_VERSION(2,6,0) */
 
     response = gtk_dialog_run (GTK_DIALOG (dialog));
     gtk_widget_destroy (dialog);
@@ -794,7 +710,7 @@ _moo_text_prompt_on_replace_dialog (GtkWidget *parent)
     GtkWidget *dialog;
     MooGladeXML *xml;
 
-    xml = moo_glade_xml_new_from_buf (MOO_TEXT_FIND_GLADE_UI, -1,
+    xml = moo_glade_xml_new_from_buf (mootextfind_glade_xml, -1,
                                       "prompt_on_replace_dialog",
                                       GETTEXT_PACKAGE, NULL);
     dialog = moo_glade_xml_get_widget (xml, "prompt_on_replace_dialog");
