@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -22,7 +21,7 @@
 #include "mooedit/mooeditdialogs.h"
 #include "mooedit/mootextbuffer.h"
 #include "mooedit/mooeditprefs.h"
-#include "mooutils/moocompat.h"
+#include "moofileview/moofile.h"
 #include "mooutils/moofilewatch.h"
 #include "mooutils/mooencodings.h"
 #include "mooutils/mooi18n.h"
@@ -65,6 +64,7 @@ static gboolean moo_edit_load_local         (MooEdit        *edit,
                                              const char     *encoding,
                                              GError        **error);
 static gboolean moo_edit_reload_local       (MooEdit        *edit,
+                                             const char     *encoding,
                                              GError        **error);
 static gboolean moo_edit_save_local         (MooEdit        *edit,
                                              const char     *filename,
@@ -95,18 +95,8 @@ static const char *
 normalize_encoding (const char *encoding,
                     gboolean    for_save)
 {
-    if (encoding)
-    {
-        if (!encoding[0] || !strcmp (encoding, MOO_ENCODING_AUTO))
-            encoding = for_save ? MOO_ENCODING_UTF8 : NULL;
-        else if (!strcmp (encoding, MOO_ENCODING_LOCALE))
-            encoding = _moo_encoding_locale ();
-    }
-    else
-    {
+    if (!encoding || !encoding[0] || !strcmp (encoding, MOO_ENCODING_AUTO))
         encoding = for_save ? MOO_ENCODING_UTF8 : NULL;
-    }
-
     return encoding;
 }
 
@@ -139,15 +129,16 @@ _moo_edit_load_file (MooEdit        *edit,
 
 
 gboolean
-_moo_edit_reload_file (MooEdit  *edit,
-                       GError  **error)
+_moo_edit_reload_file (MooEdit    *edit,
+                       const char *encoding,
+                       GError    **error)
 {
     GError *error_here = NULL;
     gboolean result;
 
     g_return_val_if_fail (MOO_IS_EDIT (edit), FALSE);
 
-    result = moo_edit_reload_local (edit, &error_here);
+    result = moo_edit_reload_local (edit, encoding, &error_here);
 
     if (error_here)
         g_propagate_error (error, error_here);
@@ -696,8 +687,9 @@ error_out:
 
 /* XXX */
 static gboolean
-moo_edit_reload_local (MooEdit  *edit,
-                       GError  **error)
+moo_edit_reload_local (MooEdit    *edit,
+                       const char *encoding,
+                       GError    **error)
 {
     GtkTextIter start, end;
     GtkTextBuffer *buffer;
@@ -716,7 +708,8 @@ moo_edit_reload_local (MooEdit  *edit,
     g_object_set (edit, "enable-highlight", FALSE, NULL);
 
     result = _moo_edit_load_file (edit, edit->priv->filename,
-                                  edit->priv->encoding, error);
+                                  encoding ? encoding : edit->priv->encoding,
+                                  error);
 
     g_object_set (edit, "enable-highlight", enable_highlight, NULL);
     gtk_text_buffer_end_user_action (buffer);
@@ -1199,18 +1192,23 @@ add_untitled (MooEdit *edit)
 }
 
 
+const char *
+_moo_edit_get_default_encoding (void)
+{
+    return moo_prefs_get_string (moo_edit_setting (MOO_EDIT_PREFS_ENCODING_SAVE));
+}
+
+
 void
 _moo_edit_set_filename (MooEdit    *edit,
                         const char *file,
                         const char *encoding)
 {
-    char *tmp1, *tmp2, *tmp3, *tmp4, *tmp5;
+    char *tmp1, *tmp3, *tmp4;
 
     tmp1 = edit->priv->filename;
-    tmp2 = edit->priv->basename;
     tmp3 = edit->priv->display_filename;
     tmp4 = edit->priv->display_basename;
-    tmp5 = edit->priv->encoding;
 
     if (!UNTITLED_NO)
         UNTITLED_NO = g_hash_table_new (g_direct_hash, g_direct_equal);
@@ -1220,7 +1218,6 @@ _moo_edit_set_filename (MooEdit    *edit,
         int n = add_untitled (edit);
 
         edit->priv->filename = NULL;
-        edit->priv->basename = NULL;
 
         if (n == 1)
             edit->priv->display_filename = g_strdup (_("Untitled"));
@@ -1231,29 +1228,30 @@ _moo_edit_set_filename (MooEdit    *edit,
     }
     else
     {
+        char *basename;
+
         remove_untitled (NULL, edit);
 
         edit->priv->filename = g_strdup (file);
-        edit->priv->basename = g_path_get_basename (file);
+        edit->priv->display_filename = _moo_edit_filename_to_utf8 (file);
 
-        edit->priv->display_filename =
-                _moo_edit_filename_to_utf8 (file);
-        edit->priv->display_basename =
-                _moo_edit_filename_to_utf8 (edit->priv->basename);
+        basename = g_path_get_basename (file);
+        edit->priv->display_basename = _moo_edit_filename_to_utf8 (basename);
+
+        g_free (basename);
     }
 
     if (!encoding)
-        encoding = moo_prefs_get_string (moo_edit_setting (MOO_EDIT_PREFS_ENCODING_SAVE));
-    edit->priv->encoding = g_strdup (encoding);
+        _moo_edit_set_encoding (edit, _moo_edit_get_default_encoding ());
+    else
+        _moo_edit_set_encoding (edit, encoding);
 
     g_signal_emit_by_name (edit, "filename-changed", edit->priv->filename, NULL);
     moo_edit_status_changed (edit);
 
     g_free (tmp1);
-    g_free (tmp2);
     g_free (tmp3);
     g_free (tmp4);
-    g_free (tmp5);
 }
 
 
@@ -1283,3 +1281,12 @@ _moo_edit_filename_to_utf8 (const char *filename)
 
     return utf_filename;
 }
+
+
+GdkPixbuf *
+_moo_edit_get_icon (MooEdit     *doc,
+                    GtkWidget   *widget,
+                    GtkIconSize  size)
+{
+    return _moo_get_icon_for_path (doc->priv->filename, widget, size);
+}
