@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -16,12 +15,11 @@
 #include "mooedit/mooedit-private.h"
 #include "mooedit/mootextiter.h"
 #include "mooedit/mootextbuffer.h"
-#include "mooutils/moocompat.h"
 #include "mooutils/mooutils-misc.h"
 #include <gdk/gdkkeysyms.h>
 
 
-inline static GtkWidgetClass*
+static GtkWidgetClass *
 parent_class (void)
 {
     static gpointer textview_class = NULL;
@@ -33,71 +31,91 @@ parent_class (void)
 }
 
 
-inline static gboolean
-is_word_char (const GtkTextIter *iter)
+static gboolean
+iter_is_word_char (const GtkTextIter *iter,
+                   MooTextView       *view)
 {
+    guint i;
     gunichar c = gtk_text_iter_get_char (iter);
-    g_return_val_if_fail (c != 0, FALSE);
-    return g_unichar_isalnum (c) || c == '_';
+
+    for (i = 0; i < view->priv->n_word_chars; ++i)
+        if (c == view->priv->word_chars[i])
+            return TRUE;
+
+    return c == '_' || g_unichar_isalnum (c);
 }
 
 /* Glib docs say: "(Note: don't use this to do word breaking; you have
  * to use Pango or equivalent to get word breaking right, the algorithm
  * is fairly complex.)"
  */
-inline static gboolean
-is_space (const GtkTextIter *iter)
+static gboolean
+iter_is_space (const GtkTextIter *iter)
 {
     gunichar c = gtk_text_iter_get_char (iter);
-    g_return_val_if_fail (c != 0, FALSE);
     return g_unichar_isspace (c);
 }
 
-inline static gboolean
-is_word_start (const GtkTextIter *iter)
+static gboolean
+iter_is_word_start (const GtkTextIter *iter,
+                    MooTextView       *view)
 {
     GtkTextIter i;
-    if (!is_word_char (iter)) return FALSE;
+
+    if (!iter_is_word_char (iter, view))
+        return FALSE;
+
     i = *iter;
-    if (!gtk_text_iter_backward_char (&i)) return TRUE;
-    return !is_word_char (&i);
+
+    return !gtk_text_iter_backward_char (&i) ||
+           !iter_is_word_char (&i, view);
 }
 
 
 static gboolean
-text_iter_forward_word_start (GtkTextIter *iter)
+text_iter_forward_word_start (GtkTextIter *iter,
+                              MooTextView *view)
 {
     gboolean moved = FALSE;
-    if (gtk_text_iter_is_end (iter)) return FALSE;
 
-    /* if iter points to word char, then go to the first non-space char after the word
-     * otherwise, go to the next word char
+    if (gtk_text_iter_is_end (iter))
+        return FALSE;
+
+    /* if iter points to a word char then go to the first non-space char after the word;
+     * otherwise, go to the next word char;
      * stop at end of line
      */
 
-    if (is_word_char (iter)) {
-        while (!gtk_text_iter_is_end (iter) && is_word_char (iter))
+    if (iter_is_word_char (iter, view))
+    {
+        while (!gtk_text_iter_is_end (iter) && iter_is_word_char (iter, view))
         {
             gtk_text_iter_forward_cursor_position (iter);
             moved = TRUE;
         }
-        if (gtk_text_iter_is_end (iter)) return FALSE;
+
+        if (gtk_text_iter_is_end (iter))
+            return FALSE;
+
         while (!gtk_text_iter_is_end (iter) &&
-               is_space (iter) &&
+               iter_is_space (iter) &&
                !gtk_text_iter_ends_line (iter))
         {
             gtk_text_iter_forward_cursor_position (iter);
             moved = TRUE;
         }
     }
-    else {
-        if (gtk_text_iter_ends_line (iter)) {
+    else
+    {
+        if (gtk_text_iter_ends_line (iter))
+        {
             gtk_text_iter_forward_cursor_position (iter);
             moved = TRUE;
         }
-        else {
+        else
+        {
             while (!gtk_text_iter_is_end (iter) &&
-                   !is_word_char (iter) &&
+                   !iter_is_word_char (iter, view) &&
                    !gtk_text_iter_ends_line (iter))
             {
                 gtk_text_iter_forward_cursor_position (iter);
@@ -105,58 +123,76 @@ text_iter_forward_word_start (GtkTextIter *iter)
             }
         }
     }
+
     return moved && !gtk_text_iter_is_end (iter);
 }
 
-inline static gboolean
-text_iter_forward_word_start_n (GtkTextIter *iter, guint count)
+static gboolean
+text_iter_forward_word_start_n (GtkTextIter *iter,
+                                guint        count,
+                                MooTextView *view)
 {
-    if (!count) return FALSE;
-    while (count) {
-        if (!text_iter_forward_word_start (iter)) {
+    if (!count)
+        return FALSE;
+
+    while (count)
+    {
+        if (!text_iter_forward_word_start (iter, view))
+        {
             gtk_text_iter_forward_to_end (iter);
             return FALSE;
         }
-        else
-            --count;
+
+        --count;
     }
+
     return TRUE;
 }
 
 
-inline static gboolean
-text_iter_backward_word_start (GtkTextIter *iter)
+static gboolean
+text_iter_backward_word_start (GtkTextIter *iter,
+                               MooTextView *view)
 {
     gboolean moved = FALSE;
-    if (gtk_text_iter_starts_line (iter)) {
+
+    if (gtk_text_iter_starts_line (iter))
+    {
         moved = gtk_text_iter_backward_cursor_position (iter);
         /* it may point now to \n in \r\n combination, fixed in gtk-2.10 */
         if (moved && !gtk_text_iter_ends_line (iter))
             gtk_text_iter_backward_char (iter);
     }
-    else {
+    else
+    {
         while (gtk_text_iter_backward_cursor_position (iter) &&
-               !is_word_start (iter) &&
+               !iter_is_word_start (iter, view) &&
                !gtk_text_iter_starts_line (iter))
             moved = TRUE;
     }
+
     return moved;
 }
 
-inline static gboolean
-text_iter_backward_word_start_n (GtkTextIter *iter, guint count)
+static gboolean
+text_iter_backward_word_start_n (GtkTextIter *iter,
+                                 guint        count,
+                                 MooTextView *view)
 {
     gboolean moved = FALSE;
-    while (count && text_iter_backward_word_start (iter)) {
+
+    while (count && text_iter_backward_word_start (iter, view))
+    {
         moved = TRUE;
         --count;
     }
+
     return moved;
 }
 
 
 /* TODO: may I do this? */
-inline static void
+static void
 text_view_reset_im_context (GtkTextView *text_view)
 {
   if (text_view->need_im_reset)
@@ -189,14 +225,14 @@ move_cursor_to (GtkTextView *text_view,
 
 
 static void
-moo_text_view_move_cursor_words (G_GNUC_UNUSED MooTextView *view,
+moo_text_view_move_cursor_words (MooTextView *view,
                                  GtkTextIter *iter,
                                  gint         count)
 {
     if (count < 0)
-        text_iter_backward_word_start_n (iter, -count);
+        text_iter_backward_word_start_n (iter, -count, view);
     else if (count > 0)
-        text_iter_forward_word_start_n (iter, count);
+        text_iter_forward_word_start_n (iter, count, view);
 }
 
 
@@ -216,7 +252,7 @@ moo_text_view_home_end (MooTextView *view,
 
         while (!gtk_text_iter_ends_line (&first))
         {
-            if (is_space (&first))
+            if (iter_is_space (&first))
                 gtk_text_iter_forward_cursor_position (&first);
             else
                 break;
@@ -237,7 +273,7 @@ moo_text_view_home_end (MooTextView *view,
             {
                 gtk_text_iter_backward_cursor_position (&last);
 
-                if (!is_space (&last))
+                if (!iter_is_space (&last))
                 {
                     gtk_text_iter_forward_cursor_position (&last);
                     break;
@@ -266,6 +302,24 @@ moo_text_view_home_end (MooTextView *view,
 }
 
 
+static void
+_moo_text_view_move_cursor_call_parent (GtkTextView        *text_view,
+                                        GtkMovementStep     step,
+                                        gint                count,
+                                        gboolean            extend_selection)
+{
+#if !GTK_CHECK_VERSION(2,12,0)
+    MooTextView *view = MOO_TEXT_VIEW (text_view);
+    if (view->priv->overwrite_mode)
+        gtk_text_view_set_cursor_visible (text_view, TRUE);
+#endif
+    GTK_TEXT_VIEW_CLASS (parent_class())->move_cursor (text_view, step, count, extend_selection);
+#if !GTK_CHECK_VERSION(2,12,0)
+    if (view->priv->overwrite_mode)
+        gtk_text_view_set_cursor_visible (text_view, FALSE);
+#endif
+}
+
 void
 _moo_text_view_move_cursor (GtkTextView        *text_view,
                             GtkMovementStep     step,
@@ -275,11 +329,10 @@ _moo_text_view_move_cursor (GtkTextView        *text_view,
     GtkTextBuffer *buffer;
     GtkTextMark *insert;
     GtkTextIter iter;
-    MooTextView *view = MOO_TEXT_VIEW (text_view);
 
-    if (!text_view->cursor_visible && !view->priv->overwrite_mode)
+    if (!text_view->cursor_visible)
     {
-        GTK_TEXT_VIEW_CLASS (parent_class())->move_cursor (text_view, step, count, extend_selection);
+        _moo_text_view_move_cursor_call_parent (text_view, step, count, extend_selection);
         return;
     }
 
@@ -287,30 +340,27 @@ _moo_text_view_move_cursor (GtkTextView        *text_view,
     insert = gtk_text_buffer_get_insert (buffer);
     gtk_text_buffer_get_iter_at_mark (buffer, &iter, insert);
 
-    switch (step)
+    if (step == GTK_MOVEMENT_WORDS)
     {
-        case GTK_MOVEMENT_WORDS:
-            moo_text_view_move_cursor_words (MOO_TEXT_VIEW (text_view), &iter, count);
-            break;
-
-        case GTK_MOVEMENT_DISPLAY_LINE_ENDS:
-            moo_text_view_home_end (MOO_TEXT_VIEW (text_view), &iter, count);
-            break;
-
-        default:
-            if (view->priv->overwrite_mode)
-                gtk_text_view_set_cursor_visible (text_view, TRUE);
-            GTK_TEXT_VIEW_CLASS (parent_class())->move_cursor (text_view, step, count, extend_selection);
-            if (view->priv->overwrite_mode)
-                gtk_text_view_set_cursor_visible (text_view, FALSE);
-            return;
+        moo_text_view_move_cursor_words (MOO_TEXT_VIEW (text_view), &iter, count);
+    }
+    else if (step == GTK_MOVEMENT_DISPLAY_LINE_ENDS &&
+             gtk_text_view_get_wrap_mode (text_view) == GTK_WRAP_NONE)
+    {
+        moo_text_view_home_end (MOO_TEXT_VIEW (text_view), &iter, count);
+    }
+    else
+    {
+        _moo_text_view_move_cursor_call_parent (text_view, step, count, extend_selection);
+        return;
     }
 
     move_cursor_to (text_view, &iter, extend_selection);
-    _moo_text_view_pend_cursor_blink (view);
+    _moo_text_view_pend_cursor_blink (MOO_TEXT_VIEW (text_view));
 }
 
 
+#if !GTK_CHECK_VERSION(2,12,0)
 void
 _moo_text_view_page_horizontally (GtkTextView *text_view,
                                   int          count,
@@ -324,6 +374,7 @@ _moo_text_view_page_horizontally (GtkTextView *text_view,
         gtk_text_view_set_cursor_visible (text_view, FALSE);
     _moo_text_view_pend_cursor_blink (view);
 }
+#endif
 
 
 void
@@ -334,6 +385,7 @@ _moo_text_view_delete_from_cursor (GtkTextView        *text_view,
     GtkTextBuffer *buf;
     GtkTextMark *insert_mark;
     GtkTextIter insert, start, end;
+    MooTextView *view = MOO_TEXT_VIEW (text_view);
 
     if (type != GTK_DELETE_WORD_ENDS)
     {
@@ -359,9 +411,9 @@ _moo_text_view_delete_from_cursor (GtkTextView        *text_view,
     end = insert;
 
     if (count > 0)
-        text_iter_forward_word_start_n (&end, count);
+        text_iter_forward_word_start_n (&end, count, view);
     else if (count < 0)
-        text_iter_backward_word_start_n (&start, -count);
+        text_iter_backward_word_start_n (&start, -count, view);
 
     if (!gtk_text_iter_equal (&start, &end))
     {
@@ -375,7 +427,7 @@ _moo_text_view_delete_from_cursor (GtkTextView        *text_view,
 /* Mouse
  */
 
-inline static gboolean
+static gboolean
 extend_selection (MooTextView          *view,
                   MooTextSelectionType  type,
                   GtkTextIter          *insert,
@@ -384,7 +436,7 @@ extend_selection (MooTextView          *view,
     return MOO_TEXT_VIEW_GET_CLASS(view)->extend_selection (view, type, insert, selection_bound);
 }
 
-inline static void
+static void
 clear_drag_stuff (MooTextView *view)
 {
     view->priv->dnd.moved = FALSE;
@@ -394,11 +446,8 @@ clear_drag_stuff (MooTextView *view)
     view->priv->dnd.button = GDK_BUTTON_RELEASE;
 }
 
-#define get_buf(view) \
-    GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)))
-
 /* from gtktextview.c */
-inline static void
+static void
 text_view_unobscure_mouse_cursor (GtkTextView *text_view)
 {
     if (text_view->mouse_cursor_obscured)
@@ -809,9 +858,7 @@ _moo_text_view_motion_event (GtkWidget          *widget,
     if (view->priv->dnd.type == MOO_TEXT_VIEW_DRAG_SELECT_LINES)
     {
         GdkRectangle rect;
-        GdkWindow *window;
 
-        window = gtk_text_view_get_window (text_view, GTK_TEXT_WINDOW_LEFT);
         gtk_text_view_get_visible_rect (text_view, &rect);
 
         select_lines (view, y);
@@ -862,7 +909,6 @@ _moo_text_view_motion_event (GtkWidget          *widget,
     else
     {
         /* this piece is from gtktextview.c */
-        int x, y;
         GtkTextIter start;
         gdk_window_get_pointer (gtk_text_view_get_window (text_view, GTK_TEXT_WINDOW_TEXT),
                                 &x, &y, NULL);
@@ -887,7 +933,6 @@ text_view_start_selection_dnd (GtkTextView       *text_view,
                                G_GNUC_UNUSED const GtkTextIter *iter,
                                GdkEventMotion    *event)
 {
-    GdkDragContext *context;
     GtkTargetList *target_list;
 
 #if !GTK_CHECK_VERSION(2,10,0)
@@ -906,21 +951,28 @@ text_view_start_selection_dnd (GtkTextView       *text_view,
     text_view->drag_start_y = -1;
     text_view->pending_place_cursor_button = 0;
 
-    context = gtk_drag_begin (GTK_WIDGET (text_view), target_list,
-                              (GdkDragAction) (GDK_ACTION_COPY | GDK_ACTION_MOVE),
-                              1, (GdkEvent*)event);
+    gtk_drag_begin (GTK_WIDGET (text_view), target_list,
+                    GDK_ACTION_COPY | GDK_ACTION_MOVE,
+                    1, (GdkEvent*) event);
 
     gtk_target_list_unref (target_list);
 }
 
 
-inline static int
-char_class (const GtkTextIter *iter)
+static int
+iter_get_char_class (const GtkTextIter *iter,
+                     MooTextView       *view)
 {
-    if (gtk_text_iter_ends_line (iter)) return -1;
-    else if (is_space (iter)) return 0;
-    else if (is_word_char (iter)) return 1;
-    else return 2;
+    if (gtk_text_iter_ends_line (iter))
+        return -1;
+
+    if (iter_is_space (iter))
+        return 0;
+
+    if (iter_is_word_char (iter, view))
+        return 1;
+
+    return 2;
 }
 
 #define FIND_BRACKET_LIMIT 2000
@@ -978,23 +1030,23 @@ _moo_text_view_extend_selection (MooTextView        *view,
             start = end; end = tmp;
         }
 
-        ch_class = char_class (end);
+        ch_class = iter_get_char_class (end, view);
 
         while (!gtk_text_iter_ends_line (end) &&
-                char_class (end) == ch_class)
+               iter_get_char_class (end, view) == ch_class)
         {
             gtk_text_iter_forward_cursor_position (end);
         }
 
-        ch_class = char_class (start);
+        ch_class = iter_get_char_class (start, view);
 
         while (!gtk_text_iter_starts_line (start) &&
-                char_class (start) == ch_class)
+               iter_get_char_class (start, view) == ch_class)
         {
             gtk_text_iter_backward_cursor_position (start);
         }
 
-        if (char_class (start) != ch_class)
+        if (iter_get_char_class (start, view) != ch_class)
             gtk_text_iter_forward_cursor_position (start);
 
         return gtk_text_iter_compare (start, end);
@@ -1129,7 +1181,13 @@ _moo_text_view_key_press_event (GtkWidget          *widget,
     if (view->priv->qs.in_search)
         return FALSE;
 
-    if (!mods)
+    if (keyval == GDK_KP_Enter || keyval == GDK_Return)
+    {
+        gtk_text_buffer_begin_user_action (buffer);
+        handled = handle_enter (view, event);
+        gtk_text_buffer_end_user_action (buffer);
+    }
+    else if (!mods)
     {
         switch (keyval)
         {
@@ -1142,19 +1200,12 @@ _moo_text_view_key_press_event (GtkWidget          *widget,
                 handled = handle_backspace (view, event);
                 gtk_text_buffer_end_user_action (buffer);
                 break;
-            case GDK_KP_Enter:
-            case GDK_Return:
-                gtk_text_buffer_begin_user_action (buffer);
-                handled = handle_enter (view, event);
-                gtk_text_buffer_end_user_action (buffer);
-                break;
         }
     }
     else if (mods == GDK_SHIFT_MASK)
     {
         switch (keyval)
         {
-            /* TODO TODO stupid X and gtk !!! */
             case GDK_ISO_Left_Tab:
             case GDK_KP_Tab:
                 handled = handle_tab (view, event);
@@ -1359,41 +1410,21 @@ moo_text_view_unindent (MooTextView *view)
 }
 
 
-static void
-move_focus (GtkWidget       *widget,
-            GtkDirectionType direction)
-{
-    GtkWidget *toplevel = gtk_widget_get_toplevel (widget);
-    g_signal_emit_by_name (toplevel, "move-focus", direction);
-}
-
 static gboolean
 handle_tab (MooTextView *view,
             GdkEventKey *event)
 {
-    switch (view->priv->tab_key_action)
+    if (view->priv->tab_indents)
     {
-        case MOO_TEXT_TAB_KEY_DO_NOTHING:
-            move_focus (GTK_WIDGET (view),
-                        event->state & GDK_SHIFT_MASK ?
-                                GTK_DIR_TAB_BACKWARD : GTK_DIR_TAB_FORWARD);
-            return TRUE;
-
-        case MOO_TEXT_TAB_KEY_FIND_PLACEHOLDER:
-            if (event->state & GDK_SHIFT_MASK)
-                moo_text_view_prev_placeholder (view);
-            else
-                moo_text_view_next_placeholder (view);
-            return TRUE;
-
-        case MOO_TEXT_TAB_KEY_INDENT:
-            if (event->state & GDK_SHIFT_MASK)
-                return tab_unindent (view);
-            else
-                return tab_indent (view);
+        if (event->state & GDK_SHIFT_MASK)
+            return tab_unindent (view);
+        else
+            return tab_indent (view);
+    }
+    else
+    {
+        return FALSE;
     }
-
-    g_return_val_if_reached (FALSE);
 }
 
 
@@ -1461,10 +1492,14 @@ handle_enter (MooTextView        *view,
     GtkTextBuffer *buffer;
     GtkTextIter start, end;
     gboolean has_selection;
+    gboolean indent;
 
-    if (!view->priv->indenter || !view->priv->enter_indents)
+    if (!gtk_text_view_get_editable (GTK_TEXT_VIEW (view)))
         return FALSE;
 
+    indent = view->priv->indenter && view->priv->enter_indents &&
+             !(event->state & GDK_SHIFT_MASK);
+
     buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
     has_selection = gtk_text_buffer_get_selection_bounds (buffer, &start, &end);
 
@@ -1472,9 +1507,12 @@ handle_enter (MooTextView        *view,
 
     if (has_selection)
         gtk_text_buffer_delete (buffer, &start, &end);
+
     /* XXX insert "\r\n" on windows? */
     gtk_text_buffer_insert (buffer, &start, "\n", 1);
-    moo_indenter_character (view->priv->indenter, '\n', &start);
+
+    if (indent)
+        moo_indenter_character (view->priv->indenter, '\n', &start);
 
     gtk_text_buffer_end_user_action (buffer);
 
