@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -23,7 +22,6 @@
 #include "mooedit/mooedit-ui.h"
 #include "mooedit/medit-ui.h"
 #include "mooutils/moomenuaction.h"
-#include "mooutils/moocompat.h"
 #include "mooutils/moomarshals.h"
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooutils-gobject.h"
@@ -31,7 +29,9 @@
 #include "mooutils/mooutils-fs.h"
 #include "mooutils/moostock.h"
 #include "mooutils/mooi18n.h"
+#include <glib/gbase64.h>
 #include <string.h>
+#include <stdlib.h>
 
 
 #define RECENT_ACTION_ID "OpenRecent"
@@ -69,12 +69,20 @@ static WindowInfo   *window_list_find_file  (MooEditor      *editor,
 static void          set_single_window      (MooEditor      *editor,
                                              gboolean        single);
 
+#if !GTK_CHECK_VERSION(2,12,0)
 static GtkAction    *create_recent_action   (MooEditWindow  *window);
+static void          activate_history_item  (MooEditor      *editor,
+                                             MooHistoryItem *item,
+                                             MooEditWindow  *window);
+#endif
 
 static MooEditWindow *create_window         (MooEditor      *editor);
 static void          moo_editor_add_doc     (MooEditor      *editor,
                                              MooEditWindow  *window,
                                              MooEdit        *doc);
+static gboolean      close_window_handler   (MooEditor      *editor,
+                                             MooEditWindow  *window,
+                                             gboolean        ask_confirm);
 static void          do_close_window        (MooEditor      *editor,
                                              MooEditWindow  *window);
 static void          do_close_doc           (MooEditor      *editor,
@@ -84,10 +92,6 @@ static gboolean      close_docs_real        (MooEditor      *editor,
                                              gboolean        ask_confirm);
 static GSList       *find_modified          (GSList         *docs);
 
-static void          activate_history_item  (MooEditor      *editor,
-                                             MooHistoryItem *item,
-                                             MooEditWindow  *window);
-
 static void          add_new_window_action      (void);
 static void          remove_new_window_action   (void);
 
@@ -108,7 +112,9 @@ struct _MooEditorPrivate {
     MooUIXML        *doc_ui_xml;
     MooUIXML        *ui_xml;
     MooFilterMgr    *filter_mgr;
+#if !GTK_CHECK_VERSION(2,12,0)
     MooHistoryList  *history;
+#endif
     MooLangMgr      *lang_mgr;
     MooFileWatch    *file_watch;
     gboolean         open_single;
@@ -156,7 +162,7 @@ enum {
 };
 
 enum {
-    ALL_WINDOWS_CLOSED,
+    CLOSE_WINDOW,
     LAST_SIGNAL
 };
 
@@ -171,12 +177,14 @@ static void
 moo_editor_class_init (MooEditorClass *klass)
 {
     GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-    MooWindowClass *edit_window_class;
+    G_GNUC_UNUSED MooWindowClass *edit_window_class;
 
     gobject_class->finalize = moo_editor_finalize;
     gobject_class->set_property = moo_editor_set_property;
     gobject_class->get_property = moo_editor_get_property;
 
+    klass->close_window = close_window_handler;
+
     _moo_edit_init_config ();
     g_type_class_unref (g_type_class_ref (MOO_TYPE_EDIT));
     g_type_class_unref (g_type_class_ref (MOO_TYPE_EDIT_WINDOW));
@@ -255,19 +263,24 @@ moo_editor_class_init (MooEditorClass *klass)
                                              MOO_TYPE_EDIT,
                                              G_PARAM_READABLE));
 
-    signals[ALL_WINDOWS_CLOSED] =
-            _moo_signal_new_cb ("all-windows-closed",
-                                G_OBJECT_CLASS_TYPE (klass),
-                                G_SIGNAL_RUN_LAST,
-                                NULL, NULL, NULL,
-                                _moo_marshal_VOID__VOID,
-                                G_TYPE_NONE, 0);
+    signals[CLOSE_WINDOW] =
+            g_signal_new ("close-window",
+                          G_OBJECT_CLASS_TYPE (klass),
+                          G_SIGNAL_RUN_LAST,
+                          G_STRUCT_OFFSET (MooEditorClass, close_window),
+                          g_signal_accumulator_true_handled, NULL,
+                          _moo_marshal_BOOLEAN__OBJECT_BOOLEAN,
+                          G_TYPE_BOOLEAN, 2,
+                          MOO_TYPE_EDIT_WINDOW,
+                          G_TYPE_BOOLEAN);
 
+#if !GTK_CHECK_VERSION(2,12,0)
     edit_window_class = g_type_class_ref (MOO_TYPE_EDIT_WINDOW);
     moo_window_class_new_action_custom (edit_window_class, RECENT_ACTION_ID, NULL,
                                         (MooWindowActionFunc) create_recent_action,
                                         NULL, NULL);
     g_type_class_unref (edit_window_class);
+#endif
 
     add_new_window_action ();
 }
@@ -291,7 +304,9 @@ moo_editor_init (MooEditor *editor)
 
     editor->priv->filter_mgr = moo_filter_mgr_new ();
 
-    editor->priv->history = moo_history_list_new ("Editor");
+#if !GTK_CHECK_VERSION(2,12,0)
+    editor->priv->history = moo_history_list_get ("Editor");
+    g_object_ref (editor->priv->history);
     moo_history_list_set_display_func (editor->priv->history,
                                        moo_history_list_display_basename,
                                        NULL);
@@ -300,6 +315,7 @@ moo_editor_init (MooEditor *editor)
                                    NULL);
     g_signal_connect_swapped (editor->priv->history, "activate-item",
                               G_CALLBACK (activate_history_item), editor);
+#endif
 
     editor->priv->windows = NULL;
     editor->priv->windowless = window_info_new (NULL);
@@ -435,8 +451,10 @@ moo_editor_finalize (GObject *object)
         g_object_unref (editor->priv->ui_xml);
     if (editor->priv->filter_mgr)
         g_object_unref (editor->priv->filter_mgr);
+#if !GTK_CHECK_VERSION(2,12,0)
     if (editor->priv->history)
         g_object_unref (editor->priv->history);
+#endif
     g_object_unref (editor->priv->lang_mgr);
     g_object_unref (editor->priv->doc_ui_xml);
 
@@ -677,18 +695,23 @@ file_info_list_free (GSList *list)
 
 
 void
-moo_editor_set_app_name (MooEditor      *editor,
-                         const char     *name)
+moo_editor_set_app_name (MooEditor  *editor,
+                         const char *name)
 {
-    GSList *l;
-
     g_return_if_fail (MOO_IS_EDITOR (editor));
+    MOO_ASSIGN_STRING (editor->priv->app_name, name);
+    _moo_edit_window_update_title ();
+}
 
-    g_free (editor->priv->app_name);
-    editor->priv->app_name = g_strdup (name);
+const char *
+moo_editor_get_app_name (MooEditor *editor)
+{
+    g_return_val_if_fail (MOO_IS_EDITOR (editor), NULL);
 
-    for (l = editor->priv->windows; l != NULL; l = l->next)
-        moo_edit_window_set_title_prefix (MOO_EDIT_WINDOW (l->data), name);
+    if (!editor->priv->app_name)
+        return g_get_prgname ();
+    else
+        return editor->priv->app_name;
 }
 
 
@@ -700,14 +723,6 @@ moo_editor_get_filter_mgr (MooEditor *editor)
 }
 
 
-MooHistoryList *
-moo_editor_get_history (MooEditor *editor)
-{
-    g_return_val_if_fail (MOO_IS_EDITOR (editor), NULL);
-    return editor->priv->history;
-}
-
-
 MooUIXML *
 moo_editor_get_ui_xml (MooEditor *editor)
 {
@@ -794,11 +809,18 @@ window_info_remove (WindowInfo     *win,
 static int
 edit_and_file_cmp (MooEdit *edit, const char *filename)
 {
-    const char *edit_filename;
+    char *edit_filename;
+
     g_return_val_if_fail (MOO_IS_EDIT (edit) && filename != NULL, TRUE);
+
     edit_filename = moo_edit_get_filename (edit);
+
     if (edit_filename)
-        return strcmp (edit_filename, filename);
+    {
+        int result = strcmp (edit_filename, filename);
+        g_free (edit_filename);
+        return result;
+    }
     else
         return TRUE;
 }
@@ -935,16 +957,43 @@ window_list_find_file (MooEditor  *editor,
 }
 
 
+#if GTK_CHECK_VERSION(2,12,0)
+static void
+add_recent_file (G_GNUC_UNUSED MooEditor *editor,
+                 const char *filename)
+{
+    char *uri;
+
+    uri = g_filename_to_uri (filename, NULL, NULL);
+    g_return_if_fail (uri != NULL);
+
+    gtk_recent_manager_add_item (gtk_recent_manager_get_default (), uri);
+
+    g_free (uri);
+}
+#else
+static void
+add_recent_file (MooEditor  *editor,
+                 const char *filename)
+{
+    moo_history_list_add_filename (editor->priv->history, filename);
+}
+#endif
+
+
+#if !GTK_CHECK_VERSION(2,12,0)
+
 static GtkAction*
 create_recent_action (MooEditWindow  *window)
 {
     MooEditor *editor = moo_edit_window_get_editor (window);
     GtkAction *action;
-    MooMenuMgr *mgr;
+    G_GNUC_UNUSED MooMenuMgr *mgr;
 
     g_return_val_if_fail (editor != NULL, NULL);
 
     action = moo_menu_action_new (RECENT_ACTION_ID, _("Open Recent"));
+
     mgr = moo_history_list_get_menu_mgr (editor->priv->history);
     moo_menu_mgr_set_show_tooltips (mgr, TRUE);
     moo_menu_action_set_mgr (MOO_MENU_ACTION (action), mgr);
@@ -956,7 +1005,6 @@ create_recent_action (MooEditWindow  *window)
     return action;
 }
 
-
 static void
 activate_history_item (MooEditor      *editor,
                        MooHistoryItem *item,
@@ -975,6 +1023,8 @@ activate_history_item (MooEditor      *editor,
         moo_history_list_remove (editor->priv->history, item->data);
 }
 
+#endif /* !GTK_CHECK_VERSION(2,12,0) */
+
 
 /*****************************************************************************/
 
@@ -986,7 +1036,6 @@ create_window (MooEditor *editor)
                                           "ui-xml",
                                           moo_editor_get_ui_xml (editor),
                                           NULL);
-    moo_edit_window_set_title_prefix (window, editor->priv->app_name);
     window_list_add (editor, window);
     _moo_window_attach_plugins (window);
     gtk_widget_show (GTK_WIDGET (window));
@@ -1015,9 +1064,9 @@ moo_editor_add_doc (MooEditor      *editor,
 
     window_info_add (info, doc);
 
-    if (!moo_edit_get_filename (doc) &&
-         !moo_edit_config_get_string (doc->config, "lang") &&
-         editor->priv->default_lang)
+    if (moo_edit_is_untitled (doc) &&
+        !moo_edit_config_get_string (doc->config, "lang") &&
+        editor->priv->default_lang)
     {
         moo_edit_config_set (doc->config, MOO_EDIT_CONFIG_SOURCE_FILENAME,
                              "lang", editor->priv->default_lang, NULL);
@@ -1113,9 +1162,13 @@ _moo_editor_move_doc (MooEditor     *editor,
 
     g_return_if_fail (MOO_IS_EDITOR (editor));
     g_return_if_fail (MOO_IS_EDIT (doc) && doc->priv->editor == editor);
-    g_return_if_fail (MOO_IS_EDIT_WINDOW (dest) && moo_edit_window_get_editor (dest) == editor);
+    g_return_if_fail (!dest || (MOO_IS_EDIT_WINDOW (dest) && moo_edit_window_get_editor (dest) == editor));
 
     old = window_list_find_doc (editor, doc);
+
+    if (!dest)
+        dest = moo_editor_new_window (editor);
+
     new = window_list_find (editor, dest);
     g_return_if_fail (old != NULL && new != NULL);
 
@@ -1127,11 +1180,8 @@ _moo_editor_move_doc (MooEditor     *editor,
     {
         _moo_edit_window_remove_doc (old->window, doc, FALSE);
 
-        if (!moo_edit_window_get_active_doc (old->window) &&
-            !editor->priv->allow_empty_window)
-        {
-            moo_editor_new_doc (editor, old->window);
-        }
+        if (!moo_edit_window_get_active_doc (old->window))
+            moo_editor_close_window (editor, old->window, FALSE);
     }
 
     old_doc = moo_edit_window_get_active_doc (dest);
@@ -1154,6 +1204,88 @@ _moo_editor_move_doc (MooEditor     *editor,
 }
 
 
+static gboolean
+moo_editor_load_file (MooEditor       *editor,
+                      MooEditWindow   *window,
+                      GtkWidget       *parent,
+                      MooEditFileInfo *info,
+                      gboolean         silent,
+                      gboolean         add_history,
+                      MooEdit        **docp)
+{
+    GError *error = NULL;
+    gboolean new_doc = FALSE;
+    MooEdit *doc = NULL;
+    char *filename;
+    gboolean result = TRUE;
+
+    *docp = NULL;
+    filename = _moo_normalize_file_path (info->filename);
+
+    if (window_list_find_file (editor, filename, docp))
+    {
+        if (add_history)
+            add_recent_file (editor, filename);
+        g_free (filename);
+        return FALSE;
+    }
+
+    if (window)
+    {
+        doc = moo_edit_window_get_active_doc (window);
+
+        if (doc && moo_edit_is_empty (doc))
+            g_object_ref (doc);
+        else
+            doc = NULL;
+    }
+
+    if (!doc)
+    {
+        doc = g_object_new (get_doc_type (editor), "editor", editor, NULL);
+        MOO_OBJECT_REF_SINK (doc);
+        new_doc = TRUE;
+    }
+
+    /* XXX open_single */
+    if (!_moo_edit_load_file (doc, filename, info->encoding, &error))
+    {
+        if (!silent)
+        {
+            if (!parent && !window)
+                window = moo_editor_get_active_window (editor);
+            if (!parent && window)
+                parent = GTK_WIDGET (window);
+            _moo_edit_open_error_dialog (parent, filename, info->encoding, error);
+        }
+        g_error_free (error);
+        result = FALSE;
+    }
+    else
+    {
+        if (!window)
+            window = moo_editor_get_active_window (editor);
+        if (!window)
+            window = create_window (editor);
+
+        if (new_doc)
+        {
+            _moo_edit_window_insert_doc (window, doc, -1);
+            moo_editor_add_doc (editor, window, doc);
+        }
+
+        if (add_history)
+            add_recent_file (editor, filename);
+    }
+
+    if (result)
+        *docp = doc;
+
+    g_free (filename);
+    g_object_unref (doc);
+    return result;
+}
+
 gboolean
 moo_editor_open (MooEditor      *editor,
                  MooEditWindow  *window,
@@ -1174,7 +1306,9 @@ moo_editor_open (MooEditor      *editor,
 
     if (!files)
     {
-        files = _moo_edit_open_dialog (parent, editor->priv->filter_mgr);
+        MooEdit *active = window ? moo_edit_window_get_active_doc (window) : NULL;
+
+        files = _moo_edit_open_dialog (parent, editor->priv->filter_mgr, active);
 
         if (!files)
             return FALSE;
@@ -1185,78 +1319,21 @@ moo_editor_open (MooEditor      *editor,
         return result;
     }
 
+    bring_to_front = NULL;
+
     for (l = files; l != NULL; l = l->next)
     {
         MooEditFileInfo *info = l->data;
-        GError *error = NULL;
-        gboolean new_doc = FALSE;
         MooEdit *doc = NULL;
-        char *filename;
-
-        filename = _moo_normalize_file_path (info->filename);
-
-        if (window_list_find_file (editor, filename, &bring_to_front))
-        {
-            moo_history_list_add_filename (editor->priv->history, filename);
-            g_free (filename);
-            continue;
-        }
-        else
-        {
-            bring_to_front = NULL;
-        }
 
         if (!window)
             window = moo_editor_get_active_window (editor);
 
-        if (window)
-        {
-            doc = moo_edit_window_get_active_doc (window);
-            if (doc && moo_edit_is_empty (doc))
-                g_object_ref (doc);
-            else
-                doc = NULL;
-        }
+        if (moo_editor_load_file (editor, window, parent, info, editor->priv->silent, TRUE, &doc))
+            parent = GTK_WIDGET (doc);
 
-        if (!doc)
-        {
-            doc = g_object_new (get_doc_type (editor), "editor", editor, NULL);
-            MOO_OBJECT_REF_SINK (doc);
-            new_doc = TRUE;
-        }
-
-        /* XXX open_single */
-        if (!_moo_edit_load_file (doc, filename, info->encoding, &error))
-        {
-            if (!editor->priv->silent)
-                _moo_edit_open_error_dialog (parent, filename, info->encoding, error);
-            g_error_free (error);
-            result = FALSE;
-        }
-        else
-        {
-            if (!window)
-                window = moo_editor_get_active_window (editor);
-            if (!window)
-                window = create_window (editor);
-
-            if (new_doc)
-            {
-                _moo_edit_window_insert_doc (window, doc, -1);
-                moo_editor_add_doc (editor, window, doc);
-            }
-            else
-            {
-                bring_to_front = doc;
-            }
-
-            moo_history_list_add_filename (editor->priv->history, filename);
-
-            parent = GTK_WIDGET (window);
-        }
-
-        g_free (filename);
-        g_object_unref (doc);
+        if (doc)
+            bring_to_front = doc;
     }
 
     if (bring_to_front)
@@ -1351,13 +1428,13 @@ find_busy (GSList *docs)
 }
 
 
-gboolean
-moo_editor_close_window (MooEditor      *editor,
-                         MooEditWindow  *window,
-                         gboolean        ask_confirm)
+static gboolean
+close_window_handler (MooEditor     *editor,
+                      MooEditWindow *window,
+                      gboolean       ask_confirm)
 {
     WindowInfo *info;
-    MooEditDialogResponse response;
+    MooSaveChangesDialogResponse response;
     GSList *modified;
     gboolean do_close = FALSE;
     MooEdit *busy = NULL;
@@ -1373,7 +1450,7 @@ moo_editor_close_window (MooEditor      *editor,
     if (busy)
     {
         moo_editor_set_active_doc (editor, busy);
-        return FALSE;
+        return TRUE;
     }
 
     modified = find_modified (info->docs);
@@ -1391,12 +1468,12 @@ moo_editor_close_window (MooEditor      *editor,
 
         switch (response)
         {
-            case MOO_EDIT_RESPONSE_SAVE:
+            case MOO_SAVE_CHANGES_RESPONSE_SAVE:
                 if (_moo_editor_save (editor, modified->data, NULL))
                     do_close = TRUE;
                 break;
 
-            case MOO_EDIT_RESPONSE_CANCEL:
+            case MOO_SAVE_CHANGES_RESPONSE_CANCEL:
                 break;
 
             default:
@@ -1413,7 +1490,7 @@ moo_editor_close_window (MooEditor      *editor,
 
         switch (response)
         {
-            case MOO_EDIT_RESPONSE_SAVE:
+            case MOO_SAVE_CHANGES_RESPONSE_SAVE:
                 for (l = to_save; l != NULL; l = l->next)
                     if (!_moo_editor_save (editor, l->data, NULL))
                     {
@@ -1427,7 +1504,7 @@ moo_editor_close_window (MooEditor      *editor,
                 g_slist_free (to_save);
                 break;
 
-            case MOO_EDIT_RESPONSE_CANCEL:
+            case MOO_SAVE_CHANGES_RESPONSE_CANCEL:
                 break;
 
             default:
@@ -1436,11 +1513,34 @@ moo_editor_close_window (MooEditor      *editor,
         }
     }
 
-    if (do_close)
+    g_slist_free (modified);
+    return !do_close;
+}
+
+
+gboolean
+moo_editor_close_window (MooEditor      *editor,
+                         MooEditWindow  *window,
+                         gboolean        ask_confirm)
+{
+    gboolean stopped = FALSE;
+
+    g_return_val_if_fail (MOO_IS_EDITOR (editor), FALSE);
+    g_return_val_if_fail (MOO_IS_EDIT_WINDOW (window), FALSE);
+
+    if (!window_list_find (editor, window))
+        return TRUE;
+
+    g_object_ref (window);
+
+    g_signal_emit (editor, signals[CLOSE_WINDOW], 0, window, ask_confirm, &stopped);
+
+    if (!stopped && window_list_find (editor, window))
         do_close_window (editor, window);
 
-    g_slist_free (modified);
-    return do_close;
+    g_object_unref (window);
+
+    return !stopped;
 }
 
 
@@ -1464,9 +1564,6 @@ do_close_window (MooEditor      *editor,
     _moo_window_detach_plugins (window);
     gtk_widget_destroy (GTK_WIDGET (window));
 
-    if (!editor->priv->windows)
-        g_signal_emit (editor, signals[ALL_WINDOWS_CLOSED], 0);
-
     g_slist_free (list);
 }
 
@@ -1560,7 +1657,7 @@ close_docs_real (MooEditor      *editor,
                  GSList         *docs,
                  gboolean        ask_confirm)
 {
-    MooEditDialogResponse response;
+    MooSaveChangesDialogResponse response;
     GSList *modified, *l;
     gboolean do_close = FALSE;
 
@@ -1582,12 +1679,12 @@ close_docs_real (MooEditor      *editor,
 
         switch (response)
         {
-            case MOO_EDIT_RESPONSE_SAVE:
+            case MOO_SAVE_CHANGES_RESPONSE_SAVE:
                 if (_moo_editor_save (editor, modified->data, NULL))
                     do_close = TRUE;
                 break;
 
-            case MOO_EDIT_RESPONSE_CANCEL:
+            case MOO_SAVE_CHANGES_RESPONSE_CANCEL:
                 break;
 
             default:
@@ -1604,7 +1701,7 @@ close_docs_real (MooEditor      *editor,
 
         switch (response)
         {
-            case MOO_EDIT_RESPONSE_SAVE:
+            case MOO_SAVE_CHANGES_RESPONSE_SAVE:
                 for (l = to_save; l != NULL; l = l->next)
                     if (!_moo_editor_save (editor, l->data, NULL))
                 {
@@ -1618,7 +1715,7 @@ close_docs_real (MooEditor      *editor,
                 g_slist_free (to_save);
                 break;
 
-            case MOO_EDIT_RESPONSE_CANCEL:
+            case MOO_SAVE_CHANGES_RESPONSE_CANCEL:
                 break;
 
             default:
@@ -1686,6 +1783,238 @@ moo_editor_close_all (MooEditor *editor,
 }
 
 
+static char *
+filename_from_utf8 (const char *encoded)
+{
+    if (g_str_has_prefix (encoded, "base64"))
+    {
+        guchar *filename;
+        gsize len;
+
+        filename = g_base64_decode (encoded + strlen ("base64"), &len);
+
+        if (!filename || !len || filename[len-1] != 0)
+        {
+            g_critical ("%s: oops", G_STRLOC);
+            return NULL;
+        }
+
+        return (char*) filename;
+    }
+    else
+    {
+        return g_strdup (encoded);
+    }
+}
+
+static char *
+filename_to_utf8 (const char *filename)
+{
+    char *encoded, *ret;
+
+    if (g_utf8_validate (filename, -1, NULL))
+        return g_strdup (filename);
+
+    encoded = g_base64_encode ((const guchar *) filename, strlen (filename) + 1);
+    ret = g_strdup_printf ("base64%s", encoded);
+    g_free (encoded);
+    return ret;
+}
+
+static MooEdit *
+load_doc_session (MooEditor     *editor,
+                  MooEditWindow *window,
+                  MooMarkupNode *elm)
+{
+    char *filename;
+    const char *utf8_filename;
+    const char *encoding;
+    MooEdit *doc = NULL;
+    MooEditFileInfo *info;
+
+    utf8_filename = moo_markup_get_content (elm);
+
+    if (!utf8_filename || !utf8_filename[0])
+        return moo_editor_new_doc (editor, window);
+
+    filename = filename_from_utf8 (utf8_filename);
+    g_return_val_if_fail (filename != NULL, NULL);
+
+    encoding = moo_markup_get_prop (elm, "encoding");
+    info = moo_edit_file_info_new (filename, encoding);
+
+    moo_editor_load_file (editor, window, GTK_WIDGET (window), info, TRUE, FALSE, &doc);
+
+    if (doc)
+    {
+        int line = moo_markup_get_int_prop (elm, "line", 0);
+        moo_text_view_move_cursor (MOO_TEXT_VIEW (doc), line, 0, FALSE, TRUE);
+    }
+
+    moo_edit_file_info_free (info);
+    g_free (filename);
+
+    return doc;
+}
+
+static MooMarkupNode *
+save_doc_session (MooEdit       *doc,
+                  MooMarkupNode *elm)
+{
+    char *filename;
+    const char *encoding;
+    MooMarkupNode *node;
+
+    filename = moo_edit_get_filename (doc);
+    encoding = moo_edit_get_encoding (doc);
+
+    if (filename)
+    {
+        char *utf8_filename;
+
+        utf8_filename = filename_to_utf8 (filename);
+        g_return_val_if_fail (utf8_filename != NULL, NULL);
+
+        node = moo_markup_create_text_element (elm, "document", utf8_filename);
+        moo_markup_set_int_prop (node, "line", moo_text_view_get_cursor_line (MOO_TEXT_VIEW (doc)));
+
+        if (encoding && encoding[0])
+            moo_markup_set_prop (node, "encoding", encoding);
+
+        g_free (utf8_filename);
+    }
+    else
+    {
+        node = moo_markup_create_element (elm, "document");
+    }
+
+    g_free (filename);
+    return node;
+}
+
+static MooEditWindow *
+load_window_session (MooEditor     *editor,
+                     MooMarkupNode *elm)
+{
+    MooEditWindow *window;
+    MooEdit *active_doc = NULL;
+    MooMarkupNode *node;
+
+    window = create_window (editor);
+
+    for (node = elm->children; node != NULL; node = node->next)
+    {
+        if (MOO_MARKUP_IS_ELEMENT (node))
+        {
+            MooEdit *doc;
+
+            doc = load_doc_session (editor, window, node);
+
+            if (doc && moo_markup_get_bool_prop (node, "active", FALSE))
+                active_doc = doc;
+        }
+    }
+
+    if (active_doc)
+        moo_edit_window_set_active_doc (window, active_doc);
+
+    return window;
+}
+
+static MooMarkupNode *
+save_window_session (MooEditWindow *window,
+                     MooMarkupNode *elm)
+{
+    MooMarkupNode *node;
+    MooEdit *active_doc;
+    GSList *docs;
+
+    active_doc = moo_edit_window_get_active_doc (window);
+    docs = moo_edit_window_list_docs (window);
+
+    node = moo_markup_create_element (elm, "window");
+
+    while (docs)
+    {
+        MooMarkupNode *doc_node;
+        MooEdit *doc = docs->data;
+
+        doc_node = save_doc_session (doc, node);
+
+        if (doc_node && doc == active_doc)
+            moo_markup_set_bool_prop (doc_node, "active", TRUE);
+
+        docs = g_slist_delete_link (docs, docs);
+    }
+
+    return node;
+}
+
+void
+_moo_editor_load_session (MooEditor     *editor,
+                          MooMarkupNode *xml)
+{
+    MooMarkupNode *editor_node;
+
+    g_return_if_fail (MOO_IS_EDITOR (editor));
+    g_return_if_fail (MOO_MARKUP_IS_ELEMENT (xml));
+
+    editor_node = moo_markup_get_element (xml, "editor");
+
+    if (editor_node)
+    {
+        MooEditWindow *active_window = NULL;
+        MooMarkupNode *node;
+
+        for (node = editor_node->children; node != NULL; node = node->next)
+        {
+            MooEditWindow *window;
+
+            if (!MOO_MARKUP_IS_ELEMENT (node))
+                continue;
+
+            window = load_window_session (editor, node);
+
+            if (window && moo_markup_get_bool_prop (node, "active", FALSE))
+                active_window = window;
+        }
+
+        if (active_window)
+            moo_editor_set_active_window (editor, active_window);
+    }
+}
+
+void
+_moo_editor_save_session (MooEditor     *editor,
+                          MooMarkupNode *xml)
+{
+    MooMarkupNode *node;
+    MooEditWindow *active_window;
+    GSList *windows;
+
+    g_return_if_fail (MOO_IS_EDITOR (editor));
+    g_return_if_fail (MOO_MARKUP_IS_ELEMENT (xml));
+
+    active_window = moo_editor_get_active_window (editor);
+    windows = moo_editor_list_windows (editor);
+
+    node = moo_markup_create_element (xml, "editor");
+
+    while (windows)
+    {
+        MooEditWindow *window = windows->data;
+        MooMarkupNode *window_node;
+
+        window_node = save_window_session (window, node);
+
+        if (window_node && window == active_window)
+            moo_markup_set_bool_prop (window_node, "active", TRUE);
+
+        windows = g_slist_delete_link (windows, windows);
+    }
+}
+
+
 GSList*
 moo_editor_list_windows (MooEditor *editor)
 {
@@ -1884,6 +2213,7 @@ moo_editor_open_uri (MooEditor      *editor,
 void
 _moo_editor_reload (MooEditor      *editor,
                     MooEdit        *doc,
+                    const char     *encoding,
                     GError        **error)
 {
     WindowInfo *info;
@@ -1900,7 +2230,7 @@ _moo_editor_reload (MooEditor      *editor,
     g_return_if_fail (info != NULL);
 
     /* XXX */
-    g_return_if_fail (moo_edit_get_filename (doc) != NULL);
+    g_return_if_fail (!moo_edit_is_untitled (doc));
 
     if (!editor->priv->silent &&
          !MOO_EDIT_IS_CLEAN (doc) &&
@@ -1912,13 +2242,19 @@ _moo_editor_reload (MooEditor      *editor,
     cursor_line = gtk_text_iter_get_line (&iter);
     cursor_offset = moo_text_iter_get_visual_line_offset (&iter, 8);
 
-    if (!_moo_edit_reload_file (doc, &error_here))
+    if (!_moo_edit_reload_file (doc, encoding, &error_here))
     {
         if (!editor->priv->silent)
+        {
             _moo_edit_reload_error_dialog (doc, error_here);
+            g_error_free (error_here);
+        }
         else
+        {
             g_propagate_error (error, error_here);
+        }
 
+        moo_text_view_undo (MOO_TEXT_VIEW (doc));
         g_object_set_data (G_OBJECT (doc), "moo-scroll-to", NULL);
         return;
     }
@@ -1949,11 +2285,15 @@ do_save (MooEditor    *editor,
 {
     gboolean result;
     gboolean strip;
+    gboolean add_newline;
 
     strip = moo_edit_config_get_bool (doc->config, "strip");
+    add_newline = moo_edit_config_get_bool (doc->config, "add-newline");
 
     if (strip)
         moo_text_view_strip_whitespace (MOO_TEXT_VIEW (doc));
+    if (add_newline)
+        _moo_edit_ensure_newline (doc);
 
     g_signal_emit_by_name (doc, "save-before");
     result = _moo_edit_save_file (doc, filename, encoding,
@@ -1984,10 +2324,10 @@ _moo_editor_save (MooEditor      *editor,
     info = window_list_find_doc (editor, doc);
     g_return_val_if_fail (info != NULL, FALSE);
 
-    if (!moo_edit_get_filename (doc))
+    if (moo_edit_is_untitled (doc))
         return _moo_editor_save_as (editor, doc, NULL, NULL, error);
 
-    filename = g_strdup (moo_edit_get_filename (doc));
+    filename = moo_edit_get_filename (doc);
     encoding = g_strdup (moo_edit_get_encoding (doc));
 
     if (!editor->priv->silent &&
@@ -2016,7 +2356,7 @@ _moo_editor_save (MooEditor      *editor,
         goto out;
     }
 
-    moo_history_list_add_filename (editor->priv->history, filename);
+    add_recent_file (editor, filename);
     result = TRUE;
 
     /* fall through */
@@ -2085,7 +2425,7 @@ _moo_editor_save_as (MooEditor      *editor,
         goto out;
     }
 
-    moo_history_list_add_filename (editor->priv->history, file_info->filename);
+    add_recent_file (editor, file_info->filename);
     result = TRUE;
 
     /* fall through */
@@ -2175,6 +2515,9 @@ moo_editor_apply_prefs (MooEditor *editor)
     int autosave_interval;
     const char *color_scheme, *default_lang;
 
+    _moo_edit_window_update_title ();
+    _moo_edit_window_set_use_tabs ();
+
     default_lang = moo_prefs_get_string (moo_edit_setting (MOO_EDIT_PREFS_DEFAULT_LANG));
 
     if (default_lang && !strcmp (default_lang, MOO_LANG_NONE))
@@ -2203,3 +2546,171 @@ moo_editor_apply_prefs (MooEditor *editor)
                   "save-backups", backups,
                   NULL);
 }
+
+
+char *
+_moo_edit_filename_to_uri (const char *filename,
+                           guint       line,
+                           guint       options)
+{
+    GString *string;
+    char *uri;
+    GError *error = NULL;
+
+    g_return_val_if_fail (filename != NULL, NULL);
+
+    if (!(uri = g_filename_to_uri (filename, NULL, &error)))
+    {
+        g_warning ("%s: could not convert filename to URI: %s",
+                   G_STRLOC, error->message);
+        g_error_free (error);
+        return NULL;
+    }
+
+    if (!line && !options)
+        return uri;
+
+    string = g_string_new (uri);
+    g_string_append (string, "?");
+
+    if (line > 0)
+        g_string_append_printf (string, "line=%u;", line);
+
+    if (options)
+    {
+        g_string_append (string, "options=");
+        if (options & MOO_EDIT_OPEN_NEW_WINDOW)
+            g_string_append (string, "new-window");
+        if (options & MOO_EDIT_OPEN_NEW_TAB)
+            g_string_append (string, "new-tab");
+        g_string_append (string, ";");
+    }
+
+    g_free (uri);
+    return g_string_free (string, FALSE);
+}
+
+static void
+parse_options (const char *optstring,
+               guint      *line,
+               guint      *options)
+{
+    char **p, **comps;
+
+    comps = g_strsplit (optstring, ";", 0);
+
+    for (p = comps; p && *p; ++p)
+    {
+        if (!strncmp (*p, "line=", strlen ("line=")))
+        {
+            /* doesn't matter if there is an error */
+            *line = strtoul (*p + strlen ("line="), NULL, 10);
+        }
+        else if (!strncmp (*p, "options=", strlen ("options=")))
+        {
+            char **opts, **op;
+            opts = g_strsplit (*p + strlen ("options="), ",", 0);
+            for (op = opts; op && *op; ++op)
+            {
+                if (!strcmp (*op, "new-window"))
+                    *options |= MOO_EDIT_OPEN_NEW_WINDOW;
+                else if (!strcmp (*op, "new-tab"))
+                    *options |= MOO_EDIT_OPEN_NEW_TAB;
+            }
+            g_strfreev (opts);
+        }
+    }
+
+    g_strfreev (comps);
+}
+
+char *
+_moo_edit_uri_to_filename (const char *uri,
+                           guint      *line,
+                           guint      *options)
+{
+    const char *question_mark;
+    const char *optstring = NULL;
+    char *freeme = NULL;
+    char *filename;
+    GError *error = NULL;
+
+    g_return_val_if_fail (uri != NULL, NULL);
+    g_return_val_if_fail (line != NULL, NULL);
+    g_return_val_if_fail (options != NULL, NULL);
+
+    *line = 0;
+    *options = 0;
+
+    question_mark = strchr (uri, '?');
+
+    if (question_mark && question_mark > uri)
+    {
+        freeme = g_strndup (uri, question_mark - uri);
+        optstring = question_mark + 1;
+        uri = freeme;
+    }
+
+    if (!(filename = g_filename_from_uri (uri, NULL, &error)))
+    {
+        g_warning ("%s: could not convert URI to filename: %s",
+                   G_STRLOC, error->message);
+        g_error_free (error);
+        g_free (freeme);
+        return NULL;
+    }
+
+    if (optstring)
+        parse_options (optstring, line, options);
+
+    g_free (freeme);
+    return filename;
+}
+
+void
+_moo_editor_open_file (MooEditor  *editor,
+                       const char *filename,
+                       guint       line,
+                       guint       options)
+{
+    MooEdit *doc;
+    MooEditWindow *window;
+
+    g_return_if_fail (MOO_IS_EDITOR (editor));
+    g_return_if_fail (filename != NULL);
+
+    doc = moo_editor_get_doc (editor, filename);
+
+    if (doc)
+    {
+        if (line > 0)
+            moo_text_view_move_cursor (MOO_TEXT_VIEW (doc), line - 1, 0, FALSE, FALSE);
+        moo_editor_set_active_doc (editor, doc);
+        gtk_widget_grab_focus (GTK_WIDGET (doc));
+        return;
+    }
+
+    window = moo_editor_get_active_window (editor);
+    doc = window ? moo_edit_window_get_active_doc (window) : NULL;
+
+    if (!doc || !moo_edit_is_empty (doc))
+    {
+        gboolean new_window = moo_prefs_get_bool (moo_edit_setting (MOO_EDIT_PREFS_OPEN_NEW_WINDOW));
+
+        if (options & MOO_EDIT_OPEN_NEW_TAB)
+            new_window = FALSE;
+        else if (options & MOO_EDIT_OPEN_NEW_WINDOW)
+            new_window = TRUE;
+
+        if (new_window)
+            window = moo_editor_new_window (editor);
+    }
+
+    doc = moo_editor_new_file (editor, window, NULL, filename, NULL);
+    g_return_if_fail (doc != NULL);
+
+    moo_editor_set_active_doc (editor, doc);
+    if (line > 0)
+        moo_text_view_move_cursor (MOO_TEXT_VIEW (doc), line - 1, 0, FALSE, TRUE);
+    gtk_widget_grab_focus (GTK_WIDGET (doc));
+}
