@@ -1,12 +1,11 @@
 /*
  *   mooindenter.c
  *
- *   Copyright (C) 2004-2006 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -86,55 +85,33 @@ moo_indenter_class_init (MooIndenterClass *klass)
     klass->config_changed = config_changed_default;
 
     settings[SETTING_USE_TABS] = moo_edit_config_install_setting (
-            g_param_spec_boolean ("indent-use-tabs",
-                                  "indent-use-tabs",
-                                  "indent-use-tabs",
-                                  TRUE,
-                                  G_PARAM_READWRITE));
+        g_param_spec_boolean ("indent-use-tabs", "indent-use-tabs", "indent-use-tabs",
+                              TRUE, G_PARAM_READWRITE));
     moo_edit_config_install_alias ("indent-use-tabs", "use-tabs");
 
     settings[SETTING_INDENT_WIDTH] = moo_edit_config_install_setting (
-            g_param_spec_uint ("indent-width",
-                               "indent-width",
-                               "indent-width",
-                               1, G_MAXUINT, 8,
-                               G_PARAM_READWRITE));
+        g_param_spec_uint ("indent-width", "indent-width", "indent-width",
+                           1, G_MAXUINT, 8, G_PARAM_READWRITE));
 
-    g_object_class_install_property (gobject_class,
-                                     PROP_DOC,
-                                     g_param_spec_object ("doc",
-                                             "doc",
-                                             "doc",
-                                             MOO_TYPE_EDIT,
-                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+    g_object_class_install_property (gobject_class, PROP_DOC,
+        g_param_spec_object ("doc", "doc", "doc",
+                             MOO_TYPE_EDIT,
+                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
-    g_object_class_install_property (gobject_class,
-                                     PROP_TAB_WIDTH,
-                                     g_param_spec_uint ("tab-width",
-                                             "tab-width",
-                                             "tab-width",
-                                             1,
-                                             G_MAXUINT,
-                                             8,
-                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+    g_object_class_install_property (gobject_class, PROP_TAB_WIDTH,
+        g_param_spec_uint ("tab-width", "tab-width", "tab-width",
+                           1, G_MAXUINT, 8,
+                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
-    g_object_class_install_property (gobject_class,
-                                     PROP_USE_TABS,
-                                     g_param_spec_boolean ("use-tabs",
-                                             "use-tabs",
-                                             "use-tabs",
-                                             TRUE,
-                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+    g_object_class_install_property (gobject_class, PROP_USE_TABS,
+        g_param_spec_boolean ("use-tabs", "use-tabs", "use-tabs",
+                              TRUE,
+                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
-    g_object_class_install_property (gobject_class,
-                                     PROP_INDENT,
-                                     g_param_spec_uint ("indent",
-                                             "indent",
-                                             "indent",
-                                             1,
-                                             G_MAXUINT,
-                                             8,
-                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+    g_object_class_install_property (gobject_class, PROP_INDENT,
+        g_param_spec_uint ("indent", "indent", "indent",
+                           1, G_MAXUINT, 8,
+                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
     signals[CONFIG_CHANGED] =
             g_signal_new ("config-changed",
@@ -207,6 +184,7 @@ moo_indenter_finalize (GObject *object)
         g_signal_handlers_disconnect_by_func (indent->doc,
                                               (gpointer) config_notify,
                                               indent);
+    indent->doc = NULL;
 
     G_OBJECT_CLASS(moo_indenter_parent_class)->finalize (object);
 }
@@ -370,7 +348,8 @@ moo_indenter_make_space (MooIndenter    *indenter,
 
 /* computes amount of leading white space on the given line;
    returns TRUE if line contains some non-whitespace chars;
-   if returns TRUE, then iter points to the first non-white-space char */
+   if returns TRUE, then iter points to the first non-white-space char,
+   otherwise it points to the end of line */
 static gboolean
 compute_line_offset (GtkTextIter    *dest,
                      guint           tab_width,
@@ -378,6 +357,7 @@ compute_line_offset (GtkTextIter    *dest,
 {
     guint offset = 0;
     GtkTextIter iter = *dest;
+    gboolean retval = FALSE;
 
     while (!gtk_text_iter_ends_line (&iter))
     {
@@ -394,15 +374,16 @@ compute_line_offset (GtkTextIter    *dest,
         }
         else
         {
-            *offsetp = offset;
-            *dest = iter;
-            return TRUE;
+            retval = TRUE;
+            break;
         }
 
         gtk_text_iter_forward_char (&iter);
     }
 
-    return FALSE;
+    *offsetp = offset;
+    *dest = iter;
+    return retval;
 }
 
 
@@ -411,29 +392,21 @@ character_default (MooIndenter    *indenter,
                    gunichar        inserted_char,
                    GtkTextIter    *where)
 {
-    int i;
     char *indent_string = NULL;
     GtkTextBuffer *buffer = gtk_text_iter_get_buffer (where);
+    guint offset;
+    GtkTextIter iter;
 
     if (inserted_char != '\n')
         return;
 
-    for (i = gtk_text_iter_get_line (where) - 1; i >= 0; --i)
-    {
-        guint offset;
-        GtkTextIter iter;
+    iter = *where;
+    gtk_text_iter_backward_line (&iter);
+    compute_line_offset (&iter, indenter->tab_width, &offset);
 
-        gtk_text_buffer_get_iter_at_line (buffer, &iter, i);
-
-        if (compute_line_offset (&iter, indenter->tab_width, &offset))
-        {
-            indent_string = moo_indenter_make_space (indenter, offset, 0);
-            break;
-        }
-    }
-
-    if (indent_string)
+    if (offset)
     {
+        indent_string = moo_indenter_make_space (indenter, offset, 0);
         gtk_text_buffer_insert (buffer, where, indent_string, -1);
         g_free (indent_string);
     }
@@ -529,7 +502,7 @@ iter_get_visual_offset (GtkTextIter *iter,
                         int         *offsetp,
                         int         *white_spacep)
 {
-    GtkTextIter start, white_space_start;
+    GtkTextIter start, white_space_start = {0};
     guint offset, white_space;
 
     start = *iter;
