@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -14,7 +13,6 @@
 #include "mooedit/mookeyfile.h"
 #include "mooutils/mooutils-gobject.h"
 #include "mooutils/mooutils-misc.h"
-#include "mooutils/moocompat.h"
 #include <string.h>
 
 
@@ -130,18 +128,7 @@ static gboolean
 line_is_empty_or_comment (const char *line,
                           gsize       line_len)
 {
-    gsize i;
-
-    for (i = 0; i < line_len; ++i)
-    {
-        if (line[i] == '#')
-            return TRUE;
-
-        if (!CHAR_IS_SPACE (line[i]))
-            return FALSE;
-    }
-
-    return TRUE;
+    return line_len == 0 || line[0] == '#';
 }
 
 static guint
@@ -171,6 +158,8 @@ line_is_blank (const char *line,
 {
     guint i;
 
+    *indent = line_len;
+
     for (i = 0; i < line_len; ++i)
     {
         if (!CHAR_IS_SPACE (line[i]))
@@ -284,12 +273,7 @@ parse_content (Parser         *parser,
         line = parser->ptr;
         get_line (line, parser->len, &line_len, &next_line);
 
-        if (line_is_blank (line, line_len, &indent_here))
-        {
-            g_string_append (content, "\n");
-            parser_next_line (parser, next_line);
-            continue;
-        }
+        line_is_blank (line, line_len, &indent_here);
 
         if (!indent_here)
             break;
@@ -850,7 +834,7 @@ format_content (const char *content,
                 const char *indent)
 {
     char **p;
-    char **lines = _moo_splitlines (content);
+    char **lines = moo_splitlines (content);
 
     for (p = lines; p && *p; ++p)
         g_string_append_printf (string, "%s%s\n", indent, *p);
