@@ -1,12 +1,11 @@
 /*
  *   moolangmgr-private.h
  *
- *   Copyright (C) 2004-2006 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -34,7 +33,7 @@ typedef struct _MooLangMgrClass MooLangMgrClass;
 struct _MooLangMgr {
     GtkSourceLanguageManager base;
 
-    GtkSourceStyleManager *style_mgr;
+    GtkSourceStyleSchemeManager *style_mgr;
 
     GHashTable *langs;
     GHashTable *config;
