@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -75,8 +74,8 @@ moo_lang_mgr_init (MooLangMgr *mgr)
 
     dirs = moo_get_data_subdirs (LANGUAGE_DIR, MOO_DATA_SHARE, &n_dirs);
     g_object_set (mgr, "search-path", dirs, NULL);
-    mgr->style_mgr = gtk_source_style_manager_new ();
-    gtk_source_style_manager_set_search_path (mgr->style_mgr, dirs);
+    mgr->style_mgr = gtk_source_style_scheme_manager_new ();
+    gtk_source_style_scheme_manager_set_search_path (mgr->style_mgr, dirs);
 
     load_config (mgr);
 
@@ -111,12 +110,12 @@ moo_lang_mgr_class_init (MooLangMgrClass *klass)
 {
     G_OBJECT_CLASS(klass)->dispose = moo_lang_mgr_dispose;
 
-    _moo_signal_new_cb ("loaded",
-                        G_OBJECT_CLASS_TYPE (klass),
-                        G_SIGNAL_RUN_LAST,
-                        NULL, NULL, NULL,
-                        _moo_marshal_VOID__VOID,
-                        G_TYPE_NONE, 0);
+    g_signal_new ("loaded",
+                  G_OBJECT_CLASS_TYPE (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  _moo_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
 }
 
 
@@ -229,6 +228,8 @@ _moo_lang_mgr_find_lang (MooLangMgr *mgr,
         name = "cpp";
     else if (!g_ascii_strcasecmp (name, "nxml"))
         name = "xml";
+    else if (!g_ascii_strcasecmp (name, "cperl"))
+        name = "perl";
 
     read_langs (mgr);
     id = _moo_lang_id_from_name (name);
@@ -386,6 +387,16 @@ moo_lang_mgr_get_lang_for_file (MooLangMgr *mgr,
     if (file_blacklisted (mgr, filename))
         return NULL;
 
+    lang = get_lang_by_extension (mgr, filename);
+
+    if (lang)
+        return lang;
+
+    lang = lang_mgr_get_lang_for_bak_filename (mgr, filename);
+
+    if (lang)
+        return lang;
+
 #ifdef MOO_USE_XDGMIME
     /* XXX: xdgmime wants utf8-encoded filename here. is it a problem? */
     /* It's a big problem! */
@@ -402,16 +413,6 @@ moo_lang_mgr_get_lang_for_file (MooLangMgr *mgr,
 #endif
 #endif /* MOO_USE_XDGMIME */
 
-    lang = get_lang_by_extension (mgr, filename);
-
-    if (lang)
-        return lang;
-
-    lang = lang_mgr_get_lang_for_bak_filename (mgr, filename);
-
-    if (lang)
-        return lang;
-
     return NULL;
 }
 
@@ -530,7 +531,7 @@ get_lang_for_mime_type (MooLangMgr *mgr,
 static void
 read_langs (MooLangMgr *mgr)
 {
-    const GSList *langs = NULL;
+    const char * const *ids = NULL;
 
     if (mgr->got_langs)
         return;
@@ -540,26 +541,29 @@ read_langs (MooLangMgr *mgr)
     mgr->got_langs = TRUE;
 
 #ifdef MOO_USE_XML
-    langs = gtk_source_language_manager_get_available_languages (GTK_SOURCE_LANGUAGE_MANAGER (mgr));
+    ids = gtk_source_language_manager_get_language_ids (GTK_SOURCE_LANGUAGE_MANAGER (mgr));
 #endif
 
     get_lang_info (mgr, MOO_LANG_NONE, TRUE);
 
-    while (langs)
+    while (ids && *ids)
     {
         LangInfo *info;
-        MooLang *lang = langs->data;
-        const char *id = _moo_lang_id (lang);
+        const char *id;
+        GtkSourceLanguage *lang;
+
+        id = *ids;
+        lang = gtk_source_language_manager_get_language (GTK_SOURCE_LANGUAGE_MANAGER (mgr), id);
 
         info = get_lang_info (mgr, id, TRUE);
         info->lang = g_object_ref (lang);
 
         if (!info->globs_modified)
-            info->globs = _moo_lang_get_globs (lang);
+            info->globs = _moo_lang_get_globs (info->lang);
         if (!info->mime_types_modified)
-            info->mime_types = _moo_lang_get_mime_types (lang);
+            info->mime_types = _moo_lang_get_mime_types (info->lang);
 
-        langs = langs->next;
+        ids++;
     }
 
     g_signal_emit_by_name (mgr, "loaded");
@@ -632,6 +636,7 @@ static void
 set_default_config (MooLangMgr *mgr)
 {
     _moo_lang_mgr_set_config (mgr, "makefile", "use-tabs: true");
+    _moo_lang_mgr_set_config (mgr, "diff", "strip: false");
 }
 
 static void
@@ -671,7 +676,7 @@ load_config (MooLangMgr *mgr)
 
 
 MooTextStyleScheme *
-_moo_lang_mgr_get_active_scheme (MooLangMgr *mgr)
+moo_lang_mgr_get_active_scheme (MooLangMgr *mgr)
 {
     g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
 
@@ -713,7 +718,7 @@ prepend_scheme (G_GNUC_UNUSED const char *name,
 }
 
 GSList *
-_moo_lang_mgr_list_schemes (MooLangMgr *mgr)
+moo_lang_mgr_list_schemes (MooLangMgr *mgr)
 {
     GSList *list = NULL;
 
@@ -729,25 +734,33 @@ _moo_lang_mgr_list_schemes (MooLangMgr *mgr)
 static void
 read_schemes (MooLangMgr *mgr)
 {
-    const GSList *list;
+    const char *const *ids;
 
     if (mgr->got_schemes)
         return;
 
     mgr->got_schemes = TRUE;
 
-    list = gtk_source_style_manager_list_schemes (mgr->style_mgr);
+    ids = gtk_source_style_scheme_manager_get_scheme_ids (mgr->style_mgr);
 
-    while (list)
+    while (ids && *ids)
     {
-        MooTextStyleScheme *scheme = list->data;
+        MooTextStyleScheme *scheme;
 
-        if (!mgr->active_scheme || !strcmp (moo_text_style_scheme_get_id (scheme), SCHEME_DEFAULT))
-            mgr->active_scheme = scheme;
+        scheme = MOO_TEXT_STYLE_SCHEME (gtk_source_style_scheme_manager_get_scheme (mgr->style_mgr, *ids));
 
-        g_hash_table_insert (mgr->schemes, g_strdup (moo_text_style_scheme_get_id (scheme)), g_object_ref (scheme));
+        if (!scheme)
+        {
+            g_critical ("%s: oops", G_STRLOC);
+        }
+        else
+        {
+            if (!mgr->active_scheme || !strcmp (*ids, SCHEME_DEFAULT))
+                mgr->active_scheme = scheme;
+            g_hash_table_insert (mgr->schemes, g_strdup (*ids), g_object_ref (scheme));
+        }
 
-        list = list->next;
+        ids++;
     }
 }
 
@@ -756,22 +769,30 @@ GSList *
 moo_lang_mgr_get_sections (MooLangMgr *mgr)
 {
     GSList *sections = NULL;
-    const GSList *list = NULL;
+    const char * const *ids = NULL;
 
     g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
 
     read_langs (mgr);
 
 #ifdef MOO_USE_XML
-    list = gtk_source_language_manager_get_available_languages (GTK_SOURCE_LANGUAGE_MANAGER (mgr));
+    ids = gtk_source_language_manager_get_language_ids (GTK_SOURCE_LANGUAGE_MANAGER (mgr));
 #endif
 
-    while (list)
+    while (ids && *ids)
     {
-        const char *section = _moo_lang_get_section (list->data);
+        GtkSourceLanguage *lang;
+        const char *section;
+        const char *id;
+
+        id = *ids;
+        lang = gtk_source_language_manager_get_language (GTK_SOURCE_LANGUAGE_MANAGER (mgr), id);
+        section = _moo_lang_get_section (MOO_LANG (lang));
+
         if (section && !g_slist_find_custom (sections, section, (GCompareFunc) strcmp))
             sections = g_slist_prepend (sections, g_strdup (section));
-        list = list->next;
+
+        ids++;
     }
 
     return sections;
@@ -781,24 +802,27 @@ moo_lang_mgr_get_sections (MooLangMgr *mgr)
 GSList *
 moo_lang_mgr_get_available_langs (MooLangMgr *mgr)
 {
-    GSList *langs = NULL;
-    const GSList *list = NULL;
+    GSList *list = NULL;
+    const char *const *ids = NULL;
 
     g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
 
     read_langs (mgr);
 
 #ifdef MOO_USE_XML
-    list = gtk_source_language_manager_get_available_languages (GTK_SOURCE_LANGUAGE_MANAGER (mgr));
+    ids = gtk_source_language_manager_get_language_ids (GTK_SOURCE_LANGUAGE_MANAGER (mgr));
 #endif
 
-    while (list)
+    while (ids && *ids)
     {
-        langs = g_slist_prepend (langs, g_object_ref (list->data));
-        list = list->next;
+        GtkSourceLanguage *lang;
+        lang = gtk_source_language_manager_get_language (GTK_SOURCE_LANGUAGE_MANAGER (mgr), *ids);
+        if (lang)
+            list = g_slist_prepend (list, g_object_ref (lang));
+        ids++;
     }
 
-    return g_slist_reverse (langs);
+    return g_slist_reverse (list);
 }
 
 
@@ -884,8 +908,14 @@ set_globs_or_mime_types (MooLangMgr *mgr,
                   _moo_lang_mgr_get_mime_types (mgr, id);
 
     if (string_list_equal (new, old))
-        goto out;
+    {
+        g_free (id);
+        string_list_free (old);
+        string_list_free (new);
+        return;
+    }
 
+    string_list_free (old);
     mgr->modified = TRUE;
 
     if (info->lang)
@@ -913,10 +943,6 @@ set_globs_or_mime_types (MooLangMgr *mgr,
         info->globs_modified = modified;
     else
         info->mime_types_modified = modified;
-
-out:
-    g_free (id);
-    string_list_free (old);
 }
 
 
