@@ -1,12 +1,11 @@
 /*
  *   moolinemark.c
  *
- *   Copyright (C) 2004-2006 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -15,6 +14,7 @@
 #include "mooedit/mootext-private.h"
 #include "mooedit/mootextbuffer.h"
 #include "mooutils/moomarshals.h"
+#include "mooutils/mooutils-misc.h"
 
 
 struct _MooLineMarkPrivate {
@@ -368,16 +368,12 @@ void
 moo_line_mark_set_markup (MooLineMark    *mark,
                           const char     *markup)
 {
-    char *tmp;
-
     g_return_if_fail (MOO_IS_LINE_MARK (mark));
 
     if (!markup || !markup[0])
         markup = NULL;
 
-    tmp = mark->priv->markup;
-    mark->priv->markup = g_strdup (markup);
-    g_free (tmp);
+    MOO_ASSIGN_STRING (mark->priv->markup, markup);
 }
 
 
