@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -20,15 +19,21 @@
 #include "mooutils/mooutils-gobject.h"
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/moomarkup.h"
+#include "mooutils/mooi18n.h"
 #include <glib/gregex.h>
 #include <string.h>
 
+#define FILTERS_VERSION     "1.0"
+
 #define FILTERS_FILE        "filters.xml"
+#define ELEMENT_ROOT        "medit-filters"
 #define ELEMENT_FILTER      "filter"
 #define ELEMENT_MATCH       "match"
 #define ELEMENT_ACTION      "action"
+#define PROP_VERSION        "version"
 #define PROP_FILTER_ID      "id"
 #define PROP_FILTER_NAME    "name"
+#define PROP_FILTER__NAME   "_name"
 #define PROP_OUTPUT_TYPE    "what"
 #define PROP_SPAN           "span"
 #define PROP_PATTERN        "pattern"
@@ -215,7 +220,7 @@ parse_file_line (MooOutputFilterRegex *filter,
                  const char           *character)
 {
     MooFileLineData *data;
-    char *freeme;
+    char *freeme = NULL;
 
     file = file && *file ? file : NULL;
     line = line && *line ? line : NULL;
@@ -239,6 +244,7 @@ parse_file_line (MooOutputFilterRegex *filter,
     data->line = _moo_convert_string_to_int (line, 0) - 1;
     data->character = _moo_convert_string_to_int (character, 0) - 1;
 
+    g_free (freeme);
     return data;
 }
 
@@ -370,22 +376,16 @@ get_tag (MooLineView *view,
 
         tag = moo_line_view_create_tag (view, name, NULL);
         lang_mgr = moo_editor_get_lang_mgr (moo_editor_instance ());
-        scheme = _moo_lang_mgr_get_active_scheme (lang_mgr);
+        scheme = moo_lang_mgr_get_active_scheme (lang_mgr);
 
         if (scheme)
             style = _moo_text_style_scheme_lookup_style (scheme, name);
 
         if (style)
-        {
             _moo_text_style_apply_to_tag (style, tag);
-            moo_text_style_free (style);
-        }
-        else
-        {
-            if (type == OUTPUT_STDERR || !strcmp (name, "output-error") ||
-                !strcmp (name, "output-stderr"))
-                    g_object_set (tag, "foreground", "red", NULL);
-        }
+        else if (type == OUTPUT_STDERR || !strcmp (name, "output-error") ||
+                 !strcmp (name, "output-stderr"))
+            g_object_set (tag, "foreground", "red", NULL);
     }
 
     return tag;
@@ -933,12 +933,14 @@ parse_filter_node (MooMarkupNode *elm,
 {
     const char *id;
     const char *name;
+    const char *_name;
     FilterInfo *info;
     GSList *patterns = NULL;
     MooMarkupNode *child;
 
     id = moo_markup_get_prop (elm, PROP_FILTER_ID);
     name = moo_markup_get_prop (elm, PROP_FILTER_NAME);
+    _name = moo_markup_get_prop (elm, PROP_FILTER__NAME);
 
     if (!id || !id[0])
     {
@@ -946,7 +948,11 @@ parse_filter_node (MooMarkupNode *elm,
         return NULL;
     }
 
-    if (!name || !name[0])
+    if (_name && _name[0])
+    {
+        name = Q_(_name);
+    }
+    else if (!name || !name[0])
     {
         g_warning ("in file %s: filter name missing", file);
         return NULL;
@@ -989,7 +995,7 @@ static void
 parse_filter_file (const char *file)
 {
     MooMarkupDoc *doc;
-    MooMarkupNode *node;
+    MooMarkupNode *root, *node;
     GError *error = NULL;
 
     if (!g_file_test (file, G_FILE_TEST_EXISTS))
@@ -1004,7 +1010,26 @@ parse_filter_file (const char *file)
         return;
     }
 
-    for (node = doc->children; node != NULL; node = node->next)
+    root = moo_markup_get_root_element (doc, ELEMENT_ROOT);
+
+    if (root)
+    {
+        const char *version = moo_markup_get_prop (root, PROP_VERSION);
+
+        if (!version || strcmp (version, FILTERS_VERSION) != 0)
+        {
+            g_warning ("in file %s: invalid version '%s'",
+                       file, version ? version : "(null)");
+            moo_markup_doc_unref (doc);
+            return;
+        }
+    }
+    else
+    {
+        root = MOO_MARKUP_NODE (doc);
+    }
+
+    for (node = root->children; node != NULL; node = node->next)
     {
         FilterInfo *info;
 
