@@ -1,12 +1,11 @@
 /*
  *   mooplugin.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -23,6 +22,9 @@
 #include "mooutils/mooprefsdialog.h"
 #include "mooutils/moostock.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/mooutils-debug.h"
+#include "mooutils/moohelp.h"
+#include "help-sections.h"
 #include <string.h>
 #include <gmodule.h>
 #include <gobject/gvaluecollector.h>
@@ -906,11 +908,13 @@ moo_plugin_get_dirs (void)
 static void
 moo_plugin_init_builtin (void)
 {
-#ifndef __WIN32__
-    _moo_find_plugin_init ();
-#endif
-#if GTK_CHECK_VERSION(2,6,0)
     _moo_file_selector_plugin_init ();
+    _moo_file_list_plugin_init ();
+#ifndef __WIN32__
+#ifdef MOO_BUILD_CTAGS
+    _moo_ctags_plugin_init ();
+#endif
+    _moo_find_plugin_init ();
 #endif
 #if 0
     _moo_completion_plugin_init ();
@@ -1077,7 +1081,7 @@ _moo_doc_detach_plugins (MooEditWindow *window,
 {
     GSList *l;
 
-    g_return_if_fail (MOO_IS_EDIT_WINDOW (window));
+    g_return_if_fail (!window || MOO_IS_EDIT_WINDOW (window));
     g_return_if_fail (MOO_IS_EDIT (doc));
 
     plugin_store_init ();
@@ -1483,10 +1487,12 @@ moo_plugin_attach_prefs (GtkWidget *dialog)
     g_return_if_fail (MOO_IS_PREFS_DIALOG (dialog));
 
     page = moo_prefs_dialog_page_new_from_xml ("Plugins", MOO_STOCK_PLUGINS,
-                                               NULL, MOO_PLUGIN_PREFS_GLADE_UI,
+                                               NULL, moopluginprefs_glade_xml,
                                                "page", MOO_PLUGIN_PREFS_ROOT);
     g_return_if_fail (page != NULL);
 
+    moo_help_set_id (GTK_WIDGET (page), HELP_SECTION_PREFS_PLUGINS);
+
     xml = page->xml;
 
     treeview = moo_glade_xml_get_widget (xml, "treeview");
