@@ -1,12 +1,11 @@
 /*
  *   mooprintpreview.c
  *
- *   Copyright (C) 2004-2006 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -21,8 +20,11 @@
 #include "mooedit/mooprintpreview-glade.h"
 #include "mooutils/mooutils-gobject.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/mooutils-debug.h"
 #include <cairo-pdf.h>
 
+MOO_DEBUG_INIT(printing, TRUE)
+
 #ifdef __WIN32__
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
@@ -89,8 +91,12 @@ _moo_print_preview_init (MooPrintPreview *preview)
 
     preview->priv->xml = moo_glade_xml_new_empty (GETTEXT_PACKAGE);
     moo_glade_xml_fill_widget (preview->priv->xml, GTK_WIDGET (preview),
-                               MOO_PRINT_PREVIEW_GLADE_XML, -1,
+                               mooprintpreview_glade_xml, -1,
                                "dialog", NULL);
+    gtk_dialog_set_alternative_button_order (GTK_DIALOG (preview),
+                                             GTK_RESPONSE_APPLY,
+                                             GTK_RESPONSE_CLOSE,
+                                             -1);
 
     preview->priv->context = NULL;
     preview->priv->gtk_preview = NULL;
@@ -298,7 +304,7 @@ moo_print_preview_set_page (MooPrintPreview *preview,
     gtk_widget_set_sensitive (moo_glade_xml_get_widget (preview->priv->xml, "prev"),
                               n > 0);
 
-    text = g_strdup_printf ("%d", n + 1);
+    text = g_strdup_printf ("%u", n + 1);
     gtk_entry_set_text (preview->priv->entry, text);
     g_free (text);
 }
@@ -367,9 +373,9 @@ moo_print_preview_set_zoom (MooPrintPreview *preview,
     gtk_widget_queue_draw (preview->priv->darea);
 
     if (zoom_to_fit)
-        _moo_message ("zoom_to_fit");
+        moo_dmsg ("zoom_to_fit");
     else
-        _moo_message ("zoom: %s", zoom_factor_names[zoom]);
+        moo_dmsg ("zoom: %s", zoom_factor_names[zoom]);
 }
 
 static void
@@ -393,9 +399,9 @@ moo_print_preview_zoom_in (MooPrintPreview *preview,
     g_return_if_fail (change == -1 || change == 1);
 
     if (change < 0)
-        _moo_message ("zoom out");
+        moo_dmsg ("zoom out");
     else
-        _moo_message ("zoom in");
+        moo_dmsg ("zoom in");
 }
 
 
@@ -583,7 +589,7 @@ _moo_print_preview_start (MooPrintPreview *preview)
                               G_CALLBACK (size_allocate), preview);
 
     entry_item = moo_glade_xml_get_widget (preview->priv->xml, "entry_item");
-    xml = moo_glade_xml_new_from_buf (MOO_PRINT_PREVIEW_GLADE_XML, -1,
+    xml = moo_glade_xml_new_from_buf (mooprintpreview_glade_xml, -1,
                                       "entry_hbox", GETTEXT_PACKAGE, NULL);
     box = moo_glade_xml_get_widget (xml, "entry_hbox");
     gtk_container_add (GTK_CONTAINER (entry_item), box);
@@ -610,7 +616,7 @@ _moo_print_preview_start (MooPrintPreview *preview)
     preview->priv->zoom_to_fit = TRUE;
     moo_print_preview_set_zoom (preview, FALSE, ZOOM_100);
 
-    text = g_strdup_printf ("of %d", preview->priv->n_pages);
+    text = g_strdup_printf ("of %u", preview->priv->n_pages);
     gtk_label_set_text (moo_glade_xml_get_widget (xml, "label"), text);
     g_free (text);
 
