@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -136,7 +135,7 @@ static guint signals[LAST_SIGNAL];
 
 enum {
     PROP_0,
-    PROP_HIGHLIGHT, /* mimic GtkSourceBuffer */
+    PROP_HIGHLIGHT_SYNTAX, /* mimic GtkSourceBuffer */
     PROP_HIGHLIGHT_MATCHING_BRACKETS,
     PROP_HIGHLIGHT_MISMATCHING_BRACKETS,
     PROP_BRACKET_MATCH_STYLE,
@@ -175,10 +174,10 @@ moo_text_buffer_class_init (MooTextBufferClass *klass)
     g_type_class_add_private (klass, sizeof (MooTextBufferPrivate));
 
     g_object_class_install_property (gobject_class,
-                                     PROP_HIGHLIGHT,
-                                     g_param_spec_boolean ("highlight",
-                                             "highlight",
-                                             "highlight",
+                                     PROP_HIGHLIGHT_SYNTAX,
+                                     g_param_spec_boolean ("highlight-syntax",
+                                             "highlight-syntax",
+                                             "highlight-syntax",
                                              TRUE,
                                              G_PARAM_READWRITE));
 
@@ -200,7 +199,7 @@ moo_text_buffer_class_init (MooTextBufferClass *klass)
 
     g_object_class_install_property (gobject_class,
                                      PROP_BRACKET_MATCH_STYLE,
-                                     g_param_spec_boxed ("bracket-match-style",
+                                     g_param_spec_object ("bracket-match-style",
                                              "bracket-match-style",
                                              "bracket-match-style",
                                              MOO_TYPE_TEXT_STYLE,
@@ -208,7 +207,7 @@ moo_text_buffer_class_init (MooTextBufferClass *klass)
 
     g_object_class_install_property (gobject_class,
                                      PROP_BRACKET_MISMATCH_STYLE,
-                                     g_param_spec_boxed ("bracket-mismatch-style",
+                                     g_param_spec_object ("bracket-mismatch-style",
                                              "bracket-mismatch-style",
                                              "bracket-mismatch-style",
                                              MOO_TYPE_TEXT_STYLE,
@@ -239,14 +238,14 @@ moo_text_buffer_class_init (MooTextBufferClass *klass)
                                              G_PARAM_READWRITE));
 
     signals[HIGHLIGHT_UPDATED] =
-            _moo_signal_new_cb ("highlight_updated",
-                                G_OBJECT_CLASS_TYPE (klass),
-                                G_SIGNAL_RUN_LAST,
-                                NULL, NULL, NULL,
-                                _moo_marshal_VOID__BOXED_BOXED,
-                                G_TYPE_NONE, 2,
-                                GTK_TYPE_TEXT_ITER | G_SIGNAL_TYPE_STATIC_SCOPE,
-                                GTK_TYPE_TEXT_ITER | G_SIGNAL_TYPE_STATIC_SCOPE);
+            g_signal_new ("highlight_updated",
+                          G_OBJECT_CLASS_TYPE (klass),
+                          G_SIGNAL_RUN_LAST,
+                          0, NULL, NULL,
+                          _moo_marshal_VOID__BOXED_BOXED,
+                          G_TYPE_NONE, 2,
+                          GTK_TYPE_TEXT_ITER | G_SIGNAL_TYPE_STATIC_SCOPE,
+                          GTK_TYPE_TEXT_ITER | G_SIGNAL_TYPE_STATIC_SCOPE);
 
     signals[CURSOR_MOVED] =
             g_signal_new ("cursor-moved",
@@ -808,8 +807,8 @@ moo_text_buffer_get_lang (MooTextBuffer  *buffer)
 
 
 void
-moo_text_buffer_set_highlight (MooTextBuffer      *buffer,
-                               gboolean            highlight)
+moo_text_buffer_set_highlight (MooTextBuffer *buffer,
+                               gboolean       highlight)
 {
     g_return_if_fail (MOO_IS_TEXT_BUFFER (buffer));
 
@@ -819,7 +818,7 @@ moo_text_buffer_set_highlight (MooTextBuffer      *buffer,
         return;
 
     buffer->priv->do_highlight = highlight;
-    g_object_notify (G_OBJECT (buffer), "highlight");
+    g_object_notify (G_OBJECT (buffer), "highlight-syntax");
 }
 
 
@@ -841,7 +840,7 @@ moo_text_buffer_set_property (GObject        *object,
 
     switch (prop_id)
     {
-        case PROP_HIGHLIGHT:
+        case PROP_HIGHLIGHT_SYNTAX:
             moo_text_buffer_set_highlight (buffer, g_value_get_boolean (value));
             break;
 
@@ -856,11 +855,11 @@ moo_text_buffer_set_property (GObject        *object,
             break;
 
         case PROP_BRACKET_MATCH_STYLE:
-            moo_text_buffer_set_bracket_match_style (buffer, g_value_get_boxed (value));
+            moo_text_buffer_set_bracket_match_style (buffer, g_value_get_object (value));
             break;
 
         case PROP_BRACKET_MISMATCH_STYLE:
-            moo_text_buffer_set_bracket_mismatch_style (buffer, g_value_get_boxed (value));
+            moo_text_buffer_set_bracket_mismatch_style (buffer, g_value_get_object (value));
             break;
 
         case PROP_LANG:
@@ -884,7 +883,7 @@ moo_text_buffer_get_property (GObject        *object,
 
     switch (prop_id)
     {
-        case PROP_HIGHLIGHT:
+        case PROP_HIGHLIGHT_SYNTAX:
             g_value_set_boolean (value, buffer->priv->do_highlight);
             break;
 
@@ -982,11 +981,9 @@ _moo_text_buffer_set_style_scheme (MooTextBuffer      *buffer,
 
     style = _moo_text_style_scheme_get_bracket_match_style (scheme);
     moo_text_buffer_set_bracket_match_style (buffer, style);
-    moo_text_style_free (style);
 
     style = _moo_text_style_scheme_get_bracket_mismatch_style (scheme);
     moo_text_buffer_set_bracket_mismatch_style (buffer, style);
-    moo_text_style_free (style);
 
     if (buffer->priv->engine)
         _gtk_source_engine_set_style_scheme (buffer->priv->engine,
@@ -1243,13 +1240,8 @@ static void
 moo_text_buffer_set_bracket_match_style (MooTextBuffer      *buffer,
                                          const MooTextStyle *style)
 {
-    static MooTextStyle empty;
-
     g_return_if_fail (MOO_IS_TEXT_BUFFER (buffer));
 
-    if (!style)
-        style = &empty;
-
     if (!buffer->priv->correct_match_tag)
         buffer->priv->correct_match_tag =
                 gtk_text_buffer_create_tag (GTK_TEXT_BUFFER (buffer), NULL, NULL);
@@ -1263,13 +1255,8 @@ static void
 moo_text_buffer_set_bracket_mismatch_style (MooTextBuffer      *buffer,
                                             const MooTextStyle *style)
 {
-    static MooTextStyle empty;
-
     g_return_if_fail (MOO_IS_TEXT_BUFFER (buffer));
 
-    if (!style)
-        style = &empty;
-
     if (!buffer->priv->incorrect_match_tag)
         buffer->priv->incorrect_match_tag =
                 gtk_text_buffer_create_tag (GTK_TEXT_BUFFER (buffer), NULL, NULL);
@@ -1575,21 +1562,19 @@ action_new (ActionType     type,
 {
     EditAction *action;
     gsize size = 0;
-    MooUndoActionClass *klass = NULL;
     GtkTextBuffer *text_buffer = GTK_TEXT_BUFFER (buffer);
 
     switch (type)
     {
         case ACTION_INSERT:
             size = sizeof (InsertAction);
-            klass = &InsertActionClass;
             break;
         case ACTION_DELETE:
             size = sizeof (DeleteAction);
-            klass = &DeleteActionClass;
             break;
     }
 
+    g_assert (size != 0);
     action = g_malloc0 (size);
     action->interactive = (!buffer->priv->non_interactive && text_buffer->user_action_count) ? TRUE : FALSE;
 
