@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -21,49 +20,50 @@
 #include "mooedit/mooedit-enums.h"
 #include "mooutils/moohistorycombo.h"
 #include "mooutils/mooentry.h"
-#include "mooutils/moocompat.h"
 #include "mooutils/moodialogs.h"
 #include "mooutils/mooi18n.h"
+#include "mooutils/moohelp.h"
+#include "help-sections.h"
 #include <gtk/gtk.h>
-#include <glib/gregex.h>
 #include <glib/gprintf.h>
 
 
 #define REGEX_FREE(re) G_STMT_START {   \
     if (re)                             \
-        g_regex_unref (re);             \
+        _moo_regex_unref (re);          \
     (re) = NULL;                        \
 } G_STMT_END
 
 static char *last_search;
-static GRegex *last_regex;
+static MooRegex *last_regex;
 static MooFindFlags last_search_flags;
 static MooHistoryList *search_history;
 static MooHistoryList *replace_history;
+static gboolean last_replace_empty;
 
 
-static void     init_find_history       (void);
+static void         init_find_history       (void);
 
-static GObject *moo_find_constructor    (GType           type,
-                                         guint           n_props,
-                                         GObjectConstructParam *props);
-static void     moo_find_set_property   (GObject        *object,
-                                         guint           prop_id,
-                                         const GValue   *value,
-                                         GParamSpec     *pspec);
-static void     moo_find_get_property   (GObject        *object,
-                                         guint           prop_id,
-                                         GValue         *value,
-                                         GParamSpec     *pspec);
-static void     moo_find_finalize       (GObject        *object);
+static GObject     *moo_find_constructor    (GType           type,
+                                             guint           n_props,
+                                             GObjectConstructParam *props);
+static void         moo_find_set_property   (GObject        *object,
+                                             guint           prop_id,
+                                             const GValue   *value,
+                                             GParamSpec     *pspec);
+static void         moo_find_get_property   (GObject        *object,
+                                             guint           prop_id,
+                                             GValue         *value,
+                                             GParamSpec     *pspec);
+static void      moo_find_finalize          (GObject        *object);
 
-static void     moo_find_set_flags      (MooFind        *find,
-                                         MooFindFlags    flags);
-static MooFindFlags moo_find_get_flags  (MooFind        *find);
+static void      moo_find_set_flags         (MooFind        *find,
+                                             MooFindFlags    flags);
+static MooFindFlags moo_find_get_flags      (MooFind        *find);
 /* returned string/regex must be freed/unrefed */
-static char    *moo_find_get_text       (MooFind        *find);
-static GRegex  *moo_find_get_regex      (MooFind        *find);
-static char    *moo_find_get_replacement (MooFind       *find);
+static char     *moo_find_get_text          (MooFind        *find);
+static MooRegex *moo_find_get_regex         (MooFind        *find);
+static char     *moo_find_get_replacement   (MooFind        *find);
 
 
 G_DEFINE_TYPE(MooFind, moo_find, GTK_TYPE_DIALOG)
@@ -84,9 +84,9 @@ init_find_history (void)
     {
         been_here = TRUE;
 
-        search_history = moo_history_list_new ("MooFind");
+        search_history = moo_history_list_get ("MooFind");
         moo_history_list_set_max_entries (search_history, 20);
-        replace_history = moo_history_list_new ("MooReplace");
+        replace_history = moo_history_list_get ("MooReplace");
         moo_history_list_set_max_entries (replace_history, 20);
         last_search = moo_history_list_get_last_item (search_history);
 
@@ -129,7 +129,7 @@ moo_find_init (MooFind *find)
     moo_glade_xml_map_id (find->xml, "search_entry", MOO_TYPE_HISTORY_COMBO);
     moo_glade_xml_map_id (find->xml, "replace_entry", MOO_TYPE_HISTORY_COMBO);
 
-    if (!moo_glade_xml_parse_memory (find->xml, MOO_TEXT_FIND_GLADE_UI, -1, "vbox", NULL))
+    if (!moo_glade_xml_parse_memory (find->xml, mootextfind_glade_xml, -1, "vbox", NULL))
     {
         g_object_unref (find->xml);
         find->xml = NULL;
@@ -189,10 +189,16 @@ moo_find_constructor (GType           type,
                             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                             stock_id, GTK_RESPONSE_OK,
                             NULL);
-    gtk_dialog_set_alternative_button_order (GTK_DIALOG (find), GTK_RESPONSE_OK,
-                                             GTK_STOCK_CANCEL, -1);
+    gtk_dialog_set_alternative_button_order (GTK_DIALOG (find),
+                                             GTK_RESPONSE_OK,
+                                             GTK_RESPONSE_CANCEL,
+                                             -1);
     gtk_dialog_set_default_response (GTK_DIALOG (find), GTK_RESPONSE_OK);
 
+    moo_help_set_id (GTK_WIDGET (find), find->replace ?
+                                HELP_SECTION_DIALOG_REPLACE : HELP_SECTION_DIALOG_FIND);
+    moo_help_connect_keys (GTK_WIDGET (find));
+
     return object;
 }
 
@@ -392,12 +398,16 @@ moo_find_setup (MooFind        *find,
 
     if (find->replace)
     {
-        char *replace_with = moo_history_list_get_last_item (replace_history);
+        const char *replace_with = "";
+        char *freeme = NULL;
+
+        if (!last_replace_empty)
+            replace_with = freeme = moo_history_list_get_last_item (replace_history);
 
         if (replace_with)
             gtk_entry_set_text (GTK_ENTRY (replace_entry->entry), replace_with);
 
-        g_free (replace_with);
+        g_free (freeme);
     }
 
     has_selection = gtk_text_buffer_get_selection_bounds (buffer, &sel_start, &sel_end);
@@ -484,14 +494,14 @@ moo_find_run (MooFind        *find,
 
         if (flags & MOO_FIND_REGEX)
         {
-            GRegex *regex;
+            MooRegex *regex;
             GRegexCompileFlags re_flags = 0;
             GError *error = NULL;
 
             if (flags & MOO_FIND_CASELESS)
                 re_flags |= G_REGEX_CASELESS;
 
-            regex = g_regex_new (search_for, re_flags | G_REGEX_OPTIMIZE, 0, &error);
+            regex = _moo_regex_compile (search_for, re_flags | G_REGEX_OPTIMIZE, 0, &error);
 
             if (!regex)
             {
@@ -521,12 +531,16 @@ moo_find_run (MooFind        *find,
         g_free (last_search);
         last_search = g_strdup (search_for);
         REGEX_FREE (last_regex);
-        last_regex = find->regex ? g_regex_ref (find->regex) : NULL;
+        last_regex = find->regex ? _moo_regex_ref (find->regex) : NULL;
 
         moo_history_list_add (search_history, search_for);
 
-        if (find->replace && replace_with[0])
-            moo_history_list_add (replace_history, replace_with);
+        if (find->replace)
+        {
+            if (replace_with[0])
+                moo_history_list_add (replace_history, replace_with);
+            last_replace_empty = !replace_with[0];
+        }
 
         return TRUE;
     }
@@ -601,11 +615,11 @@ moo_find_get_text (MooFind *find)
 }
 
 
-static GRegex *
+static MooRegex *
 moo_find_get_regex (MooFind *find)
 {
     g_return_val_if_fail (MOO_IS_FIND (find), NULL);
-    return find->regex ? g_regex_ref (find->regex) : NULL;
+    return find->regex ? _moo_regex_ref (find->regex) : NULL;
 }
 
 
@@ -623,7 +637,7 @@ static gboolean
 do_find (const GtkTextIter *start,
          const GtkTextIter *end,
          MooFindFlags       flags,
-         GRegex            *regex,
+         MooRegex          *regex,
          const char        *text,
          GtkTextIter       *match_start,
          GtkTextIter       *match_end)
@@ -631,11 +645,13 @@ do_find (const GtkTextIter *start,
     if (regex)
     {
         if (flags & MOO_FIND_BACKWARDS)
-            return _moo_text_search_regex_backward (start, end, regex, match_start,
-                                                    match_end, NULL, NULL, NULL, NULL);
+            return _moo_text_search_regex_backward (start, end, regex,
+                                                    match_start, match_end,
+                                                    NULL, NULL, NULL, NULL);
         else
-            return _moo_text_search_regex_forward (start, end, regex, match_start,
-                                                   match_end, NULL, NULL, NULL, NULL);
+            return _moo_text_search_regex_forward (start, end, regex,
+                                                   match_start, match_end,
+                                                   NULL, NULL, NULL, NULL);
     }
     else
     {
@@ -733,7 +749,7 @@ moo_text_view_run_find (GtkTextView    *view,
     GtkWidget *find;
     MooFindFlags flags;
     char *text;
-    GRegex *regex;
+    MooRegex *regex;
     GtkTextIter start, end;
     GtkTextIter match_start, match_end;
     GtkTextBuffer *buffer;
@@ -1056,7 +1072,7 @@ do_replace_silent (GtkTextIter       *start,
                    GtkTextIter       *end,
                    MooFindFlags       flags,
                    const char        *text,
-                   GRegex            *regex,
+                   MooRegex          *regex,
                    const char        *replacement)
 {
     MooTextSearchFlags search_flags = 0;
@@ -1098,7 +1114,7 @@ replaced_n_message (guint          n,
 static void
 run_replace_silent (GtkTextView   *view,
                     const char    *text,
-                    GRegex        *regex,
+                    MooRegex      *regex,
                     const char    *replacement,
                     MooFindFlags   flags,
                     MooFindMsgFunc msg_func,
@@ -1128,7 +1144,7 @@ run_replace_silent (GtkTextView   *view,
 
 static MooTextReplaceResponse
 replace_func (G_GNUC_UNUSED const char *text,
-              G_GNUC_UNUSED GRegex *regex,
+              G_GNUC_UNUSED MooRegex *regex,
               G_GNUC_UNUSED const char *replacement,
               const GtkTextIter  *to_replace_start,
               const GtkTextIter  *to_replace_end,
@@ -1175,7 +1191,7 @@ do_replace_interactive (GtkTextView       *view,
                         GtkTextIter       *end,
                         MooFindFlags       flags,
                         const char        *text,
-                        GRegex            *regex,
+                        MooRegex          *regex,
                         const char        *replacement,
                         int               *replaced)
 {
@@ -1214,7 +1230,7 @@ do_replace_interactive (GtkTextView       *view,
 static void
 run_replace_interactive (GtkTextView   *view,
                          const char    *text,
-                         GRegex        *regex,
+                         MooRegex      *regex,
                          const char    *replacement,
                          MooFindFlags   flags,
                          MooFindMsgFunc msg_func,
@@ -1254,7 +1270,7 @@ moo_text_view_run_replace (GtkTextView    *view,
     GtkWidget *find;
     MooFindFlags flags;
     char *text, *replacement;
-    GRegex *regex;
+    MooRegex *regex;
 
     g_return_if_fail (GTK_IS_TEXT_VIEW (view));
 
@@ -1337,7 +1353,7 @@ moo_text_view_run_goto_line (GtkTextView *view)
     buffer = gtk_text_view_get_buffer (view);
     line_count = gtk_text_buffer_get_line_count (buffer);
 
-    xml = moo_glade_xml_new_from_buf (MOO_TEXT_GOTO_LINE_GLADE_UI, -1,
+    xml = moo_glade_xml_new_from_buf (mootextgotoline_glade_xml, -1,
                                       NULL, GETTEXT_PACKAGE, NULL);
     g_return_if_fail (xml != NULL);
 
