@@ -1,12 +1,11 @@
 /*
  *   mootextpopup.c
  *
- *   Copyright (C) 2004-2006 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -256,9 +255,7 @@ moo_text_popup_ensure_popup (MooTextPopup *popup)
         gtk_tree_view_set_headers_visible (popup->priv->treeview, FALSE);
         gtk_tree_view_append_column (popup->priv->treeview,
                                      popup->priv->column);
-#if GTK_CHECK_VERSION(2,6,0)
         gtk_tree_view_set_hover_selection (popup->priv->treeview, TRUE);
-#endif
 
         gtk_widget_show_all (frame);
 
