@@ -1,12 +1,11 @@
 /*
  *   mootextprint.c
  *
- *   Copyright (C) 2004-2006 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -23,6 +22,7 @@
 #include "mooutils/moodialogs.h"
 #include "mooutils/mooi18n.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/mooutils-debug.h"
 #include <sys/types.h>
 #include <time.h>
 #include <errno.h>
@@ -33,6 +33,8 @@
 #include <cairo-win32.h>
 #endif
 
+MOO_DEBUG_INIT(printing, TRUE)
+
 #define SEPARATOR_POINTS 1.
 
 #define PREFS_FONT                      MOO_EDIT_PREFS_PREFIX "/print/font"
@@ -82,12 +84,12 @@ struct _MooPrintOperationPrivate {
     gpointer tm; /* struct tm * */
 
     /* aux stuff */
-    GArray *pages;          /* GtkTextIter's pointing to pages start */
+    GArray *pages;          /* offsets */
     PangoLayout *layout;
     double ln_height;
     PangoLayout *ln_layout;
 
-    Page page;                 /* text area */
+    Page page;              /* text area */
 };
 
 
@@ -343,7 +345,7 @@ moo_print_operation_finalize (GObject *object)
     g_free (op->priv->tm);
     g_free (op->priv);
 
-    _moo_message ("moo_print_operation_finalize");
+    moo_dmsg ("moo_print_operation_finalize");
 
     G_OBJECT_CLASS(_moo_print_operation_parent_class)->finalize (object);
 }
@@ -454,7 +456,7 @@ _moo_print_operation_init (MooPrintOperation *op)
 {
     op->priv = g_new0 (MooPrintOperationPrivate, 1);
 
-    _moo_message ("_moo_print_operation_init");
+    moo_dmsg ("_moo_print_operation_init");
 
     gtk_print_operation_set_print_settings (GTK_PRINT_OPERATION (op),
                                             global_print_settings);
@@ -536,7 +538,7 @@ _moo_edit_page_setup (GtkWidget *parent)
 
     g_return_if_fail (!parent || GTK_IS_WIDGET (parent));
 
-    _moo_message ("_moo_edit_page_setup");
+    moo_dmsg ("_moo_edit_page_setup");
 
     if (parent)
         parent = gtk_widget_get_toplevel (parent);
@@ -694,10 +696,10 @@ moo_print_operation_calc_page_size (MooPrintOperation    *op,
     page->text_x = page->x;
     page->text_width = page->width;
 
-    _moo_message ("page size: %f, %f - %f in, %f in",
-                  page->width, page->height,
-                  page->width / gtk_print_context_get_dpi_x (context),
-                  page->height / gtk_print_context_get_dpi_y (context));
+    moo_dmsg ("page size: %f, %f - %f in, %f in",
+              page->width, page->height,
+              page->width / gtk_print_context_get_dpi_x (context),
+              page->height / gtk_print_context_get_dpi_y (context));
 
     header_footer_get_size (settings->header, op, context,
                             default_font, MOO_PRINT_HEADER);
@@ -726,10 +728,10 @@ moo_print_operation_calc_page_size (MooPrintOperation    *op,
         page->height = gtk_print_context_get_height (context);
     }
 
-    _moo_message ("printable area size: %f, %f - %f in, %f in",
-                  page->width, page->height,
-                  page->width / gtk_print_context_get_dpi_x (context),
-                  page->height / gtk_print_context_get_dpi_y (context));
+    moo_dmsg ("printable area size: %f, %f - %f in, %f in",
+              page->width, page->height,
+              page->width / gtk_print_context_get_dpi_x (context),
+              page->height / gtk_print_context_get_dpi_y (context));
 
     if (GET_OPTION (op, MOO_PRINT_LINE_NUMBERS))
         create_ln_layout (op, context, default_font);
@@ -766,20 +768,22 @@ moo_print_operation_paginate (MooPrintOperation *op)
     double page_height;
     gboolean use_styles;
     int line_no;
+    int offset;
 
-    _moo_message ("moo_print_operation_paginate");
-    _moo_message ("page height: %f", op->priv->page.height);
+    moo_dmsg ("moo_print_operation_paginate");
+    moo_dmsg ("page height: %f", op->priv->page.height);
 
     if (op->priv->pages)
         g_array_free (op->priv->pages, TRUE);
 
-    op->priv->pages = g_array_new (FALSE, FALSE, sizeof (GtkTextIter));
+    op->priv->pages = g_array_new (FALSE, FALSE, sizeof (int));
     gtk_text_buffer_get_iter_at_line (op->priv->buffer, &iter,
                                       op->priv->first_line);
     gtk_text_buffer_get_iter_at_line (op->priv->buffer, &print_end,
                                       op->priv->last_line);
     gtk_text_iter_forward_line (&print_end);
-    g_array_append_val (op->priv->pages, iter);
+    offset = gtk_text_iter_get_offset (&iter);
+    g_array_append_val (op->priv->pages, offset);
     page_height = 0;
     line_no = op->priv->first_line;
 
@@ -851,7 +855,8 @@ moo_print_operation_paginate (MooPrintOperation *op)
                 pango_layout_iter_free (layout_iter);
             }
 
-            g_array_append_val (op->priv->pages, iter);
+            offset = gtk_text_iter_get_offset (&iter);
+            g_array_append_val (op->priv->pages, offset);
             page_height = line_height;
 
             if (!part)
@@ -867,10 +872,43 @@ moo_print_operation_paginate (MooPrintOperation *op)
 
     gtk_print_operation_set_n_pages (GTK_PRINT_OPERATION (op), op->priv->pages->len);
 
-    _moo_message ("moo_print_operation_paginate done");
+    moo_dmsg ("moo_print_operation_paginate done");
 }
 
 
+static void
+set_tabs (MooPrintOperation *op,
+          PangoLayout       *layout)
+{
+    PangoTabArray *tabs;
+    int tab_width;
+    char *string = NULL;
+
+    if (MOO_IS_TEXT_VIEW (op->priv->doc))
+    {
+        guint n_spaces;
+
+        g_object_get (op->priv->doc, "tab-width", &n_spaces, NULL);
+
+        if (n_spaces != 8)
+            string = g_strnfill (n_spaces, ' ');
+    }
+
+    if (!string)
+        return;
+
+    pango_layout_set_text (layout, string, -1);
+    pango_layout_get_size (layout, &tab_width, NULL);
+
+    tabs = pango_tab_array_new (2, FALSE);
+    pango_tab_array_set_tab (tabs, 0, PANGO_TAB_LEFT, 0);
+    pango_tab_array_set_tab (tabs, 1, PANGO_TAB_LEFT, tab_width);
+    pango_layout_set_tabs (layout, tabs);
+
+    pango_tab_array_free (tabs);
+    g_free (string);
+}
+
 static void
 moo_print_operation_begin_print (GtkPrintOperation *operation,
                                  GtkPrintContext   *context)
@@ -887,7 +925,7 @@ moo_print_operation_begin_print (GtkPrintOperation *operation,
     g_return_if_fail (op->priv->first_line < gtk_text_buffer_get_line_count (op->priv->buffer));
     g_return_if_fail (op->priv->last_line < gtk_text_buffer_get_line_count (op->priv->buffer));
 
-    _moo_message ("moo_print_operation_begin_print");
+    moo_dmsg ("moo_print_operation_begin_print");
 
     moo_print_operation_load_prefs (op);
     settings = op->priv->settings;
@@ -953,10 +991,12 @@ moo_print_operation_begin_print (GtkPrintOperation *operation,
         pango_font_description_free (font);
     }
 
+    set_tabs (op, op->priv->layout);
+
     moo_print_operation_paginate (op);
 
-    _moo_message ("begin_print: %d pages in %f s", op->priv->pages->len,
-                  g_timer_elapsed (timer, NULL));
+    moo_dmsg ("begin_print: %u pages in %f s", op->priv->pages->len,
+              g_timer_elapsed (timer, NULL));
     g_timer_destroy (timer);
 
     if (!op->priv->tm)
@@ -983,7 +1023,7 @@ moo_print_operation_begin_print (GtkPrintOperation *operation,
     if (op->priv->preview)
         _moo_print_preview_start (op->priv->preview);
 
-    _moo_message ("moo_print_operation_begin_print done");
+    moo_dmsg ("moo_print_operation_begin_print done");
 }
 
 
@@ -1149,7 +1189,7 @@ fill_layout (MooPrintOperation *op,
     segm_start = *start;
     start_index = gtk_text_iter_get_line_index (start);
 #if 0
-    _moo_message ("line %d, start at %d\n", gtk_text_iter_get_line (start), start_index);
+    moo_dmsg ("line %d, start at %d\n", gtk_text_iter_get_line (start), start_index);
 #endif
 
     while (gtk_text_iter_compare (&segm_start, end) < 0)
@@ -1320,7 +1360,7 @@ print_page (MooPrintOperation *op,
     double offset;
     int line_no;
 
-    _moo_message ("print_page %d", page);
+    moo_dmsg ("print_page %d", page);
 
     cairo_set_source_rgb (cr, 0., 0., 0.);
 
@@ -1379,7 +1419,7 @@ print_page (MooPrintOperation *op,
         line_no += 1;
     }
 
-    _moo_message ("print_page done");
+    moo_dmsg ("print_page done");
 }
 
 
@@ -1389,6 +1429,7 @@ moo_print_operation_draw_page (GtkPrintOperation *operation,
                                int                page)
 {
     cairo_t *cr;
+    int offset;
     GtkTextIter start, end;
     MooPrintOperation *op = MOO_PRINT_OPERATION (operation);
     GTimer *timer;
@@ -1402,14 +1443,20 @@ moo_print_operation_draw_page (GtkPrintOperation *operation,
 
     cr = gtk_print_context_get_cairo_context (context);
 
-    start = g_array_index (op->priv->pages, GtkTextIter, page);
+    offset = g_array_index (op->priv->pages, int, page);
+    gtk_text_buffer_get_iter_at_offset (op->priv->buffer, &start, offset);
 
     if (page + 1 < (int) op->priv->pages->len)
-        end = g_array_index (op->priv->pages, GtkTextIter, page + 1);
+    {
+        offset = g_array_index (op->priv->pages, int, page + 1);
+        gtk_text_buffer_get_iter_at_offset (op->priv->buffer, &end, offset);
+    }
     else
+    {
         gtk_text_buffer_get_end_iter (op->priv->buffer, &end);
+    }
 
-#if defined(__WIN32__) && 0
+#if 0 && defined(__WIN32__)
     if (page == 0)
     {
         HDC dc = cairo_win32_surface_get_dc (cairo_get_target (cr));
@@ -1423,7 +1470,7 @@ moo_print_operation_draw_page (GtkPrintOperation *operation,
     }
 #endif
 
-#if defined(MOO_DEBUG) && !defined(__WIN32__)
+#if 0 && defined(MOO_DEBUG_ENABLED) && !defined(__WIN32__)
     cairo_save (cr);
     cairo_set_line_width (cr, 1.);
     cairo_set_source_rgb (cr, 1., 0., 0.);
@@ -1450,7 +1497,7 @@ moo_print_operation_draw_page (GtkPrintOperation *operation,
         pango_cairo_update_layout (cr, op->priv->settings->footer->layout);
     print_page (op, &start, &end, page, cr);
 
-    _moo_message ("page %d: %f s", page, g_timer_elapsed (timer, NULL));
+    moo_dmsg ("page %d: %f s", page, g_timer_elapsed (timer, NULL));
     g_timer_destroy (timer);
 }
 
@@ -1463,7 +1510,7 @@ moo_print_operation_end_print (GtkPrintOperation  *operation,
 
     g_return_if_fail (op->priv->buffer != NULL);
 
-    _moo_message ("moo_print_operation_end_print");
+    moo_dmsg ("moo_print_operation_end_print");
 
     if (MOO_IS_EDIT (op->priv->doc))
         _moo_edit_set_state (MOO_EDIT (op->priv->doc),
@@ -1533,17 +1580,9 @@ moo_print_operation_set_filename (MooPrintOperation *op,
                                   const char        *filename,
                                   const char        *basename)
 {
-    char *tmp;
-
     g_return_if_fail (MOO_IS_PRINT_OPERATION (op));
-
-    tmp = op->priv->filename;
-    op->priv->filename = g_strdup (filename);
-    g_free (tmp);
-
-    tmp = op->priv->basename;
-    op->priv->basename = g_strdup (basename);
-    g_free (tmp);
+    MOO_ASSIGN_STRING (op->priv->filename, filename);
+    MOO_ASSIGN_STRING (op->priv->basename, basename);
 }
 
 
@@ -1597,7 +1636,7 @@ static void
 do_print_operation (GtkTextView            *view,
                     GtkWidget              *parent,
                     GtkPrintOperationAction action,
-                    const char             *filename)
+                    const char             *exp_filename)
 {
     MooPrintOperation *op;
     GtkPrintOperationResult res;
@@ -1605,11 +1644,11 @@ do_print_operation (GtkTextView            *view,
     GtkWidget *error_dialog;
 
     g_return_if_fail (GTK_IS_TEXT_VIEW (view));
-    g_return_if_fail (!filename || action == GTK_PRINT_OPERATION_ACTION_EXPORT);
+    g_return_if_fail (!exp_filename || action == GTK_PRINT_OPERATION_ACTION_EXPORT);
 
     op = g_object_new (MOO_TYPE_PRINT_OPERATION,
                        "doc", view,
-                       "export-filename", filename,
+                       "export-filename", exp_filename,
                        NULL);
 
     if (!parent)
@@ -1622,9 +1661,12 @@ do_print_operation (GtkTextView            *view,
         op->priv->parent = GTK_WINDOW (parent);
 
     if (MOO_IS_EDIT (view))
-        moo_print_operation_set_filename (op,
-                                          moo_edit_get_display_filename (MOO_EDIT (view)),
-                                          moo_edit_get_display_basename (MOO_EDIT (view)));
+    {
+        const char *filename, *basename;
+        filename = moo_edit_get_display_name (MOO_EDIT (view));
+        basename = moo_edit_get_display_basename (MOO_EDIT (view));
+        moo_print_operation_set_filename (op, filename, basename);
+    }
 
     res = gtk_print_operation_run (GTK_PRINT_OPERATION (op),
                                    action, op->priv->parent, &error);
@@ -1655,6 +1697,12 @@ do_print_operation (GtkTextView            *view,
             break;
     }
 
+#if !GTK_CHECK_VERSION(2,11,6)
+    /* older gtk crashes in unref() */
+    if (action == GTK_PRINT_OPERATION_ACTION_EXPORT)
+        g_object_ref (op);
+#endif
+
     g_object_unref (op);
 }
 
@@ -1870,7 +1918,7 @@ moo_print_operation_create_custom_widget (G_GNUC_UNUSED GtkPrintOperation *opera
     GtkWidget *page, *font_button, *line_numbers_hbox;
     MooGladeXML *xml;
 
-    xml = moo_glade_xml_new_from_buf (MOO_PRINT_GLADE_XML, -1, "page", GETTEXT_PACKAGE, NULL);
+    xml = moo_glade_xml_new_from_buf (mooprint_glade_xml, -1, "page", GETTEXT_PACKAGE, NULL);
     g_return_val_if_fail (xml != NULL, NULL);
 
     font_button = moo_glade_xml_get_widget (xml, "font");
@@ -1933,13 +1981,13 @@ moo_print_operation_preview (GtkPrintOperation        *op,
 {
     GtkWidget *dialog;
 
-    _moo_message ("moo_print_operation_preview");
+    moo_dmsg ("moo_print_operation_preview");
 
     dialog = _moo_print_preview_new (MOO_PRINT_OPERATION (op), preview, context);
     gtk_widget_show (dialog);
     g_signal_connect_swapped (dialog, "response", G_CALLBACK (preview_response), op);
 
-    _moo_message ("moo_print_operation_preview done");
+    moo_dmsg ("moo_print_operation_preview done");
 
     return TRUE;
 }
