@@ -1,26 +1,117 @@
 /*
  *   mootextsearch.c
  *
- *   Copyright (C) 2004-2006 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
 
 #include "mooedit/mootextsearch-private.h"
 #include "mooedit/gtksourceview/gtksourceview-api.h"
+#include "mooutils/mooutils-misc.h"
 #include <glib/gregex.h>
 #include <string.h>
 
 
+/* XXX lame */
+static int
+get_n_lines (GRegex *regex)
+{
+    const char *pattern;
+    gboolean found = FALSE;
+    guint bs = 0;
+
+    pattern = g_regex_get_pattern (regex);
+
+    while (*pattern && !found)
+    {
+        switch (*pattern++)
+        {
+            case '\\':
+                bs ^= 1;
+                break;
+            case 'n':
+            case 'r':
+            case 'R':
+                if (bs)
+                    found = TRUE;
+                else
+                    bs = 0;
+                break;
+            default:
+                bs = 0;
+                break;
+        }
+    }
+
+    return found ? 3 : 1;
+}
+
+MooRegex *
+_moo_regex_new (GRegex *re)
+{
+    MooRegex *regex;
+
+    g_return_val_if_fail (re != NULL, NULL);
+
+    regex = _moo_new (MooRegex);
+    regex->re = g_regex_ref (re);
+    regex->ref_count_ = 1;
+
+    regex->n_lines = get_n_lines (re);
+
+    return regex;
+}
+
+MooRegex *
+_moo_regex_compile (const char         *pattern,
+                    GRegexCompileFlags  compile_options,
+                    GRegexMatchFlags    match_options,
+                    GError            **error)
+{
+    GRegex *re;
+    MooRegex *regex = NULL;
+
+    g_return_val_if_fail (pattern != NULL, NULL);
+
+    if ((re = g_regex_new (pattern, compile_options, match_options, error)))
+    {
+        regex = _moo_regex_new (re);
+        g_regex_unref (re);
+    }
+
+    return regex;
+}
+
+MooRegex *
+_moo_regex_ref (MooRegex *regex)
+{
+    g_return_val_if_fail (regex != NULL, NULL);
+    regex->ref_count_++;
+    return regex;
+}
+
+void
+_moo_regex_unref (MooRegex *regex)
+{
+    g_return_if_fail (regex != NULL);
+
+    if (!--regex->ref_count_)
+    {
+        g_regex_unref (regex->re);
+        _moo_free (MooRegex, regex);
+    }
+}
+
+
 gboolean
 _moo_text_search_regex_forward (const GtkTextIter      *search_start,
                                 const GtkTextIter      *search_end,
-                                GRegex                 *regex,
+                                MooRegex               *regex,
                                 GtkTextIter            *match_start,
                                 GtkTextIter            *match_end,
                                 char                  **string,
@@ -45,6 +136,7 @@ _moo_text_search_regex_forward (const GtkTextIter      *search_start,
         gtk_text_iter_set_line_offset (&start, 0);
 
     end = *search_start;
+    gtk_text_iter_forward_lines (&end, regex->n_lines - 1);
     if (!gtk_text_iter_ends_line (&end))
         gtk_text_iter_forward_to_line_end (&end);
 
@@ -55,7 +147,7 @@ _moo_text_search_regex_forward (const GtkTextIter      *search_start,
         text = gtk_text_buffer_get_slice (buffer, &start, &end, TRUE);
         text_start = g_utf8_offset_to_pointer (text, start_offset);
 
-        if (g_regex_match_full (regex, text, -1, text_start - text, 0, &match_info, NULL))
+        if (g_regex_match_full (regex->re, text, -1, text_start - text, 0, &match_info, NULL))
         {
             int start_pos, end_pos;
 
@@ -93,6 +185,7 @@ _moo_text_search_regex_forward (const GtkTextIter      *search_start,
         }
 
         g_match_info_free (match_info);
+        g_free (text);
 
         start = end;
         start_offset = 0;
@@ -104,7 +197,7 @@ _moo_text_search_regex_forward (const GtkTextIter      *search_start,
             break;
 
         end = start;
-
+        gtk_text_iter_forward_lines (&end, regex->n_lines - 1);
         if (!gtk_text_iter_ends_line (&end))
             gtk_text_iter_forward_to_line_end (&end);
     }
@@ -128,7 +221,6 @@ find_last_match (GRegex            *regex,
     len = strlen (text);
     start = 0;
 
-    /* XXX a leak!!! */
     while (g_regex_match_full (regex, text, len, start, flags, &match_info, NULL))
     {
         g_match_info_fetch_pos (match_info, 0, start_pos, end_pos);
@@ -154,7 +246,7 @@ find_last_match (GRegex            *regex,
 gboolean
 _moo_text_search_regex_backward (const GtkTextIter      *search_start,
                                  const GtkTextIter      *search_end,
-                                 GRegex                 *regex,
+                                 MooRegex               *regex,
                                  GtkTextIter            *match_start,
                                  GtkTextIter            *match_end,
                                  char                  **string,
@@ -174,7 +266,7 @@ _moo_text_search_regex_backward (const GtkTextIter      *search_start,
     buffer = gtk_text_iter_get_buffer (search_start);
     slice_start = *search_start;
     slice_end = slice_start;
-    gtk_text_iter_backward_line (&slice_start);
+    gtk_text_iter_backward_lines (&slice_start, regex->n_lines);
     flags = 0;
 
     if (!gtk_text_iter_ends_line (&slice_end))
@@ -186,7 +278,7 @@ _moo_text_search_regex_backward (const GtkTextIter      *search_start,
 
         text = gtk_text_buffer_get_slice (buffer, &slice_start, &slice_end, TRUE);
 
-        if (find_last_match (regex, text, flags, &start_pos, &end_pos, match_info))
+        if (find_last_match (regex->re, text, flags, &start_pos, &end_pos, match_info))
         {
             *match_start = slice_start;
             gtk_text_iter_forward_chars (match_start, g_utf8_pointer_to_offset (text, text + start_pos));
@@ -220,6 +312,8 @@ _moo_text_search_regex_backward (const GtkTextIter      *search_start,
             return TRUE;
         }
 
+        g_free (text);
+
         slice_end = slice_start;
         flags = 0;
 
@@ -229,19 +323,19 @@ _moo_text_search_regex_backward (const GtkTextIter      *search_start,
         if (search_end && gtk_text_iter_compare (&slice_end, search_end) < 0)
             break;
 
-        gtk_text_iter_backward_line (&slice_start);
+        gtk_text_iter_backward_lines (&slice_start, regex->n_lines);
     }
 
     return FALSE;
 }
 
 
-static GRegex *
-get_regex (const char          *pattern,
-           MooTextSearchFlags   flags,
-           GError             **error)
+static MooRegex *
+get_regex (const char            *pattern,
+           MooTextSearchFlags     flags,
+           GError               **error)
 {
-    static GRegex *saved_regex;
+    static MooRegex *saved_regex;
     static char *saved_pattern;
     static MooTextSearchFlags saved_flags;
 
@@ -250,7 +344,7 @@ get_regex (const char          *pattern,
         GRegexCompileFlags re_flags = 0;
 
         if (saved_regex)
-            g_regex_unref (saved_regex);
+            _moo_regex_unref (saved_regex);
         g_free (saved_pattern);
 
         saved_pattern = g_strdup (pattern);
@@ -259,9 +353,9 @@ get_regex (const char          *pattern,
         if (flags & MOO_TEXT_SEARCH_CASELESS)
             re_flags |= G_REGEX_CASELESS;
 
-        saved_regex = g_regex_new (saved_pattern,
-                                   re_flags | G_REGEX_OPTIMIZE,
-                                   0, error);
+        saved_regex = _moo_regex_compile (saved_pattern,
+                                          re_flags | G_REGEX_OPTIMIZE,
+                                          0, error);
 
         if (!saved_regex)
         {
@@ -314,7 +408,7 @@ moo_text_search_forward (const GtkTextIter      *start,
                          GtkTextIter            *match_end,
                          const GtkTextIter      *end)
 {
-    GRegex *regex;
+    MooRegex *regex;
     GError *error = NULL;
 
     g_return_val_if_fail (start != NULL, FALSE);
@@ -383,7 +477,7 @@ moo_text_search_backward (const GtkTextIter      *start,
                           GtkTextIter            *match_end,
                           const GtkTextIter      *end)
 {
-    GRegex *regex;
+    MooRegex *regex;
     GError *error = NULL;
 
     g_return_val_if_fail (start != NULL, FALSE);
@@ -435,7 +529,7 @@ moo_text_search_backward (const GtkTextIter      *start,
 static int
 moo_text_replace_regex_all_real (GtkTextIter            *start,
                                  GtkTextIter            *end,
-                                 GRegex                 *regex,
+                                 MooRegex               *regex,
                                  const char             *replacement,
                                  gboolean                replacement_literal,
                                  MooTextReplaceFunc      func,
@@ -517,7 +611,8 @@ moo_text_replace_regex_all_real (GtkTextIter            *start,
         int match_len;
         GMatchInfo *match_info = NULL;
 
-        if (!_moo_text_search_regex_forward (start, end, regex, &match_start, &match_end,
+        if (!_moo_text_search_regex_forward (start, end, regex,
+                                             &match_start, &match_end,
                                              &string, NULL, &match_len, &match_info))
             goto out;
 
@@ -634,7 +729,7 @@ out:
 int
 _moo_text_replace_regex_all (GtkTextIter            *start,
                              GtkTextIter            *end,
-                             GRegex                 *regex,
+                             MooRegex               *regex,
                              const char             *replacement,
                              gboolean                replacement_literal)
 {
@@ -650,7 +745,7 @@ _moo_text_replace_regex_all (GtkTextIter            *start,
 int
 _moo_text_replace_regex_all_interactive (GtkTextIter            *start,
                                          GtkTextIter            *end,
-                                         GRegex                 *regex,
+                                         MooRegex               *regex,
                                          const char             *replacement,
                                          gboolean                replacement_literal,
                                          MooTextReplaceFunc      func,
@@ -686,7 +781,7 @@ moo_text_replace_all (GtkTextIter            *start,
     if (flags & MOO_TEXT_SEARCH_REGEX)
     {
         GError *error = NULL;
-        GRegex *regex = get_regex (text, flags, &error);
+        MooRegex *regex = get_regex (text, flags, &error);
 
         if (error)
         {
@@ -764,7 +859,7 @@ _moo_text_replace_all_interactive (GtkTextIter            *start,
     if (flags & MOO_TEXT_SEARCH_REGEX)
     {
         GError *error = NULL;
-        GRegex *regex = get_regex (text, flags, &error);
+        MooRegex *regex = get_regex (text, flags, &error);
 
         if (error)
         {
