@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -17,13 +16,14 @@
 #include "mooedit/mooeditor.h"
 #include "mooedit/mooeditaction.h"
 #include "mooedit/mooeditaction-factory.h"
-#include "mooedit/mookeyfile.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/mooutils-debug.h"
 #include "mooutils/mooaccel.h"
 #include "mooutils/mooi18n.h"
 #include "mooutils/mooaction-private.h"
 #include "mooutils/mootype-macros.h"
 #include <string.h>
+#include <stdlib.h>
 
 
 #define N_TOOLS 2
@@ -32,6 +32,7 @@
 #define KEY_ACCEL       "accel"
 #define KEY_MENU        "menu"
 #define KEY_LANGS       "langs"
+#define KEY_FILTER      "file-filter"
 #define KEY_POSITION    "position"
 #define KEY_COMMAND     "command"
 #define KEY_NAME        "name"
@@ -40,6 +41,7 @@
 #define KEY_ID          "id"
 #define KEY_DELETED     "deleted"
 #define KEY_BUILTIN     "builtin"
+#define KEY_OPTIONS     "options"
 
 enum {
     PROP_0,
@@ -77,6 +79,9 @@ static ToolStore *tools_stores[N_TOOLS];
 static MooCommandContext   *create_command_context  (gpointer        window,
                                                      gpointer        doc);
 static MooUserToolInfo     *_moo_user_tool_info_ref (MooUserToolInfo *info);
+static void                 add_info                (MooUserToolInfo *info,
+                                                     GSList        **list,
+                                                     GHashTable     *ids);
 
 
 static void
@@ -331,9 +336,7 @@ load_tool (MooUserToolInfo *info)
             moo_edit_window_set_action_check (info->id, MOO_ACTION_CHECK_SENSITIVE,
                                               check_sensitive_func,
                                               NULL, NULL);
-
-            if (info->langs)
-                moo_edit_window_set_action_langs (info->id, MOO_ACTION_CHECK_ACTIVE, info->langs);
+            moo_edit_window_set_action_filter (info->id, MOO_ACTION_CHECK_ACTIVE, info->filter);
 
             break;
 
@@ -345,7 +348,7 @@ load_tool (MooUserToolInfo *info)
                                        "label", info->name,
                                        "accel", info->accel,
                                        "command", cmd,
-                                       "langs", info->langs,
+                                       "file-filter", info->filter,
                                        NULL);
             break;
     }
@@ -401,7 +404,7 @@ init_tools_environment (void)
 
         done = TRUE;
 
-        script_dirs = moo_get_data_subdirs ("scripts", MOO_DATA_LIB, &n_script_dirs);
+        script_dirs = moo_get_data_subdirs ("scripts", MOO_DATA_SHARE, &n_script_dirs);
         g_return_if_fail (n_script_dirs != 0);
 
         old_path = g_getenv ("PATH");
@@ -420,7 +423,7 @@ init_tools_environment (void)
 
 
 void
-_moo_edit_load_user_tools (MooUserToolType type)
+_moo_edit_load_user_tools_type (MooUserToolType type)
 {
     GSList *list;
 
@@ -428,7 +431,7 @@ _moo_edit_load_user_tools (MooUserToolType type)
 
     unload_user_tools (type);
 
-    list = _moo_edit_parse_user_tools (type);
+    list = _moo_edit_parse_user_tools (type, TRUE);
 
     while (list)
     {
@@ -438,6 +441,190 @@ _moo_edit_load_user_tools (MooUserToolType type)
     }
 }
 
+void
+_moo_edit_load_user_tools (void)
+{
+    _moo_edit_load_user_tools_type (MOO_USER_TOOL_MENU);
+    _moo_edit_load_user_tools_type (MOO_USER_TOOL_CONTEXT);
+}
+
+
+static void
+parse_params (MooUserToolInfo  *info,
+              char            **params,
+              const char       *filename)
+{
+    while (params && *params)
+    {
+        char *key = *params++;
+        char *value = *params++;
+
+        if (!strcmp (key, KEY_POSITION))
+        {
+            if (!g_ascii_strcasecmp (value, "end"))
+                info->position = MOO_USER_TOOL_POS_END;
+            else if (!g_ascii_strcasecmp (value, "start"))
+                info->position = MOO_USER_TOOL_POS_START;
+            else
+                g_warning ("unknown position type '%s' for tool %s in file %s",
+                           value, info->name, filename);
+        }
+        else if (!strcmp (key, KEY_ID))
+        {
+            g_free (info->id);
+            info->id = g_strdup (value);
+        }
+        else if (!strcmp (key, KEY_NAME))
+        {
+            g_free (info->name);
+            info->name = g_strdup (value);
+        }
+        else if (!strcmp (key, KEY_OS))
+        {
+            if (!g_ascii_strncasecmp (value, "win", 3))
+                info->os_type = MOO_USER_TOOL_WIN32;
+            else
+                info->os_type = MOO_USER_TOOL_UNIX;
+        }
+        else if (!strcmp (key, KEY_ACCEL))
+        {
+            g_free (info->accel);
+            info->accel = g_strdup (value);
+        }
+        else if (!strcmp (key, KEY_MENU))
+        {
+            g_free (info->menu);
+            info->menu = g_strdup (value);
+        }
+        else if (!strcmp (key, KEY_LANGS))
+        {
+            g_free (info->filter);
+            info->filter = g_strdup_printf ("langs: %s", value);
+        }
+        else if (!strcmp (key, KEY_FILTER))
+        {
+            g_free (info->filter);
+            info->filter = g_strdup (value);
+        }
+        else if (!strcmp (key, KEY_OPTIONS))
+        {
+            g_free (info->options);
+            info->options = g_strdup (value);
+        }
+        else
+        {
+            g_warning ("%s: unknown option %s=%s in file %s",
+                       G_STRLOC, key, value, filename);
+        }
+    }
+}
+
+static void
+load_file (const char      *filename,
+           const char      *name,
+           MooUserToolType  type,
+           GSList         **list,
+           GHashTable      *ids)
+{
+    MooUserToolInfo *info;
+    MooCommandData *cmd_data;
+    MooCommandFactory *cmd_factory;
+    char **params;
+
+    cmd_data = _moo_command_parse_file (filename, &cmd_factory, &params);
+    if (!cmd_data)
+        return;
+
+    info = _moo_user_tool_info_new ();
+    info->type = type;
+    info->file = g_strdup (filename);
+    info->id = g_strdup (name);
+    info->name = g_strdup (name);
+    info->enabled = TRUE;
+    info->os_type = MOO_USER_TOOL_THIS_OS;
+    info->cmd_data = cmd_data;
+    info->cmd_factory = cmd_factory;
+
+    parse_params (info, params, filename);
+    add_info (info, list, ids);
+
+    g_strfreev (params);
+}
+
+static int
+cmp_filenames (const void *p1,
+               const void *p2)
+{
+    char *const *sp1 = p1;
+    char *const *sp2 = p2;
+    return strcmp (*sp1, *sp2);
+}
+
+#define NAME_PREFIX_LEN 2
+static void
+load_directory (const char       *path,
+                MooUserToolType   type,
+                GSList          **list,
+                GHashTable       *ids)
+{
+    GDir *dir;
+    GError *error = NULL;
+    const char *name;
+    GPtrArray *names;
+    guint i;
+
+    dir = g_dir_open (path, 0, &error);
+
+    if (!dir)
+    {
+        g_warning ("%s: %s", G_STRLOC, error->message);
+        g_error_free (error);
+        return;
+    }
+
+    names = g_ptr_array_new ();
+
+    while ((name = g_dir_read_name (dir)))
+        if (strlen (name) > NAME_PREFIX_LEN)
+            g_ptr_array_add (names, g_strdup (name));
+
+    g_dir_close (dir);
+
+    qsort (names->pdata, names->len, sizeof (char*), cmp_filenames);
+
+    for (i = 0; i < names->len; ++i)
+    {
+        char *filename;
+
+        filename = g_build_filename (path, names->pdata[i], NULL);
+        load_file (filename, ((char*)names->pdata[i]) + NAME_PREFIX_LEN, type, list, ids);
+
+        g_free (filename);
+        g_free (names->pdata[i]);
+    }
+
+    g_ptr_array_free (names, TRUE);
+}
+
+static void
+load_directories (MooUserToolType   type,
+                  GSList          **list,
+                  GHashTable       *ids)
+{
+    char **dirs, **p;
+    guint n_dirs;
+
+    dirs = moo_get_data_subdirs (type == MOO_USER_TOOL_MENU ? "tools" : "tools-context",
+                                 MOO_DATA_SHARE, &n_dirs);
+    dirs = _moo_strv_reverse (dirs);
+
+    for (p = dirs; p && *p; ++p)
+        if (g_file_test (*p, G_FILE_TEST_IS_DIR))
+            load_directory (*p, type, list, ids);
+
+    g_strfreev (dirs);
+}
+
 
 static MooUserToolInfo *
 parse_item (MooKeyFileItem  *item,
@@ -447,6 +634,7 @@ parse_item (MooKeyFileItem  *item,
     char *os;
     char *position = NULL;
     MooUserToolInfo *info;
+    char *langs;
 
     if (strcmp (moo_key_file_item_name (item), ITEM_TOOL))
     {
@@ -485,7 +673,6 @@ parse_item (MooKeyFileItem  *item,
 
     info->accel = moo_key_file_item_steal (item, KEY_ACCEL);
     info->menu = moo_key_file_item_steal (item, KEY_MENU);
-    info->langs = moo_key_file_item_steal (item, KEY_LANGS);
     position = moo_key_file_item_steal (item, KEY_POSITION);
 
     if (position)
@@ -501,13 +688,22 @@ parse_item (MooKeyFileItem  *item,
         g_free (position);
     }
 
+    if ((langs = moo_key_file_item_steal (item, KEY_LANGS)))
+    {
+        info->filter = g_strdup_printf ("langs: %s", langs);
+        g_free (langs);
+    }
+    else
+    {
+        info->filter = moo_key_file_item_steal (item, KEY_FILTER);
+    }
+
     info->cmd_data = _moo_command_parse_item (item, info->name, file,
                                               &info->cmd_factory,
                                               &info->options);
 
     if (!info->cmd_data)
     {
-        g_warning ("could not get command data for tool '%s' in file %s", info->name, file);
         _moo_user_tool_info_unref (info);
         return NULL;
     }
@@ -565,6 +761,44 @@ parse_file_simple (const char     *filename,
     return list;
 }
 
+static void
+add_info (MooUserToolInfo *info,
+          GSList        **list,
+          GHashTable     *ids)
+{
+    MooUserToolInfo *old_info;
+    GSList *old_link;
+
+    old_link = g_hash_table_lookup (ids, info->id);
+    old_info = old_link ? old_link->data : NULL;
+
+    if (old_link)
+    {
+        *list = g_slist_delete_link (*list, old_link);
+        g_hash_table_remove (ids, info->id);
+    }
+
+    if (info->deleted)
+    {
+        _moo_user_tool_info_unref (info);
+        info = NULL;
+    }
+    else if (info->builtin)
+    {
+        _moo_user_tool_info_unref (info);
+        info = old_info ? _moo_user_tool_info_ref (old_info) : NULL;
+    }
+
+    if (info)
+    {
+        *list = g_slist_prepend (*list, info);
+        g_hash_table_insert (ids, g_strdup (info->id), *list);
+    }
+
+    if (old_info)
+        _moo_user_tool_info_unref (old_info);
+}
+
 static void
 parse_file (const char     *filename,
             MooUserToolType type,
@@ -580,41 +814,12 @@ parse_file (const char     *filename,
 
     while (new_list)
     {
-        MooUserToolInfo *info, *old_info;
-        GSList *old_link;
+        MooUserToolInfo *info;
 
         info = new_list->data;
         g_return_if_fail (info->id != NULL);
 
-        old_link = g_hash_table_lookup (ids, info->id);
-        old_info = old_link ? old_link->data : NULL;
-
-        if (old_link)
-        {
-            *list = g_slist_delete_link (*list, old_link);
-            g_hash_table_remove (ids, info->id);
-        }
-
-        if (info->deleted)
-        {
-            _moo_user_tool_info_unref (info);
-            info = NULL;
-        }
-        else if (info->builtin)
-        {
-            _moo_user_tool_info_unref (info);
-            info = old_info ? _moo_user_tool_info_ref (old_info) : NULL;
-        }
-
-        if (info)
-        {
-            *list = g_slist_prepend (*list, info);
-            g_hash_table_insert (ids, g_strdup (info->id), *list);
-        }
-
-        if (old_info)
-            _moo_user_tool_info_unref (old_info);
-
+        add_info (info, list, ids);
         new_list = g_slist_delete_link (new_list, new_list);
     }
 }
@@ -622,7 +827,8 @@ parse_file (const char     *filename,
 static GSList *
 parse_user_tools (MooUserToolType type,
                   GHashTable    **ids_p,
-                  gboolean        sys_only)
+                  gboolean        sys_only,
+                  gboolean        all)
 {
     char **sys_files, *user_file;
     GSList *list = NULL;
@@ -642,6 +848,9 @@ parse_user_tools (MooUserToolType type,
     if (!sys_only && user_file)
         parse_file (user_file, type, &list, ids);
 
+    if (all)
+        load_directories (type, &list, ids);
+
     if (ids_p)
         *ids_p = ids;
     else
@@ -654,10 +863,11 @@ parse_user_tools (MooUserToolType type,
 }
 
 GSList *
-_moo_edit_parse_user_tools (MooUserToolType type)
+_moo_edit_parse_user_tools (MooUserToolType type,
+                            gboolean        all)
 {
     g_return_val_if_fail (type < N_TOOLS, NULL);
-    return parse_user_tools (type, NULL, FALSE);
+    return parse_user_tools (type, NULL, FALSE, all);
 }
 
 
@@ -665,45 +875,22 @@ static void
 generate_id (MooUserToolInfo *info,
              GHashTable      *ids)
 {
-    char *base, *name;
+    guint i;
 
     g_return_if_fail (info->id == NULL);
 
-    if (info->name)
-        name = get_name (info->name);
-    else
-        name = NULL;
-
-    base = g_strdup_printf ("MooUserTool_%s", name ? name : "");
-    g_strcanon (base, G_CSET_A_2_Z G_CSET_a_2_z G_CSET_DIGITS "_", '_');
-
-    if (!g_hash_table_lookup (ids, base))
+    for (i = 0; ; ++i)
     {
-        info->id = base;
-        base = NULL;
-    }
-    else
-    {
-        guint i = 0;
+        char *id = g_strdup_printf ("MooUserTool%u", i);
 
-        while (TRUE)
+        if (!g_hash_table_lookup (ids, id))
         {
-            char *tmp = g_strdup_printf ("%s_%u", base, i);
-
-            if (!g_hash_table_lookup (ids, tmp))
-            {
-                info->id = tmp;
-                break;
-            }
-
-            g_free (tmp);
-
-            i += 1;
+            info->id = id;
+            break;
         }
-    }
 
-    g_free (name);
-    g_free (base);
+        g_free (id);
+    }
 }
 
 static void
@@ -738,7 +925,7 @@ assign_missing_ids (GSList *list)
 
 static void
 add_deleted (const char *id,
-             GSList     *link,
+             G_GNUC_UNUSED GSList *link,
              GSList    **list)
 {
     MooUserToolInfo *info;
@@ -759,11 +946,6 @@ info_equal (MooUserToolInfo *info1,
     if (info1 == info2)
         return TRUE;
 
-    if (info1->deleted)
-        return info2->deleted != 0;
-    if (info1->builtin)
-        return info2->builtin != 0;
-
     if (!info1->enabled != !info2->enabled)
         return FALSE;
 
@@ -775,7 +957,7 @@ info_equal (MooUserToolInfo *info1,
            _moo_str_equal (info1->name, info2->name) &&
            _moo_str_equal (info1->accel, info2->accel) &&
            _moo_str_equal (info1->menu, info2->menu) &&
-           _moo_str_equal (info1->langs, info2->langs) &&
+           _moo_str_equal (info1->filter, info2->filter) &&
            _moo_str_equal (info1->options, info2->options) &&
            _moo_command_factory_data_equal (info1->cmd_factory, info1->cmd_data, info2->cmd_data);
 }
@@ -788,7 +970,7 @@ generate_real_list (MooUserToolType  type,
     GSList *sys_list;
     GHashTable *sys_ids;
 
-    sys_list = parse_user_tools (type, &sys_ids, TRUE);
+    sys_list = parse_user_tools (type, &sys_ids, TRUE, FALSE);
     assign_missing_ids (list);
 
     while (list)
@@ -873,8 +1055,8 @@ _moo_edit_save_user_tools (MooUserToolType  type,
                 moo_key_file_item_set (item, KEY_ACCEL, info->accel);
             if (info->menu && info->menu[0])
                 moo_key_file_item_set (item, KEY_MENU, info->menu);
-            if (info->langs && info->langs[0])
-                moo_key_file_item_set (item, KEY_LANGS, info->langs);
+            if (info->filter && info->filter[0])
+                moo_key_file_item_set (item, KEY_FILTER, info->filter);
             if (!info->enabled)
                 moo_key_file_item_set_bool (item, KEY_ENABLED, info->enabled);
             if (info->position != MOO_USER_TOOL_POS_END)
@@ -890,9 +1072,7 @@ _moo_edit_save_user_tools (MooUserToolType  type,
         list = g_slist_delete_link (list, list);
     }
 
-    string = moo_key_file_format (key_file, "This file is autogenerated, do not edit", 2);
-
-    _moo_message ("saving file %s", FILENAMES[type]);
+    string = moo_key_file_format (key_file, "This file is autogenerated, do not edit\n# -%- strip: false -%-", 2);
 
     if (!moo_save_user_data_file (FILENAMES[type], string, -1, &error))
     {
@@ -900,8 +1080,6 @@ _moo_edit_save_user_tools (MooUserToolType  type,
         g_error_free (error);
     }
 
-    _moo_message ("done");
-
     g_free (string);
     moo_key_file_unref (key_file);
 }
@@ -937,7 +1115,7 @@ _moo_user_tool_info_unref (MooUserToolInfo *info)
     g_free (info->name);
     g_free (info->accel);
     g_free (info->menu);
-    g_free (info->langs);
+    g_free (info->filter);
     g_free (info->options);
     g_free (info->file);
 
@@ -1129,13 +1307,13 @@ create_command_context (gpointer window,
 
     ctx = moo_command_context_new (doc, window);
 
-    if (MOO_IS_EDIT (doc) && moo_edit_get_filename (doc))
+    if (MOO_IS_EDIT (doc) && !moo_edit_is_untitled (doc))
     {
-        const char *filename, *basename;
+        char *filename, *basename;
         char *dirname, *base = NULL, *extension = NULL;
 
         filename = moo_edit_get_filename (doc);
-        basename = moo_edit_get_basename (doc);
+        basename = filename ? g_path_get_basename (filename) : NULL;
         dirname = g_path_get_dirname (filename);
         get_extension (basename, &base, &extension);
 
@@ -1147,6 +1325,8 @@ create_command_context (gpointer window,
         g_free (dirname);
         g_free (base);
         g_free (extension);
+        g_free (basename);
+        g_free (filename);
     }
 
     if (MOO_IS_EDIT (doc))
