@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -28,6 +27,8 @@
 #include "mooutils/moohistorycombo.h"
 #include "mooutils/moodialogs.h"
 #include "mooutils/mooi18n.h"
+#include "mooutils/moohelp.h"
+#include "help-sections.h"
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtktogglebutton.h>
@@ -38,6 +39,9 @@
 
 #define FIND_PLUGIN_ID "Find"
 
+#define GREP_SKIP_LIST_ID "FindPlugin/grep/skip"
+#define FIND_SKIP_LIST_ID "FindPlugin/find/skip"
+
 enum {
     CMD_GREP = 1,
     CMD_FIND
@@ -217,10 +221,11 @@ ensure_output (WindowStuff *stuff)
 }
 
 
-static void
+static gboolean
 find_window_plugin_create (WindowStuff *stuff)
 {
     stuff->window = MOO_WIN_PLUGIN (stuff)->window;
+    return TRUE;
 }
 
 
@@ -325,20 +330,30 @@ map_combo (MooGladeXML *xml,
 }
 
 
+static void
+init_skip_list (void)
+{
+    MooHistoryList *list = moo_history_list_get (GREP_SKIP_LIST_ID);
+    g_return_if_fail (list != NULL);
+    if (moo_history_list_is_empty (list))
+        moo_history_list_add (list, ".svn/;CVS/");
+}
+
 static void
 create_grep_dialog (MooEditWindow  *window,
                     WindowStuff    *stuff)
 {
     GtkWidget *pattern_entry;
     GtkWidget *skip_combo;
-    MooHistoryList *skip_list;
+
+    init_skip_list ();
 
     stuff->grep_xml = moo_glade_xml_new_empty (GETTEXT_PACKAGE);
     map_combo (stuff->grep_xml, "pattern_combo", "FindPlugin/grep/pattern");
     map_combo (stuff->grep_xml, "glob_combo", "FindPlugin/grep/glob");
     map_combo (stuff->grep_xml, "dir_combo", "FindPlugin/grep/dir");
-    map_combo (stuff->grep_xml, "skip_combo", "FindPlugin/grep/skip");
-    moo_glade_xml_parse_memory (stuff->grep_xml, MOO_FIND_GLADE_XML, -1, "grep_dialog", NULL);
+    map_combo (stuff->grep_xml, "skip_combo", GREP_SKIP_LIST_ID);
+    moo_glade_xml_parse_memory (stuff->grep_xml, moofind_glade_xml, -1, "grep_dialog", NULL);
 
     stuff->grep_dialog = moo_glade_xml_get_widget (stuff->grep_xml, "grep_dialog");
     g_return_if_fail (stuff->grep_dialog != NULL);
@@ -349,6 +364,13 @@ create_grep_dialog (MooEditWindow  *window,
     gtk_dialog_set_response_sensitive (GTK_DIALOG (stuff->grep_dialog),
                                        GTK_RESPONSE_OK, FALSE);
     moo_window_set_parent (stuff->grep_dialog, GTK_WIDGET (window));
+    gtk_dialog_set_alternative_button_order (GTK_DIALOG (stuff->grep_dialog),
+                                             GTK_RESPONSE_OK,
+                                             GTK_RESPONSE_CANCEL,
+                                             -1);
+
+    moo_help_set_id (stuff->grep_dialog, HELP_SECTION_DIALOG_FIND_IN_FILES);
+    moo_help_connect_keys (stuff->grep_dialog);
 
     g_signal_connect (stuff->grep_dialog, "delete-event",
                       G_CALLBACK (gtk_widget_hide_on_delete), NULL);
@@ -361,8 +383,7 @@ create_grep_dialog (MooEditWindow  *window,
     setup_file_combo (moo_glade_xml_get_widget (stuff->grep_xml, "dir_combo"));
 
     skip_combo = moo_glade_xml_get_widget (stuff->grep_xml, "skip_combo");
-    skip_list = moo_history_combo_get_list (MOO_HISTORY_COMBO (skip_combo));
-    moo_history_list_add_full (skip_list, ".svn/;CVS/", _("CVS and SVN dirs"));
+    moo_combo_set_active (MOO_COMBO (skip_combo), 0);
 }
 
 
@@ -375,8 +396,8 @@ create_find_dialog (MooEditWindow  *window,
     stuff->find_xml = moo_glade_xml_new_empty (GETTEXT_PACKAGE);
     map_combo (stuff->find_xml, "pattern_combo", "FindPlugin/find/pattern");
     map_combo (stuff->find_xml, "dir_combo", "FindPlugin/find/dir");
-    map_combo (stuff->find_xml, "skip_combo", "FindPlugin/find/skip");
-    moo_glade_xml_parse_memory (stuff->find_xml, MOO_FIND_GLADE_XML, -1, "find_dialog", NULL);
+    map_combo (stuff->find_xml, "skip_combo", FIND_SKIP_LIST_ID);
+    moo_glade_xml_parse_memory (stuff->find_xml, moofind_glade_xml, -1, "find_dialog", NULL);
 
     stuff->find_dialog = moo_glade_xml_get_widget (stuff->find_xml, "find_dialog");
     g_return_if_fail (stuff->find_dialog != NULL);
@@ -386,8 +407,15 @@ create_find_dialog (MooEditWindow  *window,
                                      GTK_RESPONSE_OK);
     gtk_dialog_set_response_sensitive (GTK_DIALOG (stuff->find_dialog),
                                        GTK_RESPONSE_OK, FALSE);
+    gtk_dialog_set_alternative_button_order (GTK_DIALOG (stuff->find_dialog),
+                                             GTK_RESPONSE_OK,
+                                             GTK_RESPONSE_CANCEL,
+                                             -1);
     moo_window_set_parent (stuff->find_dialog, GTK_WIDGET (window));
 
+    moo_help_set_id (stuff->find_dialog, HELP_SECTION_DIALOG_FIND_FILE);
+    moo_help_connect_keys (stuff->find_dialog);
+
     g_signal_connect (stuff->find_dialog, "delete-event",
                       G_CALLBACK (gtk_widget_hide_on_delete), NULL);
 
@@ -404,23 +432,27 @@ static void
 init_dir_entry (MooHistoryCombo *hist_combo,
                 MooEdit         *doc)
 {
-    MooHistoryList *list;
     GtkWidget *entry;
 
     g_return_if_fail (hist_combo != NULL);
 
-    list = moo_history_combo_get_list (hist_combo);
     entry = MOO_COMBO (hist_combo)->entry;
 
-    if (!gtk_entry_get_text(GTK_ENTRY (entry))[0])
+    if (!gtk_entry_get_text (GTK_ENTRY (entry))[0])
+        moo_combo_set_active (MOO_COMBO (hist_combo), 0);
+
+    if (!gtk_entry_get_text (GTK_ENTRY (entry))[0])
     {
         MooFileEntryCompletion *completion;
+        char *filename;
 
         completion = g_object_get_data (G_OBJECT (entry), "find-plugin-file-completion");
 
-        if (doc && moo_edit_get_filename (doc))
+        filename = doc ? moo_edit_get_filename (doc) : NULL;
+
+        if (filename)
         {
-            char *dir = g_path_get_dirname (moo_edit_get_filename (doc));
+            char *dir = g_path_get_dirname (filename);
             _moo_file_entry_completion_set_path (completion, dir);
             g_free (dir);
         }
@@ -428,6 +460,8 @@ init_dir_entry (MooHistoryCombo *hist_combo,
         {
             _moo_file_entry_completion_set_path (completion, g_get_home_dir ());
         }
+
+        g_free (filename);
     }
 
 #if 0
@@ -535,6 +569,8 @@ do_grep (MooEditWindow  *window,
 
     execute_grep (pattern, glob, dir, skip,
                   case_sensitive, stuff);
+
+    g_free (dir);
 }
 
 
@@ -573,6 +609,8 @@ do_find (MooEditWindow  *window,
     moo_big_paned_present_pane (window->paned, pane);
 
     execute_find (pattern, dir, skip, stuff);
+
+    g_free (dir);
 }
 
 
@@ -739,12 +777,13 @@ parse_globs (const char *string,
     command = g_string_new (NULL);
 
     for (p = pieces; p && *p; ++p)
-        g_string_append_printf (command, "%s-%s \"%s%s%s\"",
-                                p == pieces ? "" : " -o ",
-                                skip ? "wholename" : "name",
-                                skip ? "*" : "",
-                                *p,
-                                skip ? "*" : "");
+        if (**p)
+            g_string_append_printf (command, "%s-%s \"%s%s%s\"",
+                                    p == pieces ? "" : " -o ",
+                                    skip ? "wholename" : "name",
+                                    skip ? "*" : "",
+                                    *p,
+                                    skip ? "*" : "");
 
     g_strfreev (pieces);
     return g_string_free (command, FALSE);
@@ -927,11 +966,8 @@ MOO_PLUGIN_DEFINE_INFO (find,
                         N_("Find"), N_("Finds everything"),
                         "Yevgen Muntyan <muntyan@tamu.edu>",
                         MOO_VERSION, NULL)
-MOO_WIN_PLUGIN_DEFINE (Find, find,
-                       find_window_plugin_create,
-                       find_window_plugin_destroy)
+MOO_WIN_PLUGIN_DEFINE (Find, find)
 MOO_PLUGIN_DEFINE_FULL (Find, find,
-                        find_plugin_init, find_plugin_deinit,
                         NULL, NULL, NULL, NULL, NULL,
                         find_window_plugin_get_type (), 0)
 
