@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -17,14 +16,12 @@
 
 #define MOO_FILE_VIEW_COMPILATION
 #include "moofileview/moobookmarkmgr.h"
-#include "moofileview/moobookmarkmgr-glade.h"
+#include "moofileview/moobookmark-editor-glade.h"
 #include "moofileview/moofileentry.h"
 #include "mooutils/mooglade.h"
 #include "mooutils/mooprefs.h"
-#include "mooutils/moocompat.h"
 #include "mooutils/moomarshals.h"
 #include "mooutils/mooactionfactory.h"
-#include "mooutils/mooutils-gobject.h"
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/moostock.h"
 #include <string.h>
@@ -815,7 +812,7 @@ _moo_bookmark_mgr_get_editor (MooBookmarkMgr *mgr)
     if (mgr->priv->editor)
         return mgr->priv->editor;
 
-    xml = moo_glade_xml_new_from_buf (MOO_BOOKMARK_MGR_GLADE_UI, -1,
+    xml = moo_glade_xml_new_from_buf (moobookmark_editor_glade_xml, -1,
                                       NULL, GETTEXT_PACKAGE, NULL);
 
     dialog = moo_glade_xml_get_widget (xml, "dialog");
