@@ -3,21 +3,14 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
 
 /*
- *  Icon handling code was copied (and modified) from gtk/gtkfilesystemunix.c,
- *  Copyright (C) 2003, Red Hat, Inc.
- *
- *  g_utf8_collate_key_for_filename is taken from glib/gunicollate.c
- *  Copyright 2001,2005 Red Hat, Inc.
- *
  *  TODO!!! fix this mess
  */
 
@@ -31,6 +24,8 @@
 #include "moofileview/moofileicons.h"
 #include "mooutils/mooutils-fs.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/mooutils-debug.h"
+#include "mooutils/moocompat.h"
 #include "mooutils/moomarshals.h"
 #include "mooutils/xdgmime/xdgmime.h"
 #include <glib/gstdio.h>
@@ -53,12 +48,6 @@ static const char *mime_type_unknown = "application/octet-stream";
 #define MIME_TYPE_UNKNOWN mime_type_unknown
 #endif
 
-#if !GLIB_CHECK_VERSION(2,8,0)
-static gchar *g_utf8_collate_key_for_filename   (const gchar *str,
-                                                 gssize       len);
-#endif
-
-
 #define NORMAL_PRIORITY         G_PRIORITY_DEFAULT_IDLE
 #define NORMAL_TIMEOUT          0.04
 #define BACKGROUND_PRIORITY     G_PRIORITY_LOW
@@ -92,8 +81,7 @@ _moo_file_find_mime_type (MooFile    *file,
     if (!file->mime_type || !file->mime_type[0])
     {
         /* this should not happen */
-        if (0)
-            _moo_message ("%s: oops, %s", G_STRLOC, file->display_name);
+        _moo_message ("%s: oops, %s", G_STRLOC, file->display_name);
         file->mime_type = MIME_TYPE_UNKNOWN;
     }
 
@@ -199,20 +187,24 @@ _moo_file_stat (MooFile    *file,
     {
         if (errno == ENOENT)
         {
-            gchar *display_name = g_filename_display_name (fullname);
-            _moo_message ("%s: file '%s' does not exist",
-                          G_STRLOC, display_name);
-            g_free (display_name);
+            MOO_DEBUG({
+                gchar *display_name = g_filename_display_name (fullname);
+                _moo_message ("%s: file '%s' does not exist",
+                              G_STRLOC, display_name);
+                g_free (display_name);
+            });
             file->info = 0;
         }
         else
         {
-            int save_errno = errno;
-            gchar *display_name = g_filename_display_name (fullname);
-            _moo_message ("%s: error getting information for '%s': %s",
-                          G_STRLOC, display_name,
-                          g_strerror (save_errno));
-            g_free (display_name);
+            MOO_DEBUG({
+                int save_errno = errno;
+                gchar *display_name = g_filename_display_name (fullname);
+                _moo_message ("%s: error getting information for '%s': %s",
+                              G_STRLOC, display_name,
+                              g_strerror (save_errno));
+                g_free (display_name);
+            });
             file->info = MOO_FILE_INFO_IS_LOCKED | MOO_FILE_INFO_EXISTS;
             file->flags = 0;
         }
@@ -232,20 +224,24 @@ _moo_file_stat (MooFile    *file,
             {
                 if (errno == ENOENT)
                 {
-                    gchar *display_name = g_filename_display_name (fullname);
-                    _moo_message ("%s: file '%s' is a broken link",
-                                  G_STRLOC, display_name);
-                    g_free (display_name);
+                    MOO_DEBUG({
+                        gchar *display_name = g_filename_display_name (fullname);
+                        _moo_message ("%s: file '%s' is a broken link",
+                                      G_STRLOC, display_name);
+                        g_free (display_name);
+                    });
                     file->info = MOO_FILE_INFO_IS_LINK;
                 }
                 else
                 {
-                    int save_errno = errno;
-                    gchar *display_name = g_filename_display_name (fullname);
-                    _moo_message ("%s: error getting information for '%s': %s",
-                                  G_STRLOC, display_name,
-                                  g_strerror (save_errno));
-                    g_free (display_name);
+                    MOO_DEBUG({
+                        int save_errno = errno;
+                        gchar *display_name = g_filename_display_name (fullname);
+                        _moo_message ("%s: error getting information for '%s': %s",
+                                      G_STRLOC, display_name,
+                                      g_strerror (save_errno));
+                        g_free (display_name);
+                    });
                     file->info = MOO_FILE_INFO_IS_LOCKED | MOO_FILE_INFO_EXISTS;
                     file->flags = 0;
                 }
@@ -255,12 +251,14 @@ _moo_file_stat (MooFile    *file,
 
             if (len == -1)
             {
-                int save_errno = errno;
-                gchar *display_name = g_filename_display_name (fullname);
-                _moo_message ("%s: error getting link target for '%s': %s",
-                              G_STRLOC, display_name,
-                              g_strerror (save_errno));
-                g_free (display_name);
+                MOO_DEBUG({
+                    int save_errno = errno;
+                    gchar *display_name = g_filename_display_name (fullname);
+                    _moo_message ("%s: error getting link target for '%s': %s",
+                                  G_STRLOC, display_name,
+                                  g_strerror (save_errno));
+                    g_free (display_name);
+                });
             }
             else
             {
@@ -476,202 +474,6 @@ _moo_file_info_get_type (void)
 }
 
 
-#if !GLIB_CHECK_VERSION(2,8,0)
-/* This is a collation key that is very very likely to sort before any
-   collation key that libc strxfrm generates. We use this before any
-   special case (dot or number) to make sure that its sorted before
-   anything else.
- */
-#define COLLATION_SENTINEL "\1\1\1"
-
-/**
- * g_utf8_collate_key_for_filename:
- * @str: a UTF-8 encoded string.
- * @len: length of @str, in bytes, or -1 if @str is nul-terminated.
- *
- * Converts a string into a collation key that can be compared
- * with other collation keys produced by the same function using strcmp().
- *
- * In order to sort filenames correctly, this function treats the dot '.'
- * as a special case. Most dictionary orderings seem to consider it
- * insignificant, thus producing the ordering "event.c" "eventgenerator.c"
- * "event.h" instead of "event.c" "event.h" "eventgenerator.c". Also, we
- * would like to treat numbers intelligently so that "file1" "file10" "file5"
- * is sorted as "file1" "file5" "file10".
- *
- * Return value: a newly allocated string. This string should
- *   be freed with g_free() when you are done with it.
- *
- * Since: 2.8
- */
-static gchar *g_utf8_collate_key_for_filename   (const gchar *str,
-                                                 gssize       len)
-{
-    GString *result;
-    GString *append;
-    const gchar *p;
-    const gchar *prev;
-    gchar *collate_key;
-    gint digits;
-    gint leading_zeros;
-
-  /*
-   * How it works:
-    *
-    * Split the filename into collatable substrings which do
-    * not contain [.0-9] and special-cased substrings. The collatable
-    * substrings are run through the normal g_utf8_collate_key() and the
-    * resulting keys are concatenated with keys generated from the
-    * special-cased substrings.
-    *
-    * Special cases: Dots are handled by replacing them with '\1' which
-    * implies that short dot-delimited substrings are before long ones,
-    * e.g.
-    *
-    *   a\1a   (a.a)
-    *   a-\1a  (a-.a)
-    *   aa\1a  (aa.a)
-    *
-    * Numbers are handled by prepending to each number d-1 superdigits
-    * where d = number of digits in the number and SUPERDIGIT is a
-    * character with an integer value higher than any digit (for instance
-    * ':'). This ensures that single-digit numbers are sorted before
-    * double-digit numbers which in turn are sorted separately from
-    * triple-digit numbers, etc. To avoid strange side-effects when
-    * sorting strings that already contain SUPERDIGITs, a '\2'
-    * is also prepended, like this
-    *
-    *   file\21      (file1)
-    *   file\25      (file5)
-    *   file\2:10    (file10)
-    *   file\2:26    (file26)
-    *   file\2::100  (file100)
-    *   file:foo     (file:foo)
-    *
-    * This has the side-effect of sorting numbers before everything else (except
-    * dots), but this is probably OK.
-    *
-    * Leading digits are ignored when doing the above. To discriminate
-    * numbers which differ only in the number of leading digits, we append
-    * the number of leading digits as a byte at the very end of the collation
-    * key.
-    *
-    * To try avoid conflict with any collation key sequence generated by libc we
-    * start each switch to a special cased part with a sentinel that hopefully
-    * will sort before anything libc will generate.
-  */
-
-    if (len < 0)
-        len = strlen (str);
-
-    result = g_string_sized_new (len * 2);
-    append = g_string_sized_new (0);
-
-    /* No need to use utf8 functions, since we're only looking for ascii chars */
-    for (prev = p = str; *p != '\0'; p++)
-    {
-        switch (*p)
-        {
-            case '.':
-                if (prev != p)
-                {
-                    collate_key = g_utf8_collate_key (prev, p - prev);
-                    g_string_append (result, collate_key);
-                    g_free (collate_key);
-                }
-
-                g_string_append (result, COLLATION_SENTINEL "\1");
-
-                /* skip the dot */
-                prev = p + 1;
-                break;
-
-            case '0':
-            case '1':
-            case '2':
-            case '3':
-            case '4':
-            case '5':
-            case '6':
-            case '7':
-            case '8':
-            case '9':
-                if (prev != p)
-                {
-                    collate_key = g_utf8_collate_key (prev, p - prev);
-                    g_string_append (result, collate_key);
-                    g_free (collate_key);
-                }
-
-                g_string_append (result, COLLATION_SENTINEL "\2");
-
-                prev = p;
-
-                /* write d-1 colons */
-                if (*p == '0')
-                {
-                    leading_zeros = 1;
-                    digits = 0;
-                }
-                else
-                {
-                    leading_zeros = 0;
-                    digits = 1;
-                }
-
-                do
-                {
-                    p++;
-
-                    if (*p == '0' && !digits)
-                        ++leading_zeros;
-                    else if (g_ascii_isdigit(*p))
-                        ++digits;
-                    else
-                        break;
-                }
-                while (*p != '\0');
-
-                while (digits > 1)
-                {
-                    g_string_append_c (result, ':');
-                    --digits;
-                }
-
-                if (leading_zeros > 0)
-                {
-                    g_string_append_c (append, (char)leading_zeros);
-                    prev += leading_zeros;
-                }
-
-                /* write the number itself */
-                g_string_append_len (result, prev, p - prev);
-
-                prev = p;
-                --p;	  /* go one step back to avoid disturbing outer loop */
-                break;
-
-            default:
-                /* other characters just accumulate */
-                break;
-        }
-    }
-
-    if (prev != p)
-    {
-        collate_key = g_utf8_collate_key (prev, p - prev);
-        g_string_append (result, collate_key);
-        g_free (collate_key);
-    }
-
-    g_string_append (result, append->str);
-    g_string_free (append, TRUE);
-
-    return g_string_free (result, FALSE);
-}
-#endif /* !GLIB_CHECK_VERSION(2,8,0) */
-
-
 static GdkPixbuf *
 render_icon (const MooFile  *file,
              GtkWidget      *widget,
