@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -21,6 +20,7 @@
 #include "mooutils/moomarshals.h"
 #include "mooutils/mooutils-gobject.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/mooutils-debug.h"
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
@@ -1322,7 +1322,8 @@ completion_visible_func (GtkTreeModel           *model,
 }
 
 
-/* XXX _gtk_entry_get_borders from gtkentry.c */
+/* XXX _gtk_entry_get_borders from gtkentry.c,
+ * this is just not good, there must be public gtk API for this */
 static void
 entry_get_borders (GtkEntry *entry,
                    gint     *xborder,
