@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -27,7 +26,7 @@
 #include <io.h>
 #endif
 
-#if 0 && MOO_DEBUG
+#if 0 && MOO_DEBUG_ENABLED
 #define DEBUG_MESSAGE g_message
 #else
 static void DEBUG_MESSAGE (G_GNUC_UNUSED const char *format, ...)
@@ -225,7 +224,7 @@ debug_timeout (MooFileSystem *fs)
 {
     gsize mem[2] = {0, 0};
     g_hash_table_foreach (fs->priv->folders, (GHFunc) calc_mem_hash_cb, mem);
-    g_print ("%u bytes in %u files\n", mem[0], mem[1]);
+    g_print ("%" G_GSIZE_FORMAT " bytes in %" G_GSIZE_FORMAT " files\n", mem[0], mem[1]);
     return TRUE;
 }
 
