@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -18,7 +17,6 @@
 #define MOO_FILE_VIEW_COMPILATION
 #include "moofileview/moofilesystem.h"
 #include "moofileview/moofolder-private.h"
-#include "moofileview/symlink.h"
 #include "mooutils/mooutils-fs.h"
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/moomarshals.h"
@@ -1043,7 +1041,7 @@ static char *
 get_size_string (struct stat *statbuf)
 {
     g_return_val_if_fail (statbuf != NULL, NULL);
-    return g_strdup_printf ("%" G_GINT64_FORMAT, (MooFileSize) statbuf->st_size);
+    return g_strdup_printf ("%" G_GINT64_FORMAT, (gint64) statbuf->st_size);
 }
 
 
