@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -416,12 +415,12 @@ G_STMT_START {                          \
     ITER_SET_DIR(ip, is_dir);           \
 } G_STMT_END
 
-#ifdef MOO_DEBUG
+#ifdef MOO_DEBUG_ENABLED
 #if 0
 #define DEFINE_CHECK_ITER
 static void CHECK_ITER (MooFolderModel *model, GtkTreeIter *iter);
 #endif
-#endif /* MOO_DEBUG */
+#endif /* MOO_DEBUG_ENABLED */
 
 #ifndef DEFINE_CHECK_ITER
 #define CHECK_ITER(model,iter)
@@ -611,7 +610,7 @@ model_remove_moo_file (MooFile        *file,
 }
 
 
-#ifdef MOO_DEBUG
+#ifdef MOO_DEBUG_ENABLED
 #if 1
 #define CHECK_ORDER(order,n)                \
 G_STMT_START {                              \
@@ -626,7 +625,7 @@ G_STMT_START {                              \
     g_free (check);                         \
 } G_STMT_END
 #endif
-#endif /* MOO_DEBUG */
+#endif /* MOO_DEBUG_ENABLED */
 #ifndef CHECK_ORDER
 #define CHECK_ORDER(order,n)
 #endif
