@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -705,7 +704,12 @@ _moo_tree_view_widget_to_abs_coords (gpointer        view,
                                      int            *absy)
 {
     if (GTK_IS_TREE_VIEW (view))
+        /* Nice! */
+#if !GTK_CHECK_VERSION(2,12,0)
         gtk_tree_view_widget_to_tree_coords (view, wx, wy, absx, absy);
+#else
+        gtk_tree_view_convert_widget_to_bin_window_coords (view, wx, wy, absx, absy);
+#endif
     else if (MOO_IS_ICON_VIEW (view))
         _moo_icon_view_widget_to_abs_coords (view, wx, wy, absx, absy);
     else
