@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -27,7 +26,7 @@ _moo_app_mod_init (void)
 {
     PyObject *mod;
 
-    mod = Py_InitModule3 ("_moo_app", _moo_app_functions, moo_app_module_doc);
+    mod = Py_InitModule3 ("_moo_app", (PyMethodDef*) _moo_app_functions, moo_app_module_doc);
     PyImport_AddModule ("moo.app");
 
     if (!mod)
