@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -50,7 +49,7 @@ typedef enum
     CHECK_ITER(iter__)
 
 
-#ifdef MOO_DEBUG
+#ifdef MOO_DEBUG_ENABLED
 static void CHECK_ITER (const MooTermIter *iter)
 {
     g_assert (iter->term != NULL);
