@@ -1,12 +1,11 @@
 /*
  *   mooterm-vt.h
  *
- *   Copyright (C) 2004-2006 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -248,8 +247,8 @@ enum {
             break;                                  \
                                                     \
         default:                                    \
-            g_warning ("%s: unknown mode %d",       \
-                       G_STRLOC, code);             \
+            _moo_message ("%s: unknown mode %d",    \
+                          G_STRLOC, code);          \
     }
 
 #define GET_ANSI_MODE(code, mode)                   \
@@ -287,8 +286,8 @@ enum {
             break;                                  \
                                                     \
         default:                                    \
-            g_warning ("%s: unknown mode %d",       \
-                       G_STRLOC, code);             \
+            _moo_message ("%s: unknown mode %d",    \
+                          G_STRLOC, code);          \
     }
 
 
