@@ -1,12 +1,11 @@
 /*
- *   mooterm/mooterm.c
+ *   mooterm.c
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -18,10 +17,9 @@
 #include "mooterm/mootermpt.h"
 #include "mooterm/mootermbuffer-private.h"
 #include "mooterm/mooterm-text.h"
-#include "mooutils/moocompat.h"
 #include "mooutils/moomarshals.h"
-#include "mooutils/mooutils-gobject.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/mooutils-debug.h"
 #include <string.h>
 
 
@@ -230,14 +228,13 @@ static void moo_term_class_init (MooTermClass *klass)
                           GTK_TYPE_MENU);
 
     signals[SET_WIDTH] =
-            _moo_signal_new_cb ("set-width",
-                                G_OBJECT_CLASS_TYPE (gobject_class),
-                                G_SIGNAL_RUN_LAST,
-                                NULL, /* handler */
-                                NULL, NULL,
-                                _moo_marshal_VOID__UINT,
-                                G_TYPE_NONE, 1,
-                                G_TYPE_UINT);
+            g_signal_new ("set-width",
+                          G_OBJECT_CLASS_TYPE (gobject_class),
+                          G_SIGNAL_RUN_LAST,
+                          0, NULL, NULL,
+                          _moo_marshal_VOID__UINT,
+                          G_TYPE_NONE, 1,
+                          G_TYPE_UINT);
 
     signals[APPLY_SETTINGS] =
             g_signal_new ("apply-settings",
@@ -283,7 +280,7 @@ moo_term_init (MooTerm *term)
     term->priv->incoming = g_string_new_len (NULL, INPUT_CHUNK_SIZE);
     term->priv->parser = _moo_term_parser_new (term);
 
-    _moo_term_init_palette (term);
+    term->priv->color_scheme = NULL;
 
     term->priv->primary_buffer = moo_term_buffer_new (80, 24);
     term->priv->alternate_buffer = moo_term_buffer_new (80, 24);
@@ -307,7 +304,7 @@ moo_term_init (MooTerm *term)
     term->priv->settings.scroll_on_input = TRUE;
     term->priv->settings.backspace_binding = MOO_TERM_ERASE_AUTO;
     term->priv->settings.delete_binding = MOO_TERM_ERASE_AUTO;
-    term->priv->settings.bold_pango = TRUE;
+    term->priv->settings.bold_pango = FALSE;
     term->priv->settings.bold_offset = FALSE;
 
     term->priv->pointer_visible = TRUE;
@@ -383,7 +380,7 @@ moo_term_init (MooTerm *term)
 static void
 moo_term_finalize (GObject *object)
 {
-    guint i, j;
+    guint i;
     MooTerm *term = MOO_TERM (object);
 
     if (term->priv->process_timeout)
@@ -393,6 +390,7 @@ moo_term_finalize (GObject *object)
     if (term->priv->cursor_blink_timeout_id)
         g_source_remove (term->priv->cursor_blink_timeout_id);
 
+    g_free (term->priv->color_scheme);
     _moo_term_release_selection (term);
 
     OBJECT_UNREF (term->priv->pt);
@@ -417,14 +415,6 @@ moo_term_finalize (GObject *object)
         if (term->priv->pointer[i])
             gdk_cursor_unref (term->priv->pointer[i]);
 
-    for (j = 0; j < COLOR_MAX; ++j)
-        OBJECT_UNREF (term->priv->color[j]);
-
-    for (i = 0; i < 1; ++i)
-        OBJECT_UNREF (term->priv->fg[i]);
-
-    OBJECT_UNREF (term->priv->bg);
-
     if (term->priv->pending_adjustment_changed)
         g_source_remove (term->priv->pending_adjustment_changed);
     if (term->priv->pending_adjustment_value_changed)
@@ -477,8 +467,9 @@ static void moo_term_get_property   (GObject        *object,
 }
 
 
-static void moo_term_size_allocate          (GtkWidget          *widget,
-                                             GtkAllocation      *allocation)
+static void
+moo_term_size_allocate (GtkWidget     *widget,
+                        GtkAllocation *allocation)
 {
     int old_width, old_height;
     MooTerm *term = MOO_TERM (widget);
@@ -558,7 +549,7 @@ moo_term_realize (GtkWidget *widget)
     widget->style = gtk_style_attach (widget->style, widget->window);
 
     _moo_term_init_font_stuff (term);
-    _moo_term_update_palette (term);
+    _moo_term_update_gcs (term);
     _moo_term_update_size (term, FALSE);
 
     term->priv->im = gtk_im_multicontext_new ();
@@ -597,6 +588,8 @@ moo_term_unrealize (GtkWidget *widget)
     _moo_term_font_free (term->priv->font);
     term->priv->font = NULL;
 
+    _moo_term_unrealize_gcs (term);
+
     GTK_WIDGET_CLASS(moo_term_parent_class)->unrealize (widget);
 }
 
@@ -922,7 +915,7 @@ _moo_term_update_size (MooTerm *term,
     width = CLAMP (width, MIN_TERMINAL_WIDTH, MAX_TERMINAL_WIDTH);
     height = height < MIN_TERMINAL_HEIGHT ? MIN_TERMINAL_HEIGHT : height;
 
-    _moo_term_pt_set_size (term->priv->pt, width, height);
+    moo_term_pt_set_size (term->priv->pt, width, height);
     moo_term_buffer_set_screen_size (term->priv->primary_buffer,
                                      width, height);
     moo_term_buffer_set_screen_size (term->priv->alternate_buffer,
@@ -1477,7 +1470,7 @@ _moo_term_set_mode (MooTerm    *term,
             break;
 
         default:
-            g_warning ("%s: unknown mode %d", G_STRLOC, mode);
+            _moo_message ("%s: unknown mode %d", G_STRLOC, mode);
     }
 }
 
@@ -1860,7 +1853,7 @@ _moo_term_do_popup_menu (MooTerm        *term,
     g_return_if_fail (MOO_IS_TERM (term));
 
     menu = gtk_menu_new ();
-    gtk_menu_attach_to_widget (GTK_MENU (menu), GTK_WIDGET (term), NULL);
+    MOO_OBJECT_REF_SINK (menu);
 
     item = gtk_image_menu_item_new_from_stock (GTK_STOCK_COPY, NULL);
     gtk_widget_set_sensitive (item, moo_term_get_selection_bounds (term, NULL, NULL));
@@ -1885,6 +1878,8 @@ _moo_term_do_popup_menu (MooTerm        *term,
         gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
                         (GtkMenuPositionFunc) menu_position_func,
                         term, 0, gtk_get_current_event_time ());
+
+    g_object_unref (menu);
 }
 
 
