@@ -1,12 +1,11 @@
 /*
- *   mooterm/mootermpt-cygwin.c
+ *   mootermpt-cygwin.c
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -17,7 +16,6 @@
 #include "mooterm/mootermhelper.h"
 #include "mooterm/mooterm-private.h"
 #include "mooutils/moomarshals.h"
-#include "mooutils/moocompat.h"
 #include "mooutils/mooutils-misc.h"
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
