@@ -1,12 +1,11 @@
 /*
- *   mooterm/mootermpt-unix.c
+ *   mootermpt-unix.c
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -20,8 +19,8 @@
 #include "mooterm/mooterm-private.h"
 #include "mooterm/pty.h"
 #include "mooutils/moomarshals.h"
-#include "mooutils/moocompat.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/mooutils-debug.h"
 
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -37,6 +36,11 @@
 #  include <sys/poll.h>
 #endif
 
+#undef WATCH_CHILD
+#if defined(MOO_OS_DARWIN)
+#define WATCH_CHILD
+#endif
+
 #define TERM_EMULATION          "xterm"
 #define WRITE_CHUNK_SIZE        4096
 #define POLL_TIME               5
@@ -63,9 +67,11 @@ struct _MooTermPtUnix {
     gboolean    non_block;
     GIOChannel *io;
     guint       io_watch_id;
+#ifdef WATCH_CHILD
+    guint       child_watch_id;
+#endif
 };
 
-
 struct _MooTermPtUnixClass {
     MooTermPtClass  parent_class;
 };
@@ -139,6 +145,9 @@ _moo_term_pt_unix_init (MooTermPtUnix *pt)
     pt->non_block = FALSE;
     pt->io = NULL;
     pt->io_watch_id = 0;
+#ifdef WATCH_CHILD
+    pt->child_watch_id = 0;
+#endif
 }
 
 
@@ -192,7 +201,7 @@ setup_master_fd (MooTermPtUnix *pt,
 
     g_return_val_if_fail (master >= 0, FALSE);
 
-#if 1
+#if 0
     _moo_message ("%s: attached to fd %d",
                   G_STRLOC, pt->master);
 #endif
@@ -213,7 +222,7 @@ setup_master_fd (MooTermPtUnix *pt,
 
     pt->io_watch_id = _moo_io_add_watch_full (pt->io,
                                               MOO_TERM_PT(pt)->priority,
-                                              G_IO_IN | G_IO_PRI | G_IO_HUP,
+                                              G_IO_IN | G_IO_PRI | G_IO_HUP | G_IO_ERR,
                                               (GIOFunc) read_child_out,
                                               pt, NULL);
 
@@ -225,6 +234,19 @@ setup_master_fd (MooTermPtUnix *pt,
     return TRUE;
 }
 
+#ifdef WATCH_CHILD
+static void
+child_died (GPid pid,
+            G_GNUC_UNUSED int status,
+            MooTermPtUnix *pt)
+{
+    g_return_if_fail (pid == pt->child_pid);
+    _moo_message ("%s: child exited", G_STRLOC);
+    gdk_threads_enter ();
+    kill_child (MOO_TERM_PT (pt));
+    gdk_threads_leave ();
+}
+#endif
 
 static gboolean
 fork_argv (MooTermPt      *pt_gen,
@@ -236,7 +258,7 @@ fork_argv (MooTermPt      *pt_gen,
     MooTermPtUnix *pt;
     int env_len = 0;
     char **new_env;
-    int status;
+    int status, ret;
     int i;
     int master;
 
@@ -293,18 +315,35 @@ fork_argv (MooTermPt      *pt_gen,
 
         return FALSE;
     }
-    else
+
+    ret = waitpid (pt->child_pid, &status, WNOHANG);
+
+    if (ret == -1 || ret > 0)
     {
-#if 1
-        _moo_message ("%s: forked child pid %d on fd %d",
-                      G_STRLOC, pt->child_pid, master);
-#endif
-        pt_gen->child_alive = TRUE;
-        pt_gen->alive = TRUE;
+        if (ret < 0)
+            g_critical ("%s: error in waitpid", G_STRLOC);
+        else
+            _moo_message ("%s: child died already", G_STRLOC);
+
+        pt->child_pid = -1;
+        kill_child (pt_gen);
+
+        return FALSE;
     }
 
-    if (waitpid (-1, &status, WNOHANG) == -1)
-        g_critical ("%s: error in waitpid", G_STRLOC);
+    if (0)
+        _moo_message ("%s: forked child pid %d on fd %d",
+                      G_STRLOC, pt->child_pid, master);
+
+    pt_gen->child_alive = TRUE;
+    pt_gen->alive = TRUE;
+
+#ifdef WATCH_CHILD
+    pt->child_watch_id = g_child_watch_add_full (pt_gen->priority,
+                                                 pt->child_pid,
+                                                 (GChildWatchFunc) child_died,
+                                                 pt, NULL);
+#endif
 
     setup_master_fd (pt, master);
 
@@ -343,6 +382,14 @@ kill_child (MooTermPt *pt_gen)
 {
     MooTermPtUnix *pt = MOO_TERM_PT_UNIX (pt_gen);
 
+#ifdef WATCH_CHILD
+    if (pt->child_watch_id)
+    {
+        g_source_remove (pt->child_watch_id);
+        pt->child_watch_id = 0;
+    }
+#endif
+
     if (pt->io_watch_id)
     {
         g_source_remove (pt->io_watch_id);
@@ -427,6 +474,10 @@ read_child_out (G_GNUC_UNUSED GIOChannel     *source,
     {
         int bytes;
 
+        if (0)
+            _moo_message ("reading %" G_GSIZE_FORMAT " bytes\n", to_read);
+
+        errno = 0;
         bytes = read (pt->master, buf, to_read);
 
         switch (bytes)
