@@ -1,12 +1,11 @@
 /*
  *   mooaccelprefs.c
  *
- *   Copyright (C) 2004-2006 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -189,7 +188,7 @@ _moo_accel_prefs_page_init (MooAccelPrefsPage *page)
     xml = moo_glade_xml_new_empty (GETTEXT_PACKAGE);
     moo_glade_xml_map_class (xml, "GtkButton", MOO_TYPE_ACCEL_BUTTON);
     moo_glade_xml_fill_widget (xml, GTK_WIDGET (page),
-                               MOO_ACCEL_PREFS_GLADE_UI, -1,
+                               mooaccelprefs_glade_xml, -1,
                                "page", NULL);
     g_object_set (page, "label", "Shortcuts", "icon-stock-id", MOO_STOCK_KEYBOARD, NULL);
 
@@ -668,14 +667,7 @@ shortcut_custom_toggled (MooAccelPrefsPage *page)
     }
     else
     {
-        const char *accel = get_action_accel (page->current_action);
-        g_hash_table_insert (page->changed,
-                             page->current_action,
-                             shortcut_new (CUSTOM, accel));
-
-        block_accel_set (page);
-        _moo_accel_button_set_accel (page->shortcut, accel);
-        unblock_accel_set (page);
+        gtk_button_clicked (GTK_BUTTON (page->shortcut));
     }
 }
 
@@ -723,7 +715,7 @@ _moo_accel_prefs_dialog_new (MooActionCollection *collection)
     GtkWidget *dialog, *page_holder;
     MooGladeXML *xml;
 
-    xml = moo_glade_xml_new_from_buf (MOO_ACCEL_PREFS_GLADE_UI, -1,
+    xml = moo_glade_xml_new_from_buf (mooaccelprefs_glade_xml, -1,
                                       "dialog", GETTEXT_PACKAGE, NULL);
     g_return_val_if_fail (xml != NULL, NULL);
 
@@ -736,13 +728,10 @@ _moo_accel_prefs_dialog_new (MooActionCollection *collection)
 
     g_object_unref (xml);
 
-#if GTK_MINOR_VERSION >= 6
     gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
                                              GTK_RESPONSE_OK,
                                              GTK_RESPONSE_CANCEL,
-                                             GTK_RESPONSE_REJECT,
                                              -1);
-#endif /* GTK_MINOR_VERSION >= 6 */
 
     g_signal_connect_swapped (dialog, "response",
                               G_CALLBACK (dialog_response), page);
