@@ -1,12 +1,11 @@
 /*
  *   mooaction.c
  *
- *   Copyright (C) 2004-2006 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -22,6 +21,16 @@ static void _moo_action_set_closure (MooAction  *action,
                                      MooClosure *closure);
 
 
+void
+_moo_action_ring_the_bells_it_has_tooltip (GtkAction *action)
+{
+    char *tooltip;
+    g_object_get (action, "tooltip", &tooltip, NULL);
+    g_object_set (action, "tooltip", tooltip, NULL);
+    g_free (tooltip);
+}
+
+
 gpointer
 _moo_action_get_window (gpointer action)
 {
