@@ -1,12 +1,11 @@
 /*
  *   moocombo.c
  *
- *   Copyright (C) 2004-2006 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -439,9 +438,7 @@ create_popup (MooCombo *combo)
     gtk_widget_set_size_request (GTK_WIDGET (combo->priv->treeview), -1, -1);
     gtk_tree_view_append_column (combo->priv->treeview, combo->priv->column);
     gtk_tree_view_set_headers_visible (combo->priv->treeview, FALSE);
-#if 1 && GTK_CHECK_VERSION(2,6,0)
     gtk_tree_view_set_hover_selection (combo->priv->treeview, TRUE);
-#endif
 
     selection = gtk_tree_view_get_selection (combo->priv->treeview);
     gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
@@ -917,33 +914,6 @@ popup_move_selection (MooCombo     *combo,
 }
 
 
-static void
-popup_select_iter (MooCombo     *combo,
-                   GtkTreeIter  *iter)
-{
-    char *text;
-
-    text = moo_combo_get_text_at_iter (combo, iter);
-    g_return_if_fail (text != NULL);
-
-    g_signal_handlers_block_by_func (combo->entry,
-                                     (gpointer) entry_changed, combo);
-    moo_entry_begin_undo_group (MOO_ENTRY (combo->entry));
-
-    moo_combo_popdown (combo);
-    gtk_entry_set_text (GTK_ENTRY (combo->entry), text);
-    gtk_editable_set_position (GTK_EDITABLE (combo->entry), -1);
-
-    moo_entry_end_undo_group (MOO_ENTRY (combo->entry));
-    g_signal_handlers_unblock_by_func (combo->entry,
-                                       (gpointer) entry_changed, combo);
-
-    moo_combo_set_active_iter (combo, iter);
-
-    g_free (text);
-}
-
-
 char*
 moo_combo_get_text_at_iter (MooCombo       *combo,
                             GtkTreeIter    *iter)
@@ -967,7 +937,7 @@ popup_return_key (MooCombo *combo)
     GtkTreeSelection *selection = gtk_tree_view_get_selection (treeview);
 
     if (gtk_tree_selection_get_selected (selection, NULL, &iter))
-        popup_select_iter (combo, &iter);
+        moo_combo_set_active_iter (combo, &iter);
     else
         moo_combo_popdown (combo);
 
@@ -1036,8 +1006,7 @@ list_button_press (MooCombo       *combo,
              !combo->priv->row_separator_func (combo->priv->model, &iter,
                                                combo->priv->row_separator_data))
         {
-            popup_select_iter (combo, &iter);
-            moo_combo_popdown (combo);
+            moo_combo_set_active_iter (combo, &iter);
             gtk_tree_path_free (path);
             return TRUE;
         }
@@ -1353,6 +1322,7 @@ moo_combo_set_active_iter (MooCombo       *combo,
                            GtkTreeIter    *iter)
 {
     GtkTreePath *path;
+    char *text;
 
     g_return_if_fail (MOO_IS_COMBO (combo));
     g_return_if_fail (combo->priv->model != NULL && iter != NULL);
@@ -1370,9 +1340,39 @@ moo_combo_set_active_iter (MooCombo       *combo,
         g_return_if_reached ();
     }
 
+    text = moo_combo_get_text_at_iter (combo, iter);
+    g_return_if_fail (text != NULL);
+
+    g_signal_handlers_block_by_func (combo->entry,
+                                     (gpointer) entry_changed, combo);
+    moo_entry_begin_undo_group (MOO_ENTRY (combo->entry));
+
+    moo_combo_popdown (combo);
+    gtk_entry_set_text (GTK_ENTRY (combo->entry), text);
+    gtk_editable_set_position (GTK_EDITABLE (combo->entry), -1);
+
+    moo_entry_end_undo_group (MOO_ENTRY (combo->entry));
+    g_signal_handlers_unblock_by_func (combo->entry,
+                                       (gpointer) entry_changed, combo);
+
     moo_combo_changed (combo);
 
     gtk_tree_path_free (path);
+    g_free (text);
+}
+
+void
+moo_combo_set_active (MooCombo *combo,
+                      int       row)
+{
+    GtkTreeIter iter;
+
+    g_return_if_fail (MOO_IS_COMBO (combo));
+    g_return_if_fail (row >= 0);
+    g_return_if_fail (combo->priv->model != NULL);
+
+    if (gtk_tree_model_iter_nth_child (combo->priv->model, &iter, NULL, row))
+        moo_combo_set_active_iter (combo, &iter);
 }
 
 
