@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -15,13 +14,13 @@
 #include "mooutils/mooi18n.h"
 #include "mooutils/mooencodings-data.h"
 #include "mooutils/mooprefs.h"
+#include "mooutils/mootype-macros.h"
+#include "mooutils/mooaction.h"
 #include <gtk/gtk.h>
 #include <string.h>
 #include <stdlib.h>
 
 #define MAX_RECENT_ENCODINGS 5
-#define ROW_RECENT(save_mode) ((save_mode) ? 1 : 3)
-#define ROW_LOCALE(save_mode) ((save_mode) ? 0 : 2)
 #define ROW_AUTO              0
 
 #define ELM_ENCODINGS   "Editor/encodings"
@@ -43,6 +42,12 @@ typedef struct {
     char *name;
 } EncodingGroup;
 
+typedef struct {
+    GtkWidget *menu;
+    MooEncodingsMenuFunc func;
+    gpointer func_data;
+} MenuData;
+
 typedef struct {
     GSList *recent;
     EncodingGroup *groups;
@@ -52,6 +57,9 @@ typedef struct {
     char *last_open;
     char *last_save;
     GHashTable *encodings;
+
+    GSList *menus;
+    guint update_menu_idle;
 } EncodingsManager;
 
 enum {
@@ -60,9 +68,13 @@ enum {
 };
 
 
-static void     combo_changed   (GtkComboBox        *combo,
-                                 gpointer            save_mode);
-static void     enc_mgr_load    (EncodingsManager   *enc_mgr);
+static void     combo_changed       (GtkComboBox        *combo,
+                                     gpointer            save_mode);
+static void     enc_mgr_load        (EncodingsManager   *enc_mgr);
+static void     enc_mgr_add_used    (EncodingsManager   *enc_mgr,
+                                     Encoding           *enc);
+static void     sync_recent_menu    (MenuData           *menu,
+                                     gboolean            need_separator);
 
 
 static Encoding *
@@ -158,7 +170,7 @@ sort_encoding_groups (EncodingsManager *mgr)
        5 - "Unicode" */
     const char *order = N_("012345");
     const char *new_order_s;
-    int new_order[N_ENCODING_GROUPS] = {0, 0, 0, 0, 0, 0};
+    int new_order[N_ENCODING_GROUPS] = {-1, -1, -1, -1, -1, -1};
     guint i;
     EncodingGroup *new_array;
 
@@ -185,7 +197,7 @@ sort_encoding_groups (EncodingsManager *mgr)
 
     for (i = 0; i < mgr->n_groups; ++i)
     {
-        if (!new_order[i])
+        if (new_order[i] == -1)
         {
             g_critical ("invalid order string %s", new_order_s);
             return;
@@ -272,6 +284,22 @@ get_enc_mgr (void)
 }
 
 
+const char *
+_moo_encoding_get_display_name (const char *enc_name)
+{
+    EncodingsManager *mgr;
+    Encoding *enc;
+
+    g_return_val_if_fail (enc_name != NULL, NULL);
+
+    mgr = get_enc_mgr ();
+    enc = get_encoding (mgr, enc_name);
+    g_return_val_if_fail (enc != NULL, enc_name);
+
+    return enc->display_name;
+}
+
+
 static void
 enc_mgr_save (EncodingsManager *enc_mgr)
 {
@@ -311,6 +339,18 @@ enc_mgr_save (EncodingsManager *enc_mgr)
 
     moo_markup_set_content (moo_markup_get_element (root, ELM_LAST_OPEN), enc_mgr->last_open);
     moo_markup_set_content (moo_markup_get_element (root, ELM_LAST_SAVE), enc_mgr->last_save);
+
+    if (string)
+        g_string_free (string, TRUE);
+}
+
+static const char *
+get_rid_of_locale (const char *enc_name)
+{
+    if (enc_name && !strcmp (enc_name, "locale"))
+        return _moo_encoding_locale ();
+    else
+        return enc_name;
 }
 
 static void
@@ -336,7 +376,10 @@ enc_mgr_load (EncodingsManager *enc_mgr)
             encs = g_strsplit (string, ",", 0);
             for (p = encs; p && *p; ++p)
             {
-                Encoding *e = get_encoding (enc_mgr, *p);
+                Encoding *e;
+
+                e = get_encoding (enc_mgr, get_rid_of_locale (*p));
+
                 if (e)
                     enc_mgr->recent = g_slist_prepend (enc_mgr->recent, e);
             }
@@ -348,35 +391,17 @@ enc_mgr_load (EncodingsManager *enc_mgr)
     if ((node = moo_markup_get_element (root, ELM_LAST_OPEN)))
     {
         g_free (enc_mgr->last_open);
-        enc_mgr->last_open = g_strdup (moo_markup_get_content (node));
+        enc_mgr->last_open = g_strdup (get_rid_of_locale (moo_markup_get_content (node)));
     }
 
     if ((node = moo_markup_get_element (root, ELM_LAST_SAVE)))
     {
         g_free (enc_mgr->last_save);
-        enc_mgr->last_save = g_strdup (moo_markup_get_content (node));
+        enc_mgr->last_save = g_strdup (get_rid_of_locale (moo_markup_get_content (node)));
     }
 }
 
 
-static const char *
-current_locale_name (EncodingsManager *enc_mgr)
-{
-    static char *display_name;
-
-    if (!display_name)
-    {
-        if (enc_mgr->locale_encoding)
-            display_name = g_strdup_printf (_("Current locale (%s)"),
-                                            enc_mgr->locale_encoding->short_display_name);
-        else
-            display_name = g_strdup (_("Current locale"));
-    }
-
-    return display_name;
-}
-
-
 typedef struct {
     const char *name;
     GtkTreeIter iter;
@@ -422,11 +447,27 @@ find_encoding_iter (GtkTreeModel *model,
 }
 
 
+static int
+get_row_recent (GtkTreeStore *store)
+{
+    return GPOINTER_TO_INT (g_object_get_data (G_OBJECT (store), "moo-encodings-model-row-recent"));
+}
+
+static void
+set_row_recent (GtkTreeStore *store,
+		int           row)
+{
+    g_object_set_data (G_OBJECT (store),
+                       "moo-encodings-model-row-recent",
+                       GINT_TO_POINTER (row));
+}
+
+
 static void
 sync_recent_list (GtkTreeStore *store,
                   guint         n_old_items,
                   GSList       *list,
-                  gboolean      save_mode)
+                  gboolean      add_separator)
 {
     GtkTreeIter iter;
     guint i;
@@ -434,14 +475,17 @@ sync_recent_list (GtkTreeStore *store,
     for (i = 0; i < n_old_items; ++i)
     {
         gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (store), &iter,
-                                       NULL, ROW_RECENT (save_mode));
+                                       NULL, get_row_recent (store));
         gtk_tree_store_remove (store, &iter);
     }
 
+    if (add_separator)
+        gtk_tree_store_insert (store, &iter, NULL, get_row_recent (store));
+
     while (list)
     {
         Encoding *enc = list->data;
-        gtk_tree_store_insert (store, &iter, NULL, ROW_RECENT (save_mode));
+        gtk_tree_store_insert (store, &iter, NULL, get_row_recent (store));
         gtk_tree_store_set (store, &iter,
                             COLUMN_DISPLAY, enc->display_name,
                             COLUMN_ENCODING, enc->name, -1);
@@ -460,6 +504,64 @@ set_last (EncodingsManager *mgr,
     g_free (tmp);
 }
 
+static gboolean
+update_menu_idle (EncodingsManager *mgr)
+{
+    GSList *l;
+
+    mgr->update_menu_idle = 0;
+
+    for (l = mgr->menus; l != NULL; l = l->next)
+    {
+        MenuData *menu = l->data;
+        sync_recent_menu (menu, FALSE);
+    }
+
+    enc_mgr_save (mgr);
+
+    return FALSE;
+}
+
+static void
+enc_mgr_add_used (EncodingsManager *mgr,
+                  Encoding         *new_enc)
+{
+    GSList *l;
+    gboolean found_recent;
+    guint n_recent;
+
+    n_recent = g_slist_length (mgr->recent);
+
+    for (l = mgr->recent, found_recent = FALSE; l != NULL; l = l->next)
+    {
+        Encoding *enc = l->data;
+
+        if (!strcmp (new_enc->name, enc->name))
+        {
+            if (l != mgr->recent)
+            {
+                mgr->recent = g_slist_remove_link (mgr->recent, l);
+                l->next = mgr->recent;
+                mgr->recent = l;
+            }
+
+            found_recent = TRUE;
+            break;
+        }
+    }
+
+    if (!found_recent)
+    {
+        mgr->recent = g_slist_prepend (mgr->recent, new_enc);
+
+        if (g_slist_length (mgr->recent) > MAX_RECENT_ENCODINGS)
+            mgr->recent = g_slist_delete_link (mgr->recent, g_slist_last (mgr->recent));
+    }
+
+    if (!mgr->update_menu_idle)
+        mgr->update_menu_idle = g_idle_add ((GSourceFunc) update_menu_idle, mgr);
+}
+
 static void
 encoding_combo_set_active (GtkComboBox *combo,
                            const char  *enc_name,
@@ -467,20 +569,17 @@ encoding_combo_set_active (GtkComboBox *combo,
 {
     GtkTreeModel *model;
     GtkTreeIter iter;
-    GSList *l, *recent_copy;
+    GSList *recent_copy;
     EncodingsManager *mgr;
-    gboolean found_recent;
-    guint n_recent;
+    guint n_old_recent;
     Encoding *new_enc;
+    gboolean need_separator = FALSE;
 
     mgr = get_enc_mgr ();
 
     if (!validate_encoding_name (enc_name))
         enc_name = save_mode ? MOO_ENCODING_UTF8 : MOO_ENCODING_AUTO;
 
-    if (!strcmp (enc_name, mgr->locale_encoding->name))
-        enc_name = MOO_ENCODING_LOCALE;
-
     set_last (mgr, enc_name, save_mode);
 
     model = gtk_combo_box_get_model (combo);
@@ -494,47 +593,19 @@ encoding_combo_set_active (GtkComboBox *combo,
         goto out;
     }
 
-    if (!strcmp (enc_name, MOO_ENCODING_LOCALE))
-    {
-        gtk_tree_model_iter_nth_child (model, &iter, NULL, ROW_LOCALE (save_mode));
-        gtk_combo_box_set_active_iter (combo, &iter);
-        goto out;
-    }
-
     new_enc = get_encoding (mgr, enc_name);
     g_return_if_fail (new_enc != NULL);
 
     set_last (mgr, new_enc->name, save_mode);
 
-    n_recent = g_slist_length (mgr->recent);
+    if (!mgr->recent)
+        need_separator = TRUE;
+    n_old_recent = g_slist_length (mgr->recent);
 
-    for (l = mgr->recent, found_recent = FALSE; l != NULL; l = l->next)
-    {
-        Encoding *enc = l->data;
-
-        if (!strcmp (new_enc->name, enc->name))
-        {
-            found_recent = TRUE;
-
-            if (l != mgr->recent)
-            {
-                mgr->recent = g_slist_remove_link (mgr->recent, l);
-                l->next = mgr->recent;
-                mgr->recent = l;
-            }
-        }
-    }
-
-    if (!found_recent)
-    {
-        mgr->recent = g_slist_prepend (mgr->recent, new_enc);
-
-        if (g_slist_length (mgr->recent) > MAX_RECENT_ENCODINGS)
-            mgr->recent = g_slist_delete_link (mgr->recent, g_slist_last (mgr->recent));
-    }
+    enc_mgr_add_used (mgr, new_enc);
 
     recent_copy = g_slist_reverse (g_slist_copy (mgr->recent));
-    sync_recent_list (GTK_TREE_STORE (model), n_recent, recent_copy, save_mode);
+    sync_recent_list (GTK_TREE_STORE (model), n_old_recent, recent_copy, need_separator);
     g_slist_free (recent_copy);
 
     if (find_encoding_iter (model, &iter, new_enc->name))
@@ -599,7 +670,8 @@ cell_data_func (G_GNUC_UNUSED GtkCellLayout *layout,
 static void
 setup_combo (GtkComboBox      *combo,
              EncodingsManager *enc_mgr,
-             gboolean          save_mode)
+             gboolean          save_mode,
+	     gboolean	       use_separators)
 {
     GtkCellRenderer *cell;
     GtkTreeStore *store;
@@ -618,14 +690,15 @@ setup_combo (GtkComboBox      *combo,
                             COLUMN_ENCODING, MOO_ENCODING_AUTO,
                             -1);
 
-        gtk_tree_store_append (store, &iter, NULL);
-    }
+	if (use_separators)
+	    gtk_tree_store_append (store, &iter, NULL);
 
-    gtk_tree_store_append (store, &iter, NULL);
-    gtk_tree_store_set (store, &iter,
-                        COLUMN_DISPLAY, current_locale_name (enc_mgr),
-                        COLUMN_ENCODING, MOO_ENCODING_LOCALE,
-                        -1);
+        set_row_recent (store, use_separators ? 2 : 1);
+    }
+    else
+    {
+        set_row_recent (store, 0);
+    }
 
     for (l = enc_mgr->recent; l != NULL; l = l->next)
     {
@@ -637,9 +710,12 @@ setup_combo (GtkComboBox      *combo,
                             -1);
     }
 
+    if (enc_mgr->recent && use_separators)
+        gtk_tree_store_append (store, &iter, NULL);
+
     gtk_tree_store_append (store, &iter, NULL);
-    gtk_tree_store_append (store, &iter, NULL);
-    gtk_tree_store_set (store, &iter, COLUMN_DISPLAY, "Other", -1);
+    /* Translators: do not translate the part before | */
+    gtk_tree_store_set (store, &iter, COLUMN_DISPLAY, Q_("Encodings combo submenu|Other"), -1);
 
     for (i = 0; i < enc_mgr->n_groups; ++i)
     {
@@ -663,9 +739,7 @@ setup_combo (GtkComboBox      *combo,
 
     gtk_combo_box_set_model (combo, GTK_TREE_MODEL (store));
     gtk_combo_box_entry_set_text_column (GTK_COMBO_BOX_ENTRY (combo), COLUMN_DISPLAY);
-    gtk_combo_box_set_row_separator_func (combo,
-                                          (GtkTreeViewRowSeparatorFunc) row_separator_func,
-                                          NULL, NULL);
+
     cell = gtk_cell_renderer_text_new ();
     gtk_cell_layout_clear (GTK_CELL_LAYOUT (combo));
     gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), cell, TRUE);
@@ -675,6 +749,10 @@ setup_combo (GtkComboBox      *combo,
                                         (GtkCellLayoutDataFunc) cell_data_func,
 					NULL, NULL);
 
+    if (use_separators)
+	gtk_combo_box_set_row_separator_func (combo, (GtkTreeViewRowSeparatorFunc) row_separator_func,
+					      NULL, NULL);
+
     if (save_mode)
     {
         if (enc_mgr->last_save)
@@ -701,10 +779,13 @@ setup_combo (GtkComboBox      *combo,
 
 void
 _moo_encodings_combo_init (GtkComboBox          *combo,
-                           MooEncodingComboType  type)
+                           MooEncodingComboType  type,
+			   gboolean		 use_separators)
 {
     g_return_if_fail (GTK_IS_COMBO_BOX (combo));
-    setup_combo (combo, get_enc_mgr (), type == MOO_ENCODING_COMBO_SAVE);
+    setup_combo (combo, get_enc_mgr (),
+		 type == MOO_ENCODING_COMBO_SAVE,
+		 use_separators);
 }
 
 void
@@ -741,7 +822,7 @@ _moo_encodings_attach_combo (GtkWidget  *dialog,
     gtk_widget_show (combo);
     gtk_box_pack_start (GTK_BOX (hbox), combo, TRUE, TRUE, 0);
 
-    setup_combo (GTK_COMBO_BOX (combo), get_enc_mgr (), save_mode);
+    setup_combo (GTK_COMBO_BOX (combo), get_enc_mgr (), save_mode, TRUE);
 
     if (save_mode && encoding)
         encoding_combo_set_active (GTK_COMBO_BOX (combo), encoding, save_mode);
@@ -796,9 +877,6 @@ combo_get (GtkComboBox *combo,
             enc_name = MOO_ENCODING_AUTO;
     }
 
-    if (!strcmp (enc_name, MOO_ENCODING_LOCALE))
-        enc_name = mgr->locale_encoding->name;
-
     return enc_name;
 }
 
@@ -854,3 +932,318 @@ _moo_encodings_equal (const char *enc1_name,
 
     return enc1 == enc2;
 }
+
+
+static void
+menu_data_free (MenuData *menu_data)
+{
+    g_free (menu_data);
+}
+
+static void
+menu_destroyed (GtkWidget *widget,
+                MenuData  *menu)
+{
+    EncodingsManager *mgr = get_enc_mgr ();
+
+    g_return_if_fail (g_slist_find (mgr->menus, menu) != NULL);
+
+    g_signal_handlers_disconnect_by_func (widget, (gpointer) menu_destroyed, menu);
+    mgr->menus = g_slist_remove (mgr->menus, menu);
+
+    menu_data_free (menu);
+}
+
+static void
+menu_item_activated (GtkWidget *item,
+                     MenuData  *menu)
+{
+    EncodingsManager *mgr = get_enc_mgr ();
+    Encoding *enc;
+
+    enc = g_object_get_data (G_OBJECT (item), "moo-encoding");
+    g_return_if_fail (enc != NULL);
+
+    menu->func (enc->name, menu->func_data);
+
+    if (g_slist_find (mgr->menus, menu) != NULL)
+        enc_mgr_add_used (mgr, enc);
+}
+
+static GtkWidget *
+create_menu_item (Encoding *enc,
+                  MenuData *menu,
+                  gboolean  recent)
+{
+    GtkWidget *item;
+
+    item = gtk_menu_item_new_with_label (enc->display_name);
+    gtk_widget_show (item);
+
+    g_signal_connect (item, "activate", G_CALLBACK (menu_item_activated), menu);
+    g_object_set_data (G_OBJECT (item), "moo-encoding", enc);
+
+    if (recent)
+        g_object_set_data (G_OBJECT (item), "moo-recent-encoding",
+                           GINT_TO_POINTER (TRUE));
+
+    return item;
+}
+
+static void
+sync_recent_menu (MenuData *menu,
+                  gboolean  first_time)
+{
+    EncodingsManager *mgr = get_enc_mgr ();
+    GList *children, *l;
+    GSList *recent_items;
+    gboolean have_separator = FALSE;
+    int pos = first_time ? 0 : 2;
+
+    children = gtk_container_get_children (GTK_CONTAINER (menu->menu));
+    for (l = children; l != NULL; l = l->next)
+        if (g_object_get_data (l->data, "moo-recent-encoding"))
+            gtk_container_remove (GTK_CONTAINER (menu->menu), l->data);
+    g_list_free (children);
+
+    recent_items = g_slist_reverse (g_slist_copy (mgr->recent));
+
+    while (recent_items)
+    {
+        Encoding *enc = recent_items->data;
+        GtkWidget *item;
+
+        if (first_time && !have_separator)
+        {
+            item = gtk_separator_menu_item_new ();
+            gtk_menu_shell_insert (GTK_MENU_SHELL (menu->menu), item, pos);
+            have_separator = TRUE;
+        }
+
+        item = create_menu_item (enc, menu, TRUE);
+        gtk_menu_shell_insert (GTK_MENU_SHELL (menu->menu), item, pos);
+
+        recent_items = g_slist_delete_link (recent_items, recent_items);
+    }
+}
+
+static void
+exclude_item (MenuData *menu_data,
+              Encoding *exclude_enc)
+{
+    GList *children, *l;
+
+    children = gtk_container_get_children (GTK_CONTAINER (menu_data->menu));
+
+    for (l = children; l != NULL; l = l->next)
+    {
+        GtkWidget *item = l->data;
+
+        if (g_object_get_data (G_OBJECT (item), "moo-recent-encoding"))
+        {
+            gboolean visible = TRUE;
+
+            if (exclude_enc)
+            {
+                Encoding *enc = g_object_get_data (G_OBJECT (item), "moo-encoding");
+                visible = exclude_enc != enc;
+            }
+
+            g_object_set (item, "visible", visible, NULL);
+        }
+    }
+
+    g_list_free (children);
+}
+
+static MenuData *
+_moo_encodings_menu_new (MooEncodingsMenuFunc   func,
+                         gpointer               data)
+{
+    MenuData *menu_data = NULL;
+    GtkWidget *widget;
+    EncodingsManager *mgr;
+    guint cgr;
+
+    g_return_val_if_fail (func != NULL, NULL);
+
+    mgr = get_enc_mgr ();
+
+    menu_data = g_new0 (MenuData, 1);
+    menu_data->func = func;
+    menu_data->func_data = data;
+
+    widget = gtk_menu_new ();
+    menu_data->menu = widget;
+
+    mgr->menus = g_slist_prepend (mgr->menus, menu_data);
+    g_signal_connect (widget, "destroy", G_CALLBACK (menu_destroyed), menu_data);
+
+    for (cgr = 0; cgr < mgr->n_groups; ++cgr)
+    {
+        EncodingGroup *grp = &mgr->groups[cgr];
+        GtkWidget *item, *submenu;
+        guint i;
+
+        item = gtk_menu_item_new_with_label (grp->name);
+        gtk_menu_shell_append (GTK_MENU_SHELL (widget), item);
+        submenu = gtk_menu_new ();
+        gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), submenu);
+
+        for (i = 0; i < grp->n_encodings; ++i)
+        {
+            Encoding *enc = grp->encodings[i];
+            item = create_menu_item (enc, menu_data, FALSE);
+            gtk_menu_shell_append (GTK_MENU_SHELL (submenu), item);
+        }
+    }
+
+    sync_recent_menu (menu_data, TRUE);
+
+    gtk_widget_show_all (widget);
+    return menu_data;
+}
+
+typedef struct {
+    MooAction base;
+    MooEncodingsMenuFunc func;
+    gpointer func_data;
+    Encoding *cur_enc;
+    GtkWidget *cur_item;
+    GtkWidget *cur_separator;
+    MenuData *menu_data;
+    guint update_idle;
+} MooEncodingsMenuAction;
+typedef MooActionClass MooEncodingsMenuActionClass;
+MOO_DEFINE_TYPE_STATIC (MooEncodingsMenuAction, moo_encodings_menu_action, MOO_TYPE_ACTION)
+#define MOO_TYPE_ENCODINGS_MENU_ACTION              (moo_encodings_menu_action_get_type ())
+#define MOO_ENCODINGS_MENU_ACTION(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), MOO_TYPE_ENCODINGS_MENU_ACTION, MooEncodingsMenuAction))
+#define MOO_ENCODINGS_MENU_ACTION_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), MOO_TYPE_ENCODINGS_MENU_ACTION, MooEncodingsMenuActionClass))
+#define MOO_IS_ENCODINGS_MENU_ACTION(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), MOO_TYPE_ENCODINGS_MENU_ACTION))
+#define MOO_IS_ENCODINGS_MENU_ACTION_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), MOO_TYPE_ENCODINGS_MENU_ACTION))
+#define ENCODINGS_MENU_ACTION_GET_CLASS(obj)        (G_TYPE_INSTANCE_GET_CLASS ((obj), MOO_TYPE_ENCODINGS_MENU_ACTION, MooEncodingsMenuActionClass))
+
+static void
+moo_encodings_menu_action_init (MooEncodingsMenuAction *action)
+{
+    action->func = NULL;
+    action->func_data = NULL;
+    action->cur_enc = NULL;
+    action->cur_item = NULL;
+    action->cur_separator = NULL;
+    action->menu_data = NULL;
+    g_object_set (action, "no-accel", TRUE, NULL);
+}
+
+static void
+action_item_activated (const char *encoding,
+                       gpointer    data)
+{
+    MooEncodingsMenuAction *action = data;
+
+    g_return_if_fail (action->func != NULL);
+
+    action->func (encoding, action->func_data);
+}
+
+static GtkWidget *
+moo_encodings_menu_action_create_menu_item (GtkAction *gtkaction)
+{
+    MooEncodingsMenuAction *action = MOO_ENCODINGS_MENU_ACTION (gtkaction);
+    GtkWidget *menu_item;
+
+    menu_item = GTK_ACTION_CLASS (moo_encodings_menu_action_parent_class)->create_menu_item (gtkaction);
+    action->menu_data = _moo_encodings_menu_new (action_item_activated, action);
+    gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), action->menu_data->menu);
+
+    action->cur_separator = gtk_separator_menu_item_new ();
+    gtk_menu_shell_prepend (GTK_MENU_SHELL (action->menu_data->menu), action->cur_separator);
+    action->cur_item = gtk_radio_menu_item_new_with_label (NULL, "");
+    gtk_menu_shell_prepend (GTK_MENU_SHELL (action->menu_data->menu), action->cur_item);
+
+    if (action->cur_enc)
+    {
+        Encoding *enc = action->cur_enc;
+        action->cur_enc = NULL;
+        _moo_encodings_menu_action_set_current (gtkaction, enc->name);
+    }
+
+    return menu_item;
+}
+
+static void
+moo_encodings_menu_action_class_init (MooEncodingsMenuActionClass *klass)
+{
+    GTK_ACTION_CLASS (klass)->create_menu_item = moo_encodings_menu_action_create_menu_item;
+}
+
+GtkAction *
+_moo_encodings_menu_action_new (const char            *id,
+                                const char            *label,
+                                MooEncodingsMenuFunc   func,
+                                gpointer               data)
+{
+    MooEncodingsMenuAction *action;
+
+    g_return_val_if_fail (id != NULL, NULL);
+    g_return_val_if_fail (label != NULL, NULL);
+    g_return_val_if_fail (func != NULL, NULL);
+
+    action = g_object_new (MOO_TYPE_ENCODINGS_MENU_ACTION,
+                           "name", id, "label", label, NULL);
+    action->func = func;
+    action->func_data = data;
+
+    return GTK_ACTION (action);
+}
+
+static gboolean
+update_recent_list_visibility (MooEncodingsMenuAction *action)
+{
+    action->update_idle = 0;
+
+    if (action->cur_enc)
+    {
+        GtkWidget *child;
+
+        gtk_widget_show (action->cur_item);
+        gtk_widget_show (action->cur_separator);
+
+        child = GTK_BIN (action->cur_item)->child;
+        gtk_label_set_text (GTK_LABEL (child), action->cur_enc->display_name);
+
+        exclude_item (action->menu_data, action->cur_enc);
+    }
+    else
+    {
+        gtk_widget_hide (action->cur_item);
+        gtk_widget_hide (action->cur_separator);
+    }
+
+    return FALSE;
+}
+
+void
+_moo_encodings_menu_action_set_current (GtkAction  *gtkaction,
+                                        const char *enc_name)
+{
+    MooEncodingsMenuAction *action;
+    Encoding *enc;
+    EncodingsManager *mgr;
+
+    g_return_if_fail (MOO_IS_ENCODINGS_MENU_ACTION (gtkaction));
+
+    action = MOO_ENCODINGS_MENU_ACTION (gtkaction);
+
+    mgr = get_enc_mgr ();
+    enc = enc_name ? get_encoding (mgr, enc_name) : NULL;
+
+    if (enc != action->cur_enc)
+    {
+        action->cur_enc = enc;
+        if (action->cur_item && !action->update_idle)
+            action->update_idle = g_idle_add_full (G_PRIORITY_DEFAULT_IDLE + 1,
+                                                   (GSourceFunc) update_recent_list_visibility,
+                                                   action, NULL);
+    }
+}
