@@ -1,23 +1,23 @@
 /*
- *   mooutils/moofiledialog.c
+ *   moofiledialog.c
  *
- *   Copyright (C) 2004-2006 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
 
+#include "config.h"
 #include <gtk/gtk.h>
 #include "mooutils/moodialogs.h"
 #include "mooutils/mooprefs.h"
 #include "mooutils/mooutils-misc.h"
-#include "mooutils/moocompat.h"
 #include "mooutils/moomarshals.h"
 #include "mooutils/mooencodings.h"
+#include "mooutils/moohelp.h"
 #include <string.h>
 
 
@@ -32,6 +32,8 @@ struct _MooFileDialogPrivate {
     char *filter_mgr_id;
     gboolean enable_encodings;
 
+    char *help_id;
+
     GSList *filenames;
     char *filename;
     char *encoding;
@@ -61,12 +63,34 @@ enum {
 
 static guint signals[LAST_SIGNAL];
 
+static char *
+get_string_maybe_stock (const char *string)
+{
+    GtkStockItem item;
+    char *underscore;
+    char *copy;
+
+    if (!gtk_stock_lookup (string, &item))
+        return g_strdup (string);
+
+    if (!(underscore = strchr (item.label, '_')))
+        return g_strdup (item.label);
+
+    copy = g_strdup (item.label);
+    memmove (copy + (underscore - item.label),
+             copy + (underscore - item.label) + 1,
+             strlen (copy + (underscore - item.label) + 1) + 1);
+
+    return copy;
+}
+
 static void
 moo_file_dialog_set_property (GObject        *object,
                               guint           prop_id,
                               const GValue   *value,
                               GParamSpec     *pspec)
 {
+    char *tmp;
     MooFileDialog *dialog = MOO_FILE_DIALOG (object);
 
     switch (prop_id)
@@ -77,20 +101,19 @@ moo_file_dialog_set_property (GObject        *object,
             break;
 
         case PROP_TITLE:
-            g_free (dialog->priv->title);
-            dialog->priv->title = g_value_dup_string (value);
+            tmp = dialog->priv->title;
+            dialog->priv->title = get_string_maybe_stock (g_value_get_string (value));
+            g_free (tmp);
             g_object_notify (object, "title");
             break;
 
         case PROP_DIR:
-            g_free (dialog->priv->dir);
-            dialog->priv->dir = g_value_dup_string (value);
+            MOO_ASSIGN_STRING (dialog->priv->dir, g_value_get_string (value));
             g_object_notify (object, "dir");
             break;
 
         case PROP_NAME:
-            g_free (dialog->priv->name);
-            dialog->priv->name = g_value_dup_string (value);
+            MOO_ASSIGN_STRING (dialog->priv->name, g_value_get_string (value));
             g_object_notify (object, "name");
             break;
 
@@ -203,6 +226,7 @@ moo_file_dialog_finalize (GObject *object)
     g_free (dialog->priv->filter_mgr_id);
     g_free (dialog->priv->filename);
     g_free (dialog->priv->encoding);
+    g_free (dialog->priv->help_id);
 
     string_slist_free (dialog->priv->filenames);
 
@@ -317,13 +341,12 @@ moo_file_dialog_class_init (MooFileDialogClass *klass)
 }
 
 
-#if GTK_CHECK_VERSION(2,4,0)
-
 inline static
 GtkWidget *file_chooser_dialog_new (const char *title,
                                     GtkFileChooserAction action,
                                     const char *okbtn,
-                                    const char *start_dir)
+                                    const char *start_dir,
+                                    const char *help_id)
 {
     GtkWidget *dialog =
             gtk_file_chooser_dialog_new (title, NULL, action,
@@ -331,16 +354,25 @@ GtkWidget *file_chooser_dialog_new (const char *title,
                                          okbtn, GTK_RESPONSE_OK,
                                          NULL);
 
-#if GTK_CHECK_VERSION(2,6,0)
     gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
                                              GTK_RESPONSE_OK,
                                              GTK_RESPONSE_CANCEL,
                                              -1);
-#endif /* GTK_CHECK_VERSION(2,6,0) */
 
     if (start_dir)
         gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(dialog),
                                              start_dir);
+
+
+#ifdef MOO_ENABLE_HELP
+    if (help_id)
+    {
+        moo_help_set_id (dialog, help_id);
+        moo_help_connect_keys (dialog);
+        gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_HELP, GTK_RESPONSE_HELP);
+    }
+#endif
+
     return dialog;
 }
 
@@ -353,52 +385,17 @@ GtkWidget *file_chooser_dialog_new (const char *title,
 #define file_chooser_set_name(dialog, name) \
     gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), name)
 
-#else /* !GTK_CHECK_VERSION(2,4,0) */
-
-#define GtkFileChooserAction int
-#define GTK_FILE_CHOOSER_ACTION_SAVE            1
-#define GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER   2
-#define GTK_FILE_CHOOSER_ACTION_OPEN            3
-
-inline static
-GtkWidget *file_chooser_dialog_new (const char              *title,
-                                    G_GNUC_UNUSED GtkFileChooserAction action,
-                                    G_GNUC_UNUSED const char *okbtn,
-                                    const char              *start_dir)
-{
-    GtkWidget *dialog = gtk_file_selection_new (title);
-
-    if (start_dir)
-    {
-        char *dir = g_strdup_printf ("%s/", start_dir);
-        gtk_file_selection_set_filename (GTK_FILE_SELECTION (dialog), dir);
-        g_free (dir);
-    }
-
-    return dialog;
-}
-
-#define file_chooser_set_select_multiple(dialog,multiple) \
-    gtk_file_selection_set_select_multiple (GTK_FILE_SELECTION (dialog), multiple)
-#define file_chooser_get_filename(dialog) \
-    g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (dialog)))
-#define file_chooser_get_filenames(dialog)  \
-    (gtk_file_selection_get_selections (GTK_FILE_SELECTION (dialog)))
-#define file_chooser_set_name(dialog, name)
-
-#endif /* !GTK_CHECK_VERSION(2,4,0) */
-
-
 const char *
 moo_file_dialog (GtkWidget  *parent,
                  MooFileDialogType type,
+                 const char *start_name,
                  const char *title,
                  const char *start_dir)
 {
     static char *filename;
     MooFileDialog *dialog;
 
-    dialog = moo_file_dialog_new (type, parent, FALSE, title, start_dir, NULL);
+    dialog = moo_file_dialog_new (type, parent, FALSE, title, start_dir, start_name);
     g_return_val_if_fail (dialog != NULL, NULL);
 
     moo_file_dialog_run (dialog);
@@ -431,7 +428,8 @@ moo_file_dialog_create_widget (MooFileDialog *dialog)
             widget = file_chooser_dialog_new (dialog->priv->title,
                                               chooser_action,
                                               GTK_STOCK_OPEN,
-                                              dialog->priv->dir);
+                                              dialog->priv->dir,
+                                              dialog->priv->help_id);
             file_chooser_set_select_multiple (widget, dialog->priv->multiple);
             break;
 
@@ -441,7 +439,8 @@ moo_file_dialog_create_widget (MooFileDialog *dialog)
             widget = file_chooser_dialog_new (dialog->priv->title,
                                               chooser_action,
                                               GTK_STOCK_SAVE,
-                                              dialog->priv->dir);
+                                              dialog->priv->dir,
+                                              dialog->priv->help_id);
 
             if (dialog->priv->name)
                 file_chooser_set_name (widget, dialog->priv->name);
@@ -515,9 +514,7 @@ static void
 set_encoding (MooFileDialog *dialog,
               const char    *encoding)
 {
-    char *tmp = dialog->priv->encoding;
-    dialog->priv->encoding = g_strdup (encoding);
-    g_free (tmp);
+    MOO_ASSIGN_STRING (dialog->priv->encoding, encoding);
 }
 
 void
@@ -588,6 +585,7 @@ moo_file_dialog_run (MooFileDialog *dialog)
     char *filename;
     GtkWidget *filechooser;
     gboolean result = FALSE;
+    int response;
 
     g_return_val_if_fail (MOO_IS_FILE_DIALOG (dialog), FALSE);
 
@@ -601,20 +599,29 @@ moo_file_dialog_run (MooFileDialog *dialog)
         case MOO_FILE_DIALOG_OPEN:
         case MOO_FILE_DIALOG_OPEN_ANY:
         case MOO_FILE_DIALOG_OPEN_DIR:
-            if (gtk_dialog_run (GTK_DIALOG (filechooser)) == GTK_RESPONSE_OK)
+            while ((response = gtk_dialog_run (GTK_DIALOG (filechooser))) == GTK_RESPONSE_HELP)
+                moo_help_open (filechooser);
+            if (response == GTK_RESPONSE_OK)
             {
                 set_filename (dialog, file_chooser_get_filename (filechooser));
                 if (dialog->priv->multiple)
                     set_filenames (dialog, file_chooser_get_filenames (filechooser));
                 result = TRUE;
             }
-
             goto out;
 
         case MOO_FILE_DIALOG_SAVE:
             while (TRUE)
             {
-                if (GTK_RESPONSE_OK == gtk_dialog_run (GTK_DIALOG (filechooser)))
+                response = gtk_dialog_run (GTK_DIALOG (filechooser));
+
+                if (response == GTK_RESPONSE_HELP)
+                {
+                    moo_help_open (filechooser);
+                    continue;
+                }
+
+                if (response == GTK_RESPONSE_OK)
                 {
                     char *msg;
 
@@ -711,6 +718,7 @@ moo_file_dialog_get_encoding (MooFileDialog *dialog)
 const char *
 moo_file_dialogp (GtkWidget          *parent,
                   MooFileDialogType   type,
+                  const char         *start_name,
                   const char         *title,
                   const char         *prefs_key,
                   const char         *alternate_prefs_key)
@@ -733,7 +741,7 @@ moo_file_dialogp (GtkWidget          *parent,
         start = moo_prefs_get_string (alternate_prefs_key);
     }
 
-    filename = moo_file_dialog (parent, type, title, start);
+    filename = moo_file_dialog (parent, type, start_name, title, start);
 
     if (filename && prefs_key)
     {
@@ -795,3 +803,13 @@ moo_file_dialog_set_filter_mgr (MooFileDialog  *dialog,
     g_return_if_fail (!mgr || MOO_IS_FILTER_MGR (mgr));
     g_object_set (dialog, "filter-mgr", mgr, "filter-mgr-id", id, NULL);
 }
+
+
+void
+moo_file_dialog_set_help_id (MooFileDialog *dialog,
+                             const char    *id)
+{
+    g_return_if_fail (MOO_IS_FILE_DIALOG (dialog));
+    g_free (dialog->priv->help_id);
+    dialog->priv->help_id = g_strdup (id);
+}
