@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -44,7 +43,6 @@
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/moofilewatch.h"
 #include "mooutils/moomarshals.h"
-#include "mooutils/moocompat.h"
 
 
 #if 1
@@ -414,7 +412,7 @@ moo_file_watch_create_monitor (MooFileWatch   *watch,
     {
         g_set_error (error, MOO_FILE_WATCH_ERROR,
                      MOO_FILE_WATCH_ERROR_CLOSED,
-                     "MooFileWatch %d closed",
+                     "MooFileWatch %u closed",
                      watch->id);
         return 0;
     }
