@@ -3,27 +3,13 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
 
-#include "mooutils/mooglade.h"
-#include "mooutils/moomarkup.h"
-#include "mooutils/mooutils-gobject.h"
-#include "mooutils/mooutils-misc.h"
-#include "mooutils/moocompat.h"
-#include "mooutils/mooprefsdialog.h"
-#include "mooutils/mooprefs.h"
-#include "mooutils/mooi18n.h"
-#include <gtk/gtk.h>
-#include <string.h>
-#include <errno.h>
-#include <stdlib.h>
-
 #ifdef GTK_DISABLE_DEPRECATED
 #undef GTK_DISABLE_DEPRECATED
 #include <gtk/gtkcombo.h>
@@ -33,6 +19,17 @@
 #define GTK_DISABLE_DEPRECATED
 #endif
 
+#include "mooutils/mooglade.h"
+#include "mooutils/moomarkup.h"
+#include "mooutils/mooutils-gobject.h"
+#include "mooutils/mooutils-misc.h"
+#include "mooutils/mooprefsdialog.h"
+#include "mooutils/mooprefs.h"
+#include "mooutils/mooi18n.h"
+#include <gtk/gtk.h>
+#include <string.h>
+#include <errno.h>
+#include <stdlib.h>
 
 G_DEFINE_TYPE (MooGladeXML, moo_glade_xml, G_TYPE_OBJECT)
 
@@ -930,12 +927,11 @@ pack_children (MooGladeXML    *xml,
         else if (child->widget->props->mask & PROP_RESPONSE_ID)
         {
             Child *parent = child->parent_node->parent_node;
-            if (!parent || !parent->internal_child ||
-                 strcmp (parent->internal_child, "action_area"))
-            {
-                g_warning ("%s: oops", G_STRLOC);
-            }
-            else
+
+            /* glade saves response_id with all buttons now, so do not complain
+             * if response_id found not in a dialog button */
+            if (parent && parent->internal_child &&
+                !strcmp (parent->internal_child, "action_area"))
             {
                 GtkWidget *dialog = moo_glade_xml_get_widget (xml, parent->internal_parent_id);
 
@@ -1017,11 +1013,13 @@ collect_signals (MooMarkupNode  *node,
             handler = moo_markup_get_prop (elm, "handler");
             object = moo_markup_get_prop (elm, "object");
             g_return_if_fail (name != NULL && handler != NULL);
-            widget->signals = g_slist_append (widget->signals,
-                                              signal_new (name, handler, object));
+            widget->signals = g_slist_prepend (widget->signals,
+                                               signal_new (name, handler, object));
         }
     }
     FOREACH_ELM_END;
+
+    widget->signals = g_slist_reverse (widget->signals);
 }
 
 
@@ -1413,7 +1411,6 @@ widget_props_add (WidgetProps  *props,
         props->mask |= PROP_HISTORY;
         props->history = parse_int (value);
     }
-#if GTK_CHECK_VERSION(2,4,0)
     else if (!strcmp (name, "items") &&
              GTK_IS_COMBO_BOX_CLASS (klass))
     {
@@ -1422,7 +1419,6 @@ widget_props_add (WidgetProps  *props,
 //             g_message ("%s: ignoring ComboBox items property", G_STRLOC);
 #endif
     }
-#endif /* GTK_CHECK_VERSION(2,4,0) */
     else
     {
         special = FALSE;
@@ -1488,10 +1484,6 @@ get_property_text (MooMarkupNode *node,
     gboolean translatable = FALSE, strip_context = FALSE;
     const char *prop, *translatable_attr;
 
-#ifndef ENABLE_NLS
-    return moo_markup_get_content (node);
-#endif
-
     prop = moo_markup_get_content (node);
 
     if (!prop || !prop[0])
@@ -1906,7 +1898,7 @@ parse_property (GParamSpec     *param_spec,
     }
     else if (param_spec->value_type == G_TYPE_STRV)
     {
-        char **strv = _moo_splitlines (value);
+        char **strv = moo_splitlines (value);
         g_value_take_boxed (&param->value, strv);
     }
     else
@@ -2619,16 +2611,12 @@ get_type_by_name (const char *name)
     if (type)
         return type;
 
-#if GTK_CHECK_VERSION(2,6,0)
     REGISTER_TYPE ("GtkAboutDialog", GTK_TYPE_ABOUT_DIALOG);
     REGISTER_TYPE ("GtkCellRendererCombo", GTK_TYPE_CELL_RENDERER_COMBO);
     REGISTER_TYPE ("GtkCellRendererProgress", GTK_TYPE_CELL_RENDERER_PROGRESS);
     REGISTER_TYPE ("GtkCellView", GTK_TYPE_CELL_VIEW);
     REGISTER_TYPE ("GtkIconView", GTK_TYPE_ICON_VIEW);
     REGISTER_TYPE ("GtkMenuToolButton", GTK_TYPE_MENU_TOOL_BUTTON);
-#endif
-
-#if GTK_CHECK_VERSION(2,4,0)
     REGISTER_TYPE ("GtkAccelMap", GTK_TYPE_ACCEL_MAP);
     REGISTER_TYPE ("GtkAction", GTK_TYPE_ACTION);
     REGISTER_TYPE ("GtkActionGroup", GTK_TYPE_ACTION_GROUP);
@@ -2659,7 +2647,6 @@ get_type_by_name (const char *name)
     REGISTER_TYPE ("GtkTreeModelFilter", GTK_TYPE_TREE_MODEL_FILTER);
     REGISTER_TYPE ("GtkUiManager", GTK_TYPE_UI_MANAGER);
     REGISTER_TYPE ("GtkUiManagerItemType", GTK_TYPE_UI_MANAGER_ITEM_TYPE);
-#endif /* GTK_CHECK_VERSION(2,4,0) */
     REGISTER_TYPE ("GtkAccelFlags", GTK_TYPE_ACCEL_FLAGS);
     REGISTER_TYPE ("GtkAccelGroup", GTK_TYPE_ACCEL_GROUP);
     REGISTER_TYPE ("GtkAccelLabel", GTK_TYPE_ACCEL_LABEL);
@@ -2827,7 +2814,6 @@ get_type_by_name (const char *name)
     REGISTER_TYPE ("GtkToolbarChildType", GTK_TYPE_TOOLBAR_CHILD_TYPE);
     REGISTER_TYPE ("GtkToolbarSpaceStyle", GTK_TYPE_TOOLBAR_SPACE_STYLE);
     REGISTER_TYPE ("GtkToolbarStyle", GTK_TYPE_TOOLBAR_STYLE);
-    REGISTER_TYPE ("GtkTooltips", GTK_TYPE_TOOLTIPS);
     REGISTER_TYPE ("GtkTreeDragDest", GTK_TYPE_TREE_DRAG_DEST);
     REGISTER_TYPE ("GtkTreeDragSource", GTK_TYPE_TREE_DRAG_SOURCE);
     REGISTER_TYPE ("GtkTreeIter", GTK_TYPE_TREE_ITER);
