@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -182,9 +181,8 @@ set_history_list_id (MooHistoryCombo *combo,
 {
     if (id)
     {
-        MooHistoryList *list = moo_history_list_new (id);
+        MooHistoryList *list = moo_history_list_get (id);
         moo_history_combo_set_list (combo, list);
-        g_object_unref (list);
     }
 }
 
@@ -482,9 +480,8 @@ moo_history_combo_new (const char *user_id)
 
     if (user_id)
     {
-        MooHistoryList *list = moo_history_list_new (user_id);
+        MooHistoryList *list = moo_history_list_get (user_id);
         moo_history_combo_set_list (combo, list);
-        g_object_unref (list);
     }
 
     return GTK_WIDGET (combo);
