@@ -1,12 +1,11 @@
 /*
- *   mooutils/moomarkup.c
+ *   moomarkup.c
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -14,8 +13,7 @@
 #include "mooutils/moomarkup.h"
 #include "mooutils/mooutils-fs.h"
 #include "mooutils/mooutils-gobject.h"
-#include "mooutils/mooutils-debug.h"
-#include "mooutils/moocompat.h"
+#include "mooutils/mooutils-misc.h"
 #include <string.h>
 #include <glib.h>
 
@@ -24,10 +22,12 @@ GType
 moo_markup_doc_get_type (void)
 {
     static GType type = 0;
+
     if (G_UNLIKELY (!type))
         type = g_boxed_type_register_static ("MooMarkupDoc",
-                                             (GBoxedCopyFunc)moo_markup_doc_ref,
-                                             (GBoxedFreeFunc)moo_markup_doc_unref);
+                                             (GBoxedCopyFunc) moo_markup_doc_ref,
+                                             (GBoxedFreeFunc) moo_markup_doc_unref);
+
     return type;
 }
 
@@ -53,16 +53,12 @@ typedef void (*markup_passthrough_func)     (GMarkupParseContext    *context,
                                              gpointer                user_data,
                                              GError                **error);
 
-
 typedef struct {
     MooMarkupDoc   *doc;
     MooMarkupNode  *current;
 } ParserState;
 
 
-#define BUFSIZE 1024
-
-
 static MooMarkupDoc     *moo_markup_doc_new_priv    (const char         *name);
 static void              moo_markup_doc_set_name    (MooMarkupDoc       *doc,
                                                      const char         *name);
@@ -75,7 +71,12 @@ static MooMarkupNode    *moo_markup_text_node_new   (MooMarkupNodeType   type,
                                                      MooMarkupDoc       *doc,
                                                      MooMarkupNode      *parent,
                                                      const char         *text,
-                                                     gsize               text_len);
+                                                     gssize              text_len);
+static MooMarkupNode    *moo_markup_text_node_new_take_string (MooMarkupNodeType   type,
+                                                     MooMarkupDoc       *doc,
+                                                     MooMarkupNode      *parent,
+                                                     char               *text,
+                                                     gssize              text_len);
 static void              add_node                   (MooMarkupDoc       *doc,
                                                      MooMarkupNode      *parent,
                                                      MooMarkupNode      *node);
@@ -89,7 +90,7 @@ static void moo_markup_node_free                (MooMarkupNode      *node);
 static void collect_text_content                (MooMarkupElement   *node);
 static void moo_markup_text_node_add_text       (MooMarkupText      *node,
                                                  const char         *text,
-                                                 gsize               text_len);
+                                                 gssize              text_len);
 
 static void moo_markup_element_print            (MooMarkupElement   *node,
                                                  GString            *dest);
@@ -127,7 +128,7 @@ set_modified (MooMarkupNode *node,
 {
     if (node->doc)
     {
-#ifdef MOO_DEBUG
+#ifdef MOO_DEBUG_ENABLED
         if (modified && !node->doc->modified && node->doc->track_modified)
         {
             g_message ("markup doc %s modified",
@@ -166,57 +167,66 @@ _moo_markup_get_modified (MooMarkupDoc *doc)
 }
 
 
-MooMarkupDoc*
-moo_markup_parse_memory (const char     *buffer,
-                         int             size,
-                         GError        **error)
+static MooMarkupDoc *
+markup_parse_memory (const char     *buffer,
+                     gssize          len,
+                     G_GNUC_UNUSED const char *filename,
+                     GError        **error)
 {
-    GMarkupParser parser = {(markup_start_element_func)start_element,
-                            (markup_end_element_func)end_element,
-                            (markup_text_func)text,
-                            (markup_passthrough_func)passthrough,
-                            NULL};
+    GMarkupParser parser = { (markup_start_element_func) start_element,
+                             (markup_end_element_func) end_element,
+                             (markup_text_func) text,
+                             (markup_passthrough_func) passthrough,
+                             NULL};
 
-    MooMarkupDoc *doc;
     ParserState state;
     GMarkupParseContext *context;
 
-    if (size < 0)
-        size = strlen (buffer);
+    if (len < 0)
+        len = strlen (buffer);
 
-    doc = moo_markup_doc_new_priv (NULL);
-    state.doc = doc;
-    state.current = MOO_MARKUP_NODE (doc);
-    context = g_markup_parse_context_new (&parser, (GMarkupParseFlags)0, &state, NULL);
+    state.doc = moo_markup_doc_new_priv (NULL);
+    _moo_markup_set_modified (state.doc, TRUE);
 
-    _moo_markup_set_modified (doc, TRUE);
+    state.current = MOO_MARKUP_NODE (state.doc);
+    context = g_markup_parse_context_new (&parser, (GMarkupParseFlags) 0, &state, NULL);
 
-    if (!g_markup_parse_context_parse (context, buffer, size, error) ||
+    if (!g_markup_parse_context_parse (context, buffer, len, error) ||
          !g_markup_parse_context_end_parse (context, error))
     {
         g_markup_parse_context_free (context);
-        moo_markup_doc_unref (doc);
+        moo_markup_doc_unref (state.doc);
         return NULL;
     }
 
     g_markup_parse_context_free (context);
-    return doc;
+    return state.doc;
+}
+
+MooMarkupDoc *
+moo_markup_parse_memory (const char     *buffer,
+                         int             size,
+                         GError        **error)
+{
+    g_return_val_if_fail (buffer != NULL, NULL);
+    return markup_parse_memory (buffer, size, "memory", error);
 }
 
 
-MooMarkupDoc*
+MooMarkupDoc *
 moo_markup_parse_file (const char     *filename,
                        GError        **error)
 {
     char *content;
     MooMarkupDoc *doc;
+    gsize len;
 
     g_return_val_if_fail (filename != NULL, NULL);
 
-    if (!g_file_get_contents (filename, &content, NULL, error))
+    if (!g_file_get_contents (filename, &content, &len, error))
         return NULL;
 
-    doc = moo_markup_parse_memory (content, -1, error);
+    doc = markup_parse_memory (content, len, filename, error);
 
     if (doc)
         moo_markup_doc_set_name (doc, filename);
@@ -291,10 +301,10 @@ passthrough (G_GNUC_UNUSED GMarkupParseContext    *ctx,
 }
 
 
-static MooMarkupDoc*
+static MooMarkupDoc *
 moo_markup_doc_new_priv (const char *name)
 {
-    MooMarkupDoc *doc = g_new0 (MooMarkupDoc, 1);
+    MooMarkupDoc *doc = _moo_new0 (MooMarkupDoc);
 
     doc->type = MOO_MARKUP_DOC_NODE;
     doc->name = g_strdup (name ? name : "");
@@ -309,38 +319,35 @@ static void
 moo_markup_doc_set_name (MooMarkupDoc *doc,
                          const char   *name)
 {
-    char *tmp;
-
     g_return_if_fail (MOO_MARKUP_IS_DOC (doc));
-
-    tmp = doc->name;
-    doc->name = g_strdup (name ? name : "");
-    g_free (tmp);
+    MOO_ASSIGN_STRING (doc->name, name ? name : "");
 }
 
 
-MooMarkupDoc*
+MooMarkupDoc *
 moo_markup_doc_new (const char *name)
 {
     return moo_markup_doc_new_priv (name);
 }
 
 
-static MooMarkupNode*
+static MooMarkupNode *
 moo_markup_element_new (MooMarkupDoc   *doc,
                         MooMarkupNode  *parent,
                         const char     *name,
                         const char    **attribute_names,
                         const char    **attribute_values)
 {
-    MooMarkupElement *elm = g_new0 (MooMarkupElement, 1);
+    MooMarkupElement *elm;
+
+    elm = _moo_new0 (MooMarkupElement);
     add_node (doc, parent, MOO_MARKUP_NODE (elm));
 
     elm->type = MOO_MARKUP_ELEMENT_NODE;
     elm->name = g_strdup (name);
 
-    elm->attr_names = g_strdupv ((char**)attribute_names);
-    elm->attr_vals = g_strdupv ((char**)attribute_values);
+    elm->attr_names = g_strdupv ((char**) attribute_names);
+    elm->attr_vals = g_strdupv ((char**) attribute_values);
     if (elm->attr_names)
         for (elm->n_attrs = 0; attribute_names[elm->n_attrs]; ++elm->n_attrs) ;
     else
@@ -350,17 +357,18 @@ moo_markup_element_new (MooMarkupDoc   *doc,
 }
 
 
-static MooMarkupNode*
-moo_markup_text_node_new (MooMarkupNodeType   type,
-                          MooMarkupDoc       *doc,
-                          MooMarkupNode      *parent,
-                          const char         *text,
-                          gsize               text_len)
+static MooMarkupNode *
+moo_markup_text_node_new_take_string (MooMarkupNodeType   type,
+                                      MooMarkupDoc       *doc,
+                                      MooMarkupNode      *parent,
+                                      char               *text,
+                                      gssize              text_len)
 {
     MooMarkupText *node;
 
     g_assert (type == MOO_MARKUP_TEXT_NODE || type == MOO_MARKUP_COMMENT_NODE);
-    node = g_new0 (MooMarkupText, 1);
+
+    node = _moo_new0 (MooMarkupText);
     add_node (doc, parent, MOO_MARKUP_NODE (node));
 
     node->type = type;
@@ -370,12 +378,30 @@ moo_markup_text_node_new (MooMarkupNodeType   type,
     else
         node->name = g_strdup ("COMMENT");
 
-    node->text = g_strndup (text, text_len);
+    if (text_len < 0)
+        text_len = strlen (text);
+
+    node->text = text;
     node->size = text_len;
 
     return MOO_MARKUP_NODE (node);
 }
 
+static MooMarkupNode *
+moo_markup_text_node_new (MooMarkupNodeType   type,
+                          MooMarkupDoc       *doc,
+                          MooMarkupNode      *parent,
+                          const char         *text,
+                          gssize              text_len)
+{
+    if (text_len < 0)
+        text_len = strlen (text);
+
+    return moo_markup_text_node_new_take_string (type, doc, parent,
+                                                 g_strndup (text, text_len),
+                                                 text_len);
+}
+
 
 static void
 add_node (MooMarkupDoc     *doc,
@@ -410,9 +436,17 @@ add_node (MooMarkupDoc     *doc,
 static void
 moo_markup_text_node_add_text (MooMarkupText  *node,
                                const char     *text,
-                               gsize           text_len)
+                               gssize          text_len)
 {
-    char *tmp = g_new (char, node->size + text_len + 1);
+    char *tmp;
+
+    if (text_len < 0)
+        text_len = strlen (text);
+
+    if (text_len == 0)
+        return;
+
+    tmp = g_new (char, node->size + text_len + 1);
     memcpy (tmp, node->text, node->size);
     memcpy (tmp + node->size, text, text_len);
     g_free (node->text);
@@ -460,15 +494,17 @@ static void
 moo_markup_node_free (MooMarkupNode *node)
 {
     MooMarkupNode *child;
-    GSList *children = NULL, *l;
 
     g_return_if_fail (node != NULL);
 
-    for (child = node->children; child != NULL; child = child->next)
-        children = g_slist_prepend (children, child);
+    child = node->children;
 
-    for (l = children; l != NULL; l = l->next)
-        moo_markup_node_free (l->data);
+    while (child)
+    {
+        MooMarkupNode *next = child->next;
+        moo_markup_node_free (child);
+        child = next;
+    }
 
     g_free (node->name);
 
@@ -476,20 +512,23 @@ moo_markup_node_free (MooMarkupNode *node)
     {
         case MOO_MARKUP_DOC_NODE:
             moo_markup_doc_unref_private (MOO_MARKUP_DOC (node));
+            _moo_free (MooMarkupDoc, (MooMarkupDoc*) node);
             break;
         case MOO_MARKUP_ELEMENT_NODE:
             moo_markup_element_free (MOO_MARKUP_ELEMENT (node));
+            _moo_free (MooMarkupElement, (MooMarkupElement*) node);
             break;
         case MOO_MARKUP_TEXT_NODE:
+            moo_markup_text_node_free (node);
+            _moo_free (MooMarkupText, (MooMarkupText*) node);
+            break;
         case MOO_MARKUP_COMMENT_NODE:
             moo_markup_text_node_free (node);
+            _moo_free (MooMarkupComment, (MooMarkupComment*) node);
             break;
         default:
             g_assert_not_reached ();
     }
-
-    g_slist_free (children);
-    g_free (node);
 }
 
 
@@ -636,7 +675,7 @@ moo_markup_comment_node_print (MooMarkupNode *node,
         *p = 0;
     }
 
-    g_string_append (str, text->text);
+    g_string_append_printf (str, "<!--%s-->", text->text);
 }
 
 
@@ -1160,9 +1199,13 @@ format_pretty_element (MooMarkupElement *elm,
     g_string_append_len (str, fill, indent);
     g_string_append_printf (str, "<%s", elm->name);
     for (i = 0; i < elm->n_attrs; ++i)
+    {
+        char *attr = g_markup_escape_text (elm->attr_vals[i], -1);
         g_string_append_printf (str, " %s=\"%s\"",
                                 elm->attr_names[i],
-                                elm->attr_vals[i]);
+                                attr);
+        g_free (attr);
+    }
 
     if (empty)
     {
@@ -1218,6 +1261,24 @@ moo_markup_save_pretty (MooMarkupDoc       *doc,
     return result;
 }
 
+char *
+moo_markup_format_pretty (MooMarkupDoc *doc,
+                          guint         indent)
+{
+    GString *str;
+    MooMarkupNode *child;
+
+    g_return_val_if_fail (MOO_MARKUP_IS_DOC (doc), FALSE);
+
+    str = g_string_new ("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
+
+    for (child = doc->children; child != NULL; child = child->next)
+        if (MOO_MARKUP_IS_ELEMENT (child))
+            format_pretty_element (MOO_MARKUP_ELEMENT (child), str, 0, indent);
+
+    return g_string_free (str, FALSE);
+}
+
 
 #if 0
 char *
