@@ -1,18 +1,16 @@
 /*
  *   moomenumgr.c
  *
- *   Copyright (C) 2004-2006 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
 
 #include "mooutils/moomenumgr.h"
-#include "mooutils/moocompat.h"
 #include "mooutils/moomarshals.h"
 #include "mooutils/mooutils-misc.h"
 #include <gtk/gtk.h>
@@ -343,6 +341,7 @@ mgr_remove (MooMenuMgr         *mgr,
 
     g_slist_free (items_to_free);
     g_slist_free (menus_to_free);
+    g_slist_free (menus);
 
     ensure_active_item (mgr);
 
