@@ -1,23 +1,20 @@
 /*
- *   mooutils/mooprefs.c
+ *   mooprefs.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
 
 #include "mooutils/mooprefs.h"
-#include "mooutils/moocompat.h"
 #include "mooutils/moomarshals.h"
 #include "mooutils/moomarkup.h"
 #include "mooutils/mooutils-fs.h"
 #include "mooutils/mooutils-gobject.h"
-#include "mooutils/mooutils-debug.h"
 #include "mooutils/mootype-macros.h"
 #include <string.h>
 #include <errno.h>
@@ -28,10 +25,7 @@
 #define PREFS_ROOT "Prefs"
 /* #define DEBUG_READWRITE 1 */
 
-#if !GLIB_CHECK_VERSION(2,4,0)
-#define g_value_take_string g_value_set_string_take_ownership
-#endif
-
+#define MOO_PREFS_SYS -1
 
 #define MOO_TYPE_PREFS              (_moo_prefs_get_type ())
 #define MOO_PREFS(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), MOO_TYPE_PREFS, MooPrefs))
@@ -67,6 +61,7 @@ typedef struct {
     GValue value;
     GValue default_value;
     guint  prefs_type : 1;
+    guint  overridden : 1;
 } PrefsItem;
 
 
@@ -78,7 +73,7 @@ static void          prefs_new_key      (MooPrefs       *prefs,
 static void          prefs_new_key_from_string (MooPrefs *prefs,
                                          const char     *key,
                                          const char     *value,
-                                         MooPrefsType    prefs_type);
+                                         int             prefs_type);
 static void          prefs_delete_key   (MooPrefs       *prefs,
                                          const char     *key);
 static GType         prefs_get_key_type (MooPrefs       *prefs,
@@ -117,16 +112,15 @@ static gboolean      item_set_default   (PrefsItem      *item,
 static void          moo_prefs_finalize (GObject        *object);
 static void          moo_prefs_new_key_from_string (const char *key,
                                          const char     *value,
-                                         MooPrefsType    prefs_type);
+                                         int             prefs_type);
 
 static void          moo_prefs_set_modified (gboolean    modified);
 
 
-/* MOO_TYPE_PREFS */
 MOO_DEFINE_TYPE_STATIC (MooPrefs, _moo_prefs, G_TYPE_OBJECT)
 
 
-static MooPrefs*
+static MooPrefs *
 instance (void)
 {
     static MooPrefs *p = NULL;
@@ -361,7 +355,7 @@ moo_prefs_delete_key (const char     *key)
 static void
 moo_prefs_new_key_from_string (const char     *key,
                                const char     *value,
-                               MooPrefsType    prefs_type)
+                               int             prefs_type)
 {
     g_return_if_fail (key != NULL);
     g_return_if_fail (value != NULL);
@@ -374,7 +368,7 @@ moo_prefs_set_modified (gboolean modified)
 {
     MooPrefs *prefs = instance ();
 
-#ifdef MOO_DEBUG
+#ifdef MOO_DEBUG_ENABLED
     if (modified && !prefs->rc_modified)
     {
         g_message ("%s: prefs modified", G_STRLOC);
@@ -385,6 +379,38 @@ moo_prefs_set_modified (gboolean modified)
 }
 
 
+static void
+prepend_key (const char *key,
+             PrefsItem  *item,
+             gpointer    pdata)
+{
+    struct {
+        GSList *list;
+        MooPrefsType prefs_type;
+    } *data = pdata;
+
+    if (data->prefs_type == item->prefs_type)
+        data->list = g_slist_prepend (data->list, g_strdup (key));
+}
+
+GSList *
+moo_prefs_list_keys (MooPrefsType prefs_type)
+{
+    MooPrefs *prefs = instance ();
+
+    struct {
+        GSList *list;
+        MooPrefsType prefs_type;
+    } data;
+
+    data.list = NULL;
+    data.prefs_type = prefs_type;
+    g_hash_table_foreach (prefs->data, (GHFunc) prepend_key, &data);
+
+    return g_slist_sort (data.list, (GCompareFunc) strcmp);
+}
+
+
 /***************************************************************************/
 /* Prefs
  */
@@ -418,10 +444,11 @@ prefs_new_key (MooPrefs       *prefs,
     {
         changed = item_set_type (item, type);
 
-        if (item_set_default (item, default_value))
+        if (!item->overridden && item_set_default (item, default_value))
             changed = TRUE;
 
-        if (item->prefs_type == MOO_PREFS_RC &&
+        if (!item->overridden &&
+            item->prefs_type == MOO_PREFS_RC &&
             item->prefs_type != prefs_type)
                 moo_prefs_set_modified (TRUE);
 
@@ -437,7 +464,7 @@ static void
 prefs_new_key_from_string (MooPrefs     *prefs,
                            const char   *key,
                            const char   *value,
-                           MooPrefsType  prefs_type)
+                           int           prefs_type)
 {
     PrefsItem *item;
     GValue val, default_val;
@@ -453,16 +480,30 @@ prefs_new_key_from_string (MooPrefs     *prefs,
     default_val.g_type = 0;
     g_value_init (&default_val, G_TYPE_STRING);
 
+    if (prefs_type == MOO_PREFS_SYS)
+        g_value_set_static_string (&default_val, value);
+
     if (!item)
     {
-        prefs_new_key (prefs, key, G_TYPE_STRING, &default_val, prefs_type);
+        prefs_new_key (prefs, key, G_TYPE_STRING, &default_val,
+                       prefs_type == MOO_PREFS_SYS ? MOO_PREFS_RC : prefs_type);
         item = prefs_get_item (prefs, key);
         item_set (item, &val);
+
+        if (prefs_type == MOO_PREFS_SYS)
+            item->overridden = TRUE;
     }
     else
     {
         _moo_value_convert (&val, &item->value);
-        item->prefs_type = prefs_type;
+
+        if (prefs_type == MOO_PREFS_SYS)
+        {
+            _moo_value_convert (&default_val, &item->default_value);
+            item->overridden = TRUE;
+        }
+        else
+            item->prefs_type = prefs_type;
     }
 
     g_value_unset (&val);
@@ -975,7 +1016,7 @@ moo_prefs_notify_disconnect (guint id)
 
 static void
 process_element (MooMarkupElement *elm,
-                 MooPrefsType      prefs_type)
+                 int               prefs_type)
 {
     MooMarkupNode *child;
     gboolean dir = FALSE;
@@ -997,7 +1038,8 @@ process_element (MooMarkupElement *elm,
 
         path = moo_markup_element_get_path (elm);
 
-        if (strlen (path) < strlen (PREFS_ROOT) + 2)
+        if (strncmp (path, PREFS_ROOT "/", strlen (PREFS_ROOT "/")) != 0 ||
+            strlen (path) < strlen (PREFS_ROOT) + 2)
         {
             g_free (path);
             g_return_if_reached ();
@@ -1024,7 +1066,7 @@ process_element (MooMarkupElement *elm,
 
 static gboolean
 load_file (const char  *file,
-           MooPrefsType prefs_type,
+           int          prefs_type,
            GError     **error)
 {
     MooMarkupDoc *xml;
@@ -1042,6 +1084,11 @@ load_file (const char  *file,
         case MOO_PREFS_STATE:
             target = &prefs->xml_state;
             break;
+        case MOO_PREFS_SYS:
+            target = NULL;
+            break;
+        default:
+            g_assert_not_reached ();
     }
 
     if (!g_file_test (file, G_FILE_TEST_EXISTS))
@@ -1052,13 +1099,16 @@ load_file (const char  *file,
     if (!xml)
         return FALSE;
 
-    if (*target)
+    if (target)
     {
-        g_warning ("%s: implement me", G_STRLOC);
-        moo_markup_doc_unref (*target);
-    }
+        if (*target)
+        {
+            g_warning ("%s: implement me", G_STRLOC);
+            moo_markup_doc_unref (*target);
+        }
 
-    *target = xml;
+        *target = moo_markup_doc_ref (xml);
+    }
 
     if (prefs_type != MOO_PREFS_STATE)
     {
@@ -1068,27 +1118,33 @@ load_file (const char  *file,
 
     root = moo_markup_get_root_element (xml, PREFS_ROOT);
 
-    if (!root)
-        return TRUE;
-
-    process_element (MOO_MARKUP_ELEMENT (root), prefs_type);
+    if (root)
+        process_element (MOO_MARKUP_ELEMENT (root), prefs_type);
 
+    moo_markup_doc_unref (xml);
     return TRUE;
 }
 
 
 gboolean
-moo_prefs_load (const char     *file_rc,
+moo_prefs_load (char          **sys_files,
+                const char     *file_rc,
                 const char     *file_state,
                 GError        **error)
 {
-    g_return_val_if_fail (file_rc != NULL, FALSE);
-    g_return_val_if_fail (file_state != NULL, FALSE);
-
     moo_prefs_set_modified (FALSE);
 
-    return load_file (file_rc, MOO_PREFS_RC, error) &&
-           load_file (file_state, MOO_PREFS_STATE, error);
+    for (; sys_files && *sys_files; ++sys_files)
+        if (!load_file (*sys_files, MOO_PREFS_SYS, error))
+            return FALSE;
+
+    if (file_rc && !load_file (file_rc, MOO_PREFS_RC, error))
+        return FALSE;
+
+    if (file_state && !load_file (file_state, MOO_PREFS_STATE, error))
+        return FALSE;
+
+    return TRUE;
 }
 
 
@@ -1250,7 +1306,8 @@ moo_prefs_save (const char  *file_rc,
 
     if (save_file (file_rc, MOO_PREFS_RC, error))
     {
-        _moo_markup_set_modified (prefs->xml_rc, FALSE);
+        if (prefs->xml_rc)
+            _moo_markup_set_modified (prefs->xml_rc, FALSE);
         moo_prefs_set_modified (FALSE);
     }
     else
