@@ -1,12 +1,11 @@
 /*
- *   mooutils/mooprefsdialogpage.c
+ *   mooprefsdialogpage.c
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -14,9 +13,9 @@
 #define MOOPREFS_COMPILATION
 #include "mooutils/moomarshals.h"
 #include "mooutils/moostock.h"
-#include "mooutils/moocompat.h"
 #include "mooutils/mooprefs.h"
 #include "mooutils/mooprefsdialogpage.h"
+#include "mooutils/mooutils-misc.h"
 #include "mooutils/mooutils-gobject.h"
 #include "mooutils/moofontsel.h"
 #include "mooutils/mooi18n.h"
@@ -60,7 +59,7 @@ enum {
 };
 
 
-static guint prefs_dialog_page_signals[LAST_SIGNAL] = {0};
+static G_GNUC_UNUSED guint prefs_dialog_page_signals[LAST_SIGNAL] = {0};
 
 
 /* MOO_TYPE_PREFS_DIALOG_PAGE */
