@@ -1,19 +1,17 @@
 /*
- *   mooui/moouixml.c
+ *   moouixml.c
  *
- *   Copyright (C) 2004-2006 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
 
 #include "mooutils/mooaction-private.h"
 #include "mooutils/moouixml.h"
-#include "mooutils/moocompat.h"
 #include "mooutils/moomarshals.h"
 #include "mooutils/moomenutoolbutton.h"
 #include "mooutils/mooutils-misc.h"
@@ -123,7 +121,6 @@ static gboolean node_is_ancestor        (Node           *node,
                                          Node           *ancestor);
 static gboolean node_is_empty           (Node           *node);
 static GSList  *node_list_children      (Node           *ndoe);
-static GSList  *node_list_all_children  (Node           *ndoe);
 static void     node_free               (Node           *node);
 static void     node_foreach            (Node           *node,
                                          NodeForeachFunc func,
@@ -1911,7 +1908,6 @@ check_separators (Node           *parent,
     GSList *children, *l;
     Node *separator = NULL;
     gboolean first = TRUE;
-    gboolean has_children = FALSE;
     GtkWidget *widget;
 
     if (!toplevel_get_widget (toplevel, parent))
@@ -1931,8 +1927,6 @@ check_separators (Node           *parent,
                 if (!widget || !GTK_WIDGET_VISIBLE (widget))
                     continue;
 
-                has_children = TRUE;
-
                 if (!first)
                 {
                     if (separator)
@@ -2051,12 +2045,8 @@ create_tool_separator (MooUIXML       *xml,
 }
 
 
-#if GTK_CHECK_VERSION(2,6,0)
 #define IS_MENU_TOOL_BUTTON(wid) (GTK_IS_MENU_TOOL_BUTTON (wid) || \
                                   MOO_IS_MENU_TOOL_BUTTON (wid))
-#else
-#define IS_MENU_TOOL_BUTTON MOO_IS_MENU_TOOL_BUTTON
-#endif
 
 static void
 menu_tool_button_set_menu (GtkWidget *button,
@@ -2064,10 +2054,8 @@ menu_tool_button_set_menu (GtkWidget *button,
 {
     if (MOO_IS_MENU_TOOL_BUTTON (button))
         moo_menu_tool_button_set_menu (MOO_MENU_TOOL_BUTTON (button), menu);
-#if GTK_CHECK_VERSION(2,6,0)
     else if (GTK_IS_MENU_TOOL_BUTTON (button))
         gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (button), menu);
-#endif
     else
         g_return_if_reached ();
 }
@@ -2078,10 +2066,8 @@ menu_tool_button_get_menu (GtkWidget *button)
 {
     if (MOO_IS_MENU_TOOL_BUTTON (button))
         return moo_menu_tool_button_get_menu (MOO_MENU_TOOL_BUTTON (button));
-#if GTK_CHECK_VERSION(2,6,0)
     else if (GTK_IS_MENU_TOOL_BUTTON (button))
         return gtk_menu_tool_button_get_menu (GTK_MENU_TOOL_BUTTON (button));
-#endif
     else
         g_return_val_if_reached (NULL);
 }
@@ -2128,7 +2114,7 @@ create_tool_item (MooUIXML       *xml,
             index = -1;
 
         gtk_toolbar_insert (toolbar, GTK_TOOL_ITEM (tool_item), index);
-        g_object_notify (G_OBJECT (action), "tooltip");
+        _moo_action_ring_the_bells_it_has_tooltip (action);
 
         if (node->children)
         {
@@ -2154,8 +2140,6 @@ create_tool_item (MooUIXML       *xml,
         tool_item = moo_menu_tool_button_new ();
         gtk_widget_show (tool_item);
 
-        if (item->tooltip)
-            _moo_widget_set_tooltip (tool_item, item->tooltip);
         if (item->icon_stock_id)
             gtk_tool_button_set_stock_id (GTK_TOOL_BUTTON (tool_item),
                                           item->icon_stock_id);
@@ -2168,6 +2152,9 @@ create_tool_item (MooUIXML       *xml,
 
         gtk_toolbar_insert (toolbar, GTK_TOOL_ITEM (tool_item), index);
 
+        if (item->tooltip)
+            _moo_widget_set_tooltip (tool_item, item->tooltip);
+
         menu = gtk_menu_new ();
         /* XXX empty menu */
         gtk_widget_show (menu);
@@ -2475,43 +2462,20 @@ toplevel_add_node (MooUIXML *xml,
 }
 
 
-static GSList*
-node_list_all_children (Node *node)
-{
-    GSList *list, *l;
-
-    g_return_val_if_fail (node != NULL, NULL);
-
-    list = g_slist_append (NULL, node);
-
-    for (l = node->children; l != NULL; l = l->next)
-        list = g_slist_append (list, node_list_all_children (l->data));
-
-    return list;
-}
-
-
 static void
 toplevel_remove_node (G_GNUC_UNUSED MooUIXML *xml,
                       Toplevel *toplevel,
                       Node     *node)
 {
-    GSList *children, *l;
+    GtkWidget *widget;
 
     g_return_if_fail (node != toplevel->node);
     g_return_if_fail (node_is_ancestor (node, toplevel->node));
 
-    children = node_list_all_children (node);
+    widget = g_hash_table_lookup (toplevel->children, node);
 
-    for (l = children; l != NULL; l = l->next)
-    {
-        GtkWidget *widget = g_hash_table_lookup (toplevel->children, node);
-
-        if (widget)
-            gtk_widget_destroy (widget);
-    }
-
-    g_slist_free (children);
+    if (widget)
+        gtk_widget_destroy (widget);
 }
 
 
