@@ -1,12 +1,11 @@
 /*
  *   mooutils-fs.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -16,7 +15,7 @@
 #endif
 
 #include "mooutils/mooutils-fs.h"
-#include "mooutils/mooutils-misc.h"
+#include "mooutils/mooutils-debug.h"
 #include "mooutils/moocompat.h"
 
 #include <string.h>
@@ -583,7 +582,7 @@ normalize_path_string (const char *path)
 #ifndef __WIN32__
 static char *
 normalize_full_path (const char *path,
-                     const char *is_folder)
+                     gboolean    is_folder)
 {
     guint len;
     char *normpath, *tmp;
@@ -713,8 +712,9 @@ normalize_full_path (const char *fullpath,
 }
 #endif
 
-char *
-_moo_normalize_file_path (const char *filename)
+static char *
+normalize_path (const char *filename,
+                gboolean    is_folder)
 {
     char *freeme = NULL;
     char *norm_filename;
@@ -729,12 +729,24 @@ _moo_normalize_file_path (const char *filename)
         filename = freeme;
     }
 
-    norm_filename = normalize_full_path (filename, FALSE);
+    norm_filename = normalize_full_path (filename, is_folder);
 
     g_free (freeme);
     return norm_filename;
 }
 
+char *
+_moo_normalize_file_path (const char *filename)
+{
+    return normalize_path (filename, FALSE);
+}
+
+char *
+_moo_normalize_dir_path (const char *filename)
+{
+    return normalize_path (filename, TRUE);
+}
+
 
 /**********************************************************************/
 /* MSLU for poor
