@@ -1,12 +1,11 @@
 /*
  *   mooutils-gobject.c
  *
- *   Copyright (C) 2004-2006 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -218,10 +217,8 @@ _moo_param_spec_gtype (const char     *name,
 /* Converting values forth and back
  */
 
-/* TODO */
 static char *
-flags_to_string (int   flags,
-                 GType type)
+flags_to_string (int flags)
 {
     if (flags)
         return g_strdup_printf ("%d", flags);
@@ -232,8 +229,7 @@ flags_to_string (int   flags,
 
 static gboolean
 string_to_flags (const char *string,
-                 int        *flags,
-                 GType       type)
+                 int        *flags)
 {
     GValue ival;
 
@@ -357,7 +353,7 @@ _moo_value_convert (const GValue *src,
 
         if (G_TYPE_IS_FLAGS (src_type))
         {
-            char *string = flags_to_string (g_value_get_flags (src), src_type);
+            char *string = flags_to_string (g_value_get_flags (src));
             g_value_set_string (dest, string);
             g_free (string);
             return TRUE;
@@ -508,7 +504,7 @@ _moo_value_convert (const GValue *src,
         {
             int flags;
 
-            if (string_to_flags (string, &flags, dest_type))
+            if (string_to_flags (string, &flags))
             {
                 g_value_set_flags (dest, flags);
                 return TRUE;
@@ -1138,26 +1134,6 @@ _moo_param_array_free (GParameter *array,
 /* Signal that does not require class method
  */
 
-static void
-void_marshal (G_GNUC_UNUSED GClosure *closure,
-              G_GNUC_UNUSED GValue *return_value,
-              G_GNUC_UNUSED guint n_param_values,
-              G_GNUC_UNUSED const GValue *param_values,
-              G_GNUC_UNUSED gpointer invocation_hint,
-              G_GNUC_UNUSED gpointer marshal_data)
-{
-}
-
-
-static GClosure*
-void_closure_new (void)
-{
-    GClosure *closure = g_closure_new_simple (sizeof (GClosure), NULL);
-    g_closure_set_marshal (closure, void_marshal);
-    return closure;
-}
-
-
 guint
 _moo_signal_new_cb (const gchar        *signal_name,
                     GType               itype,
@@ -1172,21 +1148,18 @@ _moo_signal_new_cb (const gchar        *signal_name,
 {
     va_list args;
     guint signal_id;
+    GClosure *closure = NULL;
 
     g_return_val_if_fail (signal_name != NULL, 0);
 
     va_start (args, n_params);
 
     if (handler)
-        signal_id = g_signal_new_valist (signal_name, itype, signal_flags,
-                                         g_cclosure_new (handler, NULL, NULL),
-                                         accumulator, accu_data, c_marshaller,
-                                         return_type, n_params, args);
-    else
-        signal_id = g_signal_new_valist (signal_name, itype, signal_flags,
-                                         void_closure_new (),
-                                         accumulator, accu_data, c_marshaller,
-                                         return_type, n_params, args);
+        closure = g_cclosure_new (handler, NULL, NULL);
+
+    signal_id = g_signal_new_valist (signal_name, itype, signal_flags, closure,
+                                     accumulator, accu_data, c_marshaller,
+                                     return_type, n_params, args);
 
     va_end (args);
 
