@@ -3,10 +3,9 @@
  *
  *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -17,11 +16,12 @@
 
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooutils-fs.h"
+#include "mooutils/mooutils-debug.h"
 #include "mooutils/moologwindow-glade.h"
 #include "mooutils/mooglade.h"
 #include "mooutils/mooi18n.h"
-#include "mooutils/moocompat.h"
 #include <gtk/gtk.h>
+#include <glib/gmappedfile.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -44,141 +44,110 @@
 
 
 G_LOCK_DEFINE_STATIC (moo_user_data_dir);
+G_LOCK_DEFINE_STATIC (moo_temp_dir);
+static char *moo_app_instance_name;
 static char *moo_user_data_dir;
+static char *moo_temp_dir;
+
+static void     moo_install_atexit      (void);
+static void     moo_remove_tempdir      (void);
 
 
 #ifdef __WIN32__
 
 static gboolean
 open_uri (const char *uri,
-          G_GNUC_UNUSED gboolean want_browser)
+          G_GNUC_UNUSED gboolean email)
 {
     return _moo_win32_open_uri (uri);
 }
 
 #else /* ! __WIN32__ */
 
+char *
+_moo_find_script (const char *name,
+                  gboolean    search_path)
+{
+    char *path = NULL;
 
-typedef enum {
-    KDE,
-    GNOME,
-    DEBIAN,
-    XFCE,
-    UNKNOWN
-} Desktop;
+    g_return_val_if_fail (name != NULL, NULL);
 
+    if (search_path)
+        path = g_find_program_in_path (name);
 
-static gboolean open_uri (const char *uri,
-                          gboolean    want_browser)
+    if (!path)
+    {
+        char **dirs, **p;
+
+        dirs = moo_get_data_subdirs ("scripts", MOO_DATA_SHARE, NULL);
+
+        for (p = dirs; p && *p; ++p)
+        {
+            path = g_build_filename (*p, name, NULL);
+
+            if (g_file_test (path, G_FILE_TEST_IS_EXECUTABLE))
+                break;
+
+            g_free (path);
+            path = NULL;
+        }
+
+        g_strfreev (dirs);
+    }
+
+    if (!path)
+    {
+        g_warning ("%s: could not find %s script", G_STRLOC, name);
+        path = g_strdup (name);
+    }
+
+    _moo_message ("%s: %s", name, path);
+    return path;
+}
+
+enum {
+    SCRIPT_XDG_OPEN,
+    SCRIPT_XDG_EMAIL
+};
+
+static const char *
+get_xdg_script (int type)
+{
+    static char *path[2];
+
+    g_return_val_if_fail (type < 2, NULL);
+
+    if (!path[0])
+    {
+        path[SCRIPT_XDG_OPEN] = _moo_find_script ("xdg-open", TRUE);
+        path[SCRIPT_XDG_EMAIL] = _moo_find_script ("xdg-email", TRUE);
+    }
+
+    return path[type];
+}
+
+static gboolean
+open_uri (const char *uri,
+          gboolean    email)
 {
     gboolean result = FALSE;
-    char **argv = NULL;
+    const char *argv[3];
+    GError *err = NULL;
 
-    Desktop desktop = UNKNOWN;
-    char *kfmclient = g_find_program_in_path ("kfmclient");
-    char *gnome_open = g_find_program_in_path ("gnome-open");
-    char *x_www_browser = g_find_program_in_path ("x-www-browser");
-    char *exo_open = g_find_program_in_path ("exo-open");
+    argv[0] = get_xdg_script (email ? SCRIPT_XDG_EMAIL : SCRIPT_XDG_OPEN);
+    argv[1] = uri;
+    argv[2] = NULL;
 
-    if (desktop == UNKNOWN)
+    result = g_spawn_async (NULL, (char**) argv, NULL, (GSpawnFlags) 0, NULL, NULL,
+                            NULL, &err);
+
+    if (err)
     {
-        if (kfmclient && g_getenv ("KDE_FULL_SESSION"))
-            desktop = KDE;
-        else if (gnome_open && g_getenv ("GNOME_DESKTOP_SESSION_ID"))
-            desktop = GNOME;
+        g_warning ("%s: error in g_spawn_async", G_STRLOC);
+        g_warning ("%s: %s", G_STRLOC, err->message);
+        g_error_free (err);
     }
 
-#ifdef GDK_WINDOWING_X11
-    if (desktop == UNKNOWN)
-    {
-        const char *wm =
-            gdk_x11_screen_get_window_manager_name (gdk_screen_get_default ());
-
-        if (wm)
-        {
-            if (!g_ascii_strcasecmp (wm, "kwin") && kfmclient)
-                desktop = KDE;
-            else if (!g_ascii_strcasecmp (wm, "metacity") && gnome_open)
-                desktop = GNOME;
-            else if (!g_ascii_strcasecmp (wm, "xfwm4") && exo_open)
-                desktop = XFCE;
-        }
-    }
-#endif /* GDK_WINDOWING_X11 */
-
-    if (desktop == UNKNOWN)
-    {
-        if (want_browser && x_www_browser)
-            desktop = DEBIAN;
-        else if (kfmclient)
-            desktop = KDE;
-        else if (gnome_open)
-            desktop = GNOME;
-        else if (exo_open)
-            desktop = XFCE;
-    }
-
-    switch (desktop)
-    {
-        case KDE:
-            argv = g_new0 (char*, 4);
-            argv[0] = g_strdup (kfmclient);
-            argv[1] = g_strdup ("exec");
-            argv[2] = g_strdup (uri);
-            break;
-
-        case GNOME:
-            argv = g_new0 (char*, 3);
-            argv[0] = g_strdup (gnome_open);
-            argv[1] = g_strdup (uri);
-            break;
-
-        case XFCE:
-            argv = g_new0 (char*, 3);
-            argv[0] = g_strdup (exo_open);
-            argv[1] = g_strdup (uri);
-            break;
-
-        case DEBIAN:
-            argv = g_new0 (char*, 3);
-            argv[0] = g_strdup (x_www_browser);
-            argv[1] = g_strdup (uri);
-            break;
-
-        case UNKNOWN:
-            if (uri)
-                g_warning ("could not find a way to open uri '%s'", uri);
-            else
-                g_warning ("could not find a way to open uri");
-            break;
-
-        default:
-            g_assert_not_reached ();
-    }
-
-    if (argv)
-    {
-        GError *err = NULL;
-
-        result = g_spawn_async (NULL, argv, NULL, (GSpawnFlags)0, NULL, NULL,
-                                NULL, &err);
-
-        if (err)
-        {
-            g_warning ("%s: error in g_spawn_async", G_STRLOC);
-            g_warning ("%s: %s", G_STRLOC, err->message);
-            g_error_free (err);
-        }
-    }
-
-    if (argv)
-        g_strfreev (argv);
-
-    g_free (gnome_open);
-    g_free (kfmclient);
-    g_free (x_www_browser);
-    g_free (exo_open);
-
     return result;
 }
 
@@ -203,7 +172,7 @@ moo_open_email (const char *address,
     if (body)
         g_string_append_printf (uri, "body=%s", body);
 
-    res = open_uri (uri->str, FALSE);
+    res = open_uri (uri->str, TRUE);
     g_string_free (uri, TRUE);
     return res;
 }
@@ -213,7 +182,7 @@ gboolean
 moo_open_url (const char *url)
 {
     g_return_val_if_fail (url != NULL, FALSE);
-    return open_uri (url, TRUE);
+    return open_uri (url, FALSE);
 }
 
 
@@ -375,6 +344,9 @@ _moo_get_top_window (GSList *windows)
 
     g_return_val_if_fail (windows != NULL, NULL);
 
+    if (!windows->next)
+        return windows->data;
+
     for (l = windows; l != NULL; l = l->next)
     {
         if (!GTK_IS_WINDOW (l->data))
@@ -519,6 +491,9 @@ _moo_get_top_window (GSList *windows)
 
     g_return_val_if_fail (windows != NULL, NULL);
 
+    if (!windows->next)
+        return windows->data;
+
     for (l = windows; l != NULL; l = l->next)
     {
         g_return_val_if_fail (GTK_IS_WINDOW (l->data), NULL);
@@ -672,7 +647,7 @@ moo_log_window_new (void)
     MooLogWindow *log;
     PangoFontDescription *font;
 
-    xml = moo_glade_xml_new_from_buf (MOO_LOG_WINDOW_GLADE_UI, -1,
+    xml = moo_glade_xml_new_from_buf (moologwindow_glade_xml, -1,
                                       NULL, GETTEXT_PACKAGE, NULL);
     log = g_new (MooLogWindow, 1);
 
@@ -903,12 +878,12 @@ print_func_file (const char *string)
 
     if (!moo_log_file_written)
     {
-        file = fopen (moo_log_file, "w+");
+        file = fopen (moo_log_file, "wb+");
         moo_log_file_written = TRUE;
     }
     else
     {
-        file = fopen (moo_log_file, "a+");
+        file = fopen (moo_log_file, "ab+");
     }
 
     if (file)
@@ -939,16 +914,10 @@ log_func_file (const char       *log_domain,
     }
 #endif /* __WIN32__ */
 
-#ifdef __WIN32__
-#define LT "\r\n"
-#else
-#define LT "\n"
-#endif
     if (log_domain)
-        string = g_strdup_printf ("%s: %s" LT, log_domain, message);
+        string = g_strdup_printf ("%s: %s\n", log_domain, message);
     else
-        string = g_strdup_printf ("%s" LT, message);
-#undef LT
+        string = g_strdup_printf ("%s\n", message);
 
     print_func_file (string);
     g_free (string);
@@ -1073,6 +1042,23 @@ _moo_get_modifiers (GtkWidget *widget)
 /* GtkAccelLabel helpers
  */
 
+static void
+accel_label_set_string (GtkWidget  *accel_label,
+                        const char *label)
+{
+    g_return_if_fail (GTK_IS_ACCEL_LABEL (accel_label));
+    g_free (GTK_ACCEL_LABEL(accel_label)->accel_string);
+    GTK_ACCEL_LABEL(accel_label)->accel_string = g_strdup (label);
+    gtk_widget_queue_resize (accel_label);
+}
+
+static void
+accel_label_screen_changed (GtkWidget  *accel_label)
+{
+    const char *label = g_object_get_data (G_OBJECT (accel_label), "moo-accel-label-accel");
+    accel_label_set_string (accel_label, label);
+}
+
 void
 _moo_menu_item_set_accel_label (GtkWidget  *menu_item,
                                 const char *label)
@@ -1087,10 +1073,11 @@ _moo_menu_item_set_accel_label (GtkWidget  *menu_item,
     accel_label = gtk_bin_get_child (GTK_BIN (menu_item));
     g_return_if_fail (GTK_IS_ACCEL_LABEL (accel_label));
 
-    g_free (GTK_ACCEL_LABEL(accel_label)->accel_string);
-    GTK_ACCEL_LABEL(accel_label)->accel_string = g_strdup (label);
-
-    gtk_widget_queue_resize (accel_label);
+    g_signal_connect_after (accel_label, "screen-changed",
+                            G_CALLBACK (accel_label_screen_changed), NULL);
+    g_object_set_data_full (G_OBJECT (accel_label), "moo-accel-label-accel",
+                            g_strdup (label), g_free);
+    accel_label_set_string (accel_label, label);
 }
 
 
@@ -1131,7 +1118,7 @@ _moo_get_pid_string (void)
 #ifdef __WIN32__
         moo_pid_string = g_strdup_printf ("%ld", GetCurrentProcessId ());
 #else
-        moo_pid_string = g_strdup_printf ("%d", getpid ());
+        moo_pid_string = g_strdup_printf ("%ld", (long) getpid ());
 #endif
     }
 
@@ -1158,6 +1145,123 @@ moo_get_prgname (void)
 }
 
 
+char *
+moo_tempnam (void)
+{
+    int i;
+    char *filename = NULL;
+    static int counter;
+    G_LOCK_DEFINE_STATIC (counter);
+
+    G_LOCK (moo_temp_dir);
+
+    if (!moo_temp_dir)
+    {
+        char *dirname = NULL;
+        const char *short_name;
+
+        short_name = moo_get_prgname ();
+
+        for (i = 0; i < 1000; ++i)
+        {
+            char *basename;
+
+            basename = g_strdup_printf ("%s-%08x", short_name, g_random_int ());
+            dirname = g_build_filename (g_get_tmp_dir (), basename, NULL);
+            g_free (basename);
+
+            if (_moo_mkdir (dirname))
+            {
+                g_free (dirname);
+                dirname = NULL;
+            }
+            else
+            {
+                break;
+            }
+        }
+
+        moo_temp_dir = dirname;
+        moo_install_atexit ();
+    }
+
+    G_UNLOCK (moo_temp_dir);
+
+    g_return_val_if_fail (moo_temp_dir != NULL, NULL);
+
+    G_LOCK (counter);
+
+    for (i = counter + 1; i < counter + 1000 && !filename; ++i)
+    {
+        char *basename;
+
+        basename = g_strdup_printf ("tmpfile-%03d", i);
+        filename = g_build_filename (moo_temp_dir, basename, NULL);
+
+        if (g_file_test (filename, G_FILE_TEST_EXISTS))
+        {
+            g_free (filename);
+            filename = NULL;
+        }
+
+        g_free (basename);
+    }
+
+    counter = i;
+
+    G_UNLOCK (counter);
+
+    if (!filename)
+        g_warning ("%s: could not generate temp file name", G_STRLOC);
+
+    return filename;
+}
+
+static void
+moo_remove_tempdir (void)
+{
+    if (moo_temp_dir)
+    {
+        GError *error = NULL;
+        _moo_remove_dir (moo_temp_dir, TRUE, &error);
+
+        if (error)
+        {
+            _moo_message ("%s: %s", G_STRLOC, error->message);
+            g_error_free (error);
+        }
+
+        g_free (moo_temp_dir);
+        moo_temp_dir = NULL;
+    }
+}
+
+static void
+moo_atexit_handler (void)
+{
+    moo_remove_tempdir ();
+}
+
+static void
+moo_install_atexit (void)
+{
+    static gboolean installed;
+
+    if (!installed)
+    {
+        atexit (moo_atexit_handler);
+        installed = TRUE;
+    }
+}
+
+
+void
+moo_cleanup (void)
+{
+    moo_remove_tempdir ();
+}
+
+
 static char *
 moo_get_user_cache_dir (void)
 {
@@ -1197,6 +1301,21 @@ _moo_set_user_data_dir (const char *path)
     G_UNLOCK (moo_user_data_dir);
 }
 
+void
+_moo_set_app_instance_name (const char *name)
+{
+    if (moo_app_instance_name)
+        g_critical ("%s: app instance name already set", G_STRFUNC);
+
+    if (name && strcmp (name, "main") == 0)
+        name = NULL;
+    if (name && !name[0])
+        name = NULL;
+
+    g_free (moo_app_instance_name);
+    moo_app_instance_name = g_strdup (name);
+}
+
 
 gboolean
 moo_make_user_data_dir (const char *path)
@@ -1389,12 +1508,18 @@ _moo_get_shared_data_dirs (void)
         sys_dirs = g_get_system_data_dirs ();
 
 #ifndef __WIN32__
-        dirs = g_slist_prepend (NULL, g_strdup (g_get_user_data_dir ()));
+        dirs = g_slist_prepend (NULL, _moo_normalize_dir_path (g_get_user_data_dir ()));
 #endif
 
         for (i = 0; sys_dirs[i] && sys_dirs[i][0]; ++i)
-            if (!g_slist_find_custom (dirs, sys_dirs[i], (GCompareFunc) strcmp))
-                dirs = g_slist_prepend (dirs, g_strdup (sys_dirs[i]));
+        {
+            char *norm = _moo_normalize_dir_path (sys_dirs[i]);
+
+            if (norm && !g_slist_find_custom (dirs, norm, (GCompareFunc) strcmp))
+                dirs = g_slist_prepend (dirs, norm);
+            else
+                g_free (norm);
+        }
 
         dirs = g_slist_reverse (dirs);
         n_dirs = g_slist_length (dirs);
@@ -1462,6 +1587,26 @@ get_user_data_file (const char *basename,
     return file;
 }
 
+char *
+moo_get_named_user_data_file (const char *basename)
+{
+    char *freeme = NULL;
+    char *file;
+
+    g_return_val_if_fail (basename && basename[0], NULL);
+
+    if (moo_app_instance_name)
+    {
+        freeme = g_strdup_printf ("%s-%s", moo_app_instance_name, basename);
+        basename = freeme;
+    }
+
+    file = moo_get_user_data_file (basename);
+
+    g_free (freeme);
+    return file;
+}
+
 char *
 moo_get_user_data_file (const char *basename)
 {
@@ -1491,7 +1636,8 @@ save_with_backup (const char *filename,
     if (!_moo_save_file_utf8 (tmp_file, data, len, error))
         goto out;
 
-    _moo_rename_file (filename, bak_file, &error_move);
+    if (g_file_test (filename, G_FILE_TEST_EXISTS))
+        _moo_rename_file (filename, bak_file, &error_move);
 
     if (!_moo_rename_file (tmp_file, filename, error))
     {
@@ -1633,44 +1779,11 @@ moo_data_dir_type_get_type (void)
 }
 
 
-static gboolean
-_moo_debug_enabled (void)
-{
-#ifndef MOO_DEBUG
-    static gboolean enabled;
-    static gboolean been_here;
-
-    if (!been_here)
-    {
-        been_here = TRUE;
-        enabled = g_getenv ("MOO_DEBUG") != NULL;
-    }
-
-    return enabled;
-#else
-    return TRUE;
-#endif
-}
-
-
-void
-_moo_message (const char *format,
-              ...)
-{
-    if (_moo_debug_enabled ())
-    {
-        va_list args;
-        va_start (args, format);
-        g_logv (G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE, format, args);
-        va_end (args);
-    }
-}
-
-
 void
 _moo_widget_set_tooltip (GtkWidget  *widget,
                          const char *tip)
 {
+#if !GTK_CHECK_VERSION(2,11,6)
     static GtkTooltips *tooltips;
 
     g_return_if_fail (GTK_IS_WIDGET (widget));
@@ -1682,6 +1795,14 @@ _moo_widget_set_tooltip (GtkWidget  *widget,
         gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (widget), tooltips, tip, NULL);
     else
         gtk_tooltips_set_tip (tooltips, widget, tip, tip);
+#else
+    g_return_if_fail (GTK_IS_WIDGET (widget));
+
+    if (GTK_IS_TOOL_ITEM (widget))
+        gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (widget), tip);
+    else
+        gtk_widget_set_tooltip_text (widget, tip);
+#endif
 }
 
 
@@ -1810,7 +1931,7 @@ moo_strnsplit_lines (const char *string,
 
 
 char **
-_moo_splitlines (const char *string)
+moo_splitlines (const char *string)
 {
     GPtrArray *array;
     const char *line, *p;
@@ -1853,10 +1974,9 @@ _moo_splitlines (const char *string)
         }
     }
 
-    if (p > line)
-        g_ptr_array_add (array, g_strndup (line, p - line));
-
+    g_ptr_array_add (array, g_strndup (line, p - line));
     g_ptr_array_add (array, NULL);
+
     return (char**) g_ptr_array_free (array, FALSE);
 }
 
@@ -1880,26 +2000,78 @@ _moo_strv_reverse (char **str_array)
     return str_array;
 }
 
+char **
+_moo_ascii_strnsplit (const char *string,
+                      gssize      len,
+                      guint       max_tokens)
+{
+    GPtrArray *array = NULL;
+    const char *end;
 
-#if defined(__WIN32__)
+    g_return_val_if_fail (string != NULL, NULL);
+
+    if (len < 0)
+        len = strlen (string);
+    if (max_tokens < 2)
+        max_tokens = G_MAXUINT;
+
+    end = string + len;
+
+    while (string != end)
+    {
+        const char *p;
+
+        while (string != end && g_ascii_isspace (*string))
+            string++;
+
+        if (string == end)
+            break;
+
+        for (p = string; p != end && !g_ascii_isspace (*p); ++p) ;
+
+        if (!array)
+            array = g_ptr_array_new ();
+
+        g_ptr_array_add (array, g_strndup (string, p - string));
+
+        if (array->len + 1 == max_tokens)
+        {
+            g_ptr_array_add (array, g_strndup (p, end - p));
+            break;
+        }
+
+        string = p;
+    }
+
+    if (array)
+    {
+        g_ptr_array_add (array, NULL);
+        return (char**) g_ptr_array_free (array, FALSE);
+    }
+    else
+    {
+        return NULL;
+    }
+}
+
+
+#if defined(__WIN32__) && !defined(MOO_DEBUG_ENABLED)
 static guint saved_win32_error_mode;
 #endif
 
 void
 moo_disable_win32_error_message (void)
 {
-#if defined(__WIN32__)
-    if (!_moo_debug_enabled ())
-        saved_win32_error_mode = SetErrorMode (SEM_NOOPENFILEERRORBOX | SEM_FAILCRITICALERRORS);
+#if defined(__WIN32__) && !defined(MOO_DEBUG_ENABLED)
+    saved_win32_error_mode = SetErrorMode (SEM_NOOPENFILEERRORBOX | SEM_FAILCRITICALERRORS);
 #endif
 }
 
 void
 moo_enable_win32_error_message (void)
 {
-#if defined(__WIN32__)
-    if (!_moo_debug_enabled ())
-        SetErrorMode (saved_win32_error_mode);
+#if defined(__WIN32__) && !defined(MOO_DEBUG_ENABLED)
+    SetErrorMode (saved_win32_error_mode);
 #endif
 }
 
@@ -2057,22 +2229,74 @@ _moo_io_add_watch_full (GIOChannel    *channel,
 const char *
 _moo_intern_string (const char *string)
 {
-    static GHashTable *hash;
-    gpointer original;
-    gpointer dummy;
-
     if (!string)
         return NULL;
 
-    if (G_UNLIKELY (!hash))
-        hash = g_hash_table_new (g_str_hash, g_str_equal);
-
-    if (!g_hash_table_lookup_extended (hash, string, &original, &dummy))
+#if GLIB_CHECK_VERSION(2,10,0)
+    return g_intern_string (string);
+#else
     {
-        char *copy = g_strdup (string);
-        g_hash_table_insert (hash, copy, NULL);
-        original = copy;
+        static GHashTable *hash;
+        gpointer original;
+        gpointer dummy;
+
+        if (G_UNLIKELY (!hash))
+            hash = g_hash_table_new (g_str_hash, g_str_equal);
+
+        if (!g_hash_table_lookup_extended (hash, string, &original, &dummy))
+        {
+            char *copy = g_strdup (string);
+            g_hash_table_insert (hash, copy, NULL);
+            original = copy;
+        }
+
+        return original;
+    }
+#endif
+}
+
+
+gboolean
+moo_debug_enabled (const char *domain,
+                   gboolean    def_enabled)
+{
+    const char *val;
+
+    val = g_getenv ("MOO_DEBUG");
+
+    if (!val || !val[0])
+        return def_enabled;
+    if (!strcmp (val, "none"))
+        return FALSE;
+    if (!strcmp (val, "all"))
+        return TRUE;
+
+    return strstr (val, domain) != NULL;
+}
+
+
+#undef _moo_message
+void
+_moo_message (const char *format,
+              ...)
+{
+    static int enabled = -1;
+
+    if (enabled == -1)
+    {
+        enabled = moo_debug_enabled ("misc",
+#ifdef MOO_DEBUG_ENABLED
+                                     TRUE);
+#else
+                                     FALSE);
+#endif
     }
 
-    return original;
+    if (enabled)
+    {
+        va_list args;
+        va_start (args, format);
+        g_logv (G_LOG_DOMAIN "-debug", G_LOG_LEVEL_MESSAGE, format, args);
+        va_end (args);
+    }
 }
