@@ -1,12 +1,11 @@
 /*
  *   moowindow.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License version 2.1 as published by the Free Software Foundation.
  *
  *   See COPYING file that comes with this distribution.
  */
@@ -19,7 +18,6 @@
 #include "mooutils/mooaccel.h"
 #include "mooutils/mooprefs.h"
 #include "mooutils/moomarshals.h"
-#include "mooutils/moocompat.h"
 #include "mooutils/moostock.h"
 #include "mooutils/mooactionfactory.h"
 #include "mooutils/mooi18n.h"
@@ -146,7 +144,7 @@ enum {
 };
 
 
-static guint signals[LAST_SIGNAL] = {0};
+static G_GNUC_UNUSED guint signals[LAST_SIGNAL] = {0};
 
 
 /* MOO_TYPE_WINDOW */
@@ -190,7 +188,6 @@ moo_window_class_init (MooWindowClass *klass)
                                  "action-type::", MOO_TYPE_TOGGLE_ACTION,
                                  "display-name", _("Show Menubar"),
                                  "label", _("Show Menubar"),
-                                 "no-accel", TRUE,
                                  "condition::active", "menubar-visible",
                                  NULL);
 
@@ -501,25 +498,20 @@ moo_window_set_property (GObject      *object,
                          const GValue *value,
                          GParamSpec   *pspec)
 {
-    char *tmp;
     const char *name = NULL;
     MooWindow *window = MOO_WINDOW (object);
 
     switch (prop_id)
     {
         case PROP_TOOLBAR_UI_NAME:
-            tmp = window->priv->toolbar_ui_name;
             name = g_value_get_string (value);
-            window->priv->toolbar_ui_name = name ? g_strdup (name) : g_strdup ("");
-            g_free (tmp);
+            MOO_ASSIGN_STRING (window->priv->toolbar_ui_name, name ? name : "");
             g_object_notify (object, "toolbar-ui-name");
             break;
 
         case PROP_MENUBAR_UI_NAME:
-            tmp = window->priv->menubar_ui_name;
             name = g_value_get_string (value);
-            window->priv->menubar_ui_name = name ? g_strdup (name) : g_strdup ("");
-            g_free (tmp);
+            MOO_ASSIGN_STRING (window->priv->menubar_ui_name, name ? name : "");
             g_object_notify (object, "menubar-ui-name");
             break;
 
@@ -747,10 +739,11 @@ toolbar_style_toggled (MooWindow            *window,
 }
 
 
-#define N_STYLES 3
+#define N_STYLES 4
 #define ICONS_ONLY "icons-only"
 #define LABELS_ONLY "labels-only"
 #define ICONS_AND_LABELS "icons-and-labels"
+#define ICONS_AND_LABELS_HORIZ "icons-and-labels-horiz"
 
 static GtkAction*
 create_toolbar_style_action (MooWindow      *window,
@@ -764,16 +757,18 @@ create_toolbar_style_action (MooWindow      *window,
     const char *labels[N_STYLES] = {
         N_("_Icons Only"),
         N_("_Labels Only"),
-        N_("Icons _and Labels")
+        N_("Labels _Below Icons"),
+        N_("Labels Be_side Icons")
     };
 
     const char *ids[N_STYLES] = {
         ICONS_ONLY,
         LABELS_ONLY,
-        ICONS_AND_LABELS
+        ICONS_AND_LABELS,
+        ICONS_AND_LABELS_HORIZ
     };
 
-    action = moo_menu_action_new (TOOLBAR_STYLE_ACTION_ID, "Toolbar _Style");
+    action = moo_menu_action_new (TOOLBAR_STYLE_ACTION_ID, _("Toolbar _Style"));
     menu_mgr = moo_menu_action_get_mgr (MOO_MENU_ACTION (action));
     moo_menu_mgr_set_use_mnemonic (menu_mgr, TRUE);
 
@@ -793,7 +788,8 @@ create_toolbar_style_action (MooWindow      *window,
 }
 
 
-static GtkToolbarStyle get_toolbar_style_gtk (MooWindow *window)
+static GtkToolbarStyle
+get_toolbar_style_gtk (MooWindow *window)
 {
     GtkSettings *settings = gtk_widget_get_settings (GTK_WIDGET (window));
     GtkToolbarStyle style = GTK_TOOLBAR_ICONS;
@@ -809,8 +805,8 @@ static GtkToolbarStyle get_toolbar_style_gtk (MooWindow *window)
     return style;
 }
 
-
-static void init_prefs (MooWindow *window)
+static void
+init_prefs (MooWindow *window)
 {
     moo_prefs_new_key_bool (setting (window, PREFS_REMEMBER_SIZE), TRUE);
     moo_prefs_new_key_bool (setting (window, PREFS_SHOW_TOOLBAR), TRUE);
@@ -820,17 +816,18 @@ static void init_prefs (MooWindow *window)
                             get_toolbar_style_gtk (window));
 
     moo_prefs_create_key (setting (window, PREFS_MAXIMIZED), MOO_PREFS_STATE, G_TYPE_BOOLEAN, FALSE);
-    moo_prefs_create_key (setting (window, PREFS_WIDTH), MOO_PREFS_STATE, G_TYPE_INT, -1);
-    moo_prefs_create_key (setting (window, PREFS_HEIGHT), MOO_PREFS_STATE, G_TYPE_INT, -1);
+    moo_prefs_create_key (setting (window, PREFS_WIDTH), MOO_PREFS_STATE, G_TYPE_INT, 800);
+    moo_prefs_create_key (setting (window, PREFS_HEIGHT), MOO_PREFS_STATE, G_TYPE_INT, 600);
 }
 
-
-static GtkToolbarStyle get_toolbar_style (MooWindow *window)
+static GtkToolbarStyle
+get_toolbar_style (MooWindow *window)
 {
     GtkToolbarStyle s = moo_prefs_get_enum (setting (window, PREFS_TOOLBAR_STYLE));
     g_return_val_if_fail (s < N_STYLES, GTK_TOOLBAR_ICONS);
     return s;
 }
+
 #undef N_STYLES
 
 
