@@ -348,8 +348,7 @@ cache_alias_lookup (const char *alias)
 
 static int
 cache_glob_lookup_literal (const char *file_name,
-			   const char *mime_types[],
-			   int         n_mime_types)
+			   const char *mime_types[])
 {
   const char *ptr;
   int i, min, max, mid, cmp;
@@ -478,7 +477,8 @@ cache_glob_node_lookup_suffix (XdgMimeCache *cache,
 		  if (match_char != 0)
 		    break;
 
-		  mime_types[n++] = cache->buffer + mimetype_offset;
+		  if (cache->buffer[mimetype_offset])
+		    mime_types[n++] = cache->buffer + mimetype_offset;
 		  i++;
 		}
 
@@ -578,7 +578,7 @@ cache_glob_lookup_file_name (const char *file_name,
   assert (file_name != NULL);
 
   /* First, check the literals */
-  n = cache_glob_lookup_literal (file_name, mime_types, n_mime_types);
+  n = cache_glob_lookup_literal (file_name, mime_types);
   if (n > 0)
     return n;
 
@@ -713,7 +713,7 @@ _xdg_mime_cache_get_mime_type_for_file (const char  *file_name,
   if (data == NULL)
     return XDG_MIME_TYPE_UNKNOWN;
         
-  file = fopen (file_name, "r");
+  file = fopen (file_name, "rb");
   if (file == NULL)
     {
       free (data);
